/*
 * We have experiment version already. But we want
 * make sure how the gcc command line options -g , -O
 * contribute to the performance measure. So we have
 * this version of the code which only includes the 
 * very basic ECho channel creation and source subscribing
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <io.h>
#include <atl.h>
#include <gen_thread.h>
#include <cm.h>
#include <echo.h>


static char* local_progname;
EChannel chan;
CManager cm;
ECSourceHandle sourceHandle;
EControlContext ec;

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  char *buff; 
} Raw_data, *Raw_data_ptr;


IOField Raw_data_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data_ptr,height)},
  {"buff", "char[size]", sizeof(char), IOOffset(Raw_data_ptr,buff)},
  {NULL,NULL},
};

#include <time.h>
#include <sys/timeb.h>
#include <stdarg.h>
int tprintf(const char* fmt,...)
{
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}


char file_name[] = "time_rec";
char chan_id_file[] = "chanId";
FILE* time_rec, *chan_file;
/*the interval between 2 frames, in microsec*/
unsigned long interval;
struct timeval prev_time, curr_time;
double time_diff;
int frame_transfered = 0;


static void
interruptHandler(signo)
     int signo;
{
  gettimeofday(&curr_time,NULL);

  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
  time_diff = time_diff/1000000;
  fprintf(time_rec," Streaming:%f,  frames:%d\n",time_diff,frame_transfered);
  fclose(time_rec);

  EChannel_destroy(chan);
  CManager_close(cm);

  exit(EXIT_SUCCESS);
} /* end interruptHandler */

int
main(argc, argv)
     int argc; 
     char* argv[];
{
//  EControlContext ec;

  //Raw_data rawrec;
  //Raw_data_ptr rawrecP = &rawrec;

  //char* tmp = NULL;

  local_progname = argv[0];  
  

  /*
   * Creation of channel and registration
   *
   */
  if (NULL == (time_rec=fopen(file_name, "a")) || 
      NULL == (chan_file=fopen(chan_id_file,"w"))) {
    printf("file open error!\n");
    exit(1);
  }
  printf(" file open for writing\n");
  gettimeofday(&prev_time,NULL);


  gen_pthread_init();
  cm = CManager_create();  


  CMfork_comm_thread(cm);


  ec = ECho_CM_init(cm);

  //if (signal(SIGINT, interruptHandler) == SIG_ERR) 
  //  fprintf(stderr,"Signal error\n");

  chan = EChannel_typed_create(ec, Raw_data_fld, NULL);
  if (chan == NULL) fprintf(stderr,"Failed to create raw channel.\n");

  sourceHandle = ECsource_typed_subscribe(chan, Raw_data_fld, NULL);

  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
  fprintf(stdout, "Echo channel ID:\n %s\n", ECglobal_id(chan));
  fprintf(chan_file,"%s\n",ECglobal_id(chan));
  fclose(chan_file);

  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
  gettimeofday(&curr_time,NULL);
  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
  fprintf(time_rec," Init:%f, \n",time_diff);
  fflush(time_rec);

  while (EChas_sinks(sourceHandle) < 1)
    CMusleep(cm, 1000);
  
  exit (EXIT_SUCCESS);
} /* main */
