/*
 *
 *  $Id: avs_viewMaster.cc,v 1.1.1.1 2003/06/11 18:48:52 wyan Exp $
 *
 */
#include <avs_viewMaster.h>

extern "C"
{
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include "io.h"
#include "gen_thread.h"
#include "atl.h"
#include "cm.h"
#include "echo.h"
#include "pds.h"
}

extern void exception_check (JNIEnv* e);

/*vvv*/
/* because I am not gonna use viewMaster, so the 2 var below is 
   defined as extern -- see cm and ecc in viewClient */
//CManager cm;
//EControlContext ecc;
extern CManager cm;
extern EControlContext ecc;
/*^^^*/

thr_mutex_t my_mutex;
pds_service directory;
ECSinkHandle context_change_sink_handle;
static char stat_buf[100];
jmethodID status_method_id;

static void
  status (JNIEnv *env, jobject jo, char *s)
{
  env->CallVoidMethod (jo, status_method_id, env->NewStringUTF (s));
}

#define STATUS_1(FMT,ARG1)   \
  memset (stat_buf, 0, 100);\
  sprintf (stat_buf, (FMT), (ARG1));\
  status (env, jo, stat_buf);

#define STATUS_2(FMT,ARG1,ARG2)   \
  memset (stat_buf, 0, 100);\
  sprintf (stat_buf, (FMT), (ARG1), (ARG2));\
  status (env, jo, stat_buf);

#define STATUS_3(FMT,ARG1,ARG2,ARG3)   \
  memset (stat_buf, 0, 100);\
  sprintf (stat_buf, (FMT), (ARG1), (ARG2), (ARG3);\
  status (env, jo, stat_buf);

void 
context_change_handler (void* event, void* client_data, attr_list alist)
{

}


extern "C" {

JNIEXPORT void JNICALL
Java_avs_ViewMaster_startup (JNIEnv *env, jobject jo)
{
  gen_pthread_init();
  my_mutex = thr_mutex_alloc();

  thr_mutex_lock (my_mutex);

  cm = CManager_create();
  if (cm == NULL)
    {
      fprintf (stderr, "couldn't initialize CM, stopping.\n");
      thr_mutex_unlock (my_mutex);
      return;
    }

  ecc = ECho_CM_init (cm);

  status_method_id = env->GetMethodID (env->GetObjectClass (jo),
                                       "status", 
                                       "(Ljava/lang/String;)V");

  thr_mutex_unlock (my_mutex);
  fprintf (stderr, "out of startup\n");
}


JNIEXPORT void JNICALL
Java_avs_ViewMaster_getStreams (JNIEnv *env, jobject jo, 
                                jstring hostname,
                                jint port, 
                                jstring context)
{
  attr_list contact_attrs;
  attr_list attrs;
  const char *c_hostname;
  const char *c_context;
  pds_domain_id_t domain_id;
  jmethodID new_stream_method_id;

  char** bindings;
  char** camera_bindings;

  c_hostname = env->GetStringUTFChars (hostname, (jboolean*)0);
  c_context = env->GetStringUTFChars (context, (jboolean*)0);

  contact_attrs = create_attr_list();
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_HOST"),
            Attr_String,
            strdup (c_hostname));
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_PORT"),
            Attr_Int4,
            (attr_value*)8848);

  STATUS_2("Contacting directory server at %s:%d...",c_hostname,8848);

  directory = pds_service_open (contact_attrs);

  domain_id = pds_open_domain (directory,
                               "demo",
                               "demo",
                               1,
                               "avs");

  bindings = pds_get_binding_list (domain_id, c_context);

  new_stream_method_id = env->GetMethodID (env->GetObjectClass (jo),
                                           "add_new_stream", 
                                           "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");
  exception_check (env);

  for (int j = 0; bindings[j] != NULL; j++)
    {
      char *e_name;

      STATUS_1("Found camera: %s", bindings[j]);
      
      /*
       *  for each bound thing, get the entity corresponding
       *  to it and probe the attribute list for things to
       *  return.
       *  Store the entity ids for later reference?
       */
      e_name = (char*) calloc (strlen (c_context) + strlen (bindings[j]) + 2,
                               sizeof (char));
      strcpy (e_name, c_context);
      strcat (e_name, "/");
      strcat (e_name, bindings[j]);

      /*
       *  get the bindings for each camera.  these are entities with attributes
       *  that describe their image streams
       */

      camera_bindings = pds_get_binding_list (domain_id, e_name);
      for (int k = 0; camera_bindings[k] != NULL; k++)
        {
          char cam_e_name[200];
          char *desc_str = NULL;
          char *chan_id = NULL;
          char *format = NULL;   
          char *format_tag = NULL;

          memset (cam_e_name, 0, 200);
          strcpy (cam_e_name, e_name);
          strcat (cam_e_name, "/");
          strcat (cam_e_name, camera_bindings[k]);

          attrs = create_attr_list();
          pds_get_entity_attributes (domain_id, cam_e_name, null_pds_context_id, attrs);
          //      dump_attr_list (attrs);
          
          query_attr (attrs, attr_atom_from_string ("StreamDescription"), NULL,
                      (void**)&desc_str);
          query_attr (attrs, attr_atom_from_string ("StreamChannelID"), NULL,
                      (void**)&chan_id);
          query_attr (attrs, attr_atom_from_string ("StreamFormatDescription"), NULL,
                      (void**)&format);
          query_attr (attrs, attr_atom_from_string ("StreamFormatTag"), NULL,
                      (void**)&format_tag);
          
          
          fprintf (stdout, "\ndesc is: (%s)\nchan ID is: (%s)\nformat is: (%s)\n", 
                   desc_str, chan_id, format); 
          
          env->CallVoidMethod (jo, new_stream_method_id, env->NewStringUTF (desc_str), 
                               env->NewStringUTF (chan_id), env->NewStringUTF (format),
                               env->NewStringUTF (format_tag));
          exception_check (env);

          free_attr_list (attrs);
          free (camera_bindings[k]);
        }

      free (e_name);
      free (camera_bindings);
      free (bindings[j]);
    }

  free (bindings);

  /*
   *  subscribe for changes to the context 
   */
  /*  
  context_change_sink_handle = pds_register_for_context_changes (cm,
                                                                 domain_id,
                                                                 c_context,
                                                                 null_pds_context_id,
                                                                 context_change_handler,
                                                                 NULL);
  */
  free_attr_list (contact_attrs);

  pds_service_close (directory);

  env->ReleaseStringUTFChars (hostname, c_hostname);
  env->ReleaseStringUTFChars (context, c_context);

}


}

/*
 *
 *  $Log: avs_viewMaster.cc,v $
 *  Revision 1.1.1.1  2003/06/11 18:48:52  wyan
 *  UAV1
 *
 *  Revision 1.6  2001/09/29 04:14:37  pmw
 *  more interim checkins.  crop rectangle in place for GUI.
 *
 *  Revision 1.5  2001/09/26 21:13:38  pmw
 *  interim checkin so I don't lose anything
 *
 *  Revision 1.4  2001/09/21 21:28:59  pmw
 *  interim checkin...
 *
 *  Revision 1.3  2001/09/18 20:39:16  pmw
 *  fixed several UI issues.  not quite working... ALMOST!
 *
 *  Revision 1.2  2001/09/07 18:42:58  pmw
 *  changes for directory service integration
 *
 *  Revision 1.1  2001/08/22 22:05:24  pmw
 *  changes for directory service - not yet complete
 *
 *
 */
