package avs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import graph.*;

public class BandwidthPane extends JPanel implements PlotPane, Runnable
{
    Graph2D graph;
    Label title;
    DataSet data1 = new DataSet();
    DataSet data2;// = new DataSet();//for x-axis

    Axis    yaxis_right;
    Axis    xaxis_bottom;

    Image    osi;
    Graphics osg;
    int iwidth;
    int iheight;
    
    Thread runner;
    int count;

    //The bookkeeping param
    int prev_count, curr_count;
    int prev_size, curr_size;
    double prev_fps, curr_fps;
    double prev_bandwidth, curr_bandwidth;
    //Ready to update the bookkeeping params?
    boolean write_ready = true;
    boolean read_ready = false;

    public BandwidthPane() {
	setDoubleBuffered(true);
        String st       = "Bandwidth (Mbps)";

        graph = new Graph2D();
        graph.zerocolor = new Color(0,0,0);
        graph.borderTop    = 10;
        graph.borderBottom = 10;

        graph.setDataBackground(new Color(240,240,0)/*Color.yellow*/);

        title = new Label(st, Label.CENTER);
        title.setFont(new Font("Courier",Font.BOLD,15));
	
        setLayout( new BorderLayout() );
        add(title, BorderLayout.NORTH);
	add(graph, BorderLayout.CENTER);

        try {
	    graph.setMarkers(new Markers(mfile));
        } catch(Exception e) {
           System.out.println("Failed to create Marker file!");
        }

        data1.linecolor   = new Color(0,255,0);
        data1.marker      = 8;//large point
        data1.markercolor = new Color(100,100,255);

	yaxis_right = graph.createAxis(Axis.RIGHT);
	yaxis_right.attachDataSet(data1);
	yaxis_right.setLabelFont(new Font("Helvetica",Font.PLAIN,10));

        graph.attachDataSet(data1);
        graph.attachDataSet(data2);
    }

    synchronized public void paint(Graphics g) {
                   g = graph.getGraphics();
		   Dimension d = graph.getSize();

		   if( iwidth != d.width || iheight != d.height  ) {
                       iwidth = d.width;
                       iheight = d.height;
                       osi = graph.createImage(iwidth,iheight);
                       osg = osi.getGraphics();
		   }

		   try {
		       osg.setColor(graph.getBackground());
		       osg.fillRect(0,0,iwidth,iheight);
		       osg.setColor(g.getColor());
		       osg.clipRect(0,0,iwidth,iheight);

		       graph.update(osg);

		       g.drawImage(osi,0,0,graph);

		   }
		   finally {
		       osg.dispose();
		   }

    }

    public void run() {
	int i =0;
	double data[] = new double[2];
        double xdata[] = new double[2*MAX_NUM_POINTS];

        for (int ii = 0; ii<MAX_NUM_POINTS; ii++) 
            {xdata[ii] = 0;xdata[2*ii+1] = -5*ii;}

        data2 = graph.loadDataSet(xdata,MAX_NUM_POINTS);

        xaxis_bottom = graph.createAxis(Axis.BOTTOM);
        xaxis_bottom.attachDataSet(data2);
        xaxis_bottom.setLabelFont(new Font("Helvetica",Font.PLAIN,8));
        xaxis_bottom.setTitleText("Sec");

        double keep_prev=0;//Keep the previous point to avoid spikes in graph

	while(true) {
	    count++;

	    if(count > MAX_NUM_POINTS) data1.delete(0,0);

	    if (read_ready == true) {
	        write_ready = false;

		if (prev_count >= curr_count) {
		    prev_count = 0;
		}
		else {
		    int tmp = curr_count - prev_count;
		    curr_fps = (double)tmp / (double)FREQ;
		    curr_bandwidth = curr_fps * curr_size / 131072.0; /*131072=1M/8bits*/
		    curr_bandwidth = (curr_bandwidth<=MAX_BANDWIDTH?
				      curr_bandwidth:MAX_BANDWIDTH);
		    /*
		    System.out.println(" curr_count:"+curr_count+", curr_size:"+
				       curr_size+", curr_bw"+curr_bandwidth+
				       ", prev_count:"+prev_count+", prev_size:"+
				       prev_size);
		    System.out.println("");
		    */
		}
	    }
		
	    if (prev_count == 0) {
		data[1] = keep_prev;
	    }
	    else {
		data[1] = keep_prev = curr_bandwidth;
	    }
	    data[0] = count;

	    /**/
	    prev_count = curr_count;
	    prev_size = curr_size;
	    //prev_fps = curr_fps;
	    
	    try {
		data1.append(data,1);
	    }
	    catch (Exception e) {
		System.out.println("Error appending Data!");
	    }

	    data1.yaxis.maximum = 10.0;/*10.0;*/
	    data1.yaxis.minimum = 0.0;
	    data2.xaxis.maximum = 0;//(MAX_NUM_POINTS-1)*5;
            data2.xaxis.minimum = -(MAX_NUM_POINTS-1)*5;

	    repaint();

	    write_ready = true;
	    read_ready = false;
	
	    try {  Thread.sleep(FREQ*1000); }
	    catch(Exception e) { }
	}
    }

    public void startdraw() {
	Thread  runner = new Thread(this);
	runner.start();
    }

    public synchronized void setValue(int frame_count, int size)
    {
	if (write_ready == true) {
	    curr_count = frame_count;
	    curr_size = size;
	    read_ready = true;
	}
    }
}


