package avs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class PPMQos implements AvsQos, Runnable
{
    //For now, we only consider the Qos related to the fps and certain
    //bandwidth.
    final float MAX_FPS = 15;
    final float MIN_FPS = .5f;
    final float MAX_BANDWIDTH=10;/* USB controller max at 12Mbps */
    final float MIN_BANDWIDTH=.5f;
    float FPS_LOW = 0;//init values
    float FPS_HIGH = 15;
    boolean needQos;

    //For now, only consider 2C and 3C PPM.
    Vector contractV = new Vector(2);
    //the reference to the ViewClient
    static ViewClient vc;

    //the Qos measurement
    float qosFPS;
    float qosBandwidth;

    int width;
    int height;
    
    Thread runner;
    //int count;

    //The bookkeeping param
    int prev_count, curr_count;
    int prev_size, curr_size;
    double prev_fps, curr_fps;
    double prev_bandwidth, curr_bandwidth;
    //Ready to update the bookkeeping params?
    boolean write_ready = true;
    boolean read_ready = false;

    public PPMQos(ViewClient vc) {
	this.vc = vc;
	//320x240 Color
	contractV.add(new QosContract("2C",230400));
	//160x120 Color
	contractV.add(new QosContract("3C",57600));
    }

    /*synchronized*/ public void applyQos() {
	if (width == 320 && height ==240) {
	    //Should use the following QosContract.
	    //But now just use the threshold passed from the slider
	    //in class ViewClient.
	    //QosContract qc = findElement("2C");
	    //if (qosFPS < qc.fpsLow ) {
	    if (qosFPS < FPS_LOW) {
		//320*240 color
		//Need to do QoS adaptation: scale down size for now.
		//See ../playerBBN/AdaptAvsUav_Impl.java for the meaning
		//of the arguments.
		vc.sourceAdapt("shrinkImage","2Cto3C",(short)1,(short)1);
	    }
	}
	
	if (width == 160 && height ==120) {
	    //QosContract qc = findElement("3C");
	    //if (qosFPS > qc.fpsHigh) {
	    if (qosFPS > FPS_HIGH) {
		//160x120 color
		vc.sourceAdapt("shrinkImage","2Cto3C",(short)0,(short)1);
	    }
	}
    }

    public QosContract findElement(String name) {
	QosContract qc;
	for (Enumeration e = contractV.elements();e.hasMoreElements();) {
	    qc = (QosContract)e.nextElement();
	    if (qc.imageName.equals("2C") || qc.imageName.equals("3C")) {
		return qc;
	    }
	}
	return null;
    }

    public void run() {
	int i =0;

        float keep_prev=0;//Keep the previous point to avoid spikes in graph

	while(true) {

	    if (read_ready == true) {
	        write_ready = false;

		if (prev_count >= curr_count) {
		    prev_count = 0;
		}
		else {
		    int tmp = curr_count - prev_count;
		    curr_fps = (double)tmp / (double)FREQ;
		    curr_bandwidth = curr_fps * curr_size / 131072.0; /*131072=1M/8bits*/
		    curr_bandwidth = (curr_bandwidth<=MAX_BANDWIDTH?
				      curr_bandwidth:MAX_BANDWIDTH);
		    /**/
		    System.out.println(" curr_count:"+curr_count+", curr_size:"+
				       curr_size+", curr_bw"+curr_bandwidth+
				       ", prev_count:"+prev_count+", prev_size:"+
				       prev_size);
		    System.out.println(" FPS_LOW="+FPS_LOW+
				       ", FPS_HIGH="+FPS_HIGH+
				       ", curr_fps="+curr_fps);
		    System.out.println("");
		    /**/
		}
	    }
		
	    if (prev_count == 0) {
		qosFPS = keep_prev;
	    }
	    else {
		qosFPS = keep_prev = (float)curr_fps;
	    }

	    prev_count = curr_count;
	    prev_size = curr_size;
	    //prev_fps = curr_fps;
	    if (needQos)
		applyQos();

	    write_ready = true;
	    read_ready = false;
	
	    try {  Thread.sleep(FREQ*1000); }
	    catch(Exception e) { }
	}
    }

    public void startQos() {
	Thread  runner = new Thread(this);
	runner.start();
    }

    public synchronized void setValue(int frame_count, int size, int w, int h,
				      boolean qos,float fps_low,
				      float fps_high)
    {
	if (write_ready == true) {
	    curr_count = frame_count;
	    curr_size = size;

	    needQos = qos;
	    FPS_LOW = fps_low;
	    FPS_HIGH = fps_high;
	    width = w;
	    height = h;
	    read_ready = true;
	}
    }
}

class QosContract implements AvsQos {
    public String imageName;
    public int size;//in bytes
    public float fpsLow;
    public float fpsHigh;

    public QosContract(String name,int size) {
	imageName = name;
	this.size = size;
	fpsLow = BANDWIDTH_LOWER_LIMIT*131072.0f/size;
	fpsHigh = BANDWIDTH_UPPER_LIMIT*131072.0f/size;
    }
}
