/* -*-C-*-
   @author: Yan, Wenchang
   @time: Dec, 2001
*/

#include "imagesender.h"

#undef DEBUG_WHERE
#define DEBUG_WHERE 0
/*for debug*/
int tprintf(const char* fmt,...)
{
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}

/*send the PPM header info*/
int send_preframe(u_char* PPMHeader, PPM_frame_info* pfi,
                  int socket, struct sockaddr* receiver)
{
  u_char* buf;
  int sendresult;
  int PPMHeader_length = 4 + 4 + sizeof(PPM_frame_info);
 
  if (!(buf = (u_char*)malloc(PPMHeader_length*sizeof(u_char)))) {
    if (DEBUG_WHERE) tprintf(" [imagereader::send_preframe(): 0ut of mem\n");
    exit(-1);
  }
 
  //if (DEBUG_WHERE) tprintf(" [imagereader::send_preframe(): tag=%d,ppm1=%c,ppm2=%c,size=%d,width=%d,height=%d,*buff=%u\n",pfi->tag,pfi->ppm1,pfi->ppm2,pfi->size,pfi->width,pfi->height,pfi->buff);
 
  memcpy(buf,PPMHeader,4+4);
  memcpy(buf+4+4,pfi,sizeof(PPM_frame_info));
 
  //if (DEBUG_WHERE) tprintf(" [imagereader::send_preframe(): PPMHeader_length=%d, buf[0]=%x, buf[1]=%x, buf[2]=%x, buf[3]=%x\n",PPMHeader_length,buf[0],buf[1],buf[2],buf[3]);
 
  sendresult = sendto(socket, buf, PPMHeader_length/*actual_size*/, 0,
                receiver,sizeof(*receiver));
 
  //sendresult = sendto(socket,buf,4,0,receiver,sizeof(*receiver));
 
  if (sendresult<0) {
    if (DEBUG_WHERE) tprintf(" [imagereader::send_preframe(): cannot send \n");
    exit(-1);
  }

  free(buf);

  return sendresult;
}
 
int send_frame(int sd,u_char* frame,int frame_size,int frame_num,
	       PPM_frame_info_ptr pfip, struct sockaddr_in &rec,
	       int mus, int maxpixelsize, int maxpixelperline)
{
  int sendresult;
  //int portno = RECEIVER_PORT;
  u_char* buf;
  int header_idx;
  int ii;
  uint32 icopy;
  int line_size,num_lines, pixel_size, lines_send_per_time, num_times;     
  int row;
  int stream_length,bytes_read;
  //for debug
  int j = 0;

  int ppm_frame_header = ToBigEndian32(PPMP_FRAME_HEADER);
  int ppm_pict_header = ToBigEndian32(PPMP_PICT_HEADER);
  /* packet format:
     PPMHeader 4bytes + Frame# 4bytes + PPM_frame_info
     PictHeader 4bytes + Frame# 4bytes + Data MUS bytes
     PictHeader 4bytes + Frame# 4bytes + Data MUS byte
     PictHeader 4bytes + Frame# 4bytes + Data MUS byte
     ...
     REPEAT several times to send the whole 480*640*3 bytes PPM frame
  */
  if (!(buf = (u_char*)malloc((mus+maxpixelsize*maxpixelperline
                               +12+sizeof(PPM_frame_info))*sizeof(u_char))))
    {
      if (DEBUG_WHERE) tprintf(" [imagereader::main(): 0ut of mem\n");
      exit(-1);
    }
 
  line_size = pfip->width;
  num_lines = pfip->height;
  pixel_size = pfip->size/pfip->width/pfip->height;

  /*fill up pfi and other info needed in udp trans*/
 
  lines_send_per_time = (int)(((double)mus)/((double)line_size*pixel_size) + .9999);
 
  num_times =(int)(((double)num_lines/(double)lines_send_per_time)+.9999);

  /*nothing has been read yet*/
  bytes_read=0;
  for (row=0;row<num_times;row++) {
    header_idx =0;
    
    //curr frame #
    icopy = ToBigEndian32(frame_num);
    /* if it's the first time reading from the PPM file */
    /* Send first the PPMHeader  */
    if (row == 0) {
      /*
      buf[0]=buf[1]=0;
      buf[2]=1; buf[3]=0xff;
      */
      memcpy(buf,&ppm_frame_header,4);
      //fill in frame #
      memcpy(&buf[4],&frame_num,4);
      sendresult=send_preframe(buf, pfip,
			       sd, (struct sockaddr*)&rec);
      
      usleep(100);
      //sendresult = sendto(sd, buf, 4, 0,
      //                     (struct sockaddr*)&receiver,sizeof(receiver));
    }
    else {
      //actual_size = MUS+8;
    }
    /* Now append the PictHeader*/
    /*
    buf[header_idx]=buf[header_idx+1]=0;
    buf[header_idx+2]=1;
    buf[header_idx+3]=0xf0;
    */
    memcpy(buf,&ppm_pict_header,4);
    header_idx+=4;
    /*Now the frame number*/
    memcpy(&buf[header_idx],&icopy,4);
    //if (DEBUG_WHERE) tprintf(" [Imagereader: buf[]=%x, %x, %x, %x\n",buf[header_idx],
    //buf[header_idx+1],buf[header_idx+2],buf[header_idx+3]);
    
    header_idx+=4;
    /*Now the bitmap data*/
    buf += header_idx;
    
    int tmp1 = lines_send_per_time*line_size*pixel_size;
    int tmp2 = frame_size - bytes_read;
    stream_length = (tmp1 > tmp2? tmp2: tmp1);

    //readresult=read(shotfd,buf,lines_send_per_time*line_size*pixel_size);
    memcpy(buf,frame + bytes_read,stream_length);
    bytes_read += stream_length;

    buf -= header_idx;
    sendresult = sendto(sd, buf, stream_length+8/*actual_size*/, 0,
                        (struct sockaddr*)&rec,sizeof(rec));
     
    if (sendresult<0) {
      if (DEBUG_WHERE) tprintf(" [imagesender::main(): cannot send \n");
      exit(-1);
    }
 
    j += sendresult;

    usleep(200);
  }
 
  if (DEBUG_WHERE) tprintf(" [imagesender::send_frame(): %d bytes sent to %s:%d\n",j, inet_ntoa(rec.sin_addr),ntohs(rec.sin_port));

  free(buf);

  return j;
}
