// -*- mode: Java; c-basic-offset: 2; -*-
//
//
//  $Id: ViewMaster.java,v 1.1.1.1 2003/06/11 18:49:02 wyan Exp $
//
//

package avs;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.geom.*;  // for 2D graphics stuff


public class ViewMaster
  extends JFrame
  implements ActionListener
{
  NewViewerDialog newViewerDlg;
  JPanel listPanel;
  JTree cameraTree;
  DefaultMutableTreeNode cameraTreeRoot;
  DefaultTreeModel cameraTreeModel;
  JScrollPane scpTreePane;
  JList viewerList;
  JPanel panAddNew;
  JTextField tfViewChan;
  JButton btnNewViewer;
  JScrollPane scpViewers;
  JPanel panStatus;
  JLabel lblStatus;
  Vector viewClients;
  static Properties props;

  public native void getStreams (String hostname, int port, String context);
  public native void startup();

  public static void main (String args[]) throws java.io.IOException
  {
    String hostname = "localhost";
    //    System.out.println ("path is " + System.getProperty ("java.library.path"));
    System.loadLibrary ("avsjni");
    props = new Properties();

    FileInputStream in = new FileInputStream ("avs.properties");
    props.load (in);
    in.close();
    
    if (args.length >= 1)
      hostname = args[0]; 
    ViewMaster tlw = new ViewMaster ("avs view master", hostname);
  }

  ViewMaster (String caption, String default_pds_host)
  {
    super (caption);

    String pds_context, pds_host;
    int pds_port;

    buildInterface();
    newViewerDlg = new NewViewerDialog();
    viewClients = new Vector();
    startup();
    status ("Looking for cameras...");
    pds_host = props.getProperty ("avs.pds_host", default_pds_host);
    pds_port = Integer.parseInt (props.getProperty ("avs.pds_port", "8848"));
    pds_context = props.getProperty ("avs.pds_context", "/avs/streams");
    //    getStreams ("localhost", 8848, "/avs/streams");
    getStreams (pds_host, pds_port, pds_context);
    status ("Ready");
  }

  public void add_new_stream (String desc, String chan_id, String format, String formatTag)
  {
    DefaultMutableTreeNode camNode = null;
    //  find the appropriate node to insert into.  If we can't find it,
    //   create a new one.

    Enumeration e = cameraTreeRoot.children();
    boolean found = false;
    while (e.hasMoreElements() && ! found)
      {
        camNode = (DefaultMutableTreeNode) e.nextElement();
        if (camNode.getUserObject().equals (desc))
          {
            found = true;
          }
      }

    if (!found)
      {
        camNode = new DefaultMutableTreeNode (desc);
        cameraTreeModel.insertNodeInto (camNode, cameraTreeRoot, 
                                        cameraTreeRoot.getChildCount());
      }

    DefaultMutableTreeNode sn = 
      new DefaultMutableTreeNode (new avs.StreamNode (format, chan_id, formatTag));
    cameraTreeModel.insertNodeInto (sn, camNode, camNode.getChildCount());
    //    cameraTree.makeVisible (new TreePath (sn.getPath());
    final TreeNode[] visPath = sn.getPath();
    SwingUtilities.invokeLater (new Runnable()
      {
        public void run()
        {
          cameraTree.makeVisible (new TreePath(visPath));
        }
      });

  }
  
  public void actionPerformed (ActionEvent ev)
  {
  }
  
  private void buildInterface()
  {
    JMenuBar mb = new JMenuBar();
    JMenu masterMenu = new JMenu ("ViewMaster");
    JMenuItem newViewerItem = new JMenuItem ("new viewer...");
    newViewerItem.addActionListener ( new ActionListener()
      {
        public void actionPerformed (ActionEvent e)
        {
          newViewerDlg.setVisible (true);
        }
      } );
    masterMenu.add (newViewerItem);

    JMenuItem exitItem = new JMenuItem ("exit");
    masterMenu.add (exitItem);

    exitItem.addActionListener ( new ActionListener() 
      {
        public void actionPerformed (ActionEvent e)
        {
          closeMe();
        }
      } );

    KeyStroke ks = KeyStroke.getKeyStroke (KeyEvent.VK_X, Event.ALT_MASK);
    exitItem.setAccelerator (ks);
    masterMenu.setMnemonic ('V');
    mb.add (masterMenu);
    setJMenuBar (mb);

    getContentPane().setLayout (new BoxLayout (getContentPane(), BoxLayout.Y_AXIS));

    listPanel = new JPanel();
    listPanel.setBorder (BorderFactory.createTitledBorder ("available streams"));

    cameraTreeRoot = new DefaultMutableTreeNode ("AVS Cameras");
    cameraTreeModel = new DefaultTreeModel (cameraTreeRoot);
    cameraTree = new JTree (cameraTreeModel);

    cameraTree.getSelectionModel().setSelectionMode
      (TreeSelectionModel.SINGLE_TREE_SELECTION);
    cameraTree.setScrollsOnExpand (true);
    cameraTree.setShowsRootHandles (false);
    cameraTree.setRootVisible (false);
    cameraTree.addTreeSelectionListener 
      (new TreeSelectionListener()
        {
          public void valueChanged (TreeSelectionEvent e)
          {
            DefaultMutableTreeNode n = (DefaultMutableTreeNode)
              cameraTree.getLastSelectedPathComponent();
            if (n == null) return;
            if (!n.isLeaf()) return;
            StreamNode sn = (StreamNode) n.getUserObject();
            status (sn.chanId);
            viewClients.addElement (new ViewClient ("avs view client", sn));
          }
        }
       );
    
    scpTreePane = new JScrollPane (cameraTree);
    scpTreePane.setPreferredSize (new Dimension (200, 100));
    listPanel.add (scpTreePane);
    
    getContentPane().add (listPanel);
    
    getContentPane().add (Box.createVerticalStrut (2));

    panStatus = new JPanel();
    panStatus.setBorder (BorderFactory.createEtchedBorder());
    lblStatus = new JLabel();
    lblStatus.setHorizontalAlignment (JLabel.LEFT);
    panStatus.add (lblStatus);
    getContentPane().add (panStatus);
    
    addWindowListener (new byebye());
    pack();
    setVisible (true);
  }    

  void closeMe()
  {
    System.exit (0);
  }

  void status (String s)
  {
    lblStatus.setText (s);
  }
}

class NewViewerDialog
  extends JFrame
  implements ActionListener
{
  JTextField tfViewChan;
  JButton btnNewViewer, btnCancel;
  JPanel panAddNew, panButtons;

  NewViewerDialog()
  {
    this ("");
  }

  NewViewerDialog (String fstring)
  {
    super ("avs filter");
    buildWindow (fstring);
    pack();
  }

  private void buildWindow (String s)
  {
    getContentPane().setLayout (new BoxLayout (getContentPane(), BoxLayout.Y_AXIS));

    panAddNew = new JPanel();
    tfViewChan = new JTextField (30);
    tfViewChan.addMouseListener (new MouseAdapter()
      {
        public void mouseClicked (MouseEvent e)
        {
          if (SwingUtilities.isMiddleMouseButton (e))
            tfViewChan.paste();
        }
      });


    panAddNew.add (tfViewChan);
    panAddNew.add (Box.createVerticalStrut (5));

    panButtons = new JPanel();
    panButtons.setLayout (new BoxLayout (panButtons, BoxLayout.X_AXIS));
    btnNewViewer = new JButton ("new viewer");
    btnNewViewer.addActionListener (this);
    panButtons.add (btnNewViewer);
    btnCancel = new JButton ("cancel");
    btnCancel.addActionListener (this);
    panButtons.add (btnNewViewer);
    panButtons.add (Box.createHorizontalStrut (5));
    panButtons.add (btnCancel);

    getContentPane().add (panButtons);
  }

  public void actionPerformed (ActionEvent e)
  {
    if (e.getSource() == btnNewViewer)
      {
        ViewClient vc = new ViewClient ("avs view client", new StreamNode (tfViewChan.getText(), null, null));

	/*vvv*/
	//System.out.println(" StreamNode: tfViewChan.getTesxt()="+tfViewChan.getText());
	/*^^^*/

        setVisible (false);
      }
    else if (e.getSource() == btnCancel)
      {
        setVisible (false);
      }
  }
}

//
//
//  $Log: ViewMaster.java,v $
//  Revision 1.1.1.1  2003/06/11 18:49:02  wyan
//  UAV1
//
//  Revision 1.1  2002/11/14 16:20:47  zorn
//  *** empty log message ***
//
//  Revision 1.1.1.1  2002/09/11 20:46:10  wyan
//  UAV demo
//
//  Revision 1.4  2001/10/19 18:11:16  pmw
//  various happiness :)
//
//  Revision 1.3  2001/10/10 17:41:59  pmw
//  now uses configure to do things.  also modified ViewMaster to update list
//  correctly from JNI thread.
//
//  Revision 1.2  2001/09/29 04:14:37  pmw
//  more interim checkins.  crop rectangle in place for GUI.
//
//  Revision 1.1  2001/09/21 21:28:58  pmw
//  interim checkin...
//
//  Revision 1.5  2001/09/18 20:39:17  pmw
//  fixed several UI issues.  not quite working... ALMOST!
//
//  Revision 1.4  2001/09/07 18:42:58  pmw
//  changes for directory service integration
//
//  Revision 1.3  2001/08/22 22:05:24  pmw
//  changes for directory service - not yet complete
//
//  Revision 1.2  2001/07/24 17:45:25  pmw
//   updates for new version of avs checkin.  not configure-ized yet
//
//  Revision 1.1  2001/06/14 19:21:47  pmw
//  finally displays pictures semi-correctly
//
//
//
