/* -*-C-*-
*******************************************************************************
*
* File:         avs_raw.c
* RCS:          $Id: avs_raw.c,v 1.1.1.1 2003/06/11 18:48:53 wyan Exp $
* Description:  Raw video streams and API
*
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Mon Apr 09, 2001 at 10:35:28
* Modified:     Fri Jul 20, 2001 at 13:59:43 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <io.h>

#include <avs_raw.h>

IOField Raw_data_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data_ptr,height)},
  {"buff", "char[size]", sizeof(char), IOOffset(Raw_data_ptr,buff)},
  {NULL,NULL},
};

/* 1 - 640 * 480 ***********/

IOField Raw_data1C_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data1C_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data1C_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data1C_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data1C_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data1C_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data1C_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE1C), sizeof(char), 
   IOOffset(Raw_data1C_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data1G_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data1G_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data1G_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data1G_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data1G_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data1G_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data1G_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE1G), sizeof(char), 
   IOOffset(Raw_data1G_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data1DC_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data1DC_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data1DC_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data1DC_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data1DC_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data1DC_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data1DC_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE1DC), sizeof(char), 
   IOOffset(Raw_data1DC_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data1DG_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data1DG_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data1DG_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data1DG_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data1DG_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data1DG_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data1DG_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE1DG), sizeof(char), 
   IOOffset(Raw_data1DG_ptr,buff[0])},
  {NULL,NULL},
};


/* 2 - 320 * 240 ***********/

IOField Raw_data2C_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data2C_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data2C_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data2C_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data2C_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data2C_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data2C_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE2C), sizeof(char), 
   IOOffset(Raw_data2C_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data2G_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data2G_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data2G_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data2G_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data2G_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data2G_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data2G_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE2G), sizeof(char), 
   IOOffset(Raw_data2G_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data2DC_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data2DC_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data2DC_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data2DC_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data2DC_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data2DC_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data2DC_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE2DC), sizeof(char), 
   IOOffset(Raw_data2DC_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data2DG_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data2DG_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data2DG_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data2DG_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data2DG_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data2DG_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data2DG_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE2DG), sizeof(char), 
   IOOffset(Raw_data2DG_ptr,buff[0])},
  {NULL,NULL},
};

/* 3 - 160 * 120 ***********/

IOField Raw_data3C_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data3C_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data3C_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data3C_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data3C_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data3C_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data3C_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE3C), sizeof(char), 
   IOOffset(Raw_data3C_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data3G_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data3G_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data3G_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data3G_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data3G_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data3G_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data3G_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE3G), sizeof(char), 
   IOOffset(Raw_data3G_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data3DC_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data3DC_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data3DC_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data3DC_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data3DC_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data3DC_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data3DC_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE3DC), sizeof(char), 
   IOOffset(Raw_data3DC_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data3DG_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data3DG_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data3DG_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data3DG_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data3DG_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data3DG_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data3DG_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE3DG), sizeof(char), 
   IOOffset(Raw_data3DG_ptr,buff[0])},
  {NULL,NULL},
};

/* 4 - 80 * 60 ***********/

IOField Raw_data4C_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data4C_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data4C_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data4C_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data4C_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data4C_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data4C_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE4C), sizeof(char), 
   IOOffset(Raw_data4C_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data4G_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data4G_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data4G_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data4G_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data4G_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data4G_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data4G_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE4G), sizeof(char), 
   IOOffset(Raw_data4G_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data4DC_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data4DC_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data4DC_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data4DC_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data4DC_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data4DC_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data4DC_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE4DC), sizeof(char), 
   IOOffset(Raw_data4DC_ptr,buff[0])},
  {NULL,NULL},
};

IOField Raw_data4DG_fld[] = {
  {"tag","integer",sizeof(int),IOOffset(Raw_data4DG_ptr,tag)},
  {"ppm1","char",sizeof(char),IOOffset(Raw_data4DG_ptr,ppm1)},
  {"ppm2","char",sizeof(char),IOOffset(Raw_data4DG_ptr,ppm2)},
  {"size","integer",sizeof(int),IOOffset(Raw_data4DG_ptr,size)},
  {"width","integer",sizeof(int),IOOffset(Raw_data4DG_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Raw_data4DG_ptr,height)},
  {"buff", IOArrayDecl(char,AVSIMAGE4DG), sizeof(char), 
   IOOffset(Raw_data4DG_ptr,buff[0])},
  {NULL,NULL},
};


/* Parameters */

IOField Params_rectangle_fld[] = {
  {"x","integer",sizeof(int),IOOffset(Params_rectangle_ptr,x)},
  {"y","integer",sizeof(int),IOOffset(Params_rectangle_ptr,y)},
  {"width","integer",sizeof(int),IOOffset(Params_rectangle_ptr,width)},
  {"height","integer",sizeof(int),IOOffset(Params_rectangle_ptr,height)},
  {NULL,NULL},
};

IOField Params_factor_fld[] = {
  {"factor","integer",sizeof(int),IOOffset(Params_factor_ptr,factor)},
  {NULL,NULL},
};


/*
 * avs_registerFormats: Register all our formats with give context, this way
 *   we can get them by name, later on
 *
 * Parameters: context
 *
 * Returns: -
 *
 */

int
avs_registerFormats(ioc)
     IOContext ioc;
{
  IOFormat format;

  format = register_IOcontext_format ("Raw_data1C", Raw_data1C_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data1G", Raw_data1G_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data1DC", Raw_data1DC_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data1DG", Raw_data1DG_fld, ioc);
  if (!format) return -1;

  format = register_IOcontext_format ("Raw_data2C", Raw_data2C_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data2G", Raw_data2G_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data2DC", Raw_data2DC_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data2DG", Raw_data2DG_fld, ioc);
  if (!format) return -1;

  format = register_IOcontext_format ("Raw_data3C", Raw_data3C_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data3G", Raw_data3G_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data3DC", Raw_data3DC_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data3DG", Raw_data3DG_fld, ioc);
  if (!format) return -1;

  format = register_IOcontext_format ("Raw_data4C", Raw_data4C_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data4G", Raw_data4G_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data4DC", Raw_data4DC_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Raw_data4DG", Raw_data4DG_fld, ioc);
  if (!format) return -1;

  /* Parameters */
  format = register_IOcontext_format ("Params_rectangle", 
				      Params_rectangle_fld, ioc);
  if (!format) return -1;
  format = register_IOcontext_format ("Params_factor", 
				      Params_factor_fld, ioc);
  if (!format) return -1;

  return 0;
} /* end avs_registerFormats */


/*
 *
 *
 *
 */

IOFieldList
avs_getFormatFieldList (ioc, formatName)
     IOContext ioc;
     char *formatName;
{
  IOFormat format;

  format = get_IOformat_by_name_IOcontext(ioc, formatName);
  if (!format) return NULL;
  return field_list_of_IOformat (format);
} /* end avs_getFormatFieldList */
