/* -*-C-*-
*******************************************************************************
*
* File:         avs_raw.h
* RCS:          $Id: avs_raw.h,v 1.1.1.1 2003/06/11 18:48:52 wyan Exp $
* Description:  Raw video streams and API
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Mon Apr 09, 2001 at 10:29:54
* Modified:     Fri Jul 20, 2001 at 14:51:04 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/


#ifndef RAW_ECHO_INCLUDED
#define RAW_ECHO_INCLUDED

#include <io.h>
#include <common.h>

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  char *buff; 
} Raw_data, *Raw_data_ptr;

extern IOField Raw_data_fld[];


/* 1 - 640 * 480 ***********/

#define AVSIMAGE1C	921600	/* 640 * 480 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE1C];
} Raw_data1C, *Raw_data1C_ptr;

extern IOField Raw_data1C_fld[];


#define AVSIMAGE1G	307200	/* 640 * 480 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE1G];
} Raw_data1G, *Raw_data1G_ptr;

extern IOField Raw_data1G_fld[];


#define AVSIMAGE1DC    1843200	/* 640 * 480 * 2 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE1DC];
} Raw_data1DC, *Raw_data1DC_ptr;

extern IOField Raw_data1DC_fld[];


#define AVSIMAGE1DG	614400	/* 640 * 480 * 2 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE1DG];
} Raw_data1DG, *Raw_data1DG_ptr;

extern IOField Raw_data1DG_fld[];


/* 2 - 320 * 240 ***********/

#define AVSIMAGE2C	230400	/* 320 * 240 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE2C];
} Raw_data2C, *Raw_data2C_ptr;

extern IOField Raw_data2C_fld[];


#define AVSIMAGE2G	76800	/* 320 * 240 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE2G];
} Raw_data2G, *Raw_data2G_ptr;

extern IOField Raw_data2G_fld[];


#define AVSIMAGE2DC    1843200	/* 320 * 240 * 2 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE2DC];
} Raw_data2DC, *Raw_data2DC_ptr;

extern IOField Raw_data2DC_fld[];


#define AVSIMAGE2DG	614400	/* 320 * 240 * 2 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE2DG];
} Raw_data2DG, *Raw_data2DG_ptr;

extern IOField Raw_data2DG_fld[];



/* 3 - 160 * 120 ***********/

#define AVSIMAGE3C	57600	/* 160 * 120 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE3C];
} Raw_data3C, *Raw_data3C_ptr;

extern IOField Raw_data3C_fld[];


#define AVSIMAGE3G	19200	/* 160 * 120 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE3G];
} Raw_data3G, *Raw_data3G_ptr;

extern IOField Raw_data3G_fld[];


#define AVSIMAGE3DC    1843200	/* 160 * 120 * 2 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE3DC];
} Raw_data3DC, *Raw_data3DC_ptr;

extern IOField Raw_data3DC_fld[];


#define AVSIMAGE3DG	614400	/* 160 * 120 * 2 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE3DG];
} Raw_data3DG, *Raw_data3DG_ptr;

extern IOField Raw_data3DG_fld[];



/* 4 - 80 * 60 ***********/

#define AVSIMAGE4C	14400	/* 80 *  60 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE4C];
} Raw_data4C, *Raw_data4C_ptr;

extern IOField Raw_data4C_fld[];


#define AVSIMAGE4G	4800	/* 80 *  60 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE4G];
} Raw_data4G, *Raw_data4G_ptr;

extern IOField Raw_data4G_fld[];


#define AVSIMAGE4DC    1843200	/* 80 *  60 * 2 - color */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE4DC];
} Raw_data4DC, *Raw_data4DC_ptr;

extern IOField Raw_data4DC_fld[];


#define AVSIMAGE4DG	614400	/* 80 *  60 * 2 - grey  */

typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  int maxval;
  char buff[AVSIMAGE4DG];
} Raw_data4DG, *Raw_data4DG_ptr;

extern IOField Raw_data4DG_fld[];


/* Parameters */

typedef struct {
  int x;
  int y;
  int width;
  int height;
} Params_rectangle, *Params_rectangle_ptr;

extern IOField Params_rectangle_fld[];

typedef struct {
  int factor;
} Params_factor, *Params_factor_ptr;

extern IOField Params_factor_fld[];

extern int avs_registerFormats PARAMS((IOContext ioc));
extern IOFieldList avs_getFormatFieldList PARAMS((IOContext ioc, char *formatName));

#endif /* RAW_ECHO_INCLUDED */
