/* -*-C-*-
 * @ Wenchang Yan
 * @date Dec, 2001
*******************************************************************************
*
* File:         avs_source.c
* RCS:          $Id: avs_source.c,v 1.1.1.1 2003/06/11 18:48:53 wyan Exp $
* Description:  AVS source
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Fri Feb 16, 2001 at 00:06:23
* Modified:     Fri Oct 12, 2001 at 10:08:18 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <io.h>
#include <atl.h>
#include <gen_thread.h>
#include <cm.h>
#include <echo.h>
//#include <pds.h>

#if !defined(DISABLE_VDEV)
#include <vdev.h>
#endif

#include <getopt.h>
#include <tv.h>
#include <hrtime.h>
#include <avs_util.h>
#include <avs_raw.h>
#include <styx.h>
#include <sensNet.h>
#include <stats.h>

static int NumRec = 0;
static boolean verbose = FALSE;
static boolean MeasureMe = FALSE;
static int Freq = 2;
static boolean debugging = FALSE;
static boolean fileimage = FALSE;
static boolean use_PDS = FALSE;
static char* local_progname;
EChannel chan;
CManager cm;
//EControlContext ec;
int NumRecSubmitted;
//pds_service directory;

struct rusage RusageBuffer;
struct timeval sendStart;	/* time for all sends */
struct timeval startCpu, stopCpu, usedCpu; /* CPU used for all sends */
struct timeval startUCpu, stopUCpu, usedUCpu; /* U CPU used for all sends */
struct timeval startSCpu, stopSCpu, usedSCpu; /* S CPU used for all sends */

/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"fileimage", required_argument, NULL, 'f'},
  {"debugging", no_argument, NULL, 'd'},
  {"help", no_argument, NULL, 'h'},
  {"clients", required_argument, NULL, 'c'},
  {"numframes", required_argument, NULL, 'n'},
  {"measure", no_argument, NULL, 'm'},
  {"sample", required_argument, NULL, 's'},
  {"report", required_argument, NULL, 'r'},
  {"order", required_argument, NULL, 'o'},
  {"use-PDS", no_argument, NULL, 'P'},
  {"host", required_argument, NULL, 'H'},
  {"context", required_argument, NULL, 'C'},
  {"name", required_argument, NULL, 'N'},
  {"description", required_argument, NULL, 'X'},
  {NULL, no_argument, NULL, 0}
};

typedef struct PDS_args_struct
{
  char *pds_hostname;
  int pds_port;
  char *pds_context;
  char *pds_name;
  char *pds_desc;
} PDS_args, *PDS_args_ptr;

/* Comment it out if you do NOT do the experiment time measuring */
//#define DO_EXP 

#define DEFAULT_FPS 15
/*for debug*/
#undef DEBUG_WHERE
#define DEBUG_WHERE 0

#include <time.h>
#include <sys/timeb.h>
#include <stdarg.h>
int tprintf(const char* fmt,...)
{
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}


/*----------------------------------------------------------------------------*/

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */

static void 
usage (stream, status)
     FILE* stream; 
     int status;
{  
  fprintf(stream, "Usage: %s [-vhm] -c nclients -n nframes [-F fps] [-s freq] [-r freq] "
	  "[-o pixorder] [-f ppm-file] [-P [-H host:port] [-C pds-context] "
	  "[-N name] [-X description]]\n"
	  "-v --verbose              \tverbose mode\n"
	  "-f --fileimage ppm-file   \timage from file\n"
	  "-d --debugging            \tdebugging mode\n"
	  "-h --help                 \tgive this help\n"
	  "-c --clients  nclients    \tnumber of sinks clients\n"
	  "-n --numframes  nframes   \tnumber of frames (0 == infinite)\n"
	  "-m --measure              \tset bandwidth measuring at client\n"
	  "-s --sample               \tsampling frequency for measurements\n"
	  "-r --report               \tfreq of bandwidth report\n"
	  "-o --order pixorder       \t[1: RGB, 2: RBG, 3: GRB, 4: GBR, 5: BRG, 6: BGR]\n"
          "-P --use-PDS              \tregister streams with PDS server\n"
          "-H --pds-host host[:port] \thost and port for PDS server\n"
          "-C --context <string>     \tcontext under which to register source\n"
          "-N --name <string>        \tname to use for the source\n"
          "-X --description <string> \tdescription to use for the source\n"
	  "-F --framerate per sec    \tfps for image capturing (1-30)\n"
     	  , local_progname);
  exit(status);
} /* end usage */

/*----------------------------------------------------------------------------*/

/*
 * dumpStats: Compute and dump stats
 *
 * Parameters: -
 *
 * Returns: -
 *
 */

void
dumpStats()
{
  struct timeval sendStop, sendTime; /* time for all sends */
  struct timeval stopCpu, usedCpu; /* CPU used for all sends */
  struct timeval stopUCpu, usedUCpu; /* U CPU used for all sends */
  struct timeval stopSCpu, usedSCpu; /* S CPU used for all sends */
  double *bwHist;

  getrusage(RUSAGE_SELF, &RusageBuffer);
  gettimeofday(&sendStop, NULL);
  tv_add2(&stopCpu, &(RusageBuffer.ru_utime), &(RusageBuffer.ru_stime));
  tv_assign(&stopUCpu, &(RusageBuffer.ru_utime));
  tv_assign(&stopSCpu, &(RusageBuffer.ru_stime));
  tv_minus2(&usedCpu, &stopCpu, &startCpu);
  tv_minus2(&usedUCpu, &stopUCpu, &startUCpu);
  tv_minus2(&usedSCpu, &stopSCpu, &startSCpu);
  tv_minus2(&sendTime, &sendStop, &sendStart);
  fprintf(stdout, "\n----------------------------------------\n");
  fprintf(stdout, "Source performance data\n");
  fprintf(stdout, "-----------------------\n");
  fprintf(stdout, "Total time:       \t%ld ms\n", tv_toMsecs(&sendTime));
  fprintf(stdout, "Time per event:   \t%.3g ms\n", 
	  (float) tv_toMsecs(&sendTime)/NumRecSubmitted);
  fprintf(stdout, "CPU usage:        \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedCpu), 
	  (float) tv_toMsecs(&usedCpu)/tv_toMsecs(&sendTime));
  fprintf(stdout, "User CPU usage:   \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedUCpu), 
	  (float) tv_toMsecs(&usedUCpu)/tv_toMsecs(&sendTime));
  fprintf(stdout, "System CPU usage: \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedSCpu), 
	  (float) tv_toMsecs(&usedSCpu)/tv_toMsecs(&sendTime));
  if (MeasureMe) {
    bwHist = (double*) malloc (((int) (NumRecSubmitted/Freq)) * sizeof(double));
    if (bwHist == NULL) Styx_errQuit("Not enough resources");
    sensNet_GetHistory(bwHist, (int) (NumRecSubmitted/Freq), SENSNET_READ);
    fprintf(stdout, "Bandwidth mean:   \t%.3g Mbps (sdev %.3g)\n",
	    Stats_mean (bwHist, (int) (NumRecSubmitted/Freq)),
	    Stats_sdev (bwHist, (int) (NumRecSubmitted/Freq)));
    sensNet_Finish();
    free(bwHist);
  }
  fprintf(stdout, "\a\a");

  fflush(stdout);
} /* end dumpStats */


/*----------------------------------------------------------------------------*/

/*
 * interruptHandler: To go out nicely
 *
 * Parameters: signal number
 * 
 * Return: -
 *
 */
#ifdef DO_EXP
//Added for the performance experiment only -- Romoved after the 
//experiment. !!!!!!!!!
char file_name[] = "time_rec";
//This is the shared dir between the 2 expr laptops
char chan_id_file[] = "/usr/local/mware/share/chanId";
FILE* time_rec, *chan_file;

/*the interval between 2 frames, in microsec*/
unsigned long interval;
struct timeval prev_time, curr_time;
double time_diff;
int frame_transfered = 0;

#endif

static void
interruptHandler(signo)
     int signo;
{
#ifdef DO_EXP
  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
  /**/
  gettimeofday(&curr_time,NULL);      
  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
    + curr_time.tv_usec - prev_time.tv_usec;
  time_diff = time_diff/1000000;
  fprintf(time_rec," Streaming:%f,  frames:%d\n",time_diff,frame_transfered);
  fclose(time_rec);
  /**/
#endif

  dumpStats();  
  EChannel_destroy(chan);
  CManager_close(cm);

  exit(EXIT_SUCCESS);
} /* end interruptHandler */

/*----------------------------------------------------------------------------*/

/*
 * register_in_PDS:  place the channel IDs in the PDS server
 *
 * Parameters: args struct, channel ID/format indicator pairs
 *
 * Returns: -
 *
 */
/*
static void
register_in_PDS (args, chanID1, ind1, chanID2, ind2)
     PDS_args_ptr args;
     char *chanID1;
     char *ind1;
     char *chanID2;
     char *ind2;
{
  attr_list contact_attrs;
  attr_list ent_attrs;
  pds_domain_id_t domain_id;
  char buf[200];

  contact_attrs = create_attr_list();
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_HOST"),
            Attr_String,
            strdup (args->pds_hostname));
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_PORT"),
            Attr_Int4,
            (attr_value*)args->pds_port);

  directory = pds_service_open (contact_attrs);

  domain_id = pds_open_domain (directory, "demo", "demo", 1, "avs");

  ent_attrs = create_attr_list();
  add_attr (ent_attrs, attr_atom_from_string ("StreamDescription"),
            Attr_String, args->pds_desc);
  add_attr (ent_attrs, attr_atom_from_string ("StreamChannelID"),
            Attr_String, chanID1);
  add_attr (ent_attrs, attr_atom_from_string ("StreamFormatDescription"),
            Attr_String, strdup ("Raw image"));
  add_attr (ent_attrs, attr_atom_from_string ("StreamFormatTag"),
            Attr_String, strdup ("RAW"));

  memset (buf, 0, 200);
  strcat (buf, "/avs/streams/");
  strcat (buf, args->pds_name);
  strcat (buf, "/");
  strcat (buf, ind1);
  
  pds_create_entity (domain_id, buf, null_pds_context_id, NULL, ent_attrs);

  set_attr (ent_attrs, attr_atom_from_string ("StreamChannelID"),
            Attr_String, chanID2);
  set_attr (ent_attrs, attr_atom_from_string ("StreamFormatDescription"),
            Attr_String, strdup ("640 x 480, color"));
  set_attr (ent_attrs, attr_atom_from_string ("StreamFormatTag"),
            Attr_String, strdup ("1C"));

  memset (buf, 0, 200);
  memset (buf, 0, 200);
  strcat (buf, "/avs/streams/");
  strcat (buf, args->pds_name);
  strcat (buf, "/");
  strcat (buf, ind2);

  pds_create_entity (domain_id, buf, null_pds_context_id, NULL, ent_attrs);

  pds_service_close (directory);

}
  
  */                   


/*----------------------------------------------------------------------------*/

int
main(argc, argv)
     int argc; 
     char* argv[];
{
  EControlContext ec;
  //ECSourceHandle sourceHandle, sourceHandle1C;
  int width,height;//size from the command line
  ECSourceHandle sourceHandle, sourceHandle1C, sourceHandle2C;
  EChannel chan1C, chan2C;
  int i, maxppmval, optc, numClient=0;
  Raw_data rawrec, debugrec;
  Raw_data_ptr rawrecP = &rawrec, debugrecP = &debugrec;

  /*Raw_data1C rawrec1C;*/
  /*Raw_data1C_ptr rawrec1CP = &rawrec1C;*/
  Raw_data1C rawrec1C;
  Raw_data1C_ptr rawrec1CP = &rawrec1C;
  Raw_data2C rawrec2C;
  Raw_data2C_ptr rawrec2CP = &rawrec2C;
  int fps=DEFAULT_FPS;
  /*the interval between 2 frames, in microsec*/
  unsigned long interval;
  //struct timeval prev_time, curr_time;
  //double time_diff;
  int imagefd;
  char* imageFilename=NULL;
  int debuggingfd; 
  char header[PPMHEADERLEN];
  char shotsentFile[FILENAMELEN];
  PDS_args my_PDS_args;
  char* tmp = NULL;
  int pixOrder = 2; /*2;*/		/* RBG */
  double *bwHist;
  int reported;
  int reportFreq;

#if !defined(DISABLE_VDEV)
  VDev_T vdev = NULL;
  struct VDev_imageSize size;
#endif 

  my_PDS_args.pds_hostname = "localhost";
  my_PDS_args.pds_port = 8848;
  my_PDS_args.pds_name = my_PDS_args.pds_context = my_PDS_args.pds_desc = NULL;


  Styx_asDaemon(0);             /* not a daemon, actually do not use syslog */
  Styx_setDebugLevel(0);        /* no debugging unless requested */
  Styx_setProgname(argv[0]);
  local_progname = argv[0];  
  
  if (argc < 3) usage(stderr, EXIT_FAILURE);
  while ((optc = getopt_long(argc, argv, "+vhmdf:o:c:n:s:r:PH:C:N:X:S:F:", 
			     longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose= TRUE;
      Styx_setDebugLevel(7);
      Styx_privateLog(0);
      break; 
    case 'd':
      debugging= TRUE;
      verbose = TRUE;
      break;
    case 'm':
      MeasureMe = TRUE;
      break; 
    case 'f':
      fileimage = TRUE;
      imageFilename = strdup(optarg);
      if (imageFilename == NULL) exit(EXIT_FAILURE);
      break;
    case 'o':
      pixOrder = atoi(optarg);
      if (pixOrder < 1 || pixOrder > 6) usage(stderr, EXIT_FAILURE);
      break; 
    case 'h':
      usage(stdout, EXIT_SUCCESS);
      break;
    case 'n':
      NumRec = atoi(optarg);
      if (NumRec < 0) usage(stderr, EXIT_FAILURE);
      break;
    case 's':
      Freq = atoi(optarg);
      if (Freq < 0) usage(stderr, 1);
      break;
    case 'r':
      reportFreq = atoi(optarg);
      if (reportFreq < 0) usage(stderr, 1);
      break;
    case 'c':
      numClient = atoi(optarg);
      if (numClient < 1 || numClient > MAXCLIENTS) usage(stderr, EXIT_FAILURE);
      break;
    case 'P':
      use_PDS = TRUE;
      break;
    case 'H':
      tmp = strchr (optarg, ':');
      if (tmp != NULL)
        {
          /* port number present */
          *tmp++ = '\0';
        }
      my_PDS_args.pds_hostname = strdup (optarg);
      if (!my_PDS_args.pds_hostname) exit(EXIT_FAILURE);
      if (tmp != NULL)
        {
          my_PDS_args.pds_port = atoi (tmp);
          if (my_PDS_args.pds_port < 0) exit(EXIT_FAILURE);
        }
      break;
    case 'C':
      my_PDS_args.pds_context = strdup (optarg);
      if (!my_PDS_args.pds_context) exit(EXIT_FAILURE);
      break;
    case 'N':
      my_PDS_args.pds_name = strdup (optarg);
      if (!my_PDS_args.pds_name) exit(EXIT_FAILURE);
      break;
    case 'X':
      my_PDS_args.pds_desc = strdup (optarg);
      if (!my_PDS_args.pds_desc) exit(EXIT_FAILURE);
      break;
    case 'F':
      fps = atoi(optarg);
      if (fps<=0) fps = DEFAULT_FPS;
      else if (fps >=15) fps = 15; /* >15 doesn't mean anything to our cam*/
      break;
    default:
      usage(stderr, EXIT_FAILURE);
    }
  }

  if (optind > argc) usage(stderr, 1); /* no extra args */

  Styx_debugMsg(1, "Number of clients: \t%d\n", numClient);

  if (fileimage) {
    fprintf(stdout, "Image file: \t%s\n", imageFilename);
    if ((imagefd = open(imageFilename, O_RDONLY)) < 0) 
      Styx_errQuit("Failed opening image file.\n");    
    read(imagefd, header, sizeof(header));   
    sscanf(header, "%c%c\n%d %d\n%d\n", 
	   &(debugrecP->ppm1), &(debugrecP->ppm2),
	   &(debugrecP->width), &(debugrecP->height), 
	   &maxppmval);
    fprintf(stdout, "PPM %c%c\n", 
	    debugrecP->ppm1, debugrecP->ppm2);
    fprintf(stdout, "Width %d Height %d\n", 
	    debugrecP->width, debugrecP->height);
    fprintf(stdout, "Max PPM val %d\n", maxppmval);
    debugrecP->size = debugrecP->width * debugrecP->height * 
      3 * sizeof(u_char);
    debugrecP->buff = (u_char*) malloc(debugrecP->size);
    if (debugrecP->buff == NULL) Styx_errQuit("Out of mem\n");
    fprintf(stdout, "Size %d\n", debugrecP->size);
    read(imagefd, debugrecP->buff, debugrecP->size);

    debuggingfd = open("shotsent-debug.ppm", 
	       O_CREAT|O_WRONLY, S_IRWXU|S_IRGRP|S_IROTH);
    sprintf(header, "%c%c\n%d %d\n%d\n", debugrecP->ppm1, debugrecP->ppm2, 
	    debugrecP->width, debugrecP->height, 255);  
    write(debuggingfd, header, sizeof(header));
    if (debugrecP->buff != NULL) 
      write(debuggingfd, debugrecP->buff, debugrecP->size);
    close(debuggingfd);
  }

  /*
   * Creation of channel and registration
   *
   */
#ifdef DO_EXP
  if (NULL == (time_rec=fopen(file_name, "a")) || 
      NULL == (chan_file=fopen(chan_id_file,"w"))) {
    printf("file open error!\n");
    exit(1);
  }
  printf(" file open for writing\n");
  gettimeofday(&prev_time,NULL);
#endif

  gen_pthread_init();
  cm = CManager_create();  

  CMfork_comm_thread(cm);

  if (signal(SIGINT, interruptHandler) == SIG_ERR) 
    Styx_errQuit("Signal error");

  ec = ECho_CM_init(cm);

  chan2C = EChannel_typed_create(ec, Raw_data2C_fld, NULL);
  if (chan2C == NULL) Styx_errQuit("Failed to create 2C channel.\n");
  sourceHandle2C = ECsource_typed_subscribe(chan2C, Raw_data2C_fld, NULL);

#ifndef DO_EXP
  chan = EChannel_typed_create(ec, Raw_data_fld, NULL);
  if (chan == NULL) Styx_errQuit("Failed to create channel.\n");
  fprintf(stdout, "Echo channel ID:\n %s\n", ECglobal_id(chan));
  sourceHandle = ECsource_typed_subscribe(chan, Raw_data_fld, NULL);
#endif

  fprintf(stdout, "\nEcho 2C (320x240-color) channel ID:\n %s\n", 
	  ECglobal_id(chan2C));

#ifdef DO_EXP
  fprintf(chan_file,"%s\n",ECglobal_id(chan2C));
  fclose(chan_file);

  gettimeofday(&curr_time,NULL);
  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
  fprintf(time_rec," Init:%f,  ",time_diff);
#endif

  
  Styx_debugMsg(1, "Source subscribed to all channels.\n");

  /*  register with PDS if indicated  */
  /*
  if (use_PDS)
    {
      register_in_PDS (&my_PDS_args, 
                       ECglobal_id(chan), "raw",
                       ECglobal_id(chan1C), "1C");
    }
  */
  /* Wait for clients */
  //- CMfork_comm_thread(cm);

  while (
#ifndef DO_EXP 
EChas_sinks(sourceHandle) < numClient &&
#endif
	 EChas_sinks(sourceHandle2C) < numClient) 
    CMusleep(cm, 1000);

  Styx_debugMsg(1, "Ready!\n");

  /* Initialize camera */
  if (!fileimage) {
    //VDev_init();
    VDev_init(320,240);

    if (VDev_numVDevices() < 1) Styx_errQuit("No camera connected.\n");
    
    vdev = VDev_getVDevice(0);
    if (vdev == NULL || !VDev_isValid(vdev)) 
      Styx_errQuit ("Something is wrong, what camera are you asking for?\n");
    
    Styx_debugMsg(1,"V4L name: %s\n", VDev_getIntfName(vdev));
    
    VDev_open(vdev, 2);
    if (VDev_getDescriptor(vdev) < 0) 
      Styx_errQuit("Something is wrong, what camera are you asking for?\n");
    
    
    VDev_getSize(vdev, &size);
    Styx_debugMsg(1,"Size (width, heigth): (%d, %d)\n", 
		  size.width, size.height);

  }

  /*
   * Blasting the guy now
   *
   */
  if (MeasureMe) {
    sensNet_Init(Freq);
    sensNet_SetHistLen(NumRec/Freq, SENSNET_WRITE);
  }
  fprintf(stdout, "\n\n");

  gettimeofday(&sendStart, NULL);
  getrusage(RUSAGE_SELF, &RusageBuffer);
  tv_add2(&startCpu, &(RusageBuffer.ru_utime), &(RusageBuffer.ru_stime));
  tv_assign(&startUCpu, &(RusageBuffer.ru_utime));
  tv_assign(&startSCpu, &(RusageBuffer.ru_stime));

  NumRecSubmitted = 0;
  reported = 1;

  /* Set up for fps */
  /* Not exactly 1 sec; Use 1.5 sec to adjust the real fps */
  //interval = 1500000/fps;
  //gettimeofday(&prev_time,NULL);
  for (i = 0; ((i < NumRec) || (NumRec == 0)); ++i) {
    /*for fps control*/
    /*
    gettimeofday(&curr_time,NULL);
    time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
    //tprintf(" [prev: [%ld:%ld] curr:[%ld:%ld] diff:%f  interval:%ld\n",prev_time.tv_sec,prev_time.tv_usec,curr_time.tv_sec,curr_time.tv_usec,time_diff,interval);
    if (time_diff>=0) {
      prev_time.tv_sec = curr_time.tv_sec;
      prev_time.tv_usec = curr_time.tv_usec;
      usleep(interval - time_diff <= 0?0:interval-time_diff);
      //tprintf(" [wait: %f\n\n", interval-time_diff);
    }
     */

    if (!fileimage) {

      /* read data */
      rawrecP->tag = i+1;
      rawrecP->ppm1 = 'P';
      rawrecP->ppm2 = '6';
      
      rawrecP->buff = (u_char*) VDev_getPPMImage(vdev, pixOrder);

      if ((rawrecP->size = VDev_getRawSize(vdev)) < 0) {
	fprintf(stderr, "Something went wrong reading the image!\n");
	rawrecP->buff = (u_char*) NULL;
	rawrecP->size = 0;
      }
      rawrecP->width = size.width;
      rawrecP->height = size.height;

#ifndef DO_EXP
      ECsubmit_typed_event(sourceHandle, rawrecP);
#endif

      rawrec2CP->tag = rawrecP->tag;
      rawrec2CP->ppm1 = 'P';
      rawrec2CP->ppm2 = '6';
      rawrec2CP->width = size.width;
      rawrec2CP->height = size.height;
      rawrec2CP->size = rawrecP->size;
      memcpy(rawrec2CP->buff, rawrecP->buff, rawrecP->size);
      
      //if (DEBUG_WHERE) tprintf(" [main(): before sends to sourceHandle2C\n");

#ifdef DO_EXP
      if (0==i)
	gettimeofday(&prev_time,NULL);
#endif
      
      ECsubmit_typed_event(sourceHandle2C, rawrec2CP);

#ifdef DO_EXP
      frame_transfered ++;
#endif

      //if (DEBUG_WHERE) tprintf(" [main(): after sends to sourceHandle2C\n");
	
      /*
      if (debugging) {
	sprintf(shotsentFile, "shotsent%d.ppm", i+1);
	debuggingfd = open(shotsentFile, 
			   O_CREAT|O_WRONLY, S_IRWXU|S_IRGRP|S_IROTH);
	sprintf(header, "%c%c\n%d %d\n%d\n", rawrec2CP->ppm1, rawrec2CP->ppm2, 
		rawrec2CP->width, rawrec2CP->height, 255);  
	write(debuggingfd, header, sizeof(header));
	if (rawrec2CP->buff != NULL) 
	  write(debuggingfd, rawrec2CP->buff, rawrec2CP->size);
	close(debuggingfd);
      }
      */

      free(rawrecP->buff);
    }
    else {/* Img read from file */
      /* Must be 320*240 color */
      rawrec2CP->tag = debugrecP->tag;
      rawrec2CP->ppm1 = 'P';
      rawrec2CP->ppm2 = '6';
      rawrec2CP->width = debugrecP->width;
      rawrec2CP->height = debugrecP->height;
      rawrec2CP->size = debugrecP->size;

      memcpy(rawrec2CP->buff, debugrecP->buff, rawrec2CP->size);

      ECsubmit_typed_event(sourceHandle2C, rawrec2CP);
    }
    ++NumRecSubmitted;

    if (MeasureMe)
      if (NumRecSubmitted == reported * reportFreq) {
	bwHist = (double*) malloc (((int) (NumRecSubmitted/Freq)) * 
				   sizeof(double));
	sensNet_GetHistory(bwHist, (int) (NumRecSubmitted/Freq), SENSNET_WRITE);
	if (bwHist == NULL) Styx_errQuit("Not enough resources.\n");

	fprintf(stdout, "Bandwidth mean:   \t%.3g Mbps [sdev %2.3g]\r",
		Stats_mean (bwHist, (int) (NumRecSubmitted/Freq)),
		Stats_sdev (bwHist, (int) (NumRecSubmitted/Freq)));
	fflush(stdout);
	free(bwHist);
	++reported;
      }

  } /* end for */
  
  dumpStats();

  if (MeasureMe) sensNet_Finish();
  //EChannel_destroy(chan);
  //CManager_close(cm);

  exit (EXIT_SUCCESS);
} /* main */
