/** -*-C-*-
 * avs_source.c: the source side of the avs demo
 * @author Wenchang Yan
 * @time 1/30/02
 */

/**
 * In order to speed up the image capture speed, mmap and other techniques
 * are used. The capture mechanism is mainly adapted from the streamer 
 * package in Xawtv-3.68.
 */

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include "../config.h"

#undef DEBUG_WHERE
#undef DEBUG_WHAT 
#define DEBUG_WHERE 1
#define DEBUG_WHAT 1


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>


#include <math.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include <sys/wait.h>
#include "grab-ng.h"
#include "writefile.h"
#include "channel.h"
#include "sound.h"
#include "capture.h"
#include "commands.h"


//#include <math.h>

/* For ECho */
#include <io.h>
#include <atl.h>
#include <gen_thread.h>
#include <cm.h>
#include <echo.h>
#include <pds.h>

//#include <vdev.h>
//#include <getopt.h>
//#include <tv.h>
//#include <hrtime.h>
//#include <avs_util.h>
#include <avs_raw.h>
//#include <styx.h>
//#include <sensNet.h>
#include <stats.h> /*macros for MAX, MIN ...*/

/* Global */
static int num_record = 0;
static boolean verbose = FALSE;
static boolean input_image_file = FALSE;
EChannel chan;
CManager cm;

extern struct movie_handle* 
ECho_movie_writer_init(char *moviename, char *audioname,
                  const struct ng_writer *writer,
                  struct ng_video_fmt *video,const void *priv_video,int fps,
                  struct ng_audio_fmt *audio,const void *priv_audio,char *dsp,
                  int slots, int threads, Raw_data_ptr rawrecP,
                  Raw_data2C_ptr rawrec2CP, ECSourceHandle sourceHandle2C);


//int NumRecSubmitted;
//pds_service directory;

//struct rusage RusageBuffer;
//struct timeval sendStart;	/* time for all sends */
//struct timeval startCpu, stopCpu, usedCpu; /* CPU used for all sends */
//struct timeval startUCpu, stopUCpu, usedUCpu; /* U CPU used for all sends */
//struct timeval startSCpu, stopSCpu, usedSCpu; /* S CPU used for all sends */

/* Command line arguments */
/*
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"fileimage", required_argument, NULL, 'f'},
  {"debugging", no_argument, NULL, 'd'},
  {"help", no_argument, NULL, 'h'},
  {"clients", required_argument, NULL, 'c'},
  {"numframes", required_argument, NULL, 'n'},
  {"measure", no_argument, NULL, 'm'},
  {"sample", required_argument, NULL, 's'},
  {"report", required_argument, NULL, 'r'},
  {"order", required_argument, NULL, 'o'},
  {"use-PDS", no_argument, NULL, 'P'},
  {"host", required_argument, NULL, 'H'},
  {"context", required_argument, NULL, 'C'},
  {"name", required_argument, NULL, 'N'},
  {"description", required_argument, NULL, 'X'},
  {NULL, no_argument, NULL, 0}
};

typedef struct PDS_args_struct
{
  char *pds_hostname;
  int pds_port;
  char *pds_context;
  char *pds_name;
  char *pds_desc;
} PDS_args, *PDS_args_ptr;
*/

/*vvv*/
/*for debug*/
#include <time.h>
#include <sys/timeb.h>
#include <stdarg.h>

extern int tprintf(const char* fmt,...);

/*^^^*/

#define PPMHEADERLENGTH 15;

static int num_client = 1;
static int       bufcount = 16;
static int       parallel = 1;
static char*     tvnorm = NULL;
static char*     input  = NULL;
static char*     moviename = NULL;
static char*     audioname = NULL;
static char*     vfmt_name;
static char*     afmt_name;
 
static const struct ng_writer     *writer;
static const void                 *video_priv;
static const void                 *audio_priv;
static struct ng_video_fmt        video = {
  width: 320, /*320,*/
  height: 240, /*240,*/
};

static struct ng_audio_fmt        audio = {
    rate: 44100,
};
static void *movie_state;
static int  absframes = 1;
static int  quiet = 0, fps = 10000;
static int  signaled = 0, wait_seconds = 0;
int debug = 0, have_dga = 0;





void list_formats(FILE *out)
{
    const struct ng_writer *wr;
    int i,j;
 
    fprintf(out,"\nmovie writers:\n");
    for (i = 0; NULL != ng_writers[i]; i++) {
        wr = ng_writers[i];
        fprintf(out,"  %s - %s\n",wr->name,
                wr->desc ? wr->desc : "-");
        if (NULL != wr->video) {
            fprintf(out,"    video formats:\n");
            for (j = 0; NULL != wr->video[j].name; j++) {
                fprintf(out,"      %-7s %-28s [%s]\n",wr->video[j].name,
                        wr->video[j].desc ? wr->video[j].desc :
                        ng_vfmt_to_desc[wr->video[j].fmtid],
                        wr->video[j].ext);
            }
        }
        if (NULL != wr->audio) {
            fprintf(out,"    audio formats:\n");
            for (j = 0; NULL != wr->audio[j].name; j++) {
                fprintf(out,"      %-7s %-28s [%s]\n",wr->audio[j].name,
                        wr->audio[j].desc ? wr->audio[j].desc :
                        ng_afmt_to_desc[wr->audio[j].fmtid],
                        wr->audio[j].ext ? wr->audio[j].ext : "-");
            }
        }
        fprintf(out,"\n");
    }
}

 
static void
find_formats(void)
{
    const struct ng_writer *wr = NULL;
    char *mext = NULL;
    char *aext = NULL;
    int w,v=-1,a=-1;
 
    if (moviename) {
        mext = strrchr(moviename,'.');
        if (mext)
            mext++;
    }
    if (audioname) {
        aext = strrchr(audioname,'.');
        if (aext)
            aext++;
    }
    for (w = 0; NULL != ng_writers[w]; w++) {
        wr = ng_writers[w];
        if (debug)
            fprintf(stderr,"checking writer %s... ",wr->name);
        if ((/*!wr->combined && */mext) || NULL != vfmt_name) {
            if (NULL == wr->video) {
                if (debug)
                    fprintf(stderr,"no video\n");
                continue;
            }
            for (v = 0; NULL != wr->video[v].name; v++) {
                if (mext && 0 != strcasecmp(wr->video[v].ext,mext))
                    continue;
                if (vfmt_name && 0 != strcasecmp(wr->video[v].name,vfmt_name))
                    continue;
                break;
            }
            if (NULL == wr->video[v].name) {
                if (debug)
                    fprintf(stderr,"video fmt not found [ext=%s,name=%s]\n",
                            mext,vfmt_name);
                continue;
            }
        }
        if ((!wr->combined && aext) || NULL != afmt_name) {
            if (NULL == wr->audio) {
                if (debug)
                    fprintf(stderr,"no audio\n");
                continue;
            }
            for (a = 0; NULL != wr->audio[a].name; a++) {
                if (!wr->combined &&
                    aext && 0 != strcasecmp(wr->audio[a].ext,aext))
                    continue;
                if (wr->combined &&
                    mext && 0 != strcasecmp(wr->audio[a].ext,mext))
                    continue;
                if (afmt_name && 0 != strcasecmp(wr->audio[a].name,afmt_name))
                    continue;
                break;
            }
            if (NULL == wr->audio[a].name) {
                if (debug)
                    fprintf(stderr,"audio fmt not found [ext=%s,name=%s]\n",
                            wr->combined ? mext : aext, afmt_name);
                continue;
            }
        }
        if (debug)
            fprintf(stderr,"ok:");
        break;
    }
    if (NULL != ng_writers[w]) {
        writer = wr;
        if (-1 != v) {
            if (debug)
                fprintf(stderr," video=%s",wr->video[v].name);
            video.fmtid = wr->video[v].fmtid;
            video_priv  = wr->video[v].priv;
        }
        if (-1 != a) {
            if (debug)
                fprintf(stderr," audio=%s",wr->audio[a].name);
            audio.fmtid = wr->audio[a].fmtid;
            audio_priv  = wr->audio[a].priv;
        }
        if (debug)
            fprintf(stderr,"\n");
    }
}
 
static int
parse_time(char *time)
{
    int hours, minutes, seconds, total=0;
 
    if (3 == sscanf(time,"%d:%d:%d",&hours,&minutes,&seconds))
        total = hours * 60*60 + minutes * 60 + seconds;
    else if (2 == sscanf(time,"%d:%d",&minutes,&seconds))
        total = minutes * 60 + seconds;
 
    if (0 != total) {
        /* hh:mm:ss => framecount */
        return total * fps / 1000;
    }
 
    return atoi(time);
}

static void 
usage (FILE* out)
{  
    fprintf(out,
            "Usage: avs_source [ options ]\n"
            "\n"
            "general options:\n"
            "  -h          print this help text\n"
	    "  -v          verbose mode\n"
            "  -q          quiet operation\n"
	    "  -l nclients number of sink clients       [%d]\n"
            "  -p n        use n compression threads    [%d]\n"
            "  -w seconds  wait before grabbing         [%d]\n"
            "\n"
            "video options:\n"
	    "  -i ppm-file image from file\n"
            "  -o file     video/movie file name\n"
            "  -f format   specify video format\n"
            "  -c device   specify video4linux device   [%s]\n"
            "  -r fps      frame rate                   [%d.%03d]\n"
            "  -s size     specify size                 [%dx%d]\n"
            "\n"
            "  -t times    number of frames or hh:mm:ss [%d]\n"
            "  -b buffers  specify # of buffers         [%d]\n"
            "  -j quality  quality for mjpeg or jpeg    [%d]\n"
            "  -n tvnorm   set pal/ntsc/secam\n"
            "  -I input    set video source\n"
            "\n"
            "audio options:\n"
            "  -O file     wav file name\n"
            "  -F format   specify audio format\n"
            "  -C device   specify dsp device           [%s]\n"
            "  -R rate     sample rate                  [%d]\n"
            "\n",

	    num_client,
            parallel,wait_seconds,
            ng_dev.video, fps/1000, fps%1000,
            video.width, video.height,
            absframes, bufcount, ng_jpeg_quality,
            ng_dev.dsp, audio.rate
        );

    list_formats(out);
} /* end usage */

static void
do_rec_status(char *message)
{
    fprintf(stderr,"%s  \r",message);
}

/*----------------------------------------------------------------------------*/

/*
 * interruptHandler: To go out nicely
 *
 * Parameters: signal number
 * 
 * Return: -
 *
 */

static void
interruptHandler(int signo)
{
  static char text[] = "^C - one moment please\n";
  write(2,text,strlen(text));
  signaled=1;

  EChannel_destroy(chan);
  CManager_close(cm);

  exit(EXIT_SUCCESS);
} /* end interruptHandler */


int
main(int argc, char* argv[])
{
  EControlContext ec;
  ECSourceHandle sourceHandle, sourceHandle1C, sourceHandle2C;
  EChannel chan1C, chan2C;
  int i, maxppmval, optc;
  Raw_data rawrec, debugrec;
  Raw_data_ptr rawrecP = &rawrec, debugrecP = &debugrec;
  Raw_data1C rawrec1C;
  Raw_data1C_ptr rawrec1CP = &rawrec1C;
  Raw_data2C rawrec2C;
  Raw_data2C_ptr rawrec2CP = &rawrec2C;
  int image_fd;
  char* image_file=NULL;
  //int debuggingfd; 
  //char header[PPMHEADERLEN];
  //char shotsentFile[FILENAMELEN];

  char* tmp = NULL;
  int pixOrder = 2;		/* RBG */
  double *bwHist;

  int opt,queued=0;
  char* raw_length = NULL;

  ng_init();
  
  if (argc < 2) usage(stderr);

  for (;;) {
    if (-1 == (opt= getopt(argc, argv, 
			   "hvqdl:p:w:" "i:o:c:f:r:s:t:n:I:b:j:" "O:C:F:R:"))) 
      break;
    switch (opt) {
            /* general options */
        case 'q':
            quiet = 1;
            break;
        case 'd':
            debug++;
            ng_debug++;
            break;
        case 'w':
            wait_seconds = atoi(optarg);
            break;
        case 'p':
            parallel = atoi(optarg);
            break;
    case 'l': /* !!!!!!!!!!!!!!*/
	  num_client = atoi(optarg);
	  break;
        case 'v':
	   verbose= TRUE;
           break; 
 
            /* video options */
        case 'i':
	  input_image_file = TRUE;
	  image_file = strdup(optarg);
	  if (image_file == NULL) exit(EXIT_FAILURE);
	  break;
    case 'o': /* !!!!!!!!!!*/
            moviename = optarg;
            break;
        case 'f':
            vfmt_name = optarg;
            break;
        case 'c':
            ng_dev.video = optarg;
            break;
        case 'r':
            fps = (int)(atof(optarg) * 1000 + 0.5);
            break;
        case 's': 
            if (2 != sscanf(optarg,"%dx%d",&video.width,&video.height))
                video.width = video.height = 0;
            break;

        case 't':
            raw_length = optarg;
            break;
        case 'b':
            bufcount = atoi(optarg);
            break;
        case 'j':
            ng_jpeg_quality = atoi(optarg);
            break;
        case 'n':
            tvnorm = optarg;
            break;
        case 'I':
            input = optarg;
            break;
 
            /* audio options */
        case 'O':
            audioname = optarg;
            break;
        case 'F':
            afmt_name = optarg;
            break;
        case 'C':
            ng_dev.dsp = optarg;
            break;
        case 'R':
            audio.rate = atoi(optarg);
            break;
        case 'h':
            usage(stdout);
            exit(0);
        default:
            usage(stderr);
            exit(1);
    }
  }/*for (;;) */

  if (raw_length)
    absframes = parse_time(raw_length);
  find_formats();

  /* sanity checks */
  if (video.fmtid == VIDEO_NONE && audio.fmtid == AUDIO_NONE) {
    fprintf(stderr,"neither audio nor video format specified/found\n");
    exit(1);
  }
  if (NULL == writer) {
    fprintf(stderr,"no output driver found\n");
    exit(1);
  }
  if (audio.fmtid != AUDIO_NONE && !writer->combined && NULL == audioname) {
    fprintf(stderr,"no audio file name specified\n");
    exit(1);
  }

  rec_status = do_rec_status;
  /* open */
  if (writer && !quiet)
    fprintf(stderr,"%s / video: %s / audio: %s\n",writer->name,
	    ng_vfmt_to_desc[video.fmtid],ng_afmt_to_desc[audio.fmtid]);
  
  if (video.fmtid != VIDEO_NONE) {
    drv = ng_vid_open(ng_dev.video,NULL,0,&h_drv);
    if (NULL == drv) {
      fprintf(stderr,"no grabber device available\n");
      exit(1);
    }
    f_drv = drv->capabilities(h_drv);
    add_attrs(drv->list_attrs(h_drv));
    if (!(f_drv & CAN_CAPTURE)) {
      fprintf(stderr,"%s: capture not supported\n",drv->name);
      exit(1);
    }
    audio_on();
    audio_init();
    
    /* modify settings */
    if (input != NULL)
      do_va_cmd(2,"setinput",input);
    if (tvnorm != NULL)
      do_va_cmd(2,"setnorm",tvnorm);
  }
 

  /* Creation of channel and registration */
  gen_pthread_init();
  cm = CManager_create();  
  ec = ECho_CM_init(cm);

  if (signal(SIGINT, interruptHandler) == SIG_ERR) 
    fprintf(stderr,"Signal error !");

  chan = EChannel_typed_create(ec, Raw_data_fld, NULL);
  if (chan == NULL) fprintf(stderr,"Failed to create channel.\n");
  fprintf(stdout, "Echo channel ID:\n %s\n", ECglobal_id(chan));
  sourceHandle = ECsource_typed_subscribe(chan, Raw_data_fld, NULL);

  //chan2C = EChannel_typed_create(ec, Raw_data2C_fld, NULL);
  chan1C = EChannel_typed_create(ec, Raw_data1C_fld, NULL);
  //if (chan2C == NULL) fprintf(stderr,"Failed to create 2C channel.\n");
  if (chan1C == NULL) fprintf(stderr,"Failed to create 1C channel.\n");
  //fprintf(stdout, "\nEcho 2C (320x240-color) channel ID:\n %s\n", 
  //  ECglobal_id(chan2C));
  fprintf(stdout, "\nEcho 1C (640x480-color) channel ID:\n %s\n", 
	  ECglobal_id(chan1C));
    
  //sourceHandle2C = ECsource_typed_subscribe(chan2C, Raw_data2C_fld, NULL);
  sourceHandle1C = ECsource_typed_subscribe(chan1C, Raw_data1C_fld, NULL);

  fprintf(stdout/*1*/, "Source subscribed to all channels.\n");


  /* Wait for clients */
  CMfork_comm_thread(cm);
  //while (EChas_sinks(sourceHandle) < num_client && 
  // EChas_sinks(sourceHandle2C) < num_client) 
  while (EChas_sinks(sourceHandle) < num_client &&
	 EChas_sinks(sourceHandle1C) < num_client)
    CMusleep(cm, 1000);
  fprintf(stdout/*1*/, "Ready!\n");


  /* init movie writer */
  ng_ratio_x = video.width;
  ng_ratio_y = video.height;
  /*
  movie_state = ECho_movie_writer_init
    (moviename, audioname, writer,
     &video, video_priv, fps,
     &audio, audio_priv, ng_dev.dsp,
     bufcount, parallel, rawrecP, rawrec2CP, sourceHandle2C);
  */
  movie_state = ECho_movie_writer_init
    (moviename, audioname, writer,
     &video, video_priv, fps,
     &audio, audio_priv, ng_dev.dsp,
     bufcount, parallel, rawrecP, rawrec1CP, sourceHandle1C);


  if (NULL == movie_state) {
    fprintf(stderr,"movie writer initialisation failed\n");
    if (video.fmtid != VIDEO_NONE) {
      audio_off();
      drv->close(h_drv);
    }

  EChannel_destroy(chan);
  CManager_close(cm);
    
  exit(1);
  }


  /* wait for some cameras to wake up and adjust light and all that */
  if (wait_seconds)
    sleep(wait_seconds);
  

  /* main loop */
  movie_writer_start(movie_state);
  for (; (absframes==0 || queued < absframes) && !signaled ;) {
    if (DEBUG_WHERE)
      tprintf(" main(): queued = %d\n",queued);

    if (video.fmtid != VIDEO_NONE) {
      /* video */
      queued = movie_grab_put_video(movie_state,NULL);
    } else {
      sleep(1);
      queued += fps / 1000;
    }
  }
  movie_writer_stop(movie_state);
  
  /* done */
  if (video.fmtid != VIDEO_NONE) {
    audio_off();
    drv->close(h_drv);
  }

  EChannel_destroy(chan);
  CManager_close(cm);

  return 0;
}


