/* -*-C-*-
*******************************************************************************
*
* File:         hrtime.h
* RCS:          $Id: hrtime.h,v 1.1.1.1 2003/06/11 18:48:52 wyan Exp $
* Description:  High resolution real-time clock portable routines
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Tue Mar 06, 2001 at 10:25:00
* Modified:     Mon Jul 09, 2001 at 15:15:49 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#ifndef HRTIME_H
#define HRTIME_H 1

#ifdef _MSC_VER

#define HAVE_WINDOWS_H
#define NEED_IOVEC_DEFINE

#include <windows.h>

#define sleep(x) Sleep(1000)
#define srand48(x) srand(x)
#define lrand48() rand()
#define drand48() (((double) rand())/ (double) RAND_MAX)
#define hrtime_t LARGE_INTEGER
#define hrtime_get(x) QueryPerformanceCounter(&x)
#define hrtime_zero(x) x.LowPart = 0; x.HighPart = 0 
#define hrtime_eqZero(x) ((x.LowPart == 0) && (x.HighPart == 0))
#define hrtime_neqZero(x) ((x.LowPart != 0) || (x.HighPart != 0))
#define hrtime_minus2(x,y,z) do_hrtime_minus2(&x, &y, &z)
#define hrtime_add2(x,y,z) do_hrtime_add2(&x, &y, &z)


static void 
do_hrtime_minus2(LARGE_INTEGER *diff, LARGE_INTEGER *src1, 
			  LARGE_INTEGER *src2)
{
  diff->QuadPart = src1->QuadPart - src2->QuadPart;
}

static void 
do_hrtime_add2(LARGE_INTEGER *diff, LARGE_INTEGER *src1, 
                          LARGE_INTEGER *src2)
{
  /*    diff->HighPart = src1->HighPart + src2->HighPart;
        diff->LowPart = src1->LowPart + src2->LowPart;
        if (diff->LowPart < 0) {*/
  diff->QuadPart = src1->QuadPart + src2->QuadPart;
}

static double
hrtime_toSecs(LARGE_INTEGER x)
{
  LARGE_INTEGER freq;
  QueryPerformanceFrequency(&freq);
  return (double)(x.QuadPart) / (double)freq.QuadPart;
}

static double
hrtime_toMsecs(LARGE_INTEGER x)
{
  LARGE_INTEGER freq;
  QueryPerformanceFrequency(&freq);
  return (double)(x.QuadPart) / (double)(freq.QuadPart/1000);
}

#elif linux

#include <sys/time.h>
#include <sys/uio.h>

#define hrtime_t unsigned long long 

#define hrtime_get(x) (__asm__("rdtsc " : "=A" (x)))
#define hrtime_zero(x) x = 0
#define hrtime_eqZero(x) x == 0
#define hrtime_neqZero(x) x != 0
#define hrtime_minus2(x,y,z) x = y - z
#define hrtime_add2(x,y,z) x = y + z
#define hrtime_toSecs(x) ((((double) x) / (450.0 * 1000000.0)))
#define hrtime_toMsecs(x) ((((double) x) / (450.0 * 1000.0)))

#ifdef UNDEF
#define hrtime_get(x) do_hrtime_get(&x)

static void 
do_hrtime_get(unsigned long long* x)
{
  __asm__("rdtsc " 
	  : "=A" (*x));
}

static double
hrtime_toSecs(unsigned long long x)
{
  return (((double) x) / (450.0 * 1000000.0));
}

static double
hrtime_toMsecs(unsigned long long x)
{
  return (((double) x) / (450.0 * 1000.0));
}
#endif /* UNDEF */

#else  /* not _MSC_VER nor linux */

#include <sys/time.h>
#include <sys/uio.h>

#define hrtime_get(x)  x = gethrtime()
#define hrtime_zero(x) x = 0
#define hrtime_eqZero(x) x == 0
#define hrtime_neqZero(x) x != 0
#define hrtime_minus2(x,y,z) x = y - z
#define hrtime_add2(x,y,z) x = y + z
#define hrtime_toSecs(x) (((double) x) / 1000000000.0)
#define hrtime_toMsecs(x) (((double) x) / 1000000.0)

#ifdef UNDEF
static double
hrtime_toSecs(hrtime_t x)
{	
  return ((double) x) / 1000000000.0;
}

static double
hrtime_toMsecs(hrtime_t x)
{
  return ((double) x) / 1000000.0;
}
#endif /* UNDEF */

#endif

#endif /* !HRTIME_H */
