/* -*-C-*-
*******************************************************************************
*
* File:         argl.h
* RCS:          $Id: argl.h,v 1.1.1.1 2003/06/11 18:49:00 wyan Exp $
* Description:  
* Author:       Fabian E. Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Wed Jul 04, 2001 at 21:56:59
* Modified:     Thu Jul 19, 2001 at 21:34:58 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#ifndef ARGL_H
#define ARGL_H 1

#include <stdio.h>

#define ARGL_MAX_ITEMS      100
#define ARGL_MAX_LINE_WIDTH 100

/*
 * ARGL_INT
 * Syntax: <parm>=<integer>. 
 *         <integer> can be prefixed by 0x... or 0..., specifying
 *         hexadecimal resp. octal base; decimal is the default. It
 *         can be suffixed by "k"/"K" resp. "M" resp. "G", meaning x
 *         1024 resp. x 1048576 resp.  x 2^30. "m" is not allowed.
 *
 * ARGL_STRING
 * Syntax: <parm>=<string>
 * 
 * ARGL_BOOL    
 * Syntax: <parm>=[no|yes]. 
 *         <parm> is <parm>=yes.
 *         Other synonyms for yes resp. no:
 *         YES,Yes,Y,y,1,true,on,enable resp.
 *         NO,No,N,n,0,false,off,disable
 *
 *
 * ARGL_DOUBLE   
 * Syntax: <parm>=<float>. 
 *         <float> is the usual floating point ASCII representation.
 *
 */

typedef enum { 
  ARGL_INT,
  ARGL_STRING,
  ARGL_BOOL,
  ARGL_DOUBLE
} Argl_item_T;

/* Error code for the last unsuccessful call to an argl function. */
extern int Argl_errno;

#define ARGL_ERRNO_VARSIZE    1
#define ARGL_ERRNO_ARG_FORMAT 2
#define MAX_ERRNO             2

#define ARGL_REGISTER_INT(NAME,VAR) do { \
      Argl_register (NAME, &(VAR), sizeof(VAR), ARGL_INT); \
    } while (0)
#define ARGL_REGISTER_STRING(NAME,VAR) do { \
      Argl_register (NAME, &(VAR), sizeof(VAR), ARGL_STRING); \
    } while (0)
#define ARGL_REGISTER_BOOL(NAME,VAR) do { \
      Argl_register (NAME, &(VAR), sizeof(VAR), ARGL_BOOL); \
    } while (0)
#define ARGL_REGISTER_DOUBLE(NAME,VAR) do { \
      Argl_register (NAME, &(VAR), sizeof(VAR), ARGL_DOUBLE); \
    } while (0)

extern char * Argl_strerror (int errno);

extern int Argl_register (char *name, void *var, unsigned long varsize, 
			  Argl_item_T ait);

extern int Argl_parse (int argc, char **argv);

extern int Argl_parse_file (FILE * f);

extern int Argl_reset ();

#endif /* !ARGL_H */
