/* -*-C-*-
 * @ Wenchang Yan
 * @date Dec, 2001
*******************************************************************************
*
* File:         avs_source.c
* RCS:          $Id: avs_source.c,v 1.1.1.1 2003/06/11 18:49:00 wyan Exp $
* Description:  AVS source
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Fri Feb 16, 2001 at 00:06:23
* Modified:     Fri Oct 12, 2001 at 10:08:18 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <signal.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <io.h>
#include <atl.h>
#include <gen_thread.h>
#include <cm.h>
#include <echo.h>
//#include <pds.h>

#include <vdev.h>
#include <getopt.h>
#include <tv.h>
#include <hrtime.h>
#include <avs_util.h>
//#include <avs_raw.h>
#include <styx.h>
#include <sensNet.h>
#include <stats.h>

#include </home/wyan/Infosphere/isg/templates/ECho2/multiUAV/allNodes.h>
#include </home/wyan/Infosphere/isg/templates/ECho2/multiUAV/sender/sender.h>


// structure for holding result of cmd line options
struct optBlock {
  int verbose;
  int debugging;
  int MeasureMe;
  int fileimage;
  char *imageFilename;
  int pixOrder;
  int NumRec;
  int Freq;
  int reportFreq;
  int fps; 
};

// global var to hold all these since original prog used a global
static struct optBlock optionBlock;


//static int NumRec = 0;
//static boolean verbose = FALSE;
//static boolean MeasureMe = FALSE;
//static int Freq = 2;
//static boolean debugging = FALSE;
//static boolean fileimage = FALSE;
//static boolean use_PDS = FALSE;
static char* local_progname;

//EChannel chan;
//CManager cm;
int NumRecSubmitted;

//pds_service directory;

struct rusage RusageBuffer;
struct timeval sendStart;	/* time for all sends */
struct timeval startCpu, stopCpu, usedCpu; /* CPU used for all sends */
struct timeval startUCpu, stopUCpu, usedUCpu; /* U CPU used for all sends */
struct timeval startSCpu, stopSCpu, usedSCpu; /* S CPU used for all sends */

/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"fileimage", required_argument, NULL, 'f'},
  {"debugging", no_argument, NULL, 'd'},
  {"help", no_argument, NULL, 'h'},
  {"clients", required_argument, NULL, 'c'},
  {"numframes", required_argument, NULL, 'n'},
  {"measure", no_argument, NULL, 'm'},
  {"sample", required_argument, NULL, 's'},
  {"report", required_argument, NULL, 'r'},
  {"order", required_argument, NULL, 'o'},
  {"use-PDS", no_argument, NULL, 'P'},
  {"host", required_argument, NULL, 'H'},
  {"context", required_argument, NULL, 'C'},
  {"name", required_argument, NULL, 'N'},
  {"description", required_argument, NULL, 'X'},
  {NULL, no_argument, NULL, 0}
};


  VDev_T vdev = NULL;
  struct VDev_imageSize vdev_size;


/*
typedef struct PDS_args_struct
{
  char *pds_hostname;
  int pds_port;
  char *pds_context;
  char *pds_name;
  char *pds_desc;
} PDS_args, *PDS_args_ptr;
*/


int parseOptions(int argc, char *argv[], struct optBlock *optionBlock );
int initializeCamera();



/*vvv*/
#define DEFAULT_FPS 15
/*for debug*/
#undef DEBUG_WHERE
#define DEBUG_WHERE 1

#include <time.h>
#include <sys/timeb.h>
#include <stdarg.h>
int tprintf(const char* fmt,...)
{
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  fprintf(stdout,"[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}
/*^^^*/


/*----------------------------------------------------------------------------*/

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */

static void usage(FILE *stream, int status)
{  
  fprintf(stream, "Usage: %s [-vhm] -c nclients -n nframes [-F fps] [-s freq] [-r freq] "
	  "[-o pixorder] [-f ppm-file] [-P [-H host:port] [-C pds-context] "
	  "[-N name] [-X description]]\n"
	  "-v --verbose              \tverbose mode\n"
	  "-f --fileimage ppm-file   \timage from file\n"
	  "-d --debugging            \tdebugging mode\n"
	  "-h --help                 \tgive this help\n"
//	  "-c --clients  nclients    \tnumber of sinks clients\n"
	  "-n --numframes  nframes   \tnumber of frames (0 == infinite)\n"
	  "-m --measure              \tset bandwidth measuring at client\n"
	  "-s --sample               \tsampling frequency for measurements\n"
	  "-r --report               \tfreq of bandwidth report\n"
	  "-o --order pixorder       \t[1: RGB, 2: RBG, 3: GRB, 4: GBR, 5: BRG, 6: BGR]\n"
//          "-P --use-PDS              \tregister streams with PDS server\n"
//          "-H --pds-host host[:port] \thost and port for PDS server\n"
//          "-C --context <string>     \tcontext under which to register source\n"
//          "-N --name <string>        \tname to use for the source\n"
//          "-X --description <string> \tdescription to use for the source\n"
	  "-F --framerate per sec    \tfps for image capturing (1-30)\n"
     	  , local_progname);
  exit(status);
} /* end usage */

/*----------------------------------------------------------------------------*/

/*
 * dumpStats: Compute and dump stats
 *
 * Parameters: -
 *
 * Returns: -
 *
 */

void dumpStats()
{
  struct timeval sendStop, sendTime; /* time for all sends */
  struct timeval stopCpu, usedCpu; /* CPU used for all sends */
  struct timeval stopUCpu, usedUCpu; /* U CPU used for all sends */
  struct timeval stopSCpu, usedSCpu; /* S CPU used for all sends */
  double *bwHist;

  getrusage(RUSAGE_SELF, &RusageBuffer);
  gettimeofday(&sendStop, NULL);
  tv_add2(&stopCpu, &(RusageBuffer.ru_utime), &(RusageBuffer.ru_stime));
  tv_assign(&stopUCpu, &(RusageBuffer.ru_utime));
  tv_assign(&stopSCpu, &(RusageBuffer.ru_stime));
  tv_minus2(&usedCpu, &stopCpu, &startCpu);
  tv_minus2(&usedUCpu, &stopUCpu, &startUCpu);
  tv_minus2(&usedSCpu, &stopSCpu, &startSCpu);
  tv_minus2(&sendTime, &sendStop, &sendStart);
  fprintf(stdout, "\n----------------------------------------\n");
  fprintf(stdout, "Source performance data\n");
  fprintf(stdout, "-----------------------\n");
  fprintf(stdout, "Total time:       \t%ld ms\n", tv_toMsecs(&sendTime));
  fprintf(stdout, "Time per event:   \t%.3g ms\n", 
	  (float) tv_toMsecs(&sendTime)/NumRecSubmitted);
  fprintf(stdout, "CPU usage:        \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedCpu), 
	  (float) tv_toMsecs(&usedCpu)/tv_toMsecs(&sendTime));
  fprintf(stdout, "User CPU usage:   \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedUCpu), 
	  (float) tv_toMsecs(&usedUCpu)/tv_toMsecs(&sendTime));
  fprintf(stdout, "System CPU usage: \t%ld ms (%.3g)\n", 
	  tv_toMsecs(&usedSCpu), 
	  (float) tv_toMsecs(&usedSCpu)/tv_toMsecs(&sendTime));
  if (optionBlock.MeasureMe) {
    bwHist = (double*) malloc (((int) (NumRecSubmitted/optionBlock.Freq)) * sizeof(double));
    if (bwHist == NULL) Styx_errQuit("Not enough resources");
    sensNet_GetHistory(bwHist, (int) (NumRecSubmitted/optionBlock.Freq), SENSNET_READ);
    fprintf(stdout, "Bandwidth mean:   \t%.3g Mbps (sdev %.3g)\n",
	    Stats_mean (bwHist, (int) (NumRecSubmitted/optionBlock.Freq)),
	    Stats_sdev (bwHist, (int) (NumRecSubmitted/optionBlock.Freq)));
    sensNet_Finish();
    free(bwHist);
  }
  fprintf(stdout, "\a\a");

  fflush(stdout);
} /* end dumpStats */


/*----------------------------------------------------------------------------*/

/*
 * interruptHandler: To go out nicely
 *
 * Parameters: signal number
 * 
 * Return: -
 *
 */
//Added for the performance experiment only -- Romoved after the 
//experiment. !!!!!!!!!
char file_name[] = "time_rec";
char chan_id_file[] = "chanId";
FILE* time_rec, *chan_file;
/*the interval between 2 frames, in microsec*/
unsigned long interval;
struct timeval prev_time, curr_time;
double time_diff;
int frame_transfered = 0;


static void interruptHandler(int signo)
{
  //Added for the performance experiment only -- Removed after the 
  //experiment. !!!!!!!!!
  gettimeofday(&curr_time,NULL);

  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
  time_diff = time_diff/1000000;
  fprintf(time_rec," Streaming:%f,  frames:%d\n",time_diff,frame_transfered);
  fclose(time_rec);

  dumpStats();  
  infopipeStop_sender();
//  EChannel_destroy(chan);
//  CManager_close(cm);

  exit(EXIT_SUCCESS);
} /* end interruptHandler */

/*----------------------------------------------------------------------------*/

/*
 * register_in_PDS:  place the channel IDs in the PDS server
 *
 * Parameters: args struct, channel ID/format indicator pairs
 *
 * Returns: -
 *
 */
/*
static void register_in_PDS ( PDS_args_ptr args, char *chanID1, char *ind1, char *chanID2, char *ind2)
{
  attr_list contact_attrs;
  attr_list ent_attrs;
  pds_domain_id_t domain_id;
  char buf[200];

  contact_attrs = create_attr_list();
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_HOST"),
            Attr_String,
            strdup (args->pds_hostname));
  set_attr (contact_attrs,
            attr_atom_from_string ("IP_PORT"),
            Attr_Int4,
            (attr_value*)args->pds_port);

  directory = pds_service_open (contact_attrs);

  domain_id = pds_open_domain (directory, "demo", "demo", 1, "avs");

  ent_attrs = create_attr_list();
  add_attr (ent_attrs, attr_atom_from_string ("StreamDescription"),
            Attr_String, args->pds_desc);
  add_attr (ent_attrs, attr_atom_from_string ("StreamChannelID"),
            Attr_String, chanID1);
  add_attr (ent_attrs, attr_atom_from_string ("StreamFormatDescription"),
            Attr_String, strdup ("Raw image"));
  add_attr (ent_attrs, attr_atom_from_string ("StreamFormatTag"),
            Attr_String, strdup ("RAW"));

  memset (buf, 0, 200);
  strcat (buf, "/avs/streams/");
  strcat (buf, args->pds_name);
  strcat (buf, "/");
  strcat (buf, ind1);
  
  pds_create_entity (domain_id, buf, null_pds_context_id, NULL, ent_attrs);

  set_attr (ent_attrs, attr_atom_from_string ("StreamChannelID"),
            Attr_String, chanID2);
  set_attr (ent_attrs, attr_atom_from_string ("StreamFormatDescription"),
            Attr_String, strdup ("640 x 480, color"));
  set_attr (ent_attrs, attr_atom_from_string ("StreamFormatTag"),
            Attr_String, strdup ("1C"));

  memset (buf, 0, 200);
  memset (buf, 0, 200);
  strcat (buf, "/avs/streams/");
  strcat (buf, args->pds_name);
  strcat (buf, "/");
  strcat (buf, ind2);

  pds_create_entity (domain_id, buf, null_pds_context_id, NULL, ent_attrs);

  pds_service_close (directory);

}
*/
  

int initOptBlock( struct optBlock *ob )
{
  ob->verbose       = 0;
  ob->debugging     = 0;
  ob->MeasureMe     = 0;
  ob->fileimage     = FALSE;
  ob->imageFilename = NULL;
  ob->pixOrder      = 2;
  ob->NumRec        = 0;
  ob->Freq          = 0;
  ob->reportFreq    = 0;
  ob->fps           = DEFAULT_FPS;
  return 0;
}

int initStyx()
{
  Styx_asDaemon(0);             /* not a daemon, actually do not use syslog */
  Styx_setDebugLevel(0);        /* no debugging unless requested */
  Styx_setProgname(local_progname);
  return 0;
}

/*----------------------------------------------------------------------------*/
int main(int argc, char *argv[]) 
{
  //EControlContext ec;
  /*vvv*/
  //ECSourceHandle sourceHandle, sourceHandle1C;
  //ECSourceHandle sourceHandle, sourceHandle1C, sourceHandle2C;
  //EChannel chan1C;
  //EChannel chan1C, chan2C;
  /*Raw_data1C rawrec1C;*/
  /*Raw_data1C_ptr rawrec1CP = &rawrec1C;*/
  //Raw_data1C rawrec1C;
  //Raw_data1C_ptr rawrec1CP = &rawrec1C;
  Raw_data_2CStruct rawrec2C;
  Raw_data_2CStruct_ptr rawrec2CP = &rawrec2C;
  /*the interval between 2 frames, in microsec*/
  //struct timeval prev_time, curr_time;
  //double time_diff;
  /*^^^*/
//  char* imageFilename=NULL;
  //int fps=DEFAULT_FPS;
  //int i, maxppmval, optc, numClient=0;
  //PDS_args my_PDS_args;
  char* tmp = NULL;
  //int pixOrder = 2; /*2;*/		/* RBG */
  
  /*^^^*/
  Raw_dataStruct rawrec, debugrec;
  Raw_dataStruct_ptr rawrecP = &rawrec, debugrecP = &debugrec;
  /*vvv*/

  int i, maxppmval; //, numClient=0;
  //int width, height;//size from the command line
  unsigned long interval;

  int imagefd;       // file des of image READ for debugging
  int debuggingfd;   // file des of image WRITTEN for debugging

  char header[PPMHEADERLEN]; // holds header info from debug ppm file
  char shotsentFile[FILENAMELEN];  // filename for debug write into file

//  VDev_T vdev = NULL;
//  struct VDev_imageSize vdev_size;

  double *bwHist;
  int reported;
  //int reportFreq;

  //my_PDS_args.pds_hostname = "localhost";
  //my_PDS_args.pds_port = 8848;
  //my_PDS_args.pds_name = my_PDS_args.pds_context = my_PDS_args.pds_desc = NULL;

  // convenience - must be done before initStyx() called
  local_progname = argv[0];  

  // initialize the debug engine (must be done after local_progname assigned)
  initStyx();

  // start up the pipe
  infopipeStart_sender();

  
  initOptBlock( &optionBlock );
  // parse options and set in option block, 
  // if failure, print usage note and exit
  if( parseOptions( argc, argv, &optionBlock ) ) 
    usage(stderr, EXIT_FAILURE);
  
  // Styx_debugMsg(1, "Number of clients: \t%d\n", numClient);

  // open file for sending in debug mode (no read from camera)
  if( optionBlock.fileimage ) {
    fprintf(stdout, "Image file: \t%s\n", optionBlock.imageFilename);
    if( (imagefd = open(optionBlock.imageFilename, O_RDONLY)) < 0 ) 
      Styx_errQuit("Failed opening image file.\n");    
    
    // read header info
    read(imagefd, header, sizeof(header));   
    sscanf(header, "%c%c\n%d %d\n%d\n", 
           &(debugrecP->ppm1), &(debugrecP->ppm2),
           &(debugrecP->width), &(debugrecP->height), 
	         &maxppmval);

    fprintf(stdout, "PPM %c%c\n", debugrecP->ppm1, debugrecP->ppm2);
    fprintf(stdout, "Width %d Height %d\n", debugrecP->width, debugrecP->height);
    fprintf(stdout, "Max PPM val %d\n", maxppmval);

    debugrecP->size = debugrecP->width * debugrecP->height 
                        * 3 * sizeof(u_char);
    //tmp = (char*)malloc(debugrecP->size);
    //&(debugrecP->buff)[0] = tmp;

    if (debugrecP->buff == NULL) 
      Styx_errQuit("Out of mem\n");
    fprintf(stdout, "Size %d\n", debugrecP->size);
    
    read(imagefd, debugrecP->buff, debugrecP->size);

    debuggingfd = open("shotsent-debug.ppm", 
                       O_CREAT|O_WRONLY, S_IRWXU|S_IRGRP|S_IROTH);
    sprintf(header, "%c%c\n%d %d\n%d\n", debugrecP->ppm1, debugrecP->ppm2, 
            debugrecP->width, debugrecP->height, 255);  
    write(debuggingfd, header, sizeof(header));
 
    if (debugrecP->buff != NULL) 
      write(debuggingfd, debugrecP->buff, debugrecP->size);
    close(debuggingfd);
  }

  /*
   * Creation of channel and registration
   *
   */
  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
//  if (NULL == (time_rec=fopen(file_name, "a")) || 
//      NULL == (chan_file=fopen(chan_id_file,"w"))) {
//    printf("file open error!\n");
//    exit(1);
//  }

//  printf(" file open for writing\n");
  gettimeofday(&prev_time,NULL);


//  gen_pthread_init();
//  cm = CManager_create();  


//  CMfork_comm_thread(cm);

  if (signal(SIGINT, interruptHandler) == SIG_ERR) 
    Styx_errQuit("Signal error");

//  ec = ECho_CM_init(cm);

  /*vvv*/
  /*
  chan2C = EChannel_typed_create(ec, Raw_data2C_fld, NULL);
  if (chan2C == NULL) Styx_errQuit("Failed to create 2C channel.\n");
  sourceHandle2C = ECsource_typed_subscribe(chan2C, Raw_data2C_fld, NULL);


  chan = EChannel_typed_create(ec, Raw_data_fld, NULL);
  if (chan == NULL) Styx_errQuit("Failed to create channel.\n");
  fprintf(stdout, "Echo channel ID:\n %s\n", ECglobal_id(chan));
  sourceHandle = ECsource_typed_subscribe(chan, Raw_data_fld, NULL);

  fprintf(stdout, "\nEcho 2C (320x240-color) channel ID:\n %s\n", 
	  ECglobal_id(chan2C));
  */

  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
  //fprintf(chan_file,"%s\n",ECglobal_id(chan2C));
  //fclose(chan_file);







  //Added for the performance experiment only -- Romoved after the 
  //experiment. !!!!!!!!!
  gettimeofday(&curr_time,NULL);
  time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
  //fprintf(time_rec," Init:%f,  ",time_diff);


  


  Styx_debugMsg(1, "Source subscribed to all channels.\n");

  /*  register with PDS if indicated  */
/*
  if (use_PDS)
    {
      register_in_PDS (&my_PDS_args, 
                       ECglobal_id(chan), "raw",
                       ECglobal_id(chan1C), "1C");
    }

*/
  /* Wait for clients */
  //- CMfork_comm_thread(cm);
  /*vvv*/
  //while (EChas_sinks(sender__send1SendHandle) < numClient /*&& 
  // EChas_sinks(sourceHandle2C) < numClient*/) 
    //CMusleep(cm, 1000);
  /*^^^*/
  Styx_debugMsg(1, "Ready!\n");


  initializeCamera(); 
  Styx_debugMsg(1, "Camera initizalized.\n");

  /*
   * Blasting the guy now
   *
   */

  if(optionBlock.MeasureMe) {
    sensNet_Init(optionBlock.Freq);
    sensNet_SetHistLen(optionBlock.NumRec/optionBlock.Freq, SENSNET_WRITE);
  }
  fprintf(stdout, "\n\n");

  gettimeofday(&sendStart, NULL);
  getrusage(RUSAGE_SELF, &RusageBuffer);
  tv_add2(&startCpu, &(RusageBuffer.ru_utime), &(RusageBuffer.ru_stime));
  tv_assign(&startUCpu, &(RusageBuffer.ru_utime));
  tv_assign(&startSCpu, &(RusageBuffer.ru_stime));

  NumRecSubmitted = 0;
  reported = 1;

  /* Set up for fps */
  /* Not exactly 1 sec; Use 1.5 sec to adjust the real fps */
  interval = 1500000/optionBlock.fps;
  gettimeofday(&prev_time,NULL);

  printf(" before loop: NumRec: %d\n",optionBlock.NumRec);

  for(i = 0; ((i < optionBlock.NumRec) || (optionBlock.NumRec == 0)); ++i) {
    printf("in loop.\n");
    fflush(stdout);
    /*for fps control*/
    /*
    gettimeofday(&curr_time,NULL);
    time_diff = (curr_time.tv_sec - prev_time.tv_sec)*1000000
      + curr_time.tv_usec - prev_time.tv_usec;
    //tprintf(" [prev: [%ld:%ld] curr:[%ld:%ld] diff:%f  interval:%ld\n",prev_time.tv_sec,prev_time.tv_usec,curr_time.tv_sec,curr_time.tv_usec,time_diff,interval);
    if (time_diff>=0) {
      prev_time.tv_sec = curr_time.tv_sec;
      prev_time.tv_usec = curr_time.tv_usec;
      usleep(interval - time_diff <= 0?0:interval-time_diff);
      //tprintf(" [wait: %f\n\n", interval-time_diff);
    }
    */

    //Added for the performance experiment only -- Romoved after the 
    //experiment. !!!!!!!!!
    if (i==0)
      gettimeofday(&prev_time,NULL);

    if (!optionBlock.fileimage) {
      char *abuff;

      /* read data */
      rawrecP->tag = i+1;
      rawrecP->ppm1 = 'P';
      rawrecP->ppm2 = '6';
      
      /*vvv*/
      //if (DEBUG_WHERE) tprintf(" [main(): before VDev_getPPMImage()\n");
      /*^^^*/

      // read from the camera
      //printf("Read from camera.\n");
      //rawrecP->buff = (char*) VDev_getPPMImage(vdev, optionBlock.pixOrder);
      abuff = (char*) VDev_getPPMImage(vdev, optionBlock.pixOrder);

      /*vvv*/
      //if (DEBUG_WHERE) tprintf(" [main(): after VDev_getPPMImage()\n");

      if ((rawrecP->size = VDev_getRawSize(vdev)) < 0) {
        fprintf(stderr, "Something went wrong reading the image!\n");
        //rawrecP->buff = NULL;
        rawrecP->size = 0;
      }
      rawrecP->width = vdev_size.width;
      rawrecP->height = vdev_size.height;
      //ECsubmit_typed_event(sourceHandle, rawrecP);

      rawrec2CP->tag = rawrecP->tag;
      rawrec2CP->ppm1 = 'P';
      rawrec2CP->ppm2 = '6';
      rawrec2CP->width = vdev_size.width;
      rawrec2CP->height = vdev_size.height;
      rawrec2CP->size = rawrecP->size;
      //memcpy(rawrec2CP->buff, rawrecP->buff, rawrecP->size);
      memcpy(rawrec2CP->buff, abuff, rawrecP->size);
      
      //if (DEBUG_WHERE) tprintf(" [main(): before sends to sourceHandle2C\n");
      
      //ECsubmit_typed_event(sourceHandle2C, rawrec2CP);
      //
      //SEND
      sender__send1Submit( (Raw_dataStruct_ptr)rawrec2CP );


      //Added for the performance experiment only -- Romoved after the 
      //experiment. !!!!!!!!!
      frame_transfered ++;
      printf("avs_source.c:main(): frame_transfered: %d\n",frame_transfered);


      //if (DEBUG_WHERE) tprintf(" [main(): after sends to sourceHandle2C\n");
	
      
      if(optionBlock.debugging) {
        sprintf(shotsentFile, "shotsent%d.ppm", i+1);
        debuggingfd = open(shotsentFile, 
                           O_CREAT|O_WRONLY, S_IRWXU|S_IRGRP|S_IROTH);
        sprintf(header, "%c%c\n%d %d\n%d\n", rawrec2CP->ppm1, rawrec2CP->ppm2, 
        rawrec2CP->width, rawrec2CP->height, 255);  
        write(debuggingfd, header, sizeof(header));
        if (rawrec2CP->buff != NULL) 
          write(debuggingfd, rawrec2CP->buff, rawrec2CP->size);
        close(debuggingfd);
      }
     


      //free(rawrecP->buff);
      free(abuff);
    }
    else {
      printf("avs_source.c():main():Doing else.\n");
      debugrecP->tag = i+1;
      //ECsubmit_typed_event(sourceHandle, debugrecP);
      //
      //SEND
      sender__send1Submit( (Raw_dataStruct_ptr)  debugrecP );
    }
    ++NumRecSubmitted;
      /*^^^*/

    if( optionBlock.MeasureMe ) {
      if( NumRecSubmitted == (reported * optionBlock.reportFreq) ) {
        bwHist = (double*) malloc (((int) (NumRecSubmitted/optionBlock.Freq)) * 
                                                        sizeof(double));
        sensNet_GetHistory(bwHist, (int) (NumRecSubmitted/optionBlock.Freq), SENSNET_WRITE);
        if( bwHist == NULL ) 
          Styx_errQuit("Not enough resources.\n");

        fprintf(stdout, "Bandwidth mean:   \t%.3g Mbps [sdev %2.3g]\r",
        Stats_mean (bwHist, (int) (NumRecSubmitted/optionBlock.Freq)),
        Stats_sdev (bwHist, (int) (NumRecSubmitted/optionBlock.Freq)));
        fflush(stdout);
        free(bwHist);
        ++reported;
      }
    }

  } /* end for */
  
  dumpStats();

  if( optionBlock.MeasureMe ) 
    sensNet_Finish();
  //EChannel_destroy(chan);
  //CManager_close(cm);

  exit(EXIT_SUCCESS);
} /* main */

int parseOptions(int argc, char *argv[], struct optBlock *optionBlock )
{
  int optc;
  /*
  if(argc < 3) {
    fprintf(stderr, "Not enough options." );
    return 1;
  } */
  // options parsing
  //while((optc = getopt_long(argc, argv, "+vhmdf:o:c:n:s:r:PH:C:N:X:S:F:", 
  while( (optc = getopt_long(argc, argv, "+vhmdf:o:c:n:s:r:F:",longopts, (int*) 0)) 
                                                         != EOF ) {
    printf("optc: %c",optc);
    switch (optc) {
    case 'v':
      optionBlock->verbose= TRUE;
      Styx_setDebugLevel(7);
      Styx_privateLog(0);
      break; 
    case 'd':
      optionBlock->debugging= TRUE;
      optionBlock->verbose = TRUE;
      break;
    case 'm':
      optionBlock->MeasureMe = TRUE;
      break; 
    case 'f':
      optionBlock->fileimage = TRUE;
      optionBlock->imageFilename = strdup(optarg);
      if(optionBlock->imageFilename == NULL) exit(EXIT_FAILURE);
      break;
    case 'o':
      optionBlock->pixOrder = atoi(optarg);
      if(optionBlock->pixOrder < 1 || optionBlock->pixOrder > 6) usage(stderr, EXIT_FAILURE);
      break; 
    case 'h':
      usage(stdout, EXIT_SUCCESS);
      break;
    case 'n':
      optionBlock->NumRec = atoi(optarg);
      if(optionBlock->reportFreq < 0) {
        fprintf(stderr, "Option Error: Number of frames (option '-f' or '--numframes') must be greater than or equal to 0.\n");
        return 1;
      }
//      if(optionBlock->NumRec < 0) usage(stderr, EXIT_FAILURE);
//      break;
    case 's':
      optionBlock->Freq = atoi(optarg);
      if(optionBlock->reportFreq < 0) {
        fprintf(stderr, "Option Error: Sampling frequency (option '-s' or '--sample') must be greater than or equal to 0.\n");
        return 1;
      }
//      if(optionBlock->Freq < 0) usage(stderr, 1);
//      break;
    case 'r':
      optionBlock->reportFreq = atoi(optarg);
      if(optionBlock->reportFreq < 0) {
        fprintf(stderr, "Option Error: Report frequency (option '-r' or '--report') must be greater than or equal to 0.\n");
        return 1;
      }
//      if(optionBlock->reportFreq < 0) usage(stderr, 1);
//      break;
/*    case 'c':
      optionBlock->numClient = atoi(optarg);
      if(optionBlock->numClient < 1 || optionBlock->numClient > MAXCLIENTS) usage(stderr, EXIT_FAILURE);
      break;*/
/*    case 'P':
      optionBlock->use_PDS = TRUE;
      break;
    case 'H':
      optionBlock->tmp = strchr (optarg, ':');
      if(optionBlock->tmp != NULL)
        {
          // port number present 
          *optionBlock->tmp++ = '\0';
        }
      my_PDS_args.pds_hostname = strdup (optarg);
      if (!my_PDS_args.pds_hostname) exit(EXIT_FAILURE);
      if (tmp != NULL)
        {
          my_PDS_args.pds_port = atoi (tmp);
          if (my_PDS_args.pds_port < 0) exit(EXIT_FAILURE);
        }
      break;
    case 'C':
      my_PDS_args.pds_context = strdup (optarg);
      if (!my_PDS_args.pds_context) exit(EXIT_FAILURE);
      break;
    case 'N':
      my_PDS_args.pds_name = strdup (optarg);
      if (!my_PDS_args.pds_name) exit(EXIT_FAILURE);
      break;
    case 'X':
      my_PDS_args.pds_desc = strdup (optarg);
      if (!my_PDS_args.pds_desc) exit(EXIT_FAILURE);
      break;
*/
    case 'F':
      optionBlock->fps = atoi(optarg);
      if(optionBlock->fps<=0) {
        fprintf(stdout,"Defaulting to %d fps.\n", DEFAULT_FPS);
        optionBlock->fps = DEFAULT_FPS;
      }
      else if(optionBlock->fps >=15) {
        fprintf(stdout, "Hardware limitation! Reducing capture fps to 15.\n");
        optionBlock->fps = 15; /* >15 doesn't mean anything to our cam*/
      }
      break;
    default:
      return 1;
      usage(stderr, EXIT_FAILURE);
    }
  }

  
  if (optind > argc) {
    fprintf(stderr, "Too many command line options.\n" );
    return 1; /* no extra args */
  }

  fprintf(stdout, "avs_source.c:parseOptions(): All options parsed normally.\n");
  return 0;
} // parseOptions()


int initializeCamera()
{
  /* Initialize camera */
  if (!optionBlock.fileimage) {
    /*vvv*/
    //VDev_init();
    VDev_init(320,240);
    /*^^^*/

    printf(" [initializeCamera(): after VDev_init()\n");

    if (VDev_numVDevices() < 1) Styx_errQuit("No camera connected.\n");
    
    vdev = VDev_getVDevice(0);
    if (vdev == NULL || !VDev_isValid(vdev)) 
      Styx_errQuit ("Something is wrong, what camera are you asking for?\n");
    
    Styx_debugMsg(1,"V4L name: %s\n", VDev_getIntfName(vdev));
    
    VDev_open(vdev, 2);
    if (VDev_getDescriptor(vdev) < 0) 
      Styx_errQuit("Something is wrong, what camera are you asking for?\n");
    
    
    VDev_getSize(vdev, &vdev_size);
    Styx_debugMsg(1,"Size (width, heigth): (%d, %d)\n", 
		  vdev_size.width, vdev_size.height);
  }
  return 0;
}
