/* -*-C-*-
*******************************************************************************
*
* File:         common.h
* RCS:          $Id: common.h,v 1.1.1.1 2003/06/11 18:49:01 wyan Exp $
* Description:  Common def to all - should eventually go as common-h.in 
*               That file should then be process with configure to produce 
*               common.h
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Tue Jul 03, 2001 at 08:23:12
* Modified:     Tue Jul 03, 2001 at 09:19:15 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#ifndef COMMON_H
#define COMMON_H 1

#include <stdlib.h>

/* HARDWIRE CONFIGURATION */
#define HAVE_STDARG_H
/* !HARDWIRE CONFIGURATION */

#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#  define VA_START(a, f) va_start(a, f)
#elif HAVE_VARARGS_H
#  include <varargs.h>
#  define VA_START(a, f) va_start(a)
#else
   error no variadic api
#endif

#ifndef WIFEXITED
#  define WIFEXITED(stat)      (((stat) & 0xff) == 0)
#endif
#ifndef WEXITSTATUS
#  define WEXITSTATUS(stat)    ((unsigned)(stat) >> 8)
#endif
#ifndef WIFSTOPPED
#  define WIFSTOPPED(stat)     (((stat) & 0xff) == 0x7f)
#endif
#ifndef WSTOPSIG
#  define WSTOPSIG(stat)       WEXITSTATUS(stat)
#endif
#ifndef WIFSIGNALED
#  define WIFSIGNALED(stat)    (!WIFEXITED(stat) && !WIFSTOPPED(stat))
#endif
#ifndef WTERMSIG
#  define WTERMSIG(stat)       ((stat) & 0x7f)
#endif

#ifndef PARAMS
#  if __STDC__
#    ifndef NOPROTOS
#      define PARAMS(args)      args
#    endif
#  endif
#  ifndef PARAMS
#    define PARAMS(args)        ()
#  endif
#endif

#ifdef __cplusplus
#  define BEGIN_C_DECLS		extern "C" {
#  define END_C_DECLS		}
#else
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif

#ifndef CMN_STMT_START
#  define CMN_STMT_START	do
#  define CMN_STMT_END		while (0)
#endif

#ifdef __GNUC__
#  ifndef const
#    define const	__const
#  endif
#  ifndef signed
#    define signed	__signed
#  endif
#  ifndef volatile
#    define volatile	__volatile
#  endif
#else
#  ifdef __STDC__
#    undef  signed
#    define signed
#    undef  volatile
#    define volatile
#  endif
#endif

#ifdef __STDC__
#define STR(x)		#x
#define CONC(x, y)	x##y
typedef void* pointer;
#else
#define STR(x)		"x"
#define CONC(x, y)	x/**/y
typedef char* pointer;
#endif

/* If FALSE is defined, we presume TRUE is defined too.  In this case,
   merely typedef boolean as being int.  Or else, define these all.  */
#ifndef FALSE
/* Do not use `enum boolean': this tag is used in SVR4 <sys/types.h>.  */
typedef enum { FALSE = 0, TRUE = 1 } myboolean;
#else
typedef int myboolean;
#endif
#ifndef boolean
#  define boolean myboolean
#endif

#ifndef EXIT_Success
#  define EXIT_SUCCESS  0
#  define EXIT_FAILURE  1
#endif

#define XCALLOC(type, num)	((type *) avs_xcalloc ((num), sizeof(type)))
#define XMALLOC(type, num)	((type *) avs_xmalloc ((num) * sizeof(type)))
#define XREALLOC(type, p, num)	((type *) avs_xrealloc ((p), (num) * sizeof(type)))
#define XFREE(stale)				CMN_STMT_START {	\
  	if (stale) { free ((pointer) stale);  stale = 0; }		\
						} CMN_STMT_END

BEGIN_C_DECLS

extern pointer avs_xcalloc		PARAMS((size_t num, size_t size));
extern pointer avs_xmalloc		PARAMS((size_t num));
extern pointer avs_xrealloc		PARAMS((pointer p, size_t num));
/* extern char *xstrdup		PARAMS((const char *string)); */

END_C_DECLS

#if WITH_DMALLOC
#  include <dmalloc.h>
#endif

#endif /* !COMMON_H */
