/* -*-C-*-
*******************************************************************************
*
* File:         stats.c
* RCS:          $Id: stats.c,v 1.1.1.1 2003/06/11 18:49:01 wyan Exp $
* Description:  
* Author:       Fabian E. Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Sat Sep 29, 2001 at 17:49:49
* Modified:     Sat Sep 29, 2001 at 18:02:48 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#if HAVE_CONFIG_H
#  include <src/config.h>
#endif

#include <math.h>
#include <stats.h>

/*----------------------------------------------------------------------------*/

/*
 * Stats_mean: Mean of an array
 * 
 * Parameters: array and its lenght
 * 
 * Returns: Mean
 *
 */

double 
Stats_mean(array, len)
     double *array; 
     int len;
{
  int i;
  double mean;
  
  if (len <= 0) return -1;
  
  mean = 0.0;
  
  for (i = 0; i < len; i++) {
    mean += array[i];
  }

  if (len > 0) {
    mean /= len;
  }

  return mean;
} /* end Stats_mean */

/*----------------------------------------------------------------------------*/

/*
 * Stats_sdev:
 * 
 * Parameters:
 * 
 * Returns:
 *
 */

double 
Stats_sdev(array, len)
     double *array; 
     int len;
{
  if (len <= 0) return -1;
  return sqrt(Stats_var(array,len));
}

/*----------------------------------------------------------------------------*/

/*
 * Stats_var:
 * 
 * Parameters:
 * 
 * Returns:
 *
 */

double 
Stats_var(array, len)
     double *array; 
     int len;
{
  int i;
  double mean;
  double var;

  if (len <= 0) return -1;

  if (len < 2) {
    return 0.0;
  }

  mean = Stats_mean(array,len);
  var= 0.0;
  for (i=0; i < len; i++) {
    var += SQUARE(array[i]-mean);
  }
  var/= len - 1;

  return var;
} /* end Stats_var */
