/* -*-C-*-
*******************************************************************************
*
* File:         strlt.c
* RCS:          $Id: strlt.c,v 1.1.1.1 2003/06/11 18:49:01 wyan Exp $
* Description:  Streamlets for avs
* Author:       Fabian E. Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Thu Jul 19, 2001 at 15:19:03
* Modified:     Tue Oct 02, 2001 at 23:36:10 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/


#if HAVE_CONFIG_H
#  include <src/config.h>
#endif

#include <strlt.h>
#include <io.h>
#include <argl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

struct Streamlet_T_struct {
  char *name;
  char *code;
  char *inputChan;
  char *outputChan;
  int takesParams;
  char *params;
};


/*----------------------------------------------------------------------------*/

/*
 * Strlt_new:
 * 
 * Parameters:
 * 
 * Returns:
 *
 */

Streamlet_T
Strlt_new() 
{
  Streamlet_T strlt;

  strlt = (Streamlet_T) malloc(sizeof(*strlt));
  if (strlt == NULL) return NULL;
  bzero((void*) (strlt), sizeof(*strlt));
  strlt->code = NULL;

  return strlt;
} /* end Strlt_new */


/*----------------------------------------------------------------------------*/

/*
 * Strlt_free:
 * 
 * Parameters:
 * 
 * Returns:
 *
 */

int
Strlt_free(strlt) 
     Streamlet_T strlt;
{
  XFREE(strlt->name);
  XFREE(strlt->code);
  XFREE(strlt->inputChan);
  XFREE(strlt->outputChan);
  if (strlt->takesParams) XFREE(strlt->params);

  return 0;
} /* end Strlt_free */


/*----------------------------------------------------------------------------*/

/*
 * Strlt_init:
 * 
 * Parameters:
 * 
 * Returns:
 *
 */

int Strlt_init( Streamlet_T strlt, char *path, char *name)
{  
  struct stat sbuf;
  FILE* cfgFile;
  int fd;
  int codesize;
  char* pathname_cfg = NULL; 
  char* pathname_strlt = NULL; 

  /* check on the streamlet config file */
  pathname_cfg = XMALLOC(char, (strlen(path) + strlen(name) + strlen("cfg") + 3));
  sprintf(pathname_cfg, "%s/%s.cfg", path, name);

  Argl_reset ();
 
  cfgFile = fopen(pathname_cfg, "r");
  ARGL_REGISTER_STRING("STRLT_name", strlt->name);
  ARGL_REGISTER_STRING("STRLT_input", strlt->inputChan);
  ARGL_REGISTER_STRING("STRLT_output", strlt->outputChan);
  ARGL_REGISTER_BOOL("STRLT_takesParams", strlt->takesParams);
  ARGL_REGISTER_STRING("STRLT_parameter", strlt->params);
  Argl_parse_file(cfgFile);

  fprintf (stderr, "after parse, strlt takes params=%d\n", strlt->takesParams);

  /* check on the streamlet now */
  pathname_strlt = XMALLOC(char, (strlen(path) + strlen(strlt->name) + 2));
  sprintf(pathname_strlt, "%s/%s", path, strlt->name);
  if (stat(pathname_strlt, &sbuf) < 0) return -1; /* cannot get to it */

  codesize = sbuf.st_size;
  strlt->code = XMALLOC (char, codesize + 1);
  memset (strlt->code, 0, codesize+1);
  if ((fd = open(pathname_strlt, O_RDONLY)) < 0) 
    return -1; /* cannot open streamlet's file */
  if ((read(fd, strlt->code, codesize)) < 0) 
    return -1; /* cannot read streamlet */

  fclose(cfgFile);
  close (fd);

  return 0;
} /* end Strlt_init */


char *
Strlt_getName (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return NULL;
  return strlt->name;
} /* end Strlt_getName */


char *
Strlt_getCode (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return NULL;
  return strlt->code;
} /* end Strlt_getCode */


char *
Strlt_getInputChan (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return NULL;
  return strlt->inputChan;
} /* end Strlt_getInputChan */


char *
Strlt_getOutputChan (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return NULL;
  return strlt->outputChan;
} /* end Strlt_getOutputChan */

int 
Strlt_takesParams (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return -1;
  return strlt->takesParams;
}

char *
Strlt_getParams (strlt)
     Streamlet_T strlt;
{
  if (!strlt)
    return NULL;
  return strlt->params;
} /* end Strlt_getParams */

