//
//$Id: Monitor.h,v 1.1.1.1 2003/06/11 19:05:35 wyan Exp $
//
//File: Monitor.h
//      The Monitor pipe
//

#ifndef _MONITOR_H
#define _MONITOR_H

#include "CtrlManager.h"
#include "orbsvcs/AV/Endpoint_Strategy.h"
#include "Pipe.h"


/////////////////////////////////////////////////
//Customerized init

//These should be put into Monitor.h to be shared by both the senderMonitor
//and the ReceiverMonitor
extern int PACKET_SIZE;
extern int PAYLOAD_SIZE;
extern int NR_PACKETS;
extern int DATA_TYPE;//char=1, int=2, float=3, double=4
extern int INTERVAL; //in milli-sec
extern char OUT_FILE[50];
//
//////////////////////////////////////////////////


/* This endpoint is type A - client */
class SenderMonitorClientStreamEndPoint;
class ReceiverMonitorClientStreamEndPoint;
/* This endpoint is type B - server */
class SenderMonitorServerStreamEndPoint;
class ReceiverMonitorServerStreamEndPoint;


class SenderMonitorClientStreamEndPoint : public TAO_Client_StreamEndPoint
{
 public:
  int get_callback (const char *flowname,
		    TAO_AV_Callback*& callback);
  
  int set_protocol_object (const char* flowname,
			 TAO_AV_Protocol_Object* object);
  
  CORBA::Boolean handle_preconnect (AVStreams::flowSpec& flow_spec);

 protected:

  TAO_AV_Callback callback;
};


typedef TAO_AV_Endpoint_Reactive_Strategy_A
        <SenderMonitorClientStreamEndPoint,
         TAO_VDev,
         AV_Null_MediaCtrl> SENDER_MONITOR_CLIENT_ENDPOINT_STRATEGY;


class SenderMonitorClient : public Pipe
{
 public:
  SenderMonitorClient(void);
  
  ~SenderMonitorClient(void);

  int init (int argc, 
	    char** argv, 
	    const char* clientName,
	    const char* serverName,
	    CORBA::Environment&);

  int sendData (void);

  CtrlManager& getCtrlManager (void);

  const char* getName(void) const;

  //Virtual method from Pipe class
  virtual int initPipe();

 protected:
  
  CtrlManager ctrlManager;

  SENDER_MONITOR_CLIENT_ENDPOINT_STRATEGY monitorClientEndpointStrategy;

  //It's a pointer, not an instance.
  TAO_MMDevice* clientMMDevice;

  ACE_CString clientName;

};


typedef ACE_Singleton<SenderMonitorClient, ACE_Null_Mutex> SENDER_MONITOR_CLIENT;

//==================================================================

class ReceiverMonitorClientStreamEndPoint : public TAO_Client_StreamEndPoint
{
 public:
  int get_callback (const char *flowname,
		    TAO_AV_Callback*& callback);
  
  int set_protocol_object (const char* flowname,
			 TAO_AV_Protocol_Object* object);
  
  CORBA::Boolean handle_preconnect (AVStreams::flowSpec& flow_spec);

 protected:

  TAO_AV_Callback callback;
};

typedef TAO_AV_Endpoint_Reactive_Strategy_A
        <ReceiverMonitorClientStreamEndPoint,
         TAO_VDev,
         AV_Null_MediaCtrl> RECEIVER_MONITOR_CLIENT_ENDPOINT_STRATEGY;


#include "ControlCenterServer.h"

class ReceiverMonitorClient : public Pipe
{
 public:
  ReceiverMonitorClient(void);
  
  ~ReceiverMonitorClient(void);

  int init (int argc, 
	    char** argv, 
	    const char* clientName,
	    const char* serverName,
	    CORBA::Environment&);

  int sendData (void);

  CtrlManager& getCtrlManager (void);

  const char* getName(void) const;

  //Virtual method from Pipe class
  virtual int initPipe();


  //Corba object
  ControlCenterImpl* cci;

 protected:
  
  CtrlManager ctrlManager;

  RECEIVER_MONITOR_CLIENT_ENDPOINT_STRATEGY monitorClientEndpointStrategy;

  //It's a pointer, not an instance.
  TAO_MMDevice* clientMMDevice;

  ACE_CString clientName;

};


typedef ACE_Singleton<ReceiverMonitorClient, ACE_Null_Mutex> RECEIVER_MONITOR_CLIENT;


//==================================================================


class SenderMonitorServerCallback : public TAO_AV_Callback
{
 public:
  SenderMonitorServerCallback (void);
  
  int receive_frame (ACE_Message_Block *frame,
                     TAO_AV_frame_info *frame_info,
                     const ACE_Addr &peer_address);

 protected:

};

class SenderMonitorServerStreamEndPoint : public TAO_Server_StreamEndPoint
{
 public:
  int get_callback (const char* flowname, TAO_AV_Callback*& callback);

 protected:
  SenderMonitorServerCallback callback;
};




typedef TAO_AV_Endpoint_Reactive_Strategy_B
        <SenderMonitorServerStreamEndPoint,
         TAO_VDev,
         AV_Null_MediaCtrl> SENDER_MONITOR_SERVER_ENDPOINT_STRATEGY;






class SenderMonitorServer : public Pipe
{
 public:
  SenderMonitorServer (void);

  int init (int argc, char** argv, const char* clientName,
	    const char* serverName, CORBA::Environment&);

  CtrlManager& getCtrlManager (void);

  const char* getName (void) const;

  //Virtual method from Pipe class
  virtual int initPipe();

 protected:
  CtrlManager ctrlManager;

  SENDER_MONITOR_SERVER_ENDPOINT_STRATEGY monitorServerEndpointStrategy;

  TAO_MMDevice* serverMMDevice;

  ACE_CString clientName;

  ACE_CString serverName;

};




typedef ACE_Singleton<SenderMonitorServer, ACE_Null_Mutex> SENDER_MONITOR_SERVER;
//=====================================================================


class ReceiverMonitorServerCallback : public TAO_AV_Callback
{
 public:
  ReceiverMonitorServerCallback (void);
  
  int receive_frame (ACE_Message_Block *frame,
                     TAO_AV_frame_info *frame_info,
                     const ACE_Addr &peer_address);

 protected:

};

class ReceiverMonitorServerStreamEndPoint : public TAO_Server_StreamEndPoint
{
 public:
  int get_callback (const char* flowname, TAO_AV_Callback*& callback);

 protected:
  ReceiverMonitorServerCallback callback;
};

typedef TAO_AV_Endpoint_Reactive_Strategy_B
        <ReceiverMonitorServerStreamEndPoint,
         TAO_VDev,
         AV_Null_MediaCtrl> RECEIVER_MONITOR_SERVER_ENDPOINT_STRATEGY;

class ReceiverMonitorServer : public Pipe
{
 public:
  ReceiverMonitorServer (void);

  int init (int argc, char** argv, const char* clientName,
	    const char* serverName, CORBA::Environment&);

  CtrlManager& getCtrlManager (void);

  const char* getName (void) const;

  //Virtual method from Pipe class
  virtual int initPipe();

 protected:
  CtrlManager ctrlManager;

  RECEIVER_MONITOR_SERVER_ENDPOINT_STRATEGY monitorServerEndpointStrategy;

  TAO_MMDevice* serverMMDevice;

  ACE_CString clientName;

  ACE_CString serverName;


};




typedef ACE_Singleton<ReceiverMonitorServer, ACE_Null_Mutex> RECEIVER_MONITOR_SERVER;


#endif
