//$Id: SenderMonitorServer.cpp,v 1.3 2003/06/25 15:16:22 wyan Exp $
// File: SenderMonitorServer.cpp
//
//Since all the measurement results are output at the SenderMonitorServer but the
//measurement results are mostly calculated at the receiver side, some msg format
//must be agreed between the Sender and the ReceiverMonitor. Here it is :
//
// INFOCP 
//         
//
// if `Type' == 1 then This is a returned msg for calculating latency
//    `Length' is used to store pkt_size.
//
//  0 1 2 3      8          16                     31
// +---+-+------+----------+-------------------------+
// |Ver|P|RecCt | Type     |     Length              |
// +---+-+------+----------+-------------------------+
// |  the returned sender time stamp                 |
// +-------------------------------------------------+
//
//  (In this demo, this kind of latency msg is never gonna be transmitted anymore.
//
// if `Type' == 2 then This is a measurement report for each receiver window
//    `Length' is used to store pkt_size.
//
//  0 1 2 3      8          16                     31
// +---+-+------+----------+-------------------------+
// |Ver|P|RecCt | Type     |     Length              |
// +---+-+------+----------+-------------------------+
// | FilterType (Or the returned sender time stamp)  |
// | bw in the last window                           |
// | jitter in the last window                       |
// | receiving rate (in nr_pkt/sec)                  |
// | missing pkt rate in  the last window            |
// | last received packet seq #                      |
// | user required target pkt_rate/sec               |
// +-------------------------------------------------+
//
//
// if `Type' == 4 then This is the measurement report for the whole sesseion
//    `Length' is used to store pkt_size.
//
//
//  0 1 2 3      8          16                     31
// +---+-+------+----------+-------------------------+
// |Ver|P|RecCt | Type     |     Length              |
// +---+-+------+----------+-------------------------+
// | the returned sender time stamp (useless)        |
// | bw                                              |
// | jitter                                          |
// | packets per second                              |
// | missing packets rate                            |
// | receiver side qos processing time               |
// | sender side qos processing time                 |
// | filter type                                     |
// +-------------------------------------------------+

#include "Monitor.h"
#include "ace/Get_Opt.h"

#define DEBUG 0

int accum_qos_time = 0;

//////////////////////////////////////
// For measurement: All defined in Monitor.cpp
#include "Share_Syn.h"
extern char* shm, *shm_buf;
extern int semid;
extern union semun sunion;
extern int DeleteSemid;

//
/////////////////////////////////////


///////////////////////////////////////////////////////////
//Custermized code.
//This makes the received frame be
//accessible to ReceiverMonitorServer class. So that
//ReceiverMonitorServer class can further process the 
//received data. 
//These two defined in Monitor.cpp
extern Type* global_buf;
extern int global_buf_length;

////////////////////////////////////////
// FlowCtrl
bool flowCtrl = false;
#define INIT_PKT_RATE 1 //? pkts/sec
int packet_size = 0;
float bw = 0;
float accum_bw = 0;
int jitter = 0;
int accum_jitter = 0;
int pkt_rate = 0;// # of pkts per second
int proc_pkt_rate = 0;//the process ability from the recv side.
float accum_missing_pkt_rate = 0;
float window_accum_missing_pkt_rate = 0;
int accum_pkt_rate = 0;// per second
int accum_qos_time_receiver = 0;
int accum_qos_time_sender = 0; 
unsigned int base;//base time

int target_fps = 1;

unsigned int FILTER=0;

int parseArgs (int argc, char** argv/*, CORBA::Environment& ACE_TRY_ENV*/)
{
  if(argc < 1 + 0){
    ACE_DEBUG((LM_DEBUG, "Usage:\n\n"));
    ACE_DEBUG((LM_DEBUG, "   SenderMonitorServer [-l flowCtrl] \n"));
    return -1;
  }

  // Parse command line arguments
  ACE_Get_Opt opts (argc, argv, "l:");

  int c;
  while ((c= opts ()) != -1)
    {
      switch (c)
        {
        case 'l':
	  if (strcmp("1", opts.optarg) == 0)
	    flowCtrl = true;
          break;
        default:
          ACE_DEBUG ((LM_DEBUG, "Unknown Option\n"));
          return -1;
        }
    }
  return 0;
}

//
///////////////////////////////////////////////////////////


//Helper class
class Signal_Handler : public ACE_Event_Handler
{
public:
  Signal_Handler() : exit_requested(0) { }

  int handle_signal(int signum, siginfo_t*,ucontext_t*) {
    exit_requested = 1;
  }

  int exit_requested;
};



int main(int argc, char** argv)
{
  ACE_DECLARE_NEW_CORBA_ENV;

  int result;
  pid_t pid;
  CORBA::ORB_var orb;

  ACE_Time_Value now = ACE_OS::gettimeofday();
  unsigned int tmp = now.sec();
  base = tmp - tmp%1000 ;//Assume SenderMonitorServer will start within 1000 secs.

  ACE_TRY
    {
      orb = CORBA::ORB_init (argc,
			     argv,
			     0,
			     ACE_TRY_ENV);
      ACE_TRY_CHECK;
 
      CORBA::Object_var obj
        = orb->resolve_initial_references ("RootPOA"
                                           );
      ACE_TRY_CHECK;

      PortableServer::POA_var root_poa
        = PortableServer::POA::_narrow (obj.in ()
                                        );
      ACE_TRY_CHECK;

      PortableServer::POAManager_var mgr
        = root_poa->the_POAManager ();
      ACE_TRY_CHECK;

      mgr->activate ();
      ACE_TRY_CHECK;


      TAO_AV_CORE::instance ()->init (orb.in (),
                                      root_poa.in ()
                                      );
      ACE_TRY_CHECK;

      /////////////////////////////////////////////////
      //For Measuremnet
      ACE_Shared_Memory_SV shm_writer(SHM_KEY_S, SHMSZ_S,
				     ACE_Shared_Memory_SV::ACE_CREATE);

      shm = (char *) shm_writer.malloc ();
      if (shm == 0)
        ACE_ERROR_RETURN ((LM_ERROR,
                       "%p\n\t(%P|%t) Cannot create shared memory segment.\n"
                           ,argv[0]), -1);
      ACE_DEBUG ((LM_INFO,
                  "(%P|%t) Shared Memory is at 0x%x\n",
                  shm ));
      /*
      //Create semaphore
      if (-1==(semid = semget(IPC_PRIVATE, 3, SHM_R|SHM_W))) 
        ACE_ERROR_RETURN((LM_ERROR,
                          "(%P|%t) Cannot create semaphore\n"),-1);
      else 
        ACE_DEBUG((LM_DEBUG,
                   "(%P|%t) Create semaphore with id: %d\n", semid));

      DeleteSemid = semid;
      atexit(&mydelete);
      signal(SIGINT, &sigdelete);
      sunion.val = 0;
      safesemctl(semid, SN_READ, SETVAL, sunion);
      sunion.val = 1;
      safesemctl(semid, SN_LOCK, SETVAL, sunion);
      safesemctl(semid, SN_WRITE, SETVAL, sunion);
      */
      //store the semid to shm
      *(int*)shm = 0;
      shm_buf = shm + sizeof(int);
      //
      //////////////////////////////////////////////////////////

      ////////////////////////////////////////////
      //Customized code
      parseArgs(argc, argv);

      if (flowCtrl) {
	cout<<" Flow Ctrl required\n";
	*(unsigned int*)shm_buf = 0x80000000; 
	*(unsigned int*)(shm_buf + 3 * sizeof(unsigned int)) = INIT_PKT_RATE;
      }
      else {
       	*(unsigned int*)shm_buf = 0;
      }

      result = SENDER_MONITOR_SERVER::instance() ->init (argc,
						  argv,
						  "ReceiverMonitor",
						  "SenderMonitor",
						  ACE_TRY_ENV);
      ACE_TRY_CHECK;
      
      if (result < 0)
	ACE_ERROR_RETURN ((LM_ERROR,
			   "SenderMonitor::main(): init failed\n"), -1);
      
      Signal_Handler signal_handler;

      ////////////////////////////
      //Customized code
      DataFlow df;
      CtrlFlow cf;
      INFOCP infocp;
      Data data;  data.SIZE=0; data.buff=NULL;
      int nr_packet_received = 0;
      int window_nr_packet_received = 0;
      double accum_time = 0;
      double window_accum_time = 0;
      double last_accum_time = 0;
      ACE_Time_Value now, last;
      float inter_time = 0;
      int feedback_T = 0;
      //
      /////////////////////////////

      global_buf = NULL;

      while (!signal_handler.exit_requested) {
	orb->perform_work();
	ACE_TRY_CHECK;

	now = ACE_OS::gettimeofday();
  
	if (!global_buf)
	  continue;

	SENDER_MONITOR_SERVER::instance() ->
	  importInCtrlFlow((char*)global_buf, global_buf_length);

	cf = SENDER_MONITOR_SERVER::instance() -> getInCtrlFlow();
	infocp = cf.getHeader();
	cf.getPayload(&data);

	packet_size = infocp.length;
	nr_packet_received ++;
	window_nr_packet_received ++;

	if (DEBUG) {
	  cout<<" infocp: size = "<<sizeof(infocp)<<endl;
	  cout<<" infocp: ver("<<infocp.ver<<"), pad("<<infocp.pad
	      <<"), rec_ct("<<infocp.rec_ct<<"), type("<<infocp.type
	      <<"), length("<<infocp.length<<endl;
	}

	//	*shm_buf = 1; 
	if (1 == infocp.type) {
	  //This kind of msg shall never be seen in this version of the demo, i.e.
	  //the latency measurement is dropped cause it's too expensive and useless
	  //in flowCtrl.
	  //this packet includes time stamp to calculate the latency
	  int tt = *(int*)data.buff;
	  tt = (now.sec() - base)*1000000 + now.usec() - tt;

	  inter_time = (double)((now.sec() - last.sec())*1000000 +
				now.usec() - last.usec())/1000;

	  accum_time += tt;
	  window_accum_time += tt;
	  last_accum_time = accum_time;
	  
	  if (DEBUG) {
	    cout<< "    packet received: "<<nr_packet_received
		<< "    window pkt recved: "<<window_nr_packet_received
		<< ", each packet size "<<packet_size<<" bytes.\n";
	    cout<< "         total time: "<<accum_time<<" usec\n";
	    cout<< "         latency: "
		<< accum_time/nr_packet_received/1000
		<< " milli-sec."<<endl;
	    cout<< " inter_time: "<<inter_time<<" ms"<<endl;
	    cout<< " now("<<now.sec()<<":"<<now.usec()<<endl;
	    cout<< "last("<<last.sec()<<":"<<last.usec()<<endl;
	    cout<< " base:"<<base<<", tt:"<<tt<<endl;
	    cout<< " llllllllllllllllllllllllllllllllllllllll \n\n";
	  }
	  last = now;
	}
	else if ( 2==infocp.type) {
	  //this packet includes report for each receiver window

	  //Get the filterType first.
	  FILTER = *(unsigned int*)data.buff;
	  if ( FILTER & 0x80000000 ) flowCtrl = true;
	  else flowCtrl = false;
	  //*(unsigned int*)shm_buf = t;
	    
	  //t = (now.sec() - base)*1000000 + now.usec() - t;
	  //accum_time += t;
	  //window_accum_time += t;
	  last_accum_time = accum_time;

	  bw = *(float*)(data.buff + 4);
	  jitter = *(int*)(data.buff + 8);
	  pkt_rate = *(int*)(data.buff + 12);
	  window_accum_missing_pkt_rate = *(float*)(data.buff + 16);
    	  unsigned int t_seq = *(int*)(data.buff + 20);//last recv'ed pkt seq #.
	  //proc_pkt_rate = *(int*)(data.buff + 24);
          target_fps = *(int*)(data.buff+24);

	  if (flowCtrl)
	    {
	      //update shm
	      //locksem(semid, SN_WRITE);
	      //if (DEBUG)
	      //printf("Waiting for lock..."),
	      //  fflush(stdout);
	      //locksem(semid, SN_LOCK);
	      //if (DEBUG)
	      //printf("Got lock...\n"),
	      //  fflush(stdout);
	      *(unsigned int*)shm_buf = FILTER; 
	      *(unsigned int*)(shm_buf + sizeof( unsigned int)) = 
		(unsigned int)(accum_time / nr_packet_received);
	      *(unsigned int*)(shm_buf + 2 * sizeof( unsigned int)) = 0;//minum_latency unused
	      *(unsigned int*)(shm_buf + 3 * sizeof( unsigned int)) = pkt_rate;
	      *(unsigned int*)(shm_buf + 4 * sizeof(unsigned int)) = t_seq;
	      *(float*)(shm_buf + 5 * sizeof( unsigned int)) = 
		window_accum_missing_pkt_rate;
              *(int*)(shm_buf + 5*sizeof(unsigned int) + sizeof(int)) = target_fps;
	      //unlocksem(semid, SN_LOCK);
	      //unlocksem(semid, SN_READ);
	    }

	  window_accum_time =0;
	  window_nr_packet_received =0;

	  if (DEBUG) {
	    cout<< " Result: \n";
	    cout<< "  total packet received: "<<nr_packet_received
		<< ", each packet size "<<packet_size<<" bytes.\n";
	    cout<< "         total time: "<<accum_time<<" usec\n";
	    cout<< "  window total time: "<<window_accum_time<<" usec\n";
	    cout<< "         latency: "
		<< accum_time/nr_packet_received/1000
		<< " milli-sec."<<endl;
	    cout<< "  window latency: "
		<< window_accum_time/window_nr_packet_received/1000
		<< " milli-sec."<<endl;
	    cout<< " window B/W: "<<bw<<" Mbps\n";
	    cout<< " window jitter: "<<jitter/1000<<" ms.\n";
	    cout<< " window Packet rate: "<<pkt_rate<<" pkts/sec \n";
	    cout<< " window missing pkt rate: "<<window_accum_missing_pkt_rate<<endl;
	    cout<< " Last received pkt seq #: "<<t_seq
		<< " inter_pkt_time: "<<inter_time<<" ms"
		<< endl;
	    cout<< " wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww "<<t_seq<<" th pkt\n\n";
	  }
	}
	else if ( 4==infocp.type ) {
	  //Tell Sender to end.
	  *shm_buf = 99;

	  //this packet includes the final report from the receiver window
	  accum_time = last_accum_time; // !!!!!!!!!!!!!!!!!
	  accum_bw = *(float*)data.buff;
	  accum_jitter = *(int*)(data.buff + 4);
	  accum_pkt_rate = *(int*)(data.buff + 8);
	  accum_missing_pkt_rate = *(float*)(data.buff + 12);
	  accum_qos_time_receiver = *(int*)(data.buff + 16);
	  accum_qos_time_sender = *(int*)(data.buff + 20);
	  int filter_type = *(int*)(data.buff + 24);

	  cout<< " Latency result: \n";
	  cout<< "         total packet received: "<<nr_packet_received-1
	      << ", each packet size "<<packet_size<<" bytes.\n";
	  cout<< "         total time: "<<accum_time<<" usec\n";
	  cout<< "         latency: "
	      << accum_time/(nr_packet_received-1)/1000
	      << " milli-sec."<<endl;

	  cout<< " Avg B/W: "<<accum_bw<<" Mbps\n";
	  cout<< " Avg jitter: "<<accum_jitter/1000<<" ms.\n";
	  cout<< " Packet rate: "<<accum_pkt_rate<<" pkts/sec \n";
	  cout<< " Missing pkt rate: "<<accum_missing_pkt_rate<<"\n";
	  cout<< " Sender qos time: "<<accum_qos_time_sender<<" ms\n";
	  cout<< " Receiver qos time: "<<accum_qos_time_receiver<<" ms\n";
	  if (flowCtrl) 
	    cout<<" Flow Ctrl applied\n";
	  else 
	    cout<<" No Flow Ctrl\n";
	  cout<< " FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF\n\n";

	  *(unsigned int*)shm_buf = 99; 
	  break;
	}//else if (4==infocp.type)
      } //while () 
    
      ACE_OS::sleep(1);

      SENDER_MONITOR_SERVER::instance ()->getCtrlManager().destroy (ACE_TRY_ENV);
      ACE_TRY_CHECK;

      shm_writer.close();
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,"SenderMonitor::main(): Exception !\n");
      return -1;
    }
  ACE_ENDTRY;
  //ACE_CHECK_RETURN (-1);


  ACE_OS::sleep (1);
  
  orb->destroy ();
  //ACE_TRY_CHECK;

  //!!!!
  //if(fd) ACE_OS::fclose (fd);

  return 0;
}
