/*
 * @author Wenchang Yan
 * @time Dec, 2001
 */

extern "C" {
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <stdio.h>

#include "connect.h"
}

#define DEBUG_WHERE 1
#define DEBUG_WHAT 1

int socket_create(const char* host, int port, const char* transport)
{
  struct hostent* p_hostent;
  struct sockaddr_in server;
  int sd,rc,type,proto_num;

  memset(&server, 0, sizeof(server));
  
  /* get server IP address (no check if input is IP address or DNS name */
  p_hostent = gethostbyname(host);
  if(p_hostent==NULL) {
    printf(" [socket_create: %s: unknown host '%s' \n", host);
    exit(1);
  }

  if (DEBUG_WHERE)
    printf(" [socket_create: Ready to send data to '%s' (IP : %s) \n",
	    p_hostent->h_name, 
	    inet_ntoa(*(struct in_addr *)p_hostent->h_addr_list[0]));

  set_server(&server, INADDR_ANY, port);

  /* which connection type */
  if (strcmp("udp",transport)==0) { 
    type = SOCK_DGRAM;
    proto_num = 17; /* from /etc/protocols */
  }
  else if (strcmp("tcp",transport)==0){ 
    type = SOCK_STREAM;
    proto_num = 6;
  }
  else {
    type = SOCK_DGRAM;
    proto_num = 0;
  }

  /* socket creation */
  sd = socket(AF_INET,type,0);/* 0 for now */
  if(sd<0) {
    printf(" [socket_create: cannot open socket \n");
    exit(1);
  }

  /* bind any port */
  rc = bind(sd, (struct sockaddr *) &server, sizeof(server));
  if(rc<0) {
    printf( "[socket_create: cannot bind port\n");
    exit(1);
  }

  if (DEBUG_WHERE) 
    printf(" [socket_create: socket binding successful, port %d\n",ntohs(server.sin_port));
  return sd;
}
  
void set_server(struct sockaddr_in* server, int addr, int port) 
{
  memset(server,0,sizeof(*server));
  server->sin_family = AF_INET;
  server->sin_addr.s_addr = htonl(addr);
  server->sin_port = htons(port);
}


int connect_udp(const char* host, int port)
{
  return socket_create(host,port,"udp");
}

int connect_tcp(const char* host, int port)
{
  return socket_create(host,port,"tcp");
}
