//
//$Id: display.cpp,v 1.1.1.1 2003/06/11 19:05:35 wyan Exp $
//
//File: display.cpp
//      Display the raw binary image
//

#include "display.h"
#include <string.h>
#include <stdio.h>

#define DEBUG_WHERE 0

/*
 * class RawImageData
 */

RawImageData::RawImageData()
{
}

void RawImageData::Create(int width,int height)
{
  image.Create(width,height);
}

void RawImageData::CreateFromExistingMemory(Pixel* data,int width,int height)
{
  //Add sth. ?????????????
  image.CreateFromExistingMemory(data,width*3,height);
  //width*3 because each Pixel consists of 3 uint8, but Pixel is uint8
}

const Pixel*const* RawImageData::AskFrame_const() const
{
  return image.AskFrame_const();
}




//==========================================================================



VideoSink_X11::VideoSink_X11() 
  : imgwin(NULL), x11img(NULL), decimg(NULL), curr_img_width(0), curr_img_height(0)
{
}


VideoSink_X11::~VideoSink_X11()
{
  if (imgwin)
    {
      delete imgwin;
      delete x11img;
    }
}


int VideoSink_X11::showPPM(char* ppmFrame, int width, int height, 
			   int bytesPerPixel, uint32 PPMFrameNum,
			   char magic1, char magic2)
{
  curr_PPM_frame_num = PPMFrameNum;

  int bytes_per_line = width* bytesPerPixel;

  if (imgwin == NULL) {
    imgwin = new ImageWindow_X11;
    x11img = new DisplayImage_X11;

    imgwin->SetPosition (-1, -1);
    imgwin->Create(width, height, "ooOOoo");
    x11img->Create(width, height, imgwin->AskWindow());
  }
  else if ((curr_img_width != 0 && curr_img_height != 0) &&
	   (curr_img_width != width || curr_img_height != height))
    {
      if (x11img) delete x11img, x11img = NULL; 
      if (imgwin) delete imgwin, imgwin = NULL;
      imgwin = new ImageWindow_X11;
      x11img = new DisplayImage_X11;

      imgwin->SetPosition (-1, -1);
      imgwin->Create(width, height, "ooOOoo");
      x11img->Create(width, height, imgwin->AskWindow());
    }

  setImage(ppmFrame, width, height, bytesPerPixel);

  XImage& ximg = x11img->AskXImage();
  imgspec.bytes_per_line = ximg.bytes_per_line;
  imgspec.bits_per_pixel = ximg.bits_per_pixel;
  imgspec.little_endian  = (ximg.byte_order==LSBFirst);
  imgspec.SetRGBMasks(ximg.red_mask, ximg.green_mask, ximg.blue_mask);
  
  if ('P' == magic1 && '6' == magic2) {
    TransformPixmap (decimg->image,
		     (uint8*)(x11img->AskXImage().data),
		     0,
		     height-1,
		     width,
		     imgspec);
  }
  else if ('P' == magic1 && '5' == magic2) {
    TransformGreymap (decimg->image,
		      (uint8*)(x11img->AskXImage().data),
		      0,
		      height-1,
		      width,
		      imgspec);
  }

  x11img->PutImage();
  
  if (decimg) decimg->FreePictureData();
    
  curr_img_width = width;
  curr_img_height = height;

  return 1;
}


void VideoSink_X11::setImage(char* ppmFrame, int width, int height, int bytesPerPixel)
{
  if (NULL == decimg) {
    decimg = new RawImageData();
  }

  if (decimg->image.IsEmpty()) {
    decimg->Create(width * bytesPerPixel , height);
    //mem is freed in ShowPPM():decimg.FreePictureData();
  }

  memcpy((void*)decimg->AskFrame_const()[0],
	 ppmFrame,
	 width * height * bytesPerPixel);

  return;
}




void VideoSink_X11::TransformPixmap(const Bitmap<Pixel>& dimg, 
			      uint8* mem, int firstline, int lastline,
			      int width, RawImageSpec_RGB& spec)
{
  if (DEBUG_WHERE) printf(" [VideoSink_X11::Transform():\n");
  if (DEBUG_WHERE) printf("      firstline=%d, lastline=%d, width=%d, depth=%d, little_endian=%c\n",firstline,lastline,width,(spec.bits_per_pixel+7)/8,spec.little_endian==true?'y':'n');

  /*
  char PPM_MAGIC1='P';
  char RPPM_MAGIC2='6';
  char header[20];
  char pixelline[3];
  */

  int row,col;
  unsigned char val1,val2,val3;

  int bpp = (spec.bits_per_pixel+7)/8;

  const Pixel*const* pPP = dimg.AskFrame_const();
  //Pixel*const* pPP = dimg.AskFrame_const();

  if (16==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_RGB16_LE
    //bbbbbggg-gggrrrrr

    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {

        val1 = (unsigned char)*pP++; 
        val2 = (unsigned char)*pP++;
        val3 = (unsigned char)*pP++;

	uint16 val;
	val = (val1 << 8) & spec.r_mask;
	val |= (val2 << 3) & spec.g_mask;
	val |= val3 >> 3;

	mem[0] = (val & 0xff);
	mem[1] = (val & 0xff00) >>8 ;

        mem += bpp;
      }
    }
  }
  else if (32==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_BGR32
    //bgr-
    rpos = 2; gpos = 1; bpos = 0;

    for ( row = firstline; row <= lastline; ++row) {
      //const Pixel* pP = pPP[row];
      Pixel* pP = (Pixel*)pPP[row];
      
      for ( col = 0; col < width; ++col ) {
        val1 = (unsigned char)*pP++; 
        val2 = (unsigned char)*pP++;
        val3 = (unsigned char)*pP++;

	/* RBG */
	mem[rpos]=val1;
	mem[gpos]=val2;
	mem[bpos]=val3;

	//For debug, write to output file
	/*
	if (row==0) {//write header
           sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,  
            640, 480, 255);  
	   write(outfile, header, sizeof(header));
	}
 
	sprintf(pixelline,"%c%c%c",val1,val3,val2);
	write(outfile,pixelline,3);
	bytes_received += 3;
	*/
        mem += bpp;
      }
    }
  }
  else {
    //TO DO
    //Other X fmt
    if (DEBUG_WHERE) 
      printf(" [VideoSink_X11::TransformPixmap(): Video Screen NOT supported yet !\n");
  }
    /*
    if (921600 <= bytes_received) 
      close (outfile);
    */
}

void VideoSink_X11::TransformGreymap(const Bitmap<Pixel>& dimg, 
			      uint8* mem, int firstline, int lastline,
			      int width, RawImageSpec_RGB& spec)
{
  if (DEBUG_WHERE) printf(" [VideoSink_X11::TransformGreymap():\n");
  if (DEBUG_WHERE) printf("      firstline=%d, lastline=%d, width=%d, depth=%d, little_endian=%c\n",firstline,lastline,width,(spec.bits_per_pixel+7)/8,spec.little_endian==true?'y':'n');

  int row,col;
  unsigned char val1,val2,val3;

  int bpp = (spec.bits_per_pixel+7)/8;

  const Pixel*const* pPP = dimg.AskFrame_const();

  if (16==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_RGB16_LE
    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      //Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {
	int tmp = (unsigned char)(*pP);
        val1 = val2 = val3 = tmp;

	uint16 val;
	val = (val1 << 8) & spec.r_mask;
	val |= (val2 << 3) & spec.g_mask;
	val |= val3 >> 3;

	mem[0] = (val & 0xff);
	mem[1] = (val & 0xff00) >>8 ;

	pP++;
        mem += bpp;
      }
    }
  }
  else if (32==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_BGR32
    rpos = 2; gpos = 1; bpos = 0;

    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {
	/*Take the value and divided by 3*/
	int tmp = (unsigned char)(*pP);
        val1 = val2 = val3 = tmp; 
        //val2 = (unsigned char)*pP++;
        //val3 = (unsigned char)*pP++;

	mem[rpos]=val2;
	mem[gpos]=val3;
	mem[bpos]=val1;

	pP++;
        mem += bpp;
      }
    }
  }
  else {
  }
}
