/*
 * ppmdecoder.cpp
 * 
 * Log: 5/15/03: in decode_ppm(), 'assert((startcode & 0x000000FF)!=0);' changes to
 *      'if ((startcode & 0x000000FF) ==0 ) {next_packet=NULL; continue;}'.
 *      This will avoid abruptily terminates the executable, but it may also not 
 *      update the video for a long time since no ppm data is pushed further because
 *      the header is not the expectable ppm header. The GZIP and ENCRYPT algorithm
 *      works both fine if seperately but fails when used together. When it fails
 *      the process exits. That's why I made this change.
 *
 */

#include "PPMP.h"
#include "ppmdecoder.h"

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/timeb.h>
#include <stdarg.h>
#include <time.h>
#include <assert.h>
}


#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHAT 0
#define DEBUG_WHERE 0


#define NEW_DISPLAYER //which replaces DVDview


PPMDecoder::PPMDecoder(int mus, int maxpixelsize, int maxlinesize)
  : PPMP_Receiver(mus)
{
  if (DEBUG_WHERE)
    printf(" [PPMDecoder::Cstr()\n");
  /*  
  if (!(next_packet = (char*)malloc(mus+maxpixelsize*maxlinesize))) {
    cerr<<" [PPMDecoder::PPMDecoder(): mem alloc error"<<endl;
    exit(-1);
  }
  */
  next_packet=NULL;
  MUS = mus;
  MAXPIXELSIZE = maxpixelsize;
  MAXLINESIZE = maxlinesize;
  //receiver = new PPMP_Receiver(mus);
}


PPMDecoder::~PPMDecoder()
{
  if (DEBUG_WHERE)
    printf(" [PPMDecoder::Dstr()\n");
}

void PPMDecoder::PushbackPacket_PPM(const char* pck)
{
  assert(next_packet!=NULL); 
  next_packet=pck;
}

/*
char* PPMDecoder::get_PPM_frame() {
  return receiver.get_img();
}
*/


void PPMDecoder::setPPMFileInfo(const char* msg)
{
  //skip the first 4 bytes
  const char* buf = msg + PPMP_FRAME_NO_OFFSET;
  //retrieve the new frame #
  memcpy(&curr_PPM_frame_num,buf,sizeof(unsigned long));
  //Now the struct PPM_frame_info
  buf += sizeof(unsigned long);
  memcpy(&pfi,buf,sizeof(PPM_frame_info));

  if (DEBUG_WHERE) printf(" [PPMDecoder::setPPMFileInfo(): curr_PPM_frame_num=%d, pfi: tag=%d,ppm1=%c,ppm2=%c,size=%d,width=%d,height=%d,*buff=%x\n",curr_PPM_frame_num,pfi.tag,pfi.ppm1,pfi.ppm2,pfi.size,pfi.width,pfi.height,pfi.buff);
}




void PPMDecoder::decode_ppm(const char* msg, size_t len)
{
  //make it non-null
  assert(msg!=NULL);
  next_packet=msg;

  while (1) {
    /**/
    if (NULL==next_packet) {
      if (DEBUG_WHERE) 
	printf(" [PPMDecoder::decode_ppm(): NULL == next_packet\n");
      break;
    }
    /**/
    unsigned long startcode, PPMFrameNum;
    memcpy (&startcode, msg, sizeof(unsigned long));
    startcode = ToBigEndian32(startcode);
    memcpy (&PPMFrameNum, msg+sizeof(unsigned long), sizeof(unsigned long));
    PPMFrameNum = ToBigEndian32(PPMFrameNum);

    if (DEBUG_WHAT) {
      printf(" \n\n\n[PPMDecoder::decode_ppm(): next_packet != NULL\n");
      printf(" [PPMDecoder::decode_ppm(): msg[]: %x:%x:%x:%x\n",msg[0],msg[1],msg[2],msg[3]);
      printf(" [PPMDecoder::decode_ppm(): startcode=%d, PPMFrameNum=%d\n",startcode, PPMFrameNum);
    }

    //assert((startcode & 0x000000FF)!=0);
    if ((startcode & 0x000000FF) ==0 ) {next_packet=NULL; continue;}
    
    if (DEBUG_WHERE) printf(" \n");//A packet is received. for debugging
    if (DEBUG_WHERE) printf(" PPMDecoder::decode_ppm(): startcode=%x,PPMFrameNum=%d,length=%d\n",startcode,PPMFrameNum,len);

    if (startcode != STARTCODE_PPM_HEADER) {
      if (startcode == STARTCODE_PPM_PICTURE ) {
        if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): STARTCODE_PPM_PICTURE\n");

        //Post header processing
        //PostPictureHeader_PPM(msg);

        //tmp = 1: img Not showed; tmp = 2: incomplete img showed
        //tmp = 3: complete img showed  
        int tmp = /*receiver->*/process_ppm(msg, len, PPMFrameNum, pfi);

        if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): after calling receiver->decode_ppm(), tmp = %d\n",tmp);

        //But if this is when a new PPM frame
        //kicks in when we're accumulating the current one.
        //Must be put before the next if(tmp) block, otherwise the
        //packet will be lost after the float returns to main()
        if (curr_PPM_frame_num == PPMFrameNum) {
          if (tmp == 2) {//Curr accumulated img must have been flushed
            PushbackPacket_PPM(msg);
            if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): curr_PPM_frame_num == PPMFrameNum -> Fallback \n");
          }
          else{
	    //everything is in order.
            //do nothing
	    if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): curr_PPM_frame_num == PPMFrameNum -> do nothing\n");
	    next_packet=NULL;
	    //break;
          } 
        }
        else {
          if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): curr_PPM_frame_num != PPMFrameNum, curr_PPM_frame_num=%x, PPMFrameNum=%x\n",curr_PPM_frame_num,PPMFrameNum);
            //Frame Number conflicts. Fallback
            PushbackPacket_PPM(msg);
            curr_PPM_frame_num = PPMFrameNum;
        }

        if (tmp==3) {
	  //return true;//Return to Main. True iff has more picture.
	  next_packet=NULL;
	  //break;
	  //return true;
        }
      }
      else {
        if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): ! STARTCODE_PPM_PICTURE\n");
        //sth. bad happens, omit the current buffer
        //break;//BREAK the LOOP ! Return to Main
	next_packet=NULL;
      }
    }
    else {
      if (DEBUG_WHERE) printf(" [PPMDecoder::decode_ppm(): STARTCODE_PPM_HEADER\n");
      //THis is a brand new PPM Frame. 
      next_packet = NULL;
      setPPMFileInfo(msg);
      //break;
      //Loop again for a new PPM frame
    }
  }

}




int PPMDecoder::process_whole_frame(const char *frame, size_t len)
{//do whatever you want on the frame
  char filename[60];
  char header[15];
  int fd;

#ifdef NEW_DISPLAYER
  displayer.showPPM (img_buffer, pfi.width, pfi.height, 
		     pfi.size/pfi.width/pfi.height,
		     curr_PPM_frame_num,
		     pfi.ppm1, pfi.ppm2);
#else

  sprintf(filename,"ImgReceived/received-%d.ppm",fileidx%40);
 
  if ((fd = open(filename,O_CREAT|O_WRONLY,S_IRWXU|S_IRGRP|S_IROTH))<0)
    {
      if (DEBUG_WHERE) printf(" file open error:>%s<\n",filename);
      exit(-1);
    }

  sprintf(header, "%c%c\n%d %d\n%d\n", pfi.ppm1, pfi.ppm2,  
	  pfi.width, pfi.height, 255);  
  write(fd, header, sizeof(header));

  write(fd,img_buffer,pfi.size);
  close(fd);
  
  fileidx ++;

#endif

  return pfi.size;
}

  
  
/*
 * Added spcifically for jpeg. If the img is a decompressed jpg img, then it
 * won't go thru the defragmenting pipeline. The whole frame is passed in 
 * completely at one time.
 * Note: setPPMFileInfo() must be called before this method.
 */
void PPMDecoder::decode_jpg (const char* msg, size_t len)
{
  //make it non-null
  assert(msg!=NULL);
  next_packet=msg;

  while (1) {
    /**/
    if (NULL==next_packet) {
      if (DEBUG_WHERE) 
	printf(" [PPMDecoder::decode_jpg(): NULL == next_packet\n");
      break;
    }
    /**/
    unsigned long startcode, PPMFrameNum;
    memcpy (&startcode, msg, sizeof(unsigned long));
    startcode = ToBigEndian32(startcode);
    memcpy (&PPMFrameNum, msg+sizeof(unsigned long), sizeof(unsigned long));
    //PPMFrameNum = ToBigEndian32(PPMFrameNum);

    if (DEBUG_WHAT) {
      printf(" \n\n\n[PPMDecoder::decode_jpg(): next_packet != NULL\n");
      printf(" [PPMDecoder::decode_jpg(): msg[]: %x:%x:%x:%x\n",msg[0],msg[1],msg[2],msg[3]);
      printf(" [PPMDecoder::decode_jpg(): startcode=%d, PPMFrameNum=%d\n",startcode, PPMFrameNum);
    }

    //assert((startcode & 0x000000FF)!=0);
    if ((startcode & 0x000000FF) ==0 ) {next_packet=NULL; continue;}

    if (DEBUG_WHERE) printf(" \n");//A packet is received. for debugging
    if (DEBUG_WHERE) printf(" PPMDecoder::decode_jpg(): startcode=%x,PPMFrameNum=%d,length=%d\n",startcode,PPMFrameNum,len);

    if (startcode != STARTCODE_PPM_HEADER) {
      if (startcode == STARTCODE_PPM_PICTURE ) {
        if (DEBUG_WHERE) printf(" [PPMDecoder::decode_jpg(): STARTCODE_PPM_PICTURE\n");

	//process_whole_frame (msg + PPMP_FRAME_INFO_OFFSET, len - PPMP_FRAME_INFO_OFFSET);
	displayer.showPPM ((char*)(msg + PPMP_FRAME_INFO_OFFSET), pfi.width, pfi.height, 
			   pfi.size/pfi.width/pfi.height,
			   curr_PPM_frame_num,
			   pfi.ppm1, pfi.ppm2);

	next_packet=NULL;
      }
      else {
        if (DEBUG_WHERE) printf(" [PPMDecoder::decode_jpg(): ! STARTCODE_PPM_PICTURE\n");
        //sth. bad happens, omit the current buffer
        //break;//BREAK the LOOP ! Return to Main
	next_packet=NULL;
      }
    }
    else {
      if (DEBUG_WHERE) printf(" [PPMDecoder::decode_jpg(): STARTCODE_PPM_HEADER\n");
      //THis is a brand new PPM Frame. 
      next_packet = NULL;
      setPPMFileInfo(msg);
      //break;
      //Loop again for a new PPM frame
    }
  }
}


unsigned long PPMDecoder::getCurrPPMFrameNum()
{
  return curr_PPM_frame_num;
}
