// author - Galen S Swint

#include <stdio.h>
#include <string.h>
#include <Python.h>

#include "csiplex.h"

static int sip2lex_initialized = 0;
static int sip2lex_dataReady = 0;
static PyObject *pSipLexDict;
static PyObject *pParseResult;

static int initialize( void );


static int initialize( void )
{
  // init python
  Py_Initialize();

  // create our Python dictionary (environment)
  pSipLexDict = PyDict_New();
  // load all built in functions
  PyDict_SetItemString(pSipLexDict, "__builtins__", PyEval_GetBuiltins());

  // import our lexer/parser
  printf("initialize(): importing siplex.\n");
  // import sys module so we can
  PyRun_String("import sys", Py_file_input, pSipLexDict, pSipLexDict);
  // set the path to include current directory
  PyRun_String("sys.path = sys.path + ['.']", Py_file_input, pSipLexDict, pSipLexDict);
  // and then import the siplex module for parsing
  PyRun_String("import siplex", Py_file_input, pSipLexDict, pSipLexDict);
  PyRun_String("print sys.path", Py_file_input, pSipLexDict, pSipLexDict);
  //printf("initialize(): printing.\n");
  PyRun_String("print dir()", Py_file_input, pSipLexDict, pSipLexDict);
  PyRun_String("print 'Hello! The parser module is now loaded.'", Py_file_input, pSipLexDict, pSipLexDict );
}

int test( void )
{
  FILE *inputFile;
  char *fileName = "multiuav.sip"; //"siptest.sip";
  char *inputString;
  size_t inputGrowSize = 1024;
  char *buf;
  int nread,i = 1;

  // open file for reading
  inputFile = fopen(fileName, "r");
  if(!inputFile) {
    fprintf(stderr, "Error opening %s", fileName);
    exit(1);
  }

  // alloc initial buffer for data string
  inputString = (char *)malloc( inputGrowSize );
  if( !inputString )
  {
    fprintf(stderr, "Error mallocing mem.");
    exit(1);
  }

  // read in the file into a string
  while( !feof( inputFile ) )
  {
    // read
    nread = fread( inputString, 1, inputGrowSize, inputFile );
    // can read more?
    if( nread == inputGrowSize )
    {
      i++;
      // grow size of buffer
      inputString = (char *) realloc( inputString, i * inputGrowSize );  
      if( !inputString )
      {
        fprintf(stderr, "Can't realloc() enough mem.");
        exit(1);
      }
    }
  }

  // null term the string
  inputString[ (i-1) * inputGrowSize + nread ] = '\0';
  printf("test():<|||-begin input dump-|||>\n");
  printf("%s<|||-end input dump-|||>:test()\n",inputString);

  // parse the file
  parseString( inputString );

  buf = getBlockSip();
  printf("%s", buf );
  free(buf);
  buf = getBlockXip();
  printf("%s", buf );
  free(buf);

}


int parseString( char *stringToParse )
{
  PyObject *pyInputString;
//  char *cmdString;
//  size_t len;
  // be sure we're initialized
  if( !sip2lex_initialized )
  {
    sip2lex_initialized = 1;
    initialize();
  }

  // convert C to Python input string
  printf("parseString(): Building python string.\n");
  pyInputString = PyString_FromString(stringToParse);
  if( pyInputString == NULL ) fprintf(stderr, "Unable to build string.");
  // add it as a value in our dict
  PyDict_SetItemString( pSipLexDict, "lexin", pyInputString );
  printf("parseString(): Set python string in dict.\n");
  PyRun_String("print dir()\n", Py_file_input, pSipLexDict, pSipLexDict);
  PyRun_String("print lexin\n", Py_file_input, pSipLexDict, pSipLexDict);
  // call the parser with the string
  PyRun_String("print dir()\n", Py_file_input, pSipLexDict, pSipLexDict);
  pParseResult = PyRun_String("result = siplex.sipparser.parse(lexin)\n", Py_file_input, pSipLexDict, pSipLexDict);
  //PyRun_String("print dir()\n", Py_file_input, pSipLexDict, pSipLexDict);
  if( !pParseResult )
    fprintf(stderr, "parseString(): Error running parser. No result.\n");
  PyRun_String("blockSip = result.BlockSip()\n", Py_file_input, pSipLexDict, pSipLexDict);
  PyRun_String("blockXip = result.Xml()\n", Py_file_input, pSipLexDict, pSipLexDict);
  
  sip2lex_dataReady = 1;
  Py_XDECREF(pyInputString);
}

// returns a string - caller must call free on returned pointer 
char *getBlockSip()
{
  size_t sip_size;
  char *buffer; 
  PyObject *pyObjectString;
  // be sure we're initialized
  if( !sip2lex_dataReady )
  {
    return NULL;
  }

  pyObjectString = PyDict_GetItemString( pSipLexDict, "blockSip");
  if( !pyObjectString ) 
  {
    fprintf(stderr, "Error retrieving block sip string.\n");
    return NULL;
  }

  sip_size  = 10 +  PyString_Size( pyObjectString );
  buffer = (char *)malloc( sip_size );
  strncpy( buffer, PyString_AsString(pyObjectString), sip_size + 1);
  
  return buffer;
}

// returns a string - caller must call free on returned pointer 
char *getBlockXip()
{
  size_t sip_size;
  char *buffer; 
  PyObject *pyObjectString;
  // be sure we're initialized
  if( !sip2lex_dataReady )
  {
    return NULL;
  }

  pyObjectString = PyDict_GetItemString( pSipLexDict, "blockXip");
  if( !pyObjectString ) 
  {
    fprintf(stderr, "Error retrieving block XIP string.\n");
    return NULL;
  }

  sip_size  = 10 +  PyString_Size( pyObjectString );
  buffer = (char *)malloc( sip_size );
  strncpy( buffer, PyString_AsString(pyObjectString), sip_size + 1);
  
  return buffer;
}
