# ---
# siplex.py - a lex file for SIP-II
#

#get file ready
lexFile = open('siptest.sip')
lexin = lexFile.read()

import lex
import yacc
import xipDoc

#tokens we accept
tokens = ( #'NL', # a newline
#           'END', #semi-colon an end of a statement
           'TYPE', #type declaration
           'ID', #identifier for a type or a pipe
           'LPAREN', 'RPAREN',
           'COLON',
           'COMMA', 
           'FLOAT','INTEGER','BYTE','CHAR','CHARSTAR',
           'PIPED', #pipe declaration 
           'IS', 'INPUTT', 'OUTPUTT',
           'COMPOSE') 

t_LPAREN = r'\('
t_RPAREN = r'\)'
t_COLON  = r':'
t_COMMA = r','
#t_END = ';'

reserved = { 'type' : 'TYPE', 
             'float' : 'FLOAT',
             'integer' : 'INTEGER',
             'char' : 'CHAR',
             'charbuf' : 'CHARSTAR',
             'byte' : 'BYTE',
             'pipe' : 'PIPED',
             'is' : 'IS',
             'input' : 'INPUTT',
             'output' : 'OUTPUTT',
             'compose' : 'COMPOSE' }
             

def t_ID(t):
  r'[a-zA-Z0-9_]+'
  t.type = reserved.get(t.value, 'ID')
  return t

#def t_NL(t):
#  r'\n+'
#  t.lineno += len(t.value)
#  print 'newline, line no.', t.lineno, 't.value:',t.value
#  return t

t_ignore = ' \t\n'

def t_error(t):
  print "Illegal Character '%s'" % t.value
  t.skip(1)


siplexer = lex.lex()

lex.input(lexin)

#while 1:
#  tok = lex.token()
#  if not tok: 
#    print 'not tok',tok
#    break
#  print tok

#print
#print 'Doing yacc now'
#print
# build our yacc rules

# GRAMMAR
# sip :  types filters pipes sip
# types : empty | type types
# type : 'type' ID '(' varlist ')'
# varlist : onevar ',' varlist | onevar
# onevar : ID ':' ptype
# ptype : 'float' | 'integer' | 'byte'
# ID : [a-zA-Z_][a-zA-Z0-9_]*

# pipes : empty | pipe pipes
# pipe : 'pipe' ID 'is' inputdecl outputdecl 
# inputdecl : empty | 'input' COLON ID
# outputdecl : empty | 'output' COLON ID

treeRoot = xipDoc.xipDoc(None,None,None,None) 

def p_sip(t):
  'sip : types pipes'
  #print 'sip'
  treeRoot.types = t[1] 
  treeRoot.pipes = t[2] 
  t[0] = treeRoot
  


def p_types_empty(t):
  'types : ' 
  #print 'p_types_empty'
  t[0] = xipDoc.typeList(treeRoot, None, None)
  
 
def p_types_multiple(t):
  'types :  type types'
  #print 'p_types_multiple'
  t[0] = xipDoc.typeList(treeRoot, t[1], t[2])

def p_type(t):
  'type : TYPE ID LPAREN varlist RPAREN'
  #print 'p_type'
  t[0] = xipDoc.typeNode( treeRoot, t[2], t[4])

def p_varlist_single(t):
  'varlist : onevar '
  #print 'p_varlist_single'
  t[0] = xipDoc.varlist(treeRoot, t[1], None)

def p_varlist_multiple(t):
  'varlist : onevar COMMA varlist'
  #print 'p_varlist_multiple'
  t[0] = xipDoc.varlist(treeRoot, t[1], t[3])

def p_onevar(t):
  'onevar : ID COLON ptype'
  #print 'p_onevar'
  t[0] = xipDoc.onevar( treeRoot, t[1], t[3])

def p_ptype_float(t):
  'ptype : FLOAT'
  t[0] = 'float'

def p_ptype_integer(t):
  'ptype : INTEGER'
  t[0] = 'integer'

def p_ptype_char(t):
  'ptype : CHAR'
  t[0] = 'char'
def p_ptype_charstar(t):
  'ptype : CHARSTAR'
  t[0] = 'char *'

def p_ptype_byte(t):
  'ptype : BYTE'
  t[0] = 'byte'

# pipes : empty | pipe pipes
# pipe : 'pipe' ID 'is' inputdecl outputdecl 
# inputdecl : empty | 'input' COLON ID
# outputdecl : empty | 'output' COLON ID

def p_empty_pipes(t):
  'pipes : '
  t[0] = xipDoc.pipeList(treeRoot, None, None)

def p_pipelist(t):
  'pipes : pipe pipes'
  #print 'Doing list of pipes',treeRoot
  t[0] = xipDoc.pipeList(treeRoot, t[1],t[2])

def p_pipe(t):
  'pipe : PIPED ID IS inputdecl outputdecl'
  t[0] = xipDoc.pipeNode(treeRoot, t[2],t[4],t[5])

def p_inputdecl_empty(t):
  'inputdecl : '
  t[0] = None

def p_inputdecl(t):
  'inputdecl : INPUTT COLON ID'
  t[0] = t[3]

def p_outputdecl_empty(t):
  'outputdecl : '
  t[0] = None

def p_outputdecl(t):
  'outputdecl : OUTPUTT COLON ID'
  t[0] = t[3]

#def p_empty(t):
#  'empty :'
#  pass

# pipe : 'pipe' ID 'is' composedecl
# composedecl : COMPOSE LPAREN pipenamelist RPAREN
# pipenamelist : ID | ID COMMA pipenamelist
def p_pipecompose(t):
  'pipe : PIPED ID IS composedecl'
  t[0] = xipDoc.pipeNodeSerialCompose(treeRoot, t[2],t[4])

def p_composedecl(t):
  'composedecl : COMPOSE LPAREN pipenamelist RPAREN'
  t[0] =  t[3]

def p_pipenamelist_multi(t):
  'pipenamelist : ID COMMA pipenamelist'
  t[0] = xipDoc.pipeNameList( treeRoot, t[1], t[3] )

def p_pipenamelist_single(t):
  'pipenamelist : ID'
  t[0] = xipDoc.pipeNameList( treeRoot, t[1], None )

def p_error(t):
   print 'Syntax error in input!'

sipparser = yacc.yacc()

#result = sipparser.parse(lexin)
#print result
#print 'Printing block SIP result'
#result.BlockSip()
#print 'Printing XIP'
#result.Xml()
