# a class to build a tree to hold the parsed
# data from the SIP file so it can be converted
# into XIP, or whatever else

#root of a tree - that describes a SIP/XIP document
#has 4 sections
#types - points to list of types
#filters - points to list of filters
#aspects - points to list of aspects

class docBase:
  def Find(self, whattype, name):
    print 'Find: Not implemented on the object'
  def BlockSip(self):
    return 'BlockSip: Not implemented on this object'
  def Sip(self):
    print 'Sip: Not implemented on this object'
  def Xml(self):
    print 'Xml: Not implemented on this object'

class xipDoc(docBase): 
  'Parent node for a XIP doc'
  blockNum = 0
  def __init__(self, types, filters, pipes, aspects):
    self.nodetype = 'xipDoc'
    self.types = types
    self.filters = filters
    self.pipes = pipes
    self.aspects = aspects
    self.root = self  # we are always the root of the tree
  def Find( self, whattype, name ):
    if whattype == 'pipe':
      return self.pipes.Find( whattype, name )
    else:
      return None
  def BlockSip(self):
    retString = '<blockSip>\n'
    if self.types: retString += self.types.BlockSip()
    if self.filters: retString += self.filters.BlockSip()
    if self.pipes: retString += self.pipes.BlockSip()
    retString += '</blockSip>\n'
    #print retString
    return retString
  def Sip(self):
    retString = ''
    if self.types: retString += self.types.Sip()
    if self.filters: retString += self.filters.Sip()
    if self.pipes: retString += self.pipes.Sip()
    if self.aspects: retString += self.aspects.Sip()
    return retString

  def Xml(self):
    # our header for an XML file
    retString = ''
    retString += '<?xml version="1.0" ?>\n'
    retString += '<!DOCTYPE InfopipeSpec SYSTEM "InfopipeSpec.dtd">\n'
    retString += '<InfopipeSpec name="sip2xip">\n'

    if self.types:
      retString += self.types.Xml()
    if self.pipes:
      retString += self.pipes.Xml()
    retString += '</InfopipeSpec>\n'
    return retString

#a (possible empty) list of type declarations
#moretypes points to rest of list
class typeList(docBase):
  def __init__(self, root, type, moretypes):
    self.nodetype = 'typeList'
    self.root = root
    self.type = type
    self.moretypes = moretypes
    #if type: self.sip  = type.sip 
    #else: self.sip = ''
  def BlockSip(self):
    #this is not a block, so no block number needed
    retString = ''
    if self.type:
      retString += self.type.BlockSip()
    if self.moretypes:
      retString += self.moretypes.BlockSip()
    return retString
  def Sip(self):
    retString = ''
    if self.type:
      retString += self.type.Sip()
    if self.moretypes:
      retString += self.moretypes.Sip()
    return retString
  def Xml(self):
    retString = ''
    if self.type:
      retString += self.type.Xml()
    if self.moretypes:
      retString += self.moretypes.Xml()
    return retString

#individual type declaration
#name - name the type referenced by
#varlist - list of the types basic elements
class typeNode(docBase):
  def __init__(self, root, name, varlist ):
    self.nodetype = 'typeNode'
    self.root = root
    self.name = name
    self.varlist = varlist
    xipDoc.blockNum = xipDoc.blockNum + 1
    self.blockNum = xipDoc.blockNum
    #self.sip = 'type' + self.name + self.varlist.sip + '</block>\n'
  def BlockSip(self):
    #this is a block
    retString = ''
    retString += '<block num="' + str(self.blockNum) + '">\n'
    retString += 'type ' + self.name
    retString += ' (' + self.varlist.BlockSip() + ')'
    retString += '\n</block>\n'
    return retString
  def Sip(self):
    retString = ''
    retString += 'type',self.name,
    retString += self.varlist.Sip()
    return retString
  def Xml(self):
    retString = ''
    retString += '<dataDef name="' + self.name + '" '
    retString += ' num="' + str(self.blockNum) + '">\n'
    retString += self.varlist.Xml()
    retString += '</dataDef>\n'
    return retString

#list of name, atomic type pairs
#var - single name/type pair
#morevars - ref to rest of ist of name/type pairs
class varlist(docBase):
  def __init__(self, root, var, morevars):
    self.nodetype = 'varlist'
    self.var = var 
    self.morevars = morevars 
    #self.sip = var.sip 
    #if morevars: self.sip = self.sip + morevars.sip
  def BlockSip(self):
    retString = ''
    retString += self.var.BlockSip()
    if self.morevars:
      retString += ', '
      retString += self.morevars.BlockSip()
    return retString
  def Sip(self):
    retString = ''
    retString =+ self.var.Sip()
    if self.morevars:
      retString += self.morevars.Sip()
    return retString
  def Xml(self):
    retString = ''
    retString += self.var.Xml()
    if self.morevars:
      retString += self.morevars.Xml()
    return retString

#one name, atomic type pair
#name - name of athe var
#ptype - primitive type (e.g. 'float', 'integer', 'char')
class onevar(docBase):
  def __init__(self, root, name, ptype):
    self.nodetype = 'onevar'
    self.root = root
    self.name = name
    self.ptype = ptype
    self.sip = name + ':' + ptype
  def BlockSip(self):
    return self.name + ':' + self.ptype 
  def Sip(self):
    retString = ''
    retString += self.name+':'+self.ptype
    return retString
  def Xml(self):
    retString = ''
    retString += '  <arg name="'+self.name+'" type="'+self.ptype+'"/>\n'
    return retString


#list of pipes
#morepipes points to rest of list
class pipeList(docBase):
  def __init__(self, root, pipe, morepipes):
    self.nodetype = 'pipeList'
    self.root = root
    self.pipe = pipe 
    self.morepipes = morepipes
    #if pipe: self.sip = pipe.sip 
    #else: self.sip = ''
    #if morepipes: self.sip = self.sip + morepipes.sip
  def Find(self, whattype, name):
    ret = self.pipe.Find(whattype, name)
    if ret:
      return ret
    else:
      return self.morepipes.Find(whattype,name)
  def BlockSip(self):
    retString = ''
    if self.pipe:
      retString += self.pipe.BlockSip()
    if self.morepipes:
      retString += self.morepipes.BlockSip()
    return retString
  def Sip(self):
    retString = ''
    if self.pipe:
      retString += self.pipe.sip()
    if self.morepipes:
      retString += self.morepipes.sip()
    return retString
  def Xml(self):
    retString = ''
    if self.pipe:
      retString += self.pipe.Xml()
    if self.morepipes:
      retString += self.morepipes.Xml()
    return retString

#individual pipe declaration
#name - name the type referenced by
#varlist - list of the types basic elements
class pipeNode(docBase):
  def __init__(self, root, name, input, output):
    self.nodetype = 'pipeNode'
    self.root = root
    self.name = name
    if input: self.input = input 
    else: self.input = 'CAPPED'
    if output: self.output = output
    else: self.output = 'CAPPED'
    #get our blocking set
    xipDoc.blockNum = xipDoc.blockNum + 1
    self.blockNum = xipDoc.blockNum
    # self.sip = 'pipe ' + name + ' is '
    # if input: self.sip = self.sip + 'input : ' +  input + '  '
    # if output: self.sip = self.sip + 'output : ' +  output
    # self.sip = self.sip + '</block>\n'
  def Find(self,whattype, name):
    if name == self.name:
      return self
    else:
      return None
  def BlockSip(self):
    retString = ''
    retString += '<block num="' + str(self.blockNum) + '">\n'
    retString += 'pipe '+self.name+' is'
    retString += ' input:' + self.input
    retString += ' output:'+ self.output
    retString += '\n</block>\n'
    return retString
  def Sip(self):
    retString = ''
    retString += 'pipe '+self.name+' is input :'
    if self.input:
      retString += 'input : '+self.input
    else:
      retString += 'CAPPED'
    if self.output:
      retString += 'output : '+self.output
    else:
      retString += 'CAPPED'
    retString += '\n'
    return retString
  def Xml(self):
    retString = ''
    retString += '<pipe name="' + self.name + '" ' 
    if self.input:
      retString += 'inType="' + self.input + '" '
    else:
      retString += 'inType="CAPPED" '
    if self.output:
      retString += 'outType="' + self.output + '" '
    else:
      retString += 'outType="CAPPED" '
    retString += 'num="' + str(self.blockNum) + '">\n'
    retString += '  <properties/>\n'
    retString += '  <connections/>\n'
    retString += '</pipe>\n'
    return retString

class pipeNodeSerialCompose(docBase):
  def __init__(self, root, name, pipeNameList):
    self.nodetype = 'pipeNodeSerialCompose'
    self.root = root
    self.name = name
    self.pipeNameList = pipeNameList
    xipDoc.blockNum = xipDoc.blockNum + 1
    self.blockNum = xipDoc.blockNum
  def Find(self,whattype, name):
    if name == self.name:
      return self
    else:
      return None
  def BlockSip(self):
    retString = ''
    retString += '<block num="' + str(self.blockNum) + '">\n'
    retString += 'pipe ' + self.name + ' is compose ('
    pipeName = self.pipeNameList
    while pipeName:
      retString += pipeName.thisName
      pipeName = pipeName.moreNames
      if pipeName:
        retString += ', '
    retString += ')'
    retString += '\n</block>\n'
    return retString
  def Sip(self):
    retString = ''
    retString += 'pipe',self.name,'is compose ('
    pipeName = self.pipeNameList
    while pipeName:
      retString += pipeName.thisName
      pipeName = pipeName.moreNames
      if pipeName:
        retString += ', '
    retString += ')\n'
    return retString
  def Xml(self):
    retString = ''
    curnamelist = self.pipeNameList
    curpipe = self.root.Find('pipe',curnamelist.thisName)
    
    retString += '<pipe name="' + curpipe.name + '" ' 
    retString += 'inType="' + curpipe.input + '" '
    retString += 'outType="' + curpipe.output + '" '
    retString += 'num="' + str(curpipe.blockNum) + '">\n'
    # don't worry about properties at this point
    retString += '  <properties/>\n'
    # connect to next pipe
    retString += '  <connections/>\n'
    retString += '</pipe>\n'
    lastname = curnamelist.thisName
    curnamelist = self.pipeNameList.moreNames
    while curnamelist:
      curpipe = self.root.Find('pipe',curnamelist.thisName)
      retString += '<pipe name="' + curpipe.name + '" ' 
      retString += 'inType="' + 'CAPPED' + '" '
      retString += 'outType="' + curpipe.output + '" '
      retString += 'num="' + str(curpipe.blockNum) + '">\n'
      retString += '  <connections>\n'
      retString += '    <join to="' + lastname + '"/>\n'
      retString += '  </connections>\n'
      retString += '</pipe>\n'
      lastname = curnamelist.thisName
      curnamelist = curnamelist.moreNames
    return retString

class pipeNameList(docBase):
  def __init__(self, root, oneName, moreNames):
    self.nodetype = 'pipeNameList'
    self.root = root
    self.thisName = oneName
    self.moreNames = moreNames
  def BlockSip(self):
    return self.thisName
  def Sip(self):
    return self.thisName
  def Xml(self):
    return self.thisName
