// isg.cpp -- infopipe stub generator
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fstream.h>

#include "isg.hpp"
#include "out.hpp"
#include "aspect.hpp"
#include "outAspect.hpp"
#include "domErrorHandler.hpp"
#include "isgUtil.hpp"

static int textprint( DOM_Element r );

static int doProperties( DOM_Element &nodeDoc, DOM_Element thePipe );
static int parseOptions( int argc, char **argv, char **specName, char **templateName, char **dumpFileName );
static int initializeXML( void );
static DOM_Element addNodeElement(DOM_Element currPipe, DOMString nodeName, DOMString pipeName, DOMString inType, DOMString outType);
static DOMString   getNextInType( DOM_Element pipe, DOM_Element nextSection  );
static DOMString   getPipeOutputType( DOMString pipeName );
int removeWhiteSpace(DOM_Node node); 
static int addNodesetMeta( DOM_Document ns );
static int addPipeMeta( DOM_Element p );
static int addFilter( DOM_Element filterDef );
static int addDataDef( DOM_Element dataDef );
static int buildPipe( DOM_Element pipeSpec );
static int processSpec( DOM_Document specDoc );

DOM_Document getDomDoc( const char *, const bool );

struct globalVars
{
  // the current specification we are processing 
  DOM_Document specDocument;

  // points to the xml template for building the stubs
  DOM_Document templateDocument;
  
  // nodeDocument holds the lists of nodes and data types for a pipe
  DOM_Document nodeSetDocument;

  int doxslt;
} isgGlobal;


static int getTemplate( char *templateName )
{
  char *filePath;
  char *templateDir;
  struct stat templateStatus;

  // check current directory first
  filePath = templateName;
  if( stat( filePath, &templateStatus ) )
    {
    // get directory of templates
    if(templateDir = getenv("ISG_TEMPLATE_PATH"))
      {
      filePath = new char[ strlen(templateDir) + strlen(templateName) + 10 ];
      // copy in
      strcpy( filePath, templateDir );
      // check to see if ends in '/' (for concat with file name)
      if( filePath[strlen(filePath)-1] != '/' )
        strcat(filePath, "/");
      strcat( filePath, templateName );
      }

    // now try again
    if( stat( filePath, &templateStatus ) )
      {
      cerr << "Can't find file " << templateName << " does not exist." << endl;
      return 1;
      }
    }

  // well it exists, is it a file?
  if( !S_ISREG( templateStatus.st_mode ) )
    {
    cerr << filePath << " is not a regular file." << endl;
    return 1;
    }

	cout << "Loading template file '" << filePath << "'" << endl;

  // set up DOM for the global template
  //cout << "template:" << endl;
  isgGlobal.templateDocument = getDomDoc( filePath, false );
  if( !isgGlobal.templateDocument.getDocumentElement().getNodeName().equals("isgTemplate") )
    {
    cerr << "File is not an isg template." << endl;
    isgGlobal.doxslt = 1;
//    return 1;
    }

  return 0;
}

static int buildNodeset( DOM_Document specDocument, char *dumpFileName )
{
  // initialize our document to hold the set of nodes
  DOM_DOMImplementation impl;
  DOM_Element allnodes;

  // create the document - I'm not exactly sure why third arg must be there -- it
  // is like this in example, but seems to create something == 0
  isgGlobal.nodeSetDocument = impl.createDocument(NULL, "nodeSet", DOM_DocumentType());
  // we add the all nodes element for attaching data types
  allnodes = isgGlobal.nodeSetDocument.createElement("allNodes");
  isgGlobal.nodeSetDocument.getDocumentElement().appendChild(allnodes);

  // pull the DOM and create the sample files
  isgGlobal.specDocument = specDocument;
  processSpec( isgGlobal.specDocument );

  // for just good looking of nodeSetDocument
  removeWhiteSpace(isgGlobal.nodeSetDocument.getDocumentElement());
  
  // check to see if user requested dump of nodeset document
  if (gFlags & FLAG_DUMP_NODESET)
  {
    // pretty print the result
    if (dumpFileName)
    {
      ofstream ofile(dumpFileName);
      ofile << DOM_ELEMENT(isgGlobal.nodeSetDocument.getDocumentElement());
    }
    else
    {
      cout << "Node descriptions: " << endl;
      cout << DOM_ELEMENT(isgGlobal.nodeSetDocument.getDocumentElement());
    }
  }
}

int oldIsgProcessing(DOM_Document templateDocument, DOM_Document specDocument, char *dumpFileName) 
{
  DOM_Node tnode;
  DOM_DOMImplementation impl;

  isgGlobal.templateDocument = templateDocument;
  
  /********************/
  /* building nodeSet */
  /********************/

  buildNodeset( specDocument, dumpFileName );

  // adds some meta information to the nodes in the nodeSet document
  addNodesetMeta( templateDocument ); 
    
  generateAspectCode(templateDocument.getDocumentElement(), isgGlobal.nodeSetDocument.getDocumentElement());

  // BACK END ISG
  generateCode( templateDocument, isgGlobal.nodeSetDocument );

  if (gFlags & FLAG_DUMP_TEMPLATE)
    cout << templateDocument;

  return 0;




  /////////////////////////////////////
  /* we don't use below code : young */
  /////////////////////////////////////
 


  // we need to split apart the pipes into different node sets
  // which means that we also have to duplicate the data defs provided
  DOM_Document *templateArray, *nodesetArray;
  DOMString watchname;
  int i,j;
  DOM_NodeList nlPipes = isgGlobal.nodeSetDocument.getDocumentElement().getElementsByTagName("pipe");

  // create new arrays to operate on
  templateArray = new DOM_Document[ nlPipes.getLength() ];
  nodesetArray = new DOM_Document[ nlPipes.getLength() ];

  // make copies of the template and nodeset docs
  for( i = 0; i < nlPipes.getLength(); i++ )
  {
    DOM_Node t;
    DOM_Node t2;
		// make the same kind of doc as what we were given (preserve
		// any XSLT-edness we have)
//  DOMString docName = isgGlobal.templateDocument.getDocumentElement().getNodeName();
// cout << thisFunc << "Creating template '" << docName << "' copy " << i << endl;
//    templateArray[i] = impl.createDocument(NULL, docName, DOM_DocumentType());
		templateArray[i] = DOM_Document::createDocument();
		t = templateArray[i].importNode(isgGlobal.templateDocument.getDocumentElement(), true);
		templateArray[i].appendChild( t );
//    t = isgGlobal.templateDocument.getDocumentElement().getFirstChild();
//		if( t == NULL ) cout << thisFunc << "First Child is NULL" << endl;
//    while( t != NULL )
//    {
//      t2 = templateArray[i].importNode(t, true); // deep copy
//      templateArray[i].getDocumentElement().appendChild(t2);
//      t = t.getNextSibling();
//    }
     // isgGlobal.templateDocument ;
//    nodesetArray[i] = impl.createDocument(NULL, "nodeSet", DOM_DocumentType());
		// copy our nodeset array
    nodesetArray[i] = DOM_Document::createDocument();
		t = nodesetArray[i].importNode(isgGlobal.nodeSetDocument.getDocumentElement(),true);
		nodesetArray[i].appendChild(t);
//    t = isgGlobal.nodeSetDocument.getDocumentElement().getFirstChild();
//    while( t != NULL )
//    {
//      t2 = nodesetArray[i].importNode(t, true); // deep copy
//      nodesetArray[i].getDocumentElement().appendChild(t2);
//      t = t.getNextSibling();
//    }
     // isgGlobal.nodeSetDocument;
  }
  // prune down the extraneous pipes in the nodeset documents
  DOM_NodeList thisnlPipes;
  for( i = 0; i < nlPipes.getLength(); i++ )
  {
    thisnlPipes = nodesetArray[i].getDocumentElement().getElementsByTagName("pipe");
    watchname = DOM_ELEMENT(thisnlPipes.item(i)).getAttribute("pipeName");
    for( j = thisnlPipes.getLength() - 1; j >= 0; j-- )
    {
      if( !DOM_ELEMENT(thisnlPipes.item(j)).getAttribute("pipeName").equals( watchname ) )
      {
        thisnlPipes.item(j).getParentNode().removeChild( thisnlPipes.item(j) );
      }
    }
  }
  // end splitting - now only one pipe per nodeset, and we have
  // one template to be sent throught with each pipe
 
  // now do the step-by-step ISG processing
//  ofstream xmlOut("xmlOut.xml");
  // temporary var to hold what comes out of the XSLT step
  DOM_Document newTemplate;
  for( i = 0; i < nlPipes.getLength(); i++ )
  {
    // FRONT END ISG

    // adds some meta information to the nodes in the nodeSet document
    addNodesetMeta( templateArray[i] );

    newTemplate = templateArray[i];
//    doXSLTstep( isgGlobal.nodeSetDocument, templateArray[i], newTemplate );

    // ASPECT ENGINE  
//    generateAspectCode( templateArray[i].getDocumentElement(), nodesetArray[i].getDocumentElement() );
    generateAspectCode( newTemplate.getDocumentElement(), nodesetArray[i].getDocumentElement() );

    // BACK END ISG
    generateCode( newTemplate, nodesetArray[i] );

    if (gFlags & FLAG_DUMP_TEMPLATE)
      cout << newTemplate;
  }

//  xmlOut.close();
  return 0;
}

static int addNodesetMeta( DOM_Document ns )
{
  int i;
  DOM_NodeList nl;
  // do metainfo for each pipe
  nl = ns.getElementsByTagName("pipe");
  for( i = 0; i < nl.getLength(); i++ )
  {
    addPipeMeta( DOM_ELEMENT( nl.item(i) ));
  }
  return 0;
}

static int addPipeMeta( DOM_Element p )
{
  DOM_NodeList nl;
  DOM_Attr a;
  int i;
  char buf[40];
  // for now just put counts on nodes
  nl = p.getElementsByTagName("node");

  // set the total number of nodes in the pipe
  a = p.getOwnerDocument().createAttribute("numNodes");
  snprintf(buf, 39, "%d", nl.getLength() );
  a.setValue( buf );
  p.setAttributeNode(a);


  // number each node sequentially
  for( i = 0; i < nl.getLength(); i++ )
  {
    a = p.getOwnerDocument().createAttribute("seqNumber");
    snprintf(buf, 39, "%d", i);
    a.setValue( buf );
    DOM_ELEMENT( nl.item(i) ).setAttributeNode(a);
  }

}

// getDomDoc() - takes file name of XML document and returns
//             - DOM_Document form of it.
DOM_Document getDomDoc( const char *filename, const bool validate )
{
  //DOMParser::ValSchemes  valScheme = DOMParser::Val_Auto;
  DOMParser dp;
  domErrorHandler isgErrorHandler;
  int ecount;


  // needed for Xalan to accept DOM tree
	//dp.setToCreateXMLDeclTypeNode(false);
  //dp.setDoNamespaces(true); // really needed?

  dp.setErrorHandler( &isgErrorHandler );
  dp.setValidationScheme( DOMParser::Val_Auto );
//  dp.setDoValidation( true );
//  cout << "getDomDoc(): parsing xml file: " << filename << "...";
  try 
    {
    dp.parse( filename );
    ecount = dp.getErrorCount();
    if( ecount )
      {
      cout << "***Errors in parsing." << endl;
      exit(1);
      }
    }
  catch(...)
    {
    cout << "parse generated an exception." << endl;

    }
  //pprint(dp.getDocument().getDocumentElement() );
//  cout << "complete." << endl;
  return ( dp.getDocument() );
}

// process spec must first pull all the type information
// then it must create the function prototypes and definitions
// then write this data out to directories/files
// to create the functions it must automatically change the
// pipe sections from links between nodes to nodes
// i.e. we give it edges, it generates vertexes in a graph
// which is where computation actually takes place
static int processSpec( DOM_Document specDoc )
{
  DOM_NodeList nodeList;
  DOM_Node current;
  DOM_Element e;
  DOMString basename;
  DOM_Attr a;
  char *name;
 

  // get our root element "<InfopipeSpec>"
  DOM_Node docNode = specDoc.getDocumentElement( );
  e = (DOM_Element &)docNode;

  // set the "basename" attribute which gives the base filenames for output
  basename = e.getAttribute("name");
  a = isgGlobal.nodeSetDocument.createAttribute("basename");
  a.setValue(basename);
  isgGlobal.nodeSetDocument.getDocumentElement().setAttributeNode( a );

  //cout << "Basename is: " << basename << endl;

  // get the first element child of the document
  // and then we can enter a loop to process them
  //cout << "Getting first child." << endl;
  current = docNode.getFirstChild();


  // loop through and get each infopipe description
  // each one will be "Incoming", "Outgoing", or "Through"
  // (will support "Through" later, maybe)
  while( current != NULL )
    {
    // check the type of the node
//    cout << "  Getting node name....";
    name = current.getNodeName().transcode();
//    cout << name << endl;

    if( !strcmp(name, "#text") )
      {
      // we just skip this (probably whitespace)
      // cout << "#text" << endl;
      }
    else if( !strcmp(name,"dataDef"))
      {
      // drop in a data description
//      cout << " isg:processSpec(): dataDef found." << endl;
      addDataDef((DOM_Element &)current);
      }
    else if(!strcmp(name,"pipe"))
      {
//      cout << "Going to buildPipe()." << endl;
      buildPipe((DOM_Element &)current);
      }
    else if(!strcmp(name,"filter"))
      {
        addFilter( (DOM_Element &)current );
      }
    else
      {
      // No match found
//      cout << "isg:processSpec():No match found for name: \"";
//      cout << name << "\"" << endl;
      }

    // free the name buffer
    //cout << "  freeing name buffer." << endl;
    delete name;

    // move to next in line
    //cout << "  Getting next node." << endl;
    current = current.getNextSibling();
    }

  return 0;
}


// adds a filter definition to the nodeDocument
// return 0 if successful
static int addFilter( DOM_Element filterDef )
{
  DOM_Node tempNode;
  DOM_Element newFilterDef, root, allNodes;

  // get ns root
  root = isgGlobal.nodeSetDocument.getDocumentElement();
  // now get allNodes element of root since these
  // defs will be avail to all nodes of pipe
  tempNode = root.getFirstChild();
  allNodes = (DOM_Element &)tempNode;
  if( !allNodes.getNodeName().equals("allNodes") )
  {
    // failure to find allNodes in the nodeset
    return 1;
  }
  // clone & import the given datadef node into the new document
  tempNode = isgGlobal.nodeSetDocument.importNode( filterDef, true );
  newFilterDef = (DOM_Element &) tempNode;

  // attach to node
  allNodes.appendChild( newFilterDef );
  return 0;
}

// adds a data definition to the nodeDocument
// return 0 if successful
static int addDataDef( DOM_Element dataDef )
{
  DOM_Node tempNode;
  DOM_Element newDataDef, root, allNodes;
  
  // get our root
  root = isgGlobal.nodeSetDocument.getDocumentElement();
  // now get allNodes element of root since these
  // defs will be avail to all nodes of pipe
  tempNode = root.getFirstChild();
  allNodes = (DOM_Element &)tempNode;
  if( !allNodes.getNodeName().equals("allNodes") )
    {
    return 1;
    }
  
  // clone the given datadef node into the new document
  tempNode = isgGlobal.nodeSetDocument.importNode( dataDef, true );
  newDataDef = (DOM_Element &) tempNode;

  // attach to node
  allNodes.appendChild( newDataDef );

  // done
  return 0;
}

DOMString makeNodeName(DOMString pipeName, int num)
{
  DOMString nodeName("");
  char buf[16];

  // whether to insert underscore or not
//  if( !pipeName.equals("") )
    // base not empty string
//    snprintf(buf, sizeof(buf), "_Node%03d", num);
//  else
    // base is empty string
    snprintf(buf, sizeof(buf), "Node%03d", num);
     
//  nodeName = pipeName.clone();
  nodeName += buf;

  return nodeName;
}

static DOMString getNextInType( DOM_Element pipe, DOM_Element nextSection  )
{
  
  if( nextSection == NULL )
  {
//    cout << "getNextInType(): nextSection is NULL" << endl;
    pipe.getAttribute("outType");
    return pipe.getAttribute("outType");
  }
  
//  addTypeToList( nextSection.getAttribute("dataRef") );
  return nextSection.getAttribute("dataRef");
}


// puts together the sections/pipes of a pipe and lets
// previous incarnation of this function worked by 
// "looking backwards" - i.e. it created nodes and kept
// track fo the last data type to connect to - this time we
// want to work forwards - look up what we need to connect
// to next. This puts in a better position to do join,
// branch since we will always have at least one node.
static int doConnections( DOM_Element newPipe, DOM_Element pipeSpec )
{
  DOM_Node connInfo;
  DOM_Node connections;
  DOM_Element currentNode;
  DOMString name, pipeName, nodeName;
  DOM_Attr newAttr;
  // list of sections in our connections list
  DOM_NodeList sectionList;

  DOMString nextInType="", thisInType;
  DOMString pipeInput="";
  DOMString pipeOutput="";
  DOM_Element telem;
  int nodeNumber = 0;

  // get the name of the pipe
  pipeName = pipeSpec.getAttribute("name");
  // get its input and output types
  pipeInput = pipeSpec.getAttribute("inType");
  pipeOutput = pipeSpec.getAttribute("outType");
//  addTypeToList( pipeInput );
//  addTypeToList( pipeOutput );

  // find our "connections" element 
  connections = pipeSpec.getElementsByTagName("connections").item(0);
  if( connections != NULL )
  {
    telem = DOM_ELEMENT( connections );
    // if we have a connections element we should pull all the sections
    // out from it - then we can walk down this list
    sectionList = telem.getElementsByTagName("section");
  }
  else
  {
    sectionList = NULL;
  }

  // create our first node - the node at the very front of this pipe
  // this will be node number 0 (first)
  nodeName = makeNodeName(pipeName, nodeNumber);

  thisInType = pipeInput;
  // now we add the node at the beginning of the pipe - 
  if( sectionList == NULL )
  {
    DOM_Element a;// = 0; // a little hack to pass a properly typed NULL value
    nextInType = getNextInType( pipeSpec, a);
    // if there is no section list, then we just connect to output
    currentNode = addNodeElement(newPipe, nodeName, pipeName, thisInType, nextInType  );
  }
  else
  {
    nextInType = getNextInType( pipeSpec, DOM_ELEMENT( sectionList.item(nodeNumber) ));
    // if there IS a section list, then we connect to the next node down the line
    currentNode = addNodeElement( newPipe, nodeName, 
                                  pipeName, thisInType, nextInType );
  }
  nodeNumber++;

  // now we iterate over all of our connection info
  if( connections != NULL )
    connInfo = connections.getFirstChild();
  else connInfo = NULL;
  while( connInfo != NULL )
  {
    name = connInfo.getNodeName();
    if( name.equals("#text") )
    {
      // do nothing
    }
    else if( name.equals("join") )
    {
      DOM_Element newInput;
      DOM_Attr newAttr;
      telem = DOM_ELEMENT(connInfo);
      // create another input for this node
      newInput = currentNode.getOwnerDocument().createElement("input");

      // get to which pipe we are connecting
      newAttr = currentNode.getOwnerDocument().createAttribute("pipe");
      newAttr.setValue( telem.getAttribute("pipe") );
      newInput.setAttributeNode( newAttr );
      
      // if connect to branch
      if( DOM_ELEMENT(connInfo).getAttribute("branch") != NULL )
      {
        // create an attribute for it
        newAttr = currentNode.getOwnerDocument().createAttribute("branch");
        // set it
        newAttr.setValue( telem.getAttribute("branch"));
        // attach attr
        newInput.setAttributeNode( newAttr );
      }

      // get to what type it is to which we need to connect
      newAttr = currentNode.getOwnerDocument().createAttribute("type");
      newAttr.setValue( getPipeOutputType( telem.getAttribute("pipe") ));
//      addTypeToList( getPipeOutputType( telem.getAttribute("pipe") ) );
      newInput.setAttributeNode( newAttr );


      // add our new input node to this node's description
      currentNode.appendChild( newInput );
    }
    else if( name.equals("branch") )
    {
        DOM_Element newOutput;
        DOM_Attr newAttr;
        telem = DOM_ELEMENT(connInfo);

        // branch is an ouput
        newOutput = currentNode.getOwnerDocument().createElement("output");
        // branches must be named and typed
        // get the name
        newAttr = currentNode.getOwnerDocument().createAttribute("name");
        newAttr.setValue( telem.getAttribute("name") );
        newOutput.setAttributeNode( newAttr );

        // get the branch type
        newAttr = currentNode.getOwnerDocument().createAttribute("type");
        newAttr.setValue( telem.getAttribute("type") );
//        addTypeToList( telem.getAttribute("type") );
        newOutput.setAttributeNode( newAttr );

        // add our new output node to this node's description
        currentNode.appendChild( newOutput );
    }
    else if( name.equals("section") )
    {
//      cout << "doConnections(): doing section." << endl;
      nodeName = makeNodeName( pipeName, nodeNumber );
      // process a section
      thisInType = nextInType;
      nextInType = getNextInType( pipeSpec, DOM_ELEMENT( sectionList.item(nodeNumber)));
      currentNode = addNodeElement( newPipe, nodeName, pipeName, thisInType, nextInType );
      // move our nodeNumber down
      nodeNumber++;
    }
    // move to next on list
    connInfo = connInfo.getNextSibling();
  }

}

static DOMString getPipeOutputType( DOMString pipeName )
{
  int i;
  DOM_NodeList tnl;
  DOM_Element telem;
  DOMString output;
  DOM_Element root = isgGlobal.specDocument.getDocumentElement();

  tnl = root.getElementsByTagName("pipe");
  for( i = 0; i < tnl.getLength(); i++ )
  {
    telem = DOM_ELEMENT(tnl.item(i));
    if( telem.getAttribute("name").equals(pipeName) )
    {
      return telem.getAttribute("outType");
    }
  }
  return "UNKNOWN";
}

/*
{


  // if "connections" element doesn't exist, set currElement = NULL
  if (connections != NULL)
  {
    currElement = DOM_ELEMENT(connections.getFirstChild());
  }
  else
    currElement = NULL;

  // inType of the pipe will become the input of the first node
  // if inType == CAPPED, addNodeElement function will handle it
  pipeInput = pipeSpec.getAttribute("inType");

  nodeNumber = 0;

  // loop until out of pipe sections, join, pipeRef, etc. 
  while( currElement != NULL )
  {
    name = currElement.getNodeName();
    if( name.equals("#text") )
    {
      // don't do anything
    }
    else if(name.equals("section"))
    {
      nodeName = makeNodeName(pipeName, nodeNumber);

      // add a section to the current pipe
      // create a node
      addNodeElement(newPipe, 
		      nodeName, pipeName,
		      lastOutType, currElement.getAttribute("dataRef"));

      // increment the node number
      nodeNumber++;

      // the output of this node will become the input of next node
      lastOutType = currElement.getAttribute("dataRef");
    }
    else if( name.equals("join") )
      {
      cout << "join" << endl;
      // join several pipes together
      // not implemented right now
      cerr << "'join' not currently supported." << endl;
      }
    else if( name.equals("branch") )
      {
      cout << "branch" << endl;
      // branch provides alternate access point to pipe
      addBranch( newPipe, nodeName );
      cerr << "'branch' not currently supported." << endl;
      }
    else if( name.equals("pipeRef") )
      {
      cout << "pipeRef" << endl;
      // join several pipes together
      // not implemented right now
      cerr << "'pipeRef' not currently supported." << endl;
      }
    else 
      {
      // error
      cerr << "***isg::buildNewPipe:: Error: Unrecognized tag." << endl;
      }

    currElement = DOM_ELEMENT(currElement.getNextSibling());
  }  // while() putting together pipe sections

  // now that we've done all the sections explicitly, do the last node 
  // even though there are "connections" element, we still do the last node
  
  nodeName = makeNodeName(pipeName, nodeNumber);

  // if outType == CAPPED, addNodeElement function will handle it
  addNodeElement(newPipe, 
		  nodeName, pipeName,
		  lastOutType, pipeSpec.getAttribute("outType"));

  return 0;
} */

static DOM_Element addNodeElement(DOM_Element currPipe, DOMString nodeName,   
                                  DOMString pipeName,   DOMString inType,     
                                  DOMString outType)
{
  DOM_Document currDoc;
  DOM_Attr newAttr;
  DOM_Element newNode;
	
  // get owner document
  currDoc = currPipe.getOwnerDocument();

  // create a node
  newNode = currDoc.createElement("node");

  // add node
  currPipe.appendChild(newNode);

  // add nodeName and pipeName attributes and block number
  newAttr = currDoc.createAttribute("nodeName");
  newAttr.setValue(nodeName);
  newNode.setAttributeNode(newAttr);
  
  newAttr = currDoc.createAttribute("pipeName");
  newAttr.setValue(pipeName);
  newNode.setAttributeNode(newAttr);

  newAttr = currDoc.createAttribute("num");
  newAttr.setValue(currPipe.getAttribute("num"));
  newNode.setAttributeNode(newAttr);

  if (inType != NULL && !inType.equals("CAPPED"))
  {
    DOM_Element nodeInput;

    // add the input type
    nodeInput = currDoc.createElement("input");
    newNode.appendChild(nodeInput);

    // add type attribute
    newAttr = currDoc.createAttribute("type");
//    addTypeToList( inType );
    newAttr.setValue(inType);
    nodeInput.setAttributeNode(newAttr);
  }

  if (outType != NULL && !outType.equals("CAPPED"))
  {
    DOM_Element nodeOutput;

    // add the output type
    nodeOutput = currDoc.createElement("output");
    newNode.appendChild(nodeOutput);

    // add type attribute
    newAttr = currDoc.createAttribute("type");
    newAttr.setValue(outType);
    nodeOutput.setAttributeNode(newAttr);
  }

  return newNode;
}


// decides what to do with a single property
static int doAspect( DOM_Element aProp, DOM_Element &nodeDoc )
{
  DOMString whichProp;
  DOM_Element aspects;

//  cout << "doAspect(): enter" << endl;
  //pprint( aProp );

  whichProp = aProp.getAttribute("name");
  
/*
  aspects = isgGlobal.nodeSetDocument.createElement("aspects");
  isgGlobal.nodeSetDocument.getDocumentElement().appendChild(aspects);
*/
  aspects = isgGlobal.nodeSetDocument.createElement("aspects");
  nodeDoc.appendChild(aspects);

  if( whichProp.equals("latency") )
    {
    doLatency( aspects, aProp );  
    }
  else
    {
    cout << "Unsupported aspect: ";
    whichProp.print();
    cout << endl;
    }

//  cout << "doAspect(): exit" << endl;
}

// does a list of properties per pipe
static int doProperties( DOM_Element &nodeDoc, DOM_Element thePipe )
{
  DOM_Node tempNode;
  DOM_Element tempElement;
  DOM_Element allProps;
  
  tempNode = thePipe.getFirstChild();
  while( tempNode != NULL )
  {
    if( tempNode.getNodeName().equals("properties") ) break;
    tempNode = tempNode.getNextSibling();
  }

  if( tempNode == NULL )
  {
    // no properties to handle
    // this is NOT an error, a pipe may be perfectly valid 
    // to have no properties
    return 0;
  }

  // otherwise, we have the properties now
  allProps = (DOM_Element &) tempNode;
  //cout << "doProperties(): properties found." << endl;

  // process all the properties one by one
  tempNode = allProps.getFirstChild();
  while( tempNode != NULL )
  {
    //tempNode.getNodeName().println();
    if( tempNode.getNodeName().equals("aspect") )
    {
      tempElement = (DOM_Element &)tempNode;
      // do all the work on this 
      doAspect( tempElement, nodeDoc );
    }
    
    tempNode = tempNode.getNextSibling();
  }

  return 0;
}

// buildPipe() -- takes an element pointing to a pipe and 
//             -- synthesizes a new one based on the sections
//             -- inside the pipe
static int buildPipe( DOM_Element pipeSpec )
{
  DOM_Element newPipe;
  DOM_Attr newAttr;
  DOMString pipeName;
  DOMString pipeBlockNum;

  // get the name of the pipe
  pipeName = pipeSpec.getAttribute("name");
  cout << "Building pipe \"" << pipeName <<  "\"" << endl;

  // create pipe element to hold all nodes
  newPipe = isgGlobal.nodeSetDocument.createElement("pipe");
  isgGlobal.nodeSetDocument.getDocumentElement().appendChild(newPipe);

  // add pipeName attribute
  newAttr = isgGlobal.nodeSetDocument.createAttribute("pipeName");
  newAttr.setValue(pipeName);
  newPipe.setAttributeNode(newAttr);

  // carry the block number through to tthe nodeset
  pipeBlockNum = pipeSpec.getAttribute("num");
  newAttr = isgGlobal.nodeSetDocument.createAttribute("num");
  newAttr.setValue(pipeBlockNum);
  newPipe.setAttributeNode(newAttr);



  // init type list

  // add aspect info here
  doProperties(newPipe, pipeSpec);

  // string together the sections, etc.
  doConnections(newPipe, pipeSpec); 

  // append type list to pipe
}

static int textprint( DOM_Element r )
{
  static int indent;
  DOM_Element child;
  DOM_Node tempNode;
  DOMString rname;
  int i;
  indent += 2;
  // print self
  if( r == NULL )
    cout << "r is null" << endl;
  rname = r.getNodeName();

  if( !rname.equals("#text") ) 
    {
    tempNode = r.getFirstChild();
    child = (DOM_Element &)tempNode;
    //if( child != NULL ) cout << endl;
    while( child != NULL )
      {
        if(textprint(child))
          return 1;
        tempNode = child.getNextSibling();
        child = (DOM_Element &) tempNode;
      }
    }
  else
    {
    //rname.print();
    r.getNodeValue().print();
    }
  indent -= 2;
  return 0;
}

// white space : space, tab, LF, CR
int isAllWhiteSpace(DOMString domStr)
{
  char *str = domStr.transcode();

  for (int i=0; i < strlen(str); i++)
  {
    if (str[i] == ' ' || str[i] == '\t' || str[i] == '\n' || str[i] == '\r')
      continue;
    else
    {
      delete str;
      return 0;
    }
  }

  delete str;

  return 1;  
}

// remove text nodes which have only white spaces recursively
int removeWhiteSpace(DOM_Node node) 
{
  DOM_Node curNode;
  DOM_Node tempNode;

  curNode = node.getFirstChild();
  while(curNode != NULL)
  {
    // if the node is text node
    if (curNode.getNodeName().equals("#text"))
    {
      // if it is has only white spaces, remove it
      if (isAllWhiteSpace(curNode.getNodeValue()))
      {
	tempNode = curNode;
	curNode = curNode.getNextSibling();
	node.removeChild(tempNode);
	continue;
      }
    }
    else
      removeWhiteSpace(curNode);

    curNode = curNode.getNextSibling();
  }

  return 0;
}

