#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h> 
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "isgUtil.hpp"

/*
  Downcast from DOM_Node to DOM_Element
*/
DOM_Element DOM_ELEMENT(DOM_Node n)
{
  DOM_Element e;
  e = (DOM_Element &)n;
  return e;
}

/*
  stream out DOMString
*/
ostream &operator<<( ostream &out, DOMString s)
{
  char *p = s.transcode();
  out << p;
  delete[] p;
  return out;
}  


/*
  stream out DOM_Element tree in XML format
*/
ostream &operator<<( ostream &out, DOM_Element r )
{
  static int indent;
  DOM_Element child;
  DOM_Node tempNode;
  DOM_NamedNodeMap atts;
  DOMString rname;
  int i; 
  int doindent=0;

  indent += 2;
  // print self
  if( r == NULL )
  {
    out << "(null)" << endl;
    return out;
  }

  rname = r.getNodeName();
  if( !rname.equals("#text") ) 
  {
    for( i = 0; i < indent; i++)
      out << " ";

    out << "<" << rname;

    // print out all attributes
    atts = r.getAttributes();
    if( atts != NULL )
    {
      for( i = 0; i < atts.getLength(); i++ )
      {
        out << " ";
        out << atts.item(i).getNodeName();
        out << "=\"";
        out << atts.item(i).getNodeValue();
        out << "\"";
      } 
    }

    out << ">";
    
    tempNode = r.getFirstChild();
    child = (DOM_Element &)tempNode;
    if( (child != NULL) )
    {
      if( !child.getNodeName().equals("#text") ) { out << endl; doindent = 1; }
    }

    while( child != NULL )
    {
      out << child;
      tempNode = child.getNextSibling();
      child = (DOM_Element &) tempNode;
    }

    for( i = 0; doindent && (i < indent); i++)
      out << " ";
    out << "</" << rname << ">" << endl;
  }
  else
  {
    out << r.getNodeValue();
  }
  indent -= 2;
  return out;
}

/*
  stream out DOM_Element tree in XML format
*/
ostream &operator<<( ostream &out, DOM_Document r )
{
  out << r.getDocumentElement();
  return out;
}


/* file writings */
// make path string like param1/param2/param3/.../param_last/
char *makePathString(char *str, ...);
// make sub-directories along the given path
int makePathDir(char *path, mode_t mode = 0755);

// make path string like param1/param2/param3/.../param_last/
char *makePathString(char *str, ...)
{
  va_list ap;
  char *arg;

  // str = ""
  str[0] = 0;
  
  va_start(ap, str);
  while(arg = va_arg(ap, char *))
  {
    strcat(str, arg);
    if (str[strlen(str)-1] != '/') strcat(str, "/");
  }
  va_end(ap);
  
  return str;
}

// make sub-directories along the given path
int makePathDir(char *path, mode_t mode)
{
  char *dir;
  char *p;

  dir = new char[strlen(path) + 1];

  p = path;

  while(1)
  {
    p = strchr(p, '/');
    if (p == NULL) 
    {
      delete dir;
      return 0;
    }
    
    p++;
    // get dir name including '/'
    memcpy(dir, path, p - path);
    dir[p - path] = 0;

    // make directory
    if (mkdir(dir, mode))
    {
      // if dir exists already, continue
      // otherwise(that is, error occurs), return -1
      if (errno != EEXIST)
      {
        delete dir;
        return -1;
      }
    }
  }

  // cannot reach here
  return 0;
}

// get "text" from a node and its children
// make it sure to be text = "" before calling getTextOfNode for the first time
//*** I don't like this function... may be modified later (young)
char *getTextOfNode(char *text, DOM_Node node)
{
  DOM_Node childNode;
  char *temp;

  if(node.getNodeType() == DOM_Node::TEXT_NODE)
  {
     temp = node.getNodeValue().transcode();
     strcat(text, temp);
     delete temp;
  }

  childNode = node.getFirstChild();
  while(childNode != NULL)
  {
    getTextOfNode(text, childNode);
    childNode = childNode.getNextSibling();
  }

  return text;
}

// textprint2stream() prints the text nodes into a stream given
// and we should only recurse on marked nodes
int textprint2stream( FILE *f, DOM_Element r )
{
  DOM_Element child;
  DOM_Node tempNode;
  char *buf;
  
  // catch null cases which shouldn't ever slip through anyways
  if( r == NULL )
    {
    cerr << "r is null" << endl;
    return 1;
    }

  if( !r.getNodeName().equals("#text") ) 
    {
    tempNode = r.getFirstChild();
    child = (DOM_Element &)tempNode;
    while( (tempNode != NULL) ) 
      { 
      // if we have an unmarked element, return immediately and ignore it
      if((tempNode.getNodeType() == DOM_Node::ELEMENT_NODE) && 
         (!child.getAttribute("mark").equals("true")))
        {
        tempNode = child.getNextSibling();
        child = (DOM_Element &) tempNode;
        }
      else
        {
        textprint2stream(f, child);
        tempNode = child.getNextSibling();
        child = (DOM_Element &) tempNode;
        }
      }
    }
  else
    {
    buf = r.getNodeValue().transcode();
    fprintf( f, "%s", buf);
    delete buf;
    }

  return 0;
}

// This will write out the set of files that describe the data types
// in the pipe as well as the connections at the nodes using the 
// already-filled-in template to do so
int writeFiles( DOM_Document domDoc )
{
  DOM_Element filledTemplate = domDoc.getDocumentElement();
  DOM_NodeList nodeList;
  DOM_Element fileElement;
  DOM_Element nameElement, contentElement;
  FILE *outFile;
  int i, maxi;
  char filename[512]; // file name should be shorter than 511 bytes

  // find <file> elements
//  cout << "isgUtil.cpp:writeFiles(): begin" << endl;
//  cout << "isgUtil.cpp:writeFiles(): begin dump" << endl << filledTemplate << endl << "isgUtil.cpp end dump." << endl;
  nodeList = filledTemplate.getElementsByTagName( "file" );
//  cout << "isgUtil.cpp:writeFiles(): list len = " << nodeList.getLength() << endl;
  for(i = 0; i < nodeList.getLength(); i++)
  {
//    cout << "isgUtil.cpp:writeFiles(): in loop " << i << endl;
    fileElement = DOM_ELEMENT(nodeList.item(i));


    // find the first <name> element among children
    nameElement = DOM_ELEMENT((fileElement.getElementsByTagName("name")).item(0));
    if (nameElement == NULL) // if not found
    {
      cerr << "<file> element should have child element of <name>" << endl;
      // should we continue? or stop and return? now, we continue..
      continue;
    }	

    // make filename = "" 
    filename[0] = 0;
    // <name> element may have several child elements, 
    // so we have to get only text from them
    getTextOfNode(filename, nameElement);
    cout << "isgUtil.cpp:writeFiles(): filename: " << filename << endl;

    // make directories in the filename
    // ex) if filename = "Node001/node.cpp", make Node001 directory
    makePathDir(filename, 0755);

    // find the first <content> element among children
    if( (contentElement = DOM_ELEMENT((fileElement.getElementsByTagName("content")).item(0) )) == NULL )
    {
      // if not found
      cerr << "<file> element should have child element of <content>" << endl;
      // should we continue? or stop and return? now, we continue..
      continue;
    }

    // open the file for output
    outFile = fopen( filename, "w" );
    if( outFile == NULL )
    {
      cerr << "***Error opening " << filename << " for writing." << endl;
			
      return 1;
    }

    // write to the file with content
    cout << "Writing " << filename << endl;
    textprint2stream( outFile, contentElement );

    // close the file
    fclose( outFile );
  }

  return 0;
}

