//////////////////////////////////////////////////////////////////////////////////////
// outPerNode.cpp - implementation of functions to output the DOM document 
//
#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "isg.hpp"
#include "out.hpp"

// from out.cpp
// (young) I moved this function to out.cpp because it can be generally used.
int fillTagWithText(DOM_Element root, char *tagName, DOMString text);
// make path string like param1/param2/param3/.../param_last/
char *makePathString(char *str, ...);

// from outAllNodes.cpp
int generateDataDefCode(DOM_Element templt, DOM_Document nodeSetDocument);

int generateNodeCode( DOM_Element templt, DOM_NodeList nodeDescList );
static int fillNodeCode( DOM_Element templt, DOM_Element nodeDesc );
static int fillInputConnection( DOM_NodeList, DOM_NodeList );
static int fillOutputConnection( DOM_NodeList, DOM_NodeList );
static int fillOutputSubData( DOM_Element nsInfo, DOM_Element tTemplt );
static int fillInputSubData( DOM_Element nsInfo, DOM_Element tTemplt );

  
int generatePerNode( DOM_Element templt, DOM_Element pipeDesc )
{
  DOM_NodeList nodeDescList;

  int perNodeTemplateNum;
  DOM_Element *perNodeTemplate;

  DOM_NodeList nodeList;

  // process <perNode>

  // we will find <node> from pipe description
  nodeDescList = pipeDesc.getElementsByTagName("node");

  // publish node description to file
  if (gFlags & FLAG_PUBLISH_FILE)
  {
    char filename[128];
    char *publishRoot;
    char *pipeName;
    char *nodeName;

    // find publishing root and if not, use pwd
    if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
    {
      publishRoot = getenv("PWD");
    }

    pipeName = pipeDesc.getAttribute("pipeName").transcode();

    // for every node description
    for (int i=0; i < nodeDescList.getLength(); i++)
    {
      nodeName = DOM_ELEMENT(nodeDescList.item(i)).getAttribute("nodeName").transcode();

      makePathString(filename, publishRoot, pipeName, nodeName, NULL);
      makePathDir(filename);

      // file name will be nodeName/nodeDescription
      strcat(filename, "nodeDescription");

      ofstream ofile(filename);
      ofile << DOM_ELEMENT(nodeDescList.item(i));

      delete nodeName;
    }

    delete pipeName;
  }    

  // get all of <perNode> in the template
  nodeList = templt.getElementsByTagName("perNode");
  perNodeTemplateNum = nodeList.getLength();

  // because nodeList changes as clones and appends
  // we copy original nodes to some array
  perNodeTemplate = new DOM_Element[perNodeTemplateNum];

  for (int i=0; i < perNodeTemplateNum; i++)
  {
    perNodeTemplate[i] = DOM_ELEMENT(nodeList.item(i));
  }

  // for all of <perNode> in the template
  for (int i=0; i < perNodeTemplateNum; i++)
  {
    generateNodeCode(perNodeTemplate[i], nodeDescList);
  }

  delete [] perNodeTemplate;

  return 0;
}

int generateNodeCode( DOM_Element templt, DOM_NodeList nodeDescList )
{
  // now for all the individual nodes
  DOM_Element currNode;
  DOM_Element currNodeTemplate;

  int i; 

  for( i = 0; i < nodeDescList.getLength(); i++ )
  {
    // for each description of a node, 
    // we need to copy the template and fill in it
    currNodeTemplate = DOM_ELEMENT(templt.cloneNode(true)); // copy deep

    // attach in tree
    // insert right before the original template
    templt.getParentNode().insertBefore( currNodeTemplate, templt );

    // get description
    currNode = DOM_ELEMENT(nodeDescList.item(i));

    // fill template
    fillNodeCode( currNodeTemplate, currNode );

    // update to print new stuff
    updateMarks( templt.getOwnerDocument().getDocumentElement() );

  } 

  // remove empty template
  // if there is no node desciptions, we don't have any node template
  templt.getParentNode().removeChild(templt);

  return 0;
}

// fills in code that goes to a single node
static int fillNodeCode( DOM_Element templt, DOM_Element nodeDesc )
{
  DOM_Node tnode;
  DOM_Element telem;
  DOM_NodeList templateList, descList;
  DOMString nodeName;
 
  //cout << "out.cpp::fillNodeCode():enter" << endl;
  //
 
  DOM_Attr newAttr;
  newAttr = templt.getOwnerDocument().createAttribute("num");
  newAttr.setValue( DOM_ELEMENT(nodeDesc.getParentNode()).getAttribute("num"));
  cout << "fillNodeCode(): " << DOM_ELEMENT(nodeDesc.getParentNode()).getAttribute("num") << endl;
  templt.setAttributeNode( newAttr );

  templateList = templt.getElementsByTagName("thisNodeName");
  nodeName = nodeDesc.getAttribute("nodeName");
  for (int i=0; i < templateList.getLength(); i++ )
  {
    makeTextAndMarkNode( templateList.item(i), nodeName );
//    templateList.item(0).appendChild(
//             templateList.item(0).getOwnerDocument().createTextNode(nodeName));
  }

  {
    DOMString nextNodeName;
    templateList = templt.getElementsByTagName("nextNodeName");
    if( nodeDesc.getNextSibling() == NULL ) nextNodeName = "end";
    else nextNodeName = DOM_ELEMENT(nodeDesc.getNextSibling()).getAttribute("nodeName");
    for (int i=0; i < templateList.getLength(); i++ )
    {
      makeTextAndMarkNode( templateList.item(i), nextNodeName );
    }
  }
  {
    templateList = templt.getElementsByTagName("prevNodeName");
    DOMString prevNodeName;
    if( nodeDesc.getPreviousSibling() == NULL ) prevNodeName = "end";
    else
      prevNodeName = DOM_ELEMENT(nodeDesc.getPreviousSibling()).getAttribute("nodeName");
    for (int i=0; i < templateList.getLength(); i++ )
    {
      makeTextAndMarkNode( templateList.item(i), prevNodeName );
    }
  }
  

  // get the incoming connection list from the template
  templateList = templt.getElementsByTagName("nodePipeInConnect");
  // get the incoming descriptions
  descList = nodeDesc.getElementsByTagName("input");
//  cout << "found " << descList.getLength() << " input elements in the description list." << endl;
//  cout << "template list is " << templateList.getLength() << " elements long." << endl;
  if( descList.getLength() )
    {
    // fill in the incoming connections
    fillInputConnection( descList, templateList );
    }
  
  // get the outgoing connection list from the template
  templateList = templt.getElementsByTagName("nodePipeOutConnect");
  // get the incoming descriptions
  descList = nodeDesc.getElementsByTagName("output");
//  cout << "found " << descList.getLength() << " output elements in the description list." << endl;
//  cout << "template list is " << templateList.getLength() << " elements long." << endl;
  if( descList.getLength() )
    {
    // fill in the outgoing connections
    fillOutputConnection( descList, templateList );
    }
  
//  cout << "out.cpp::fillNodeCode():exit" << endl;
  return 0;
}


// fillInputConnection() -- fills in connection information in a node for
// incoming connections - both joins AND the main data path
// nsInputList - the data from our nodeset doc
// tInputList - the empty template nodePipeInConnect nodes
// strategy: walk down tInputList
//   for each tInput, copy and fill with nsInputList info
// a "t" prefix for a var means that it is info for/from the template
// the "ns" prefix is for info associated w/the nodeset
static int fillInputConnection( DOM_NodeList nsInputList, DOM_NodeList tInputList )
{
  DOM_Element *tInputVector;
  DOM_Element tCopy;
  DOM_Node nsnode,tnode;
  DOM_Element telem,nselem;
  DOMString thisNodeName;
  int tInputListLength, tdex;
  int nsInputListLength, nsdex;

  // get the nodeName of this node
  telem = DOM_ELEMENT(nsInputList.item(0).getParentNode());
  thisNodeName = telem.getAttribute("nodeName");

  // length of our lists
  tInputListLength = tInputList.getLength();
  nsInputListLength = nsInputList.getLength();

  // since we have to copy our template nodes, put in a vector
  // otherwise the "liveness" of the DOM_NodeList we are given will complicate things
  tInputVector = new DOM_Element[ tInputListLength ];
  for( tdex = 0; tdex < tInputListLength; tdex++ )
  {
    tInputVector[tdex] = DOM_ELEMENT(tInputList.item(tdex));
  }

  // now we process the template and fill it with information
  for( tdex = 0; tdex < tInputListLength; tdex++ )
  {
    // iterate, but -1 since already have one copy
    for( nsdex = nsInputListLength - 1; nsdex > 0; nsdex--)
    {
      // copy (deep)
      tCopy = DOM_ELEMENT(tInputVector[tdex].cloneNode(true));
      // fill it in
      fillInputSubData( DOM_ELEMENT(nsInputList.item(nsdex)), tCopy );
      // attach to parent
      tInputVector[tdex].getParentNode().insertBefore( tCopy, tInputVector[tdex].getNextSibling() );
    }
    // do the final
    fillInputSubData( DOM_ELEMENT(nsInputList.item(0)), tInputVector[tdex] );
  }
}

static int fillInputSubData( DOM_Element nsInfo, DOM_Element tTemplt )
{
  DOM_Node tCurr;

  tCurr = tTemplt.getFirstChild();
  while( tCurr != NULL )
  {
    if( tCurr.getNodeName().equals("#text") )
    {
      // do nothing
    }
    else if( tCurr.getNodeName().equals("pipeName") )
    {
      // see if a name was given
      DOMString nsPipeName = nsInfo.getAttribute("pipe");
      // if no name given then this was just to connect to upstream node in this pipe
      if( nsPipeName == NULL )
      {
        nsPipeName = DOM_ELEMENT(DOM_ELEMENT(nsInfo.getParentNode()).getParentNode()).getAttribute("pipeName");
      }
      // transfer the info
      makeTextAndMarkNode(tCurr, nsPipeName );
    }
    else if( tCurr.getNodeName().equals("connType") )
    {
      DOMString nsConnType;

      // if a name is given then we are just looking
      // at a connection that is going back to a previous node
      // other wise when we are given a name we will be looking back
      // at a join
      if( nsInfo.getAttribute("pipe") == NULL )
      {
        nsConnType = "node";
      }
      else
      {
        // if no branch name given then it is just the pipe output
        if( nsInfo.getAttribute("branch") == NULL )
          nsConnType = "pipe";
        else
          nsConnType = "branch";
      }
      makeTextAndMarkNode( tCurr, nsConnType );

    }
    else if( tCurr.getNodeName().equals("connName") )
    {
      // get the connection name
      DOMString nsConnName = nsInfo.getAttribute("pipe");
      // if no connection supplied, this is the input from 
      // the previous node, and our name will be this node's name
      if( nsConnName == NULL )
      {
        nsConnName = DOM_ELEMENT(nsInfo.getParentNode()).getAttribute("nodeName");
//        cout << "fillInputSubData(): 1: " << nsConnName.transcode() << endl;
      }
      else 
      {
        if( nsInfo.getAttribute("branch") == NULL )
        {
//          cout << "fillInputSubData(): 2." << endl;
          nsConnName = "out";
        }
        else 
        {
//          cout << "fillInputSubData(): 3." << endl;
          nsConnName = nsInfo.getAttribute("branch");
        }
      }
      makeTextAndMarkNode( tCurr, nsConnName );
//      cout << DOM_ELEMENT(tCurr);
    }
    else if( tCurr.getNodeName().equals("dataTypeName") )
    {
      DOMString nsType = nsInfo.getAttribute("type");
      makeTextAndMarkNode( tCurr, nsType );
    }
/*    else if( tCurr.getNodeName().equals("aspect") )
    {
      // we need to fill the data required inside the aspect
      fillOutputSubData( nsInfo, DOM_ELEMENT(tCurr) );
    }*/
    tCurr = tCurr.getNextSibling();
  } // while(tCurr!=NULL)
  return 0;
}

// see notes on fillInputConnection for how this works
static int fillOutputConnection( DOM_NodeList nsOutputList, DOM_NodeList tOutputList )
{
  DOM_Element *tOutputVector;
  DOM_Element tCopy;
  DOM_Node nsnode,tnode;
  DOM_Element telem,nselem;
  DOMString thisNodeName;
  int tOutputListLength, tdex;
  int nsOutputListLength, nsdex;

  // get the nodeName of this node
  telem = DOM_ELEMENT(nsOutputList.item(0).getParentNode());
  thisNodeName = telem.getAttribute("nodeName");

  // length of our lists
  tOutputListLength = tOutputList.getLength();
  nsOutputListLength = nsOutputList.getLength();

  // since we have to copy our template nodes, put in a vector
  // otherwise the "liveness" of the DOM_NodeList we are given will complicate things
  tOutputVector = new DOM_Element[ tOutputListLength ];
  for( tdex = 0; tdex < tOutputListLength; tdex++ )
  {
    tOutputVector[tdex] = DOM_ELEMENT(tOutputList.item(tdex));
  }

  // now we process the template and fill it with information
  for( tdex = 0; tdex < tOutputListLength; tdex++ )
  {
    // iterate, but -1 since already have one copy
    for( nsdex = nsOutputListLength - 1; nsdex > 0; nsdex--)
    {
      // copy (deep)
      tCopy = DOM_ELEMENT(tOutputVector[tdex].cloneNode(true));
      // fill it in
      fillOutputSubData( DOM_ELEMENT(nsOutputList.item(nsdex)), tCopy );
      // attach to parent
      tOutputVector[tdex].getParentNode().insertBefore( tCopy, tOutputVector[tdex].getNextSibling() );
    }
    // do the final
    fillOutputSubData( DOM_ELEMENT(nsOutputList.item(0)), tOutputVector[tdex] );
  }
}

static int fillOutputSubData( DOM_Element nsInfo, DOM_Element tTemplt )
{
  DOM_Node tCurr;

  tCurr = tTemplt.getFirstChild();
  while( tCurr != NULL )
  {
    if( tCurr.getNodeName().equals("#text") )
    {
      // do nothing
    }
    else if( tCurr.getNodeName().equals("connType") )
    {
      DOMString nsConnType;
      if( nsInfo.getAttribute("name") == NULL )
      {
        nsConnType = "node";
      }
      else nsConnType = "branch";
      makeTextAndMarkNode( tCurr, nsConnType);
    }
    else if( tCurr.getNodeName().equals("connName") )
    {
      // get the connection name
      DOMString nsConnName = nsInfo.getAttribute("name");
      // if no name supplied, this is the main output from 
      // this node, and our branch name will be this node's name
      if( nsConnName == NULL )
      {
//        cout << "fillOutputSubData(): 1 -- " ;
        DOM_Element nselem = DOM_ELEMENT(nsInfo.getParentNode());
//        nselem.getNodeName().print();
        // get the branch name
        nsConnName = ( nselem.getAttribute("nodeName") );
//        nsConnName.println();
      }
      else 
      {
        cout << "fillOutputSubData(): 2" << endl;
      }
      makeTextAndMarkNode( tCurr, nsConnName );
    }
    else if( tCurr.getNodeName().equals("pipeName") )
    {
      DOMString nsPipeName = DOM_ELEMENT( nsInfo.getParentNode().getParentNode() ).getAttribute("pipeName");
      makeTextAndMarkNode( tCurr, nsPipeName );
    }
    else if( tCurr.getNodeName().equals("dataTypeName") )
    {
      DOMString nsType = nsInfo.getAttribute("type");
      makeTextAndMarkNode( tCurr, nsType );
    }
/*    else if( tCurr.getNodeName().equals("aspect") )
    {
      // we need to fill the data required inside the aspect
      fillOutputSubData( nsInfo, DOM_ELEMENT(tCurr) );
    }*/
    tCurr = tCurr.getNextSibling();
  } // while(tCurr!=NULL)
  return 0;
}

