//////////////////////////////////////////////////////////////////////////////////////////
// outPerPipe.cpp - implementation of functions to output the DOM document 
//
#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "isg.hpp"
#include "out.hpp"

int generateDataDefCode(DOM_Element templt, DOM_Document nodeSetDocument);
int generatePerNode(DOM_Element templt, DOM_Element pipeDesc);

int generatePerPipeCode(DOM_Element templt, DOM_Document nodeSetDocument);
int fillPerPipeCode(DOM_Element templt, DOM_NodeList pipeDescList);
int fillPipeTemplate(DOM_Element templt, DOM_Element pipeDesc);

// make path string like param1/param2/param3/.../param_last/
char *makePathString(char *str, ...);

int generatePerPipeCode(DOM_Element templt, DOM_Document nodeSetDocument)
{
  DOM_NodeList pipeDescList;

  int perPipeTemplateNum;
  DOM_Element *perPipeTemplate;

  DOM_NodeList nodeList;

  // process <perPipe>

  // we will find <pipe> from nodeSetDocument
  pipeDescList = nodeSetDocument.getDocumentElement().getElementsByTagName("pipe");

  // publish pipe description to file
  if (gFlags & FLAG_PUBLISH_FILE)
  {
    char filename[128];
    char *publishRoot;
    char *pipeName;

    // find publishing root and if not, use pwd
    if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
    {
      publishRoot = getenv("PWD");
    }

    // for every pipe description
    for (int i=0; i < pipeDescList.getLength(); i++)
    {
      pipeName = DOM_ELEMENT(pipeDescList.item(i)).getAttribute("pipeName").transcode();
      makePathString(filename, publishRoot, pipeName, NULL);
      makePathDir(filename);

      // file name will be pipeName/pipeDescription
      strcat(filename, "pipeDescription");

      ofstream ofile(filename);
      ofile << DOM_ELEMENT(pipeDescList.item(i));

      delete pipeName;
    }
  }    

  // get all of <perPipe> in the template
  nodeList = templt.getElementsByTagName("perPipe");
  perPipeTemplateNum = nodeList.getLength();

  // because nodeList changes as clones and appends
  // we copy original nodes to some array
  perPipeTemplate = new DOM_Element[perPipeTemplateNum];

  for (int i=0; i < perPipeTemplateNum; i++)
  {
    perPipeTemplate[i] = DOM_ELEMENT(nodeList.item(i));
  }

  // for all of <perPipe> in the template
  for (int i=0; i < perPipeTemplateNum; i++)
  {
    fillPerPipeCode(perPipeTemplate[i], pipeDescList);
  }
  
  delete [] perPipeTemplate;

  return 0;
}

int fillPerPipeCode(DOM_Element templt, DOM_NodeList pipeDescList)
{
  DOM_Element newTemplate;

  for(int i=0; i < pipeDescList.getLength(); i++)
  {
    // clone the template node and its sub-nodes (deep copy)
    newTemplate = DOM_ELEMENT(templt.cloneNode( true ));

    // attach to the same parent as original template
    // insert right before the original template
    templt.getParentNode().insertBefore( newTemplate, templt );

    // do the aspects
//    generateAspectCode( newTemplate, DOM_ELEMENT(pipeDescList.item(i)) );

    // fill in the info
    fillPipeTemplate( newTemplate, DOM_ELEMENT(pipeDescList.item(i)) );
  }

  // remove our original template
  // and then we can move to the next template
  templt.getParentNode().removeChild(templt);

  return 0;
} 

int fillPipeTemplate(DOM_Element templt, DOM_Element pipeDesc)
{
  DOMString pipeName;

  pipeName = pipeDesc.getAttribute("pipeName");
  fillTagWithText(templt, "thisPipeName", pipeName);

  generatePerNode(templt, pipeDesc);  

//  generateAspectCode(templt, pipeDesc);

  return 0;
}

