<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ================================================================= -->
<!-- $Id: CppMethod.xsl,v 1.1.1.1 2003/06/11 19:11:43 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->

                                                  
<!-- ========================================= -->
<!-- == Template: data                      == -->
<!-- == Name : name of the data member      == -->
<!-- == visibility: public, protect ..      == -->
<!-- == type : the return type              == -->
<!-- ========================================= -->

<xsl:template name="data_member">
<xsl:param name="name"/>
<xsl:param name="visibility"/>
<xsl:param name="type"/>
<xsl:text>
</xsl:text>
  <xsl:value-of select="$visibility"/><xsl:text>:
</xsl:text><xsl:text> </xsl:text>
  <xsl:if test="not($type='none')">
    <xsl:value-of select="$type"/>
  </xsl:if>
  <xsl:text> </xsl:text>
  <xsl:value-of select="$name"/>
  <xsl:text>;
</xsl:text>
</xsl:template>


       		    
<!-- ========================================= -->
<!-- == Template: method                    == -->
<!-- == Name : name of the method           == -->
<!-- == visibility: public, protect ..      == -->
<!-- == type : the return type              == -->
<!-- == params: {type param1}{type param2}..== -->
<!-- == const: the const method             == -->
<!-- ========================================= -->

<xsl:template name="method">
<xsl:param name="name"/>
<xsl:param name="visibility"/>
<xsl:param name="type"/>
<xsl:param name="params"/>
<xsl:param name="const"/>
<xsl:param name="pure_virtual"/>
<xsl:text>
</xsl:text>
  <xsl:if test="$visibility">
    <xsl:value-of select="$visibility"/><xsl:text>:
</xsl:text><xsl:text> </xsl:text>
  </xsl:if>

  <xsl:if test="not($type='none')">
    <xsl:value-of select="$type"/>
  </xsl:if>
  <xsl:text> </xsl:text>
  <xsl:value-of select="$name"/><xsl:text>(</xsl:text>
  <!-- parameters -->
  <xsl:call-template name="method_params">
    <xsl:with-param name="params" select="$params"/>
  </xsl:call-template>
  <xsl:text>)</xsl:text>
  <!-- closing modifier --> 
  <xsl:if test="($pure_virtual='true')">
    <xsl:text> = 0</xsl:text>
  </xsl:if>               
  <xsl:if test="($const='true')">
    <xsl:text>const</xsl:text>
  </xsl:if>                     
  <xsl:text>;
</xsl:text>
</xsl:template>

<!-- ========================================= -->
<!-- == Template: method_imp                == -->
<!-- == The detailed implementation of the  == -->
<!-- == class method.                       == -->
<!-- == Name : name of the method           == -->
<!-- == type : the return type              == -->
<!-- == params: {type param1}{type param2}..== -->
<!-- == content: the code comes here        == -->
<!-- ========================================= -->

<xsl:template name="method_imp">
<xsl:param name="name"/>
<xsl:param name="type"/>
<xsl:param name="params"/>
<xsl:param name="init"/>
<xsl:param name="content"/>
<xsl:text>
</xsl:text>
  <xsl:if test="not($type='none')">
    <xsl:value-of select="$type"/>
  </xsl:if>
  <xsl:text> </xsl:text>
  <xsl:value-of select="$name"/><xsl:text>(</xsl:text>
  <!-- parameters -->
  <xsl:call-template name="method_params">
    <xsl:with-param name="params" select="$params"/>
  </xsl:call-template>
  <xsl:text>)</xsl:text>
  <!-- the init value assignment -->
  <xsl:if test="$init">
    <xsl:text>
 : </xsl:text>
  </xsl:if>
  <xsl:call-template name="init_assignments">
    <xsl:with-param name="params" select="$init"/>
  </xsl:call-template>

  <!-- the content -->
  <xsl:value-of select="$content"/>
  <xsl:text>
</xsl:text>
</xsl:template>
                                                         		    

<!-- ========================================= -->
<!-- == Template: init_assignments          == -->
<!-- ========================================= -->
<xsl:template name="init_assignments">
<xsl:param name="params"/>
  <xsl:if test="starts-with($params, '{')">
    <xsl:variable name="type_arg">
      <xsl:value-of select="substring-before(substring-after($params,'{'), '}')"/>
    </xsl:variable>
    <xsl:if test="not(normalize-space($type_arg)='none')">
      <xsl:value-of select="$type_arg"/>
    </xsl:if>
    <!-- recursive -->
    <xsl:if test="contains(substring-after($params,'}'), '{')">
      <xsl:text>, </xsl:text>
      <xsl:call-template name="method_params">
        <xsl:with-param name="params" select="substring-after($params,'}')" />
      </xsl:call-template>
    </xsl:if>
  </xsl:if>
</xsl:template>



<!-- ========================================= -->
<!-- == Template: method_params             == -->
<!-- ========================================= -->
<xsl:template name="method_params">
<xsl:param name="params"/>
  <xsl:if test="starts-with($params, '{')">
    <xsl:variable name="type_arg">
      <xsl:value-of select="substring-before(substring-after($params,'{'), '}')"/>
    </xsl:variable>
    <xsl:if test="(normalize-space($type_arg)='none')">
      <xsl:text>void</xsl:text>
    </xsl:if>
    <xsl:if test="not(normalize-space($type_arg)='none')">
      <xsl:value-of select="$type_arg"/>
    </xsl:if>
    <!-- recursive -->
    <xsl:if test="contains(substring-after($params,'}'), '{')">
      <xsl:text>, </xsl:text>
      <xsl:call-template name="method_params">
        <xsl:with-param name="params" select="substring-after($params,'}')" />
      </xsl:call-template>
    </xsl:if>
  </xsl:if>
</xsl:template>




                                                         		    
</xsl:stylesheet>


