<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ================================================================= -->
<!-- $Id: PPMP.xsl,v 1.1.1.1 2003/06/11 19:11:44 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->

<!-- ================================= -->
<!--      The PPMP_CPP template      -->
<!-- ================================= -->

<xsl:template name="PPMP_CPP">
  <xsl:param name="name"/>
  <xsl:param name="filters"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
    <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.cpp</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.cpp')"/>
    </xsl:with-param>
  </xsl:call-template>


  <!-- The system include -->
  <xsl:call-template name="use">
    <xsl:with-param name="files">{stdlib.h}{stdio.h}{fcntl.h}{string.h}{sys/types.h}{sys/stat.h}{unistd.h}{sys/timeb.h}{stdarg.h}{time.h}{iostream.h}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="dependency">
    <xsl:with-param name="files">
      <xsl:value-of select="concat('{',$name,'.h}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="filters_header_include">
    <xsl:with-param name="used_filters">
      <xsl:value-of select="$filters"/>
    </xsl:with-param>
  </xsl:call-template>


#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 0
#define DEBUG_WHAT 0

FILE* outf = NULL;
static int rcvr_b = 0;
#define INTER_FRAG_DELAY 2000

typedef enum filter_type_ {GZIP=0, LZO, SHRINK, GREY, JPEG1, JPEG2, JPEG3, ENCRYPT} filter_type;
#define FILTER_GET(flag, i) ((flag) &amp; 1&lt;&lt;(i))
#define UNSET_FILTER(i) ( FILTER_GET(FILTER, i)? FILTER ^= 1&lt;&lt;(i) : FILTER)



  <xsl:value-of select="$empty_line"/>
  <!-- The implementation of related class method -->
  <!-- Cstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Fragment', '::PPMP_Fragment')"/></xsl:with-param>
      <xsl:with-param name="params">{char* frag_buf}{size_t frag_len}</xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{frag_buffer (frag_buf)}', '{frag_length(frag_len)}')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Fragment', '::PPMP_Fragment')"/></xsl:with-param>
      <xsl:with-param name="params">{unsigned long ppmp_header}{unsigned long
      frame_no}{const char* msg_buf}{size_t len}{char* frag_buf}</xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{frag_buffer (frag_buf)}', '{frag_length(len+PPMP_FRAME_INFO_OFFSET)}')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
  memmove(frag_buffer + PPMP_PICT_HEADER_OFFSET, &amp;ppmp_header, 
	 sizeof(unsigned long));
  memmove(frag_buffer + PPMP_FRAME_NO_OFFSET, &amp;frame_no,
	 sizeof(unsigned long));

  memmove(frag_buffer + PPMP_FRAME_INFO_OFFSET, msg_buf, len );
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Sender', '::PPMP_Sender')"/></xsl:with-param>
      <xsl:with-param name="params">{int mus}{int pixelsize}{int maxpixelperline}</xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{mus(mus)}', '')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
  bpp = 3;
  max_buf_size = (mus + PPMP_FRAME_INFO_OFFSET + sizeof(unsigned long)
		  + sizeof(PPM_frame_info));
  lines_send_per_time = (int)(((double)mus)/(maxpixelperline) + .9999);
  max_buf_size = max_buf_size &lt; lines_send_per_time*maxpixelperline ?
                 lines_send_per_time*maxpixelperline + PPMP_FRAME_INFO_OFFSET +
                 sizeof(unsigned long) + sizeof(PPM_frame_info) 
                 :
                 max_buf_size;

  num_times =(int)(((double)480/(double)lines_send_per_time)+.9999);
}
    </xsl:with-param>
  </xsl:call-template>



  <!-- send_whole_frame() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Sender', '::', 'send_whole_frame')"/></xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
      <xsl:with-param name="params">{const char *msg_buf}{const size_t len}{const unsigned long PPMFrame_num}{const int src_id}{const unsigned int filter_flag}</xsl:with-param>
    <xsl:with-param name="content">
{
  curr_frame_num = PPMFrame_num;
  int bytes_send_per_time = lines_send_per_time * pfi.width * bpp/8;

  char fragment_buffer[max_buf_size*sizeof(char)];
  unsigned long tmp = ToBigEndian32(PPMP_FRAME_HEADER);
  memcpy(fragment_buffer, &amp;tmp, sizeof(unsigned long));

  //fill in frame #
  memcpy(&amp;fragment_buffer[4],&amp;curr_frame_num,sizeof(unsigned long));

  //For traditional img filters: shrink, grey.
  PPM_frame_info afterImg, afterImg1; 
  afterImg.buff = NULL;
  afterImg1.buff = NULL;

  if (FILTER_GET(filter_flag, SHRINK)) {
<xsl:if test="contains($filters, 'SHRINK')">
    //&lt;block name="filter_SHRINK">
    afterImg.size = pfi.size/4;
    if (!(afterImg.buff = (char*)malloc(sizeof(char) * afterImg.size)))
      printf(" [PPMP_Sender::send_whole_frame(): mem alloc error\n");

    pfi.buff = (char*)msg_buf;
    int ret = shrinkFilter(&amp;pfi, &amp;afterImg);

    pfi = afterImg;
    setPPMFrameInfo (mus, pfi); 
    bytes_send_per_time = lines_send_per_time * pfi.width * bpp/8;
    //&lt;/block>
</xsl:if>
  }
  
  if (FILTER_GET(filter_flag, GREY)) {
<xsl:if test="contains($filters, 'GREY')">
    //&lt;block name="filter_GREY">
    afterImg1.size = pfi.size/3;
    if (!(afterImg1.buff = (char*)malloc(sizeof(char) * afterImg1.size)))
      printf(" [PPMP_Sender::send_whole_frame(): mem alloc error\n");

    if (!FILTER_GET(filter_flag, SHRINK)) {
      pfi.buff = (char*)msg_buf;
    }
   int ret = greyFilter(&amp;pfi, &amp;afterImg1);

   if (FILTER_GET(filter_flag, SHRINK)) 
      if (afterImg.buff) free(afterImg.buff), afterImg.buff = NULL;

    pfi = afterImg1;
    setPPMFrameInfo (mus, pfi); 
    bytes_send_per_time = lines_send_per_time * pfi.width * bpp/8;
    //&lt;/block>
</xsl:if>
  }
  else if (0) {
  }

  int sendresult=send_preframe(fragment_buffer, src_id, curr_frame_num,
			       filter_flag);
  usleep(100);

  int bytes_read = 0;
  int actualsend = 0;
  sendresult=0;

  //
  // JPEG compression has to be here:
  // Assume jpg compression ONLY applied ONCE which gives us compressed img
  // about the size less than 16KB, in the sense we don't need to do jpg fragment
  // in AVStreams.
  // JPEG cannot be applied together with SHRINK or GREY filter(s).
  if (FILTER_GET(filter_flag, JPEG1)) {
<xsl:if test="contains($filters, 'JPEG')">
    //&lt;block name="filter_JPEG">
    //Call jpeg
    char* jpgImage = (char*)malloc(sizeof(char)*60*1024);
    int ret_size=0;
    
    int ret = ppm_to_jpeg (msg_buf, 
			   pfi.height,
			   pfi.width,
			   jpgImage,
			   40,
			   &amp;ret_size);
    
    if (ret &lt; 0)
      cout&lt;&lt;" Jpeg compression error "&lt;&lt;endl;

    unsigned short mark = 0x0;
    if (ret_size > max_buf_size - PPMP_FRAME_INFO_OFFSET) 
      cout&lt;&lt; " [PPMP_Sender::send_whole_frame(): JPEG compressed size too large !!!!\n";

    PPMP_Fragment frag(ToBigEndian32(PPMP_PICT_HEADER), ToBigEndian32(curr_frame_num),
		       jpgImage, 
		       ret_size, 
		       fragment_buffer);
    sendresult = this->send_fragment(fragment_buffer, ret_size+PPMP_FRAME_INFO_OFFSET,
				     src_id, curr_frame_num,
				     filter_flag,
				     mark);
    if (jpgImage) free(jpgImage), jpgImage = NULL;

    usleep(INTER_FRAG_DELAY);
    actualsend += sendresult;
    //&lt;/block>
</xsl:if>
  }
  else if (FILTER_GET(filter_flag, JPEG2)) {
  }
  else if (FILTER_GET(filter_flag, JPEG3)) {
  }
  else if (FILTER_GET(filter_flag, SHRINK) ||
	   FILTER_GET(filter_flag, GREY) ) {
    for (int time=0; time&lt;num_times; time++) {
      int tmp = pfi.size - bytes_read;
      int stream_length = (bytes_send_per_time > tmp? tmp: bytes_send_per_time);

      lines_send_curr = stream_length / pfi.width/3;
      
      unsigned short mark = 0;
      PPMP_Fragment frag(ToBigEndian32(PPMP_PICT_HEADER), ToBigEndian32(curr_frame_num),
			 pfi.buff + bytes_read, 
			 stream_length, fragment_buffer);
      sendresult = this->send_fragment(fragment_buffer, frag.get_frag_length(),
				       src_id, curr_frame_num,
				       filter_flag, mark);

      bytes_read += stream_length;
      usleep(INTER_FRAG_DELAY);
      actualsend += sendresult;
    }
   if (pfi.buff) free(pfi.buff), pfi.buff =NULL;
  }
  else {

    for (int time=0; time&lt;num_times; time++) {
      //Now the PPM pixel data
      int tmp = pfi.size - bytes_read;
      int stream_length = (bytes_send_per_time > tmp? tmp: bytes_send_per_time);

      lines_send_curr = stream_length / pfi.width/3;
      
      if (0/*other img filters*/) {
      }
      else {
	//No Img filters need to be applied
	unsigned short mark = 0;
	PPMP_Fragment frag(ToBigEndian32(PPMP_PICT_HEADER), ToBigEndian32(curr_frame_num),
			   msg_buf + bytes_read, 
			   stream_length, fragment_buffer);
	sendresult = this->send_fragment(fragment_buffer, frag.get_frag_length(),
					 src_id, curr_frame_num,
					 filter_flag, mark);
      }
      
      bytes_read += stream_length;
      usleep(INTER_FRAG_DELAY);
      actualsend += sendresult;
    } /* for (int time=0; time&lt;num_times; time++) */
  } /* if (FILTER_GET(filter_flag, JPEG3) */

  return actualsend;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of send_whole_frame() -->

  <!-- send_preframe() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Sender::', 'send_preframe')"/></xsl:with-param>
      <xsl:with-param name="params">{const char *PPMHeader}{const int src_id}{const int seq}{const unsigned int filter_flag}</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="content">
{
  char* buf;
  int sendresult;
  int PPMHeader_length = PPMP_FRAME_INFO_OFFSET + sizeof(PPM_frame_info);
 
  if (!(buf = (char*)malloc(PPMHeader_length*sizeof(char)))) {
    cerr&lt;&lt;" [PPMP_Sender::send_preframe(): 0ut of mem"&lt;&lt;endl;
    return (-1);
  }
 
  memcpy(buf, PPMHeader, PPMP_FRAME_INFO_OFFSET);
  memcpy(buf+PPMP_FRAME_INFO_OFFSET, &amp;pfi, sizeof(PPM_frame_info));

  unsigned short mark = 0xF;
  sendresult = this->send_fragment(buf, PPMHeader_length, src_id, seq, filter_flag,
				   mark); 
   if (sendresult&lt;0) {
    cerr&lt;&lt;" [PPMP_Sender::send_preframe(): cannot send preframe info"&lt;&lt;endl;
    return (-1);
  }

  if (buf) free(buf);
  return sendresult;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of send_preframe() -->

///////////////////////////////////////////////////////////////////////////////

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('Flag_Array', '::Flag_Array')"/></xsl:with-param>
      <xsl:with-param name="params">{int first}{int last}</xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{start(first)}', '{end(last)}',
      '{current(0)}', '{available(NULL)}')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
  available = (bool*)malloc((end-start+1)*sizeof(bool));

  if (!available)
    cerr&lt;&lt;" [Flag_Array::Cstr(): mem alloc error"&lt;&lt;endl;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('Flag_Array', '::Flag_Array')"/></xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{start(0)}', '{end(0)}',
      '{current(0)}', '{available(NULL)}')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->


  <!-- init_available() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('Flag_Array', '::', 'init_available')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="content">
{
  for (int i=start; i&lt;=end; i++) {
    available[i] = false;
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of init_available() -->

  <!-- free_available() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('Flag_Array', '::', 'free_available')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="content">
{
  if (available) free(available), available=NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of init_available() -->



/////////////////////////////////////////////////////////////////////////////


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Receiver', '::PPMP_Receiver')"/></xsl:with-param>
      <xsl:with-param name="params">{int mus}</xsl:with-param>
    <xsl:with-param name="init">
      <xsl:value-of select="concat('{MUS(mus)}', '{first(true)}',
      '{BytesReceived(0)}')"/>
    </xsl:with-param>
    <xsl:with-param name="content">
{
  /* For debug */
  fileidx =0;
  img_buffer = NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->

  <!-- process_ppm() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Receiver', '::', 'process_ppm')"/></xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
      <xsl:with-param name="params">{const char* msg}{size_t len}{unsigned long PPMFrameNum}{PPM_frame_info&amp; pfi}</xsl:with-param>
    <xsl:with-param name="content">
{
  if (first)
    {
      if (DEBUG_WHERE) printf(" [PPMP_Receiver(): if (first)\n");

      curr_PPM_frame_num = PPMFrameNum;
 
      //P1/P4 : bitmap
      //P2/P5 : greymap
      //P3/P6 : pixmap
      //Update later for X11 display
      bits_per_pixel = (pfi.ppm2 == '6'? 24:8);
      bytes_per_line = pfi.width * bits_per_pixel / 8;
      little_endian = true; //hack

      frame_height = pfi.height; //480;
      frame_width = pfi.width;//640
      img_start_line = 0;
     start_defragment(msg, 0, frame_height-1, 0,
                            (int)((double)MUS/(double)bytes_per_line+.9999)-1,
                            img_start_line);


      first=false;
    }

  bool image_flush = false;

  if (PPMFrameNum != curr_PPM_frame_num) {
    //frame number conflicts. Flush the current accumulated image
    image_flush = true;
    first = true;
    //return false;//Be careful
  }

  char* tmp = accumulate_fragment(msg, len, pfi, image_flush);

  if (!tmp)
    return 1;//Not displayed

  //Display the image
  first = true;

  free_img_buffer();

  if (image_flush)
    return 2;//incomplete but displayed
  else
    return 3;//complete img displayed
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of process_ppm() -->

  <!-- start_defragment() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Receiver', '::', 'free_img_buffer')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="content">
{
  if (img_buffer) {
    free(img_buffer);
    img_buffer = NULL;
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of free_img_buffer() -->


  <!-- start_defragment() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Receiver', '::', 'start_defragment')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params">{const char* img}{int first}{int last}{int d_start_line}{int d_end_line}{int i_start_line}</xsl:with-param>
    <xsl:with-param name="content">
{
  //init the flag arrary
  line = Flag_Array(first,last);
  line.init_available();

  line_accumulated = 0;

  data_start_line = d_start_line;
  data_end_line = d_end_line;
  img_start_line = i_start_line;

  /* For debugging*/
  BytesReceived = 0;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of start_defragment() -->


  <!-- accumulate_fragment() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('PPMP_Receiver', '::', 'accumulate_fragment')"/></xsl:with-param>
    <xsl:with-param name="type">char*</xsl:with-param>
      <xsl:with-param name="params">{const char *frag_buf}{size_t frag_len}{PPM_frame_info&amp; pfi}{bool flushOrNot}</xsl:with-param>
    <xsl:with-param name="content">
{
  const int first = line.get_start_idx();
  const int last  = pfi.height-1 > line.get_end_idx()?
                              line.get_end_idx() : pfi.height-1;

  const int ys = img_start_line;               // first output line
  const int ye = ys + data_end_line - data_start_line;

  if (NULL == img_buffer) {
    int tmp = pfi.size > frame_height*frame_width*bits_per_pixel/8?
      pfi.size : frame_height*frame_width*bits_per_pixel/8;

    img_buffer = (char*)malloc(tmp*sizeof(char));

    if (!img_buffer) {
      cerr&lt;&lt;" [PPMP_Receiver::accumulate_fragment(): mem alloc error"&lt;&lt;endl;
      exit(-1);
    }
  }

  //If frame # conflicts, flush the curr frame
  if (flushOrNot) {
    return img_buffer;
  }

  //May change each time, depends on MUS and img size ...
  int line_to_accumulate = data_end_line+1 > pfi.height-line_accumulated ?
    pfi.height-line_accumulated : data_end_line+1;
  line_to_accumulate = line_to_accumulate > 0 ? line_to_accumulate:0;

  for (int y=0; y&lt;line_to_accumulate;y++)
    {
      memcpy((void*)(img_buffer+ (y+ys)*bytes_per_line), frag_buf +
	     PPMP_FRAME_INFO_OFFSET + y*bytes_per_line, 
	     bytes_per_line);
      line.set_available(y+ys);
      line.increase_idx(1);
      
      line_accumulated ++;
  
      BytesReceived += bytes_per_line;
    }

  img_start_line += line_to_accumulate;

  if (pfi.size &lt;= BytesReceived) {
    BytesReceived = 0;
  }

  if (false == line.all_available()) 
    return NULL;

  if (DEBUG_WHERE) {
    printf(" \n[PPMP_Receiver::accumulate_fragment(): A whole frame accumulated -------------------------\n\n");
  }

  if (1) {
    process_whole_frame(img_buffer, pfi.size);
  }

  line.free_available();

  return img_buffer;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of accumulate_fragment() -->


   </content></file>
</xsl:template>
<!-- ================================= -->
<!--  End of The PPMP_CPP template   -->
<!-- ================================= -->









  
<!-- ================================= -->
<!--      The PPMP_H template        -->
<!-- ================================= -->


<xsl:template name="PPMP_H">
  <xsl:param name="name"/>
  <xsl:param name="filters"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
     <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.h</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.h')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- include ifdef-endif -->

#ifndef _<xsl:value-of select="$name_all_capital"/>_H_
#define _<xsl:value-of select="$name_all_capital"/>_H_

  <xsl:value-of select="$empty_line"/>

  <!-- The system include -->
  <xsl:call-template name="use">
    <xsl:with-param name="files">{string.h}{netinet/in.h}{iostream}</xsl:with-param>
  </xsl:call-template>


typedef unsigned long long uint64;
typedef   signed long long  int64;
typedef unsigned long      uint32;
typedef   signed long       int32;
typedef unsigned short     uint16;
typedef   signed short      int16;
typedef unsigned char      uint8;
typedef   signed char       int8;
 
inline uint16 ChangeEndian16(uint16 v) { return (v>>8) | (v&lt;&lt;8); }
inline uint32 ChangeEndian32(uint32 v) { return (v>>24) | ((v>>8)&amp;0xFF00) | ((v&amp;0xFF00)&lt;&lt;8) | (v&lt;&lt;
24); }

#ifdef WORDS_BIGENDIAN
inline uint16 ToLittleEndian16(uint16 v) { return ChangeEndian16(v); }
inline uint32 ToLittleEndian32(uint32 v) { return ChangeEndian32(v); }
inline uint16 ToBigEndian16(uint16 v)    { return v; }
inline uint32 ToBigEndian32(uint32 v)    { return v; }
#else
inline uint16 ToLittleEndian16(uint16 v) { return v; }
inline uint32 ToLittleEndian32(uint32 v) { return v; }
inline uint16 ToBigEndian16(uint16 v)    { return ChangeEndian16(v); }
inline uint32 ToBigEndian32(uint32 v)    { return ChangeEndian32(v); }
#endif



#if defined (STRUCT_PPM_frame_info)

#else

typedef struct {
  int tag;          //Reserved
  char ppm1;        //'P'
  char ppm2;        //'1'-'6'
  int size;         //width*height*pixel_size_in_bytes
  int width;
  int height;
  char* buff;       //Reserved
} PPM_frame_info, *PPM_frame_info_ptr;

#define STRUCT_PPM_frame_info

#endif

//Only used in sending PPM_frame_info before sending
//each fragment of the whole PPM frame
#define PPMP_FRAME_HEADER_OFFSET 0
#define PPMP_FRAME_INFO_OFFSET 8
//Used in sending each fragment of the whole PPM frame
#define PPMP_PICT_HEADER_OFFSET 0
#define PPMP_FRAME_NO_OFFSET 4

//Used as to distinguished from Mpeg headers in DVDview
#define PPMP_FRAME_HEADER 0x1ff
#define PPMP_PICT_HEADER 0x1f0


  <!-- =============================== -->
  <!-- The PPMP_Fragment class -->
  <!--                                 -->

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat('class', $name)"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The class declaration -->

  <xsl:call-template name="class">
    <xsl:with-param name="name">PPMP_Fragment</xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->

  <xsl:call-template name="method">
    <xsl:with-param name="name">PPMP_Fragment</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params">{char* frag_buf}{size_t frag_len}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">PPMP_Fragment</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params">{unsigned long ppmp_header}{unsigned long frame_no}{const char* msg_buf}{size_t len}{char* frag_buf}</xsl:with-param>
  </xsl:call-template>


  size_t get_frag_length()
    {
      return frag_length;
    }

  size_t get_payload_length()
    {//No matter it's a frame or pict header, data starts from 
      //PPMP_FRAME_INFO_OFFSET
      return frag_length - PPMP_FRAME_INFO_OFFSET;
    }

  void copy_payload(char *target_buffer)
    {
      memcpy(target_buffer, frag_buffer + PPMP_FRAME_INFO_OFFSET, 
	     frag_length - PPMP_FRAME_INFO_OFFSET);
    }

  unsigned long get_ppmp_frame_header()
    {
      return ntohl(*((unsigned long *) (frag_buffer + PPMP_FRAME_HEADER_OFFSET)));
    }

  unsigned long get_ppmp_pict_header()
    {
      return ntohl(*((unsigned long *) (frag_buffer + PPMP_PICT_HEADER_OFFSET)));
    }

  unsigned long get_ppmp_frame_no()
    {
      return ntohl(*((unsigned long*) (frag_buffer+ PPMP_FRAME_NO_OFFSET)));
    }

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">frag_buffer</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">char*</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">frag_length</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">size_t</xsl:with-param>
  </xsl:call-template>

  <!-- The closing of class -->

  <xsl:text>
};// class: 

</xsl:text>

/////////////////////////////////////////////////////////////////////////////

  <xsl:call-template name="class">
    <xsl:with-param name="name">PPMP_Sender</xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->

  <xsl:call-template name="method">
    <xsl:with-param name="name">PPMP_Sender</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params">{int mus}{int pixelsize}{int maxpixelperline}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">send_preframe</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{const char* PPMHeader}{const int src_id}{const int seq}{const unsigned int filter_flag}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">send_whole_frame</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{const char *msg_buf}{const size_t len}{const unsigned long PPMFrame_num}{const int src_id}{const unsigned int filter_flag}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">send_fragment</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">virtual int</xsl:with-param>
    <xsl:with-param name="params">{const char *msg_buf}{const size_t
    len}{const int src_id}{const int seq}{const unsigned int
    filter_flag}{const unsigned short mark}</xsl:with-param>
    <xsl:with-param name="pure_virtual">true</xsl:with-param>   
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">setPPMFrameInfo</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">virtual void</xsl:with-param>
    <xsl:with-param name="params">{int mus}{PPM_frame_info&amp; ppmfi}</xsl:with-param>
    <xsl:with-param name="pure_virtual">true</xsl:with-param>
  </xsl:call-template>

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">lines_send_curr</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">pfi</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">PPM_frame_info</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">bpp</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>
  
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">max_buf_size</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">lines_send_per_time</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">num_times</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">mus</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">curr_frame_num</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">unsigned long</xsl:with-param>
  </xsl:call-template>

  <!-- The closing of class -->

  <xsl:text>
};// class: 

</xsl:text>

///////////////////////////////////////////////////////////////////////////////

  <xsl:call-template name="class">
    <xsl:with-param name="name">Flag_Array</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">Flag_Array</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">Flag_Array</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params">{int first}{int last}</xsl:with-param>
  </xsl:call-template>

  int get_current_idx() {return current;}
  int get_start_idx() {return start;}
  int get_end_idx() {return end;}
  void increase_idx(int i) {current+=i;}
  void set_available(int idx) 
    {
    available[idx] = true;}

  bool all_available() {
    bool b = false;
    for (int i=start; i&lt;=end; i++) {
      if (available[i]==false) 

	return b;
    }
    return true;
  }

  <xsl:call-template name="method">
    <xsl:with-param name="name">init_available</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">free_available</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
  </xsl:call-template>

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">start</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">end</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">current</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">available</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">bool*</xsl:with-param>
  </xsl:call-template>

  <!-- The closing of class -->

  <xsl:text>
};// class: 

</xsl:text>

///////////////////////////////////////////////////////////////////////////////

  <xsl:call-template name="class">
    <xsl:with-param name="name">PPMP_Receiver</xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->

  <xsl:call-template name="method">
    <xsl:with-param name="name">PPMP_Receiver</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params">{int mus}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">process_ppm</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{const char* msg}{size_t len}{unsigned long PPMFrameNum}{PPM_frame_info&amp; pfi}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">start_defragment</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const char* img}{int first,int last}{int d_start_line}{int d_end_line}{int i_start_line}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">accumulate_fragment</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">char*</xsl:with-param>
    <xsl:with-param name="params">{const char *frag_buf}{size_t frag_len}{PPM_frame_info&amp; pfi}{bool flushOrNot}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">process_whole_frame</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">virtual int</xsl:with-param>
    <xsl:with-param name="params">{const char *frame}{size_t len}</xsl:with-param>
    <xsl:with-param name="pure_virtual">true</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">free_img_buffer</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
  </xsl:call-template>

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">img_buffer</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">char*</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">MUS</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">line_accumulated</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">first</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">bool</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">data_start_line</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">data_end_line</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">img_start_line</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">line</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">Flag_Array</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">frame_height</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">frame_width</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">frame_no</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">unsigned long</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">curr_PPM_frame_num</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">unsigned long</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">rpos</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">gpos</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">bpos</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">bytes_per_line</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">bits_per_pixel</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="data_member">
    <xsl:with-param name="name">little_endian</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">bool</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">BytesReceived</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">fileidx</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
  </xsl:call-template>

  <!-- The closing of class -->

  <xsl:text>
};// class: 

</xsl:text>


#endif // 



  </content></file>

</xsl:template>











</xsl:stylesheet>
