<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="xml"/>

<!-- ================================================================= -->
<!-- $Id: UAV.xsl,v 1.1.1.1 2003/06/11 19:11:44 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->
<!-- This stylesheet is specifically designed for ISG as the TAO       -->
<!-- AVStream abstract machine (binding). In addition the BBN QuO may  -->
<!-- also be created as needed.                                        -->
<!-- ================================================================= -->



<!-- The general templates and defs -->
<xsl:include href="common.xsl"/>
<xsl:include href="CppClass.xsl"/>
<!-- The TAO/AVStream -->

<xsl:include href="DataFlow.xsl"/>
<xsl:include href="INFOP.xsl"/>
<xsl:include href="Sender.xsl"/>
<xsl:include href="Receiver.xsl"/>
<xsl:include href="CtrlManager.xsl"/>
<xsl:include href="Pipe.xsl"/>
<xsl:include href="PPMP.xsl"/>
<xsl:include href="Makefile.xsl"/>


<!-- Identify the components in an application which use TAO AVStream -->
<!-- The true values will be known later -->
<xsl:variable name="sender_name">Sender</xsl:variable>
<xsl:variable name="receiver_name">Receiver</xsl:variable>
<xsl:variable name="ctrlmanager_name">CtrlManager</xsl:variable>
<xsl:variable name="datatype_name">INFOP DataFlow</xsl:variable>
<xsl:variable name="pipetype_name">Pipe</xsl:variable>
<xsl:variable name="ppmreader_name">PPMP ppmudpreader</xsl:variable>
<xsl:variable name="ppmdecoder_name">PPMP ppmdecoder</xsl:variable>
<xsl:variable name="sender_other_name">Share_Syn</xsl:variable>
<xsl:variable name="receiver_other_name">Share_Syn connect display</xsl:variable>
<xsl:variable name="profile_name">PROFILE</xsl:variable>

<xsl:template match="/">
<files>

  <xsl:apply-templates select="/xip/datatypes"/>

  <xsl:apply-templates select="/xip/composedPipe/pipes/pipe"/>

  <xsl:call-template name="CtrlManager_H">
    <xsl:with-param name="name">CtrlManager</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="CtrlManager_CPP">
    <xsl:with-param name="name">CtrlManager</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="Pipe_H">
    <xsl:with-param name="name">Pipe</xsl:with-param>
    <xsl:with-param name="filters">
        <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
           <xsl:variable name="found_filter">      
              <xsl:value-of select="attribute::name"/>
              </xsl:variable>
           <xsl:value-of select="concat('{', $found_filter, '}')"/>
        </xsl:for-each>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="Pipe_CPP">
    <xsl:with-param name="name">Pipe</xsl:with-param>
    <xsl:with-param name="filters">
        <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
           <xsl:variable name="found_filter">      
              <xsl:value-of select="attribute::name"/>
              </xsl:variable>
           <xsl:value-of select="concat('{', $found_filter, '}')"/>
        </xsl:for-each>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="PPMP_H">
    <xsl:with-param name="name">PPMP</xsl:with-param>
    <xsl:with-param name="filters">
        <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
           <xsl:variable name="found_filter">      
              <xsl:value-of select="attribute::name"/>
              </xsl:variable>
           <xsl:value-of select="concat('{', $found_filter, '}')"/>
        </xsl:for-each>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="PPMP_CPP">
    <xsl:with-param name="name">PPMP</xsl:with-param>
    <xsl:with-param name="filters">
        <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
           <xsl:variable name="found_filter">      
              <xsl:value-of select="attribute::name"/>
              </xsl:variable>
           <xsl:value-of select="concat('{', $found_filter, '}')"/>
        </xsl:for-each>
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="Makefile">
     <xsl:with-param name="name">
       <xsl:value-of select="concat('Makefile', '-part')"/></xsl:with-param>
     <xsl:with-param name="sender_files">
       <xsl:value-of select="concat('{', $sender_name, '}')"/>
     </xsl:with-param>   
     <xsl:with-param name="receiver_files">
       <xsl:value-of select="concat('{', $receiver_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="ctrlmanager_files">
       <xsl:value-of select="concat('{', $ctrlmanager_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="datatype_files">
       <xsl:value-of select="concat('{', $datatype_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="pipetype_files">
       <xsl:value-of select="concat('{', $pipetype_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="ppmreader_files">
       <xsl:value-of select="concat('{', $ppmreader_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="ppmdecoder_files">
       <xsl:value-of select="concat('{', $ppmdecoder_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="sender_other_files">
       <xsl:value-of select="concat('{', $sender_other_name, '}')"/>
     </xsl:with-param>
     <xsl:with-param name="receiver_other_files">
       <xsl:value-of select="concat('{', $receiver_other_name, '}')"/>
     </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="PROFILE">
    <xsl:with-param name="filename">
      <xsl:value-of select="$profile_name"/></xsl:with-param>
  </xsl:call-template>

</files>
</xsl:template>



<!-- ======================================== -->
<!-- Generate profile                         -->
<!--                                          -->
<xsl:template name="PROFILE">
<xsl:param name="filename"/>

  <file><name>
    <xsl:value-of
    select="$dir_output_root"/>frontEnd/src/<xsl:value-of
    select="$filename"/></name><content>
<xsl:value-of select="/xip/profiles/profile/attribute::name"/>
</content>
</file>

</xsl:template>
  
<!--                                          -->
<!-- ======================================== -->



<!-- ========================================= -->
<!-- The template generates 2 major components -->
<!-- Sender & Receiver                         -->

<xsl:template match="/xip/composedPipe/pipes/pipe">

  <!-- Get the filter info from the filter segment of -->
  <!-- XIP -->


  <!-- retrieve the pipe name and try to identify -->
  <!-- if it is a server-only/client-only          -->
  <!-- class_name: the type (class) of each node_name (instance) -->
  <xsl:variable name="class_name">
    <xsl:value-of select="@class"/>
  </xsl:variable>

  <!-- node_name cannot be duplicated, otherwise overwriting. -->
  <!-- The node_name will be passed to the template to generate -->
  <!-- the code. Here're some criteria doing that:              -->
  <!-- 1. The node_name will appear in the places of file names,-->
  <!--    class names, those string values will be used in the  -->
  <!--    naming service, class method names.                   -->
  <!-- 2. Since each node will be put into different files,     -->
  <!--    The class names, class method names methioned in 1    -->
  <!--    can be identical even for all the senders.            -->

  <xsl:variable name="node_name">
    <xsl:value-of select="@name"/>
  </xsl:variable>

  <xsl:variable name="in_port_name"
     select="/xip/pipes/pipe[@name=$class_name]/inport/attribute::name"/>
  <xsl:variable name="out_port_name"
     select="/xip/pipes/pipe[@name=$class_name]/outport/attribute::name"/>

 <!-- Get the data type right -->
  <xsl:variable name="data_type_1">
    <xsl:value-of select="/xip/pipes/pipe[@name=$class_name]/inport/attribute::type"/>
  </xsl:variable>

  <xsl:variable name="in_data_type">
    <xsl:value-of select="/xip/datatypes/datatype[@name=$data_type_1]/attribute::name"/>
  </xsl:variable>

  <xsl:variable name="data_type_2">
    <xsl:value-of select="/xip/pipes/pipe[@name=$class_name]/outport/attribute::type"/>
  </xsl:variable>

  <xsl:variable name="out_data_type">
    <xsl:value-of select="/xip/datatypes/datatype[@name=$data_type_2]/attribute::name"/>
  </xsl:variable>

  <!--xsl:value-of select="$in_port_name"/-->
  <!--xsl:value-of select="$out_port_name"/-->
  <xsl:choose>
      <xsl:when test="boolean($out_port_name)">
        <xsl:choose>
          <xsl:when test="boolean($in_port_name)">
            <!-- it's a server-and-receiver -->
	    <!-- if you reach here, you're doomed -->
	    <!--
             <xsl:variable name="distributor_name">
              <xsl:value-of select="concat('{', $node_name, '}')"/>
            </xsl:variable>

            <xsl:call-template name="Distributor_H">
              <xsl:with-param name="name">
                <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="in_type">
                <xsl:value-of select="/xip/absMachines/absMachine/mapTable/mapping[@from=$in_data_type]/@to"/>
              </xsl:with-param>
              <xsl:with-param name="out_type">
                <xsl:value-of select="/xip/absMachines/absMachine/mapTable/mapping[@from=$out_data_type]/@to"/>
              </xsl:with-param>
            </xsl:call-template>

            <xsl:call-template name="Distributor_CPP">
              <xsl:with-param name="name">
                <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="in_type">
                <xsl:value-of select="/xip/absMachines/absMachine/mapTable/mapping[@from=$in_data_type]/@to"/>
              </xsl:with-param>
              <xsl:with-param name="out_type">
                <xsl:value-of select="/xip/absMachines/absMachine/mapTable/mapping[@from=$out_data_type]/@to"/>
              </xsl:with-param>
            </xsl:call-template>
            -->
          </xsl:when>

          <xsl:otherwise>
            <!-- it's a sender-only -->
            <xsl:variable name="sender_name">
              <xsl:value-of select="concat('{', $node_name, '}')"/>
            </xsl:variable>

            <xsl:call-template name="Sender_H">
              <xsl:with-param name="name">
                <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="classname">
                <xsl:value-of select="$class_name"/>
              </xsl:with-param>
	      <xsl:with-param name="filters">
	         <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
                   <xsl:variable name="found_filter">      
                     <xsl:value-of select="attribute::name"/>
                   </xsl:variable>
                   <xsl:value-of select="concat('{', $found_filter, '}')"/>
                 </xsl:for-each>
              </xsl:with-param>
            </xsl:call-template>

            <xsl:call-template name="Sender_CPP">
              <xsl:with-param name="name">
                 <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="classname">
                <xsl:value-of select="$class_name"/>
              </xsl:with-param>
	      <xsl:with-param name="filters">
	         <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
                   <xsl:variable name="found_filter">      
                     <xsl:value-of select="attribute::name"/>
                   </xsl:variable>
                   <xsl:value-of select="concat('{', $found_filter, '}')"/>
                 </xsl:for-each>
              </xsl:with-param>
            </xsl:call-template>

          </xsl:otherwise>
        </xsl:choose>
      </xsl:when>

      <xsl:when test="boolean($in_port_name)">
        <!-- its' a receiver-only -->
            <xsl:variable name="receiver_name">
              <xsl:value-of select="concat('{', $node_name, '}')"/>
            </xsl:variable>

            <xsl:call-template name="Receiver_H">
              <xsl:with-param name="name">
                <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="classname">
                <xsl:value-of select="$class_name"/>
              </xsl:with-param>
	      <xsl:with-param name="filters">
	         <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
                   <xsl:variable name="found_filter">      
                     <xsl:value-of select="attribute::name"/>
                   </xsl:variable>
                   <xsl:value-of select="concat('{', $found_filter, '}')"/>
                 </xsl:for-each>
              </xsl:with-param>
            </xsl:call-template>

            <xsl:call-template name="Receiver_CPP">
              <xsl:with-param name="name">
                 <xsl:value-of select="$node_name"/>
              </xsl:with-param>
              <xsl:with-param name="classname">
                <xsl:value-of select="$class_name"/>
              </xsl:with-param>
	      <xsl:with-param name="filters">
	         <xsl:for-each select="/xip/composedPipe/connections/connection/filters/filter">
                   <xsl:variable name="found_filter">      
                     <xsl:value-of select="attribute::name"/>
                   </xsl:variable>
                   <xsl:value-of select="concat('{', $found_filter, '}')"/>
                 </xsl:for-each>
              </xsl:with-param>
            </xsl:call-template>

      </xsl:when>

      <xsl:otherwise>
         <xsl:value-of select="$dlmt1"/><xsl:text>/** Can not find
         Sender/Receiver*/</xsl:text><xsl:value-of select="$dlmt1"/>      </xsl:otherwise>
   </xsl:choose>

</xsl:template>

<!--                                           -->
<!-- The template generates 2 major components -->
<!-- ========================================= -->




<!-- ========================================= -->
<!-- The template generates DataTypes files,   -->
<!-- INFOP files.                              -->

<xsl:template match="/xip/datatypes">

  <xsl:call-template name="DataFlow_H">
     <xsl:with-param name="name">
       <xsl:value-of select="concat('','DataFlow')"/>
     </xsl:with-param>

     <xsl:with-param name="datatypes">
       <xsl:for-each select="/xip/datatypes/datatype">
         <xsl:variable name="found_datatype">      
           <xsl:value-of select="attribute::name"/>
         </xsl:variable>
         <xsl:value-of select="concat('{', $found_datatype, '}')"/>
       </xsl:for-each>
     </xsl:with-param>

  </xsl:call-template>

  <xsl:call-template name="DataFlow_CPP">
     <xsl:with-param name="name">
       <xsl:value-of select="concat('','DataFlow')"/>
     </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="INFOP_H">
    <xsl:with-param name="name">INFOP</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="INFOP_CPP">
    <xsl:with-param name="name">INFOP</xsl:with-param>
  </xsl:call-template>

</xsl:template>





                                             
</xsl:stylesheet>
