<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!-- this is a mapping of the infopipe types -->
<xsl:variable name="maptables" select="document('ECho2mapTables.xml')/mapTables"/>

<xsl:include href="ECho2filter.xsl"/>

<xsl:template name="oneSource">
<xsl:for-each select="composedPipe/declarations/pipe">
<xsl:variable name="thisPipeName" select="@name"/>
<xsl:variable name="thisPipeClass" select="@class"/>
<file>
<name><xsl:value-of select="../../@name"/>/<xsl:value-of select="$thisPipeName"/>/<xsl:value-of select="$thisPipeName"/>.c</name>
<content>
// &lt;block name="pipe_<xsl:value-of select="$thisPipeClass"/>"&gt;
/////////////////////////////////////////////////////////////////////////////
// <xsl:value-of select="$thisPipeName"/>.c -- the implementation file for this node
/////////////////////////////////////////////////////////////////////////////
#include "<xsl:value-of select="$thisPipeName"/>.h"
#include &lt;sys/types.h&gt;
#include &lt;sys/stat.h&gt;
#include &lt;errno.h&gt;


static EControlContext ec;
static CManager cm;

// this probably shouldn't go here (bad form), but we'll cope for now
char *incomingGlobalID;

// definition of publish types
#define PUBLISH_FILE  0x0001


// functions publishing and looking up
static int publish(char *pipe, char *connType, char *connName, char *str, int type);
static int publishFile(char *pipe, char *connType, char *connName, char *str);
static int unpublish(char *pipe, char *connType, char *connName, int type);
static int unpublishFile(char *pipe, char *connType, char *connName);
static char *lookup(char *pipe, char *nodeOrBranch, char *connPoint, int type, char *str);
static char *lookupFile(char *pipe, char *connType, char *connName, char *str);

<xsl:for-each select="/xip/datatypes/datatype" >
<!--
// carrier structure - allows for packaging both data
// and meta-info (e.g. QoS) along with the packet
//typedef struct <xsl:value-of select="@name"/>CarrierStructDef
//{
//  <xsl:value-of select="@name"/>Struct payload;
//} <xsl:value-of select="@name"/>CarrierStruct, *<xsl:value-of select="@name"/>CarrierStruct_ptr; 
<xsl:value-of select="@name"/>Struct_ptr <xsl:value-of select="@name"/>; -->
</xsl:for-each>

<xsl:for-each select="/xip/datatypes/datatype" >
// &lt;block name="datatype_<xsl:value-of select="@name"/>"&gt;

static IOField <xsl:value-of select="@name"/>FieldList[] = {
  <xsl:for-each select="arg">
  <xsl:variable name="this" select="."/>
  {"<xsl:value-of select="@name"/>", "<xsl:value-of select="$maptables/mapTable[@to='ECho']/mapping[@from=$this/@type]/@to"/> ",sizeof(<xsl:value-of select="$maptables/mapTable[@to='C']/mapping[@from=$this/@type]/@to"/>), IOOffset( <xsl:value-of select="../@name"/>Struct*, <xsl:value-of select="@name"/>)},
  </xsl:for-each>
  <xsl:for-each select="arrayArg">
  <xsl:variable name="this" select="."/>
  {"<xsl:value-of select="@name"/>", "<xsl:value-of select="$maptables/mapTable[@to='ECho']/mapping[@from=$this/@type]/@to"/>[<xsl:value-of select="@size"/>]",sizeof(<xsl:value-of select="$maptables/mapTable[@to='C']/mapping[@from=$this/@type]/@to"/>), IOOffset( <xsl:value-of select="../@name"/>Struct*, <xsl:value-of select="@name"/>)},
  </xsl:for-each>
  {NULL, NULL, 0, 0}
};

static CMFormatRec <xsl:value-of select="@name"/>FormatList[] = 
{
  {"<xsl:value-of select="@name"/>Struct",<xsl:value-of select="@name"/>FieldList},
  { NULL, NULL}
};

<!--
static IOField <xsl:value-of select="@name"/>CarrierFieldList[] = {
  {"payload","<xsl:value-of select="@name"/>Struct", sizeof(<xsl:value-of select="@name"/>Struct), IOOffset( <xsl:value-of select="@name"/>CarrierStruct*, payload) },
  {NULL, NULL, 0, 0}
};
-->
// &lt;/block&gt;
</xsl:for-each> <!-- data types -->


int makePathDir(char *path, mode_t mode)
{
  char *dir; 
  char *p;
  mode = 0777;

  dir = (char *)malloc(sizeof(char) * (strlen(path) + 1));

  p = path;

  while(1)
  {
    p = strchr(p, '/');
    if (p == NULL) 
    {
      free( dir );
      return 0;
    }

    p++;
    // get first dir name including '/'
    memcpy(dir, path, p - path);
    dir[p - path] = 0;

    // make directory
    if (mkdir(dir, mode))
    {
      // if dir exists already, continue
      // otherwise(error occurs), return -1
      if (errno != EEXIST)
      {
        free(dir);
        return -1;
      }
    }
  }

  // cannot reach here
  return 0;
}

// publish
static int publish(char *pipe, char *connType, char *connName, char *str, int type)
{
  if (type &amp; PUBLISH_FILE)
  {
    publishFile(pipe, connType, connName, str);
  }

  return 0;
}

static int publishFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ofile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc( sizeof(char) * (strlen(publishRoot) + strlen(pipe) 
                                             + strlen(connType) + strlen(connName)
                                             + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  makePathDir(filename,0);

  printf("publishing file = %s\n",  filename);

  ofile = fopen(filename,"w");
  if (ofile)
  {
    fprintf(ofile,"%s\n",str);
  }
  else
    printf("Error in publishing file %s\n", filename);
 
  // close and write info
  fclose(ofile);
  return 0;
}


static int unpublish(char *pipe, char *connType, char *connName, int type)
{
  if (type &amp; PUBLISH_FILE)
  {
    unpublishFile(pipe, connType, connName);
  }

  return 0;
}

static int unpublishFile(char *pipe, char *connType, char *connName)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *) malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                             strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  return remove(filename);
}


static char *lookup(char *pipe, char *connType, char *connName, int type, char *str)
{
  if (type &amp; PUBLISH_FILE)
  {
    return lookupFile(pipe, connType, connName, str);
  }

}


static char *lookupFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ifile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                  		      strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  ifile = fopen(filename, "r");
  if (ifile &amp;&amp; !feof(ifile))
  {
    fgets(str,512,ifile);
  }
  else
    fprintf(stderr,"Error in looking up file %s\n",filename); 

  return str;
}



<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
<xsl:variable name="portName" select="@name"/>
<nodePipeInConnect mark="true">
void <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Handler( void *, void *, attr_list );
</nodePipeInConnect>
</xsl:for-each>
// variables to track
<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
// channel vars for sending info 
static ECSourceHandle <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendHandle;
static EChannel <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel;
//<xsl:value-of select="@type"/><dataTypeName/>CarrierStruct <xsl:value-of select="$portName"/><connName/>_outgoingData;
</nodePipeOutConnect>
</xsl:for-each>

<!--
// testvalues
// <xsl:for-each select="/xip/pipes/pipe/inport">
//   Name: <xsl:value-of select="$thisPipeName"/>
//   Class: <xsl:value-of select="$thisPipeClass"/>
//   <xsl:value-of select="@name"/>
// </xsl:for-each>
// testvalues
-->

extern IOContext ioc;
static int doPipeInit(void)
{
  <xsl:if test="not(/xip/pipes/pipe[@name=$thisPipeClass]/inport)">
  int forked;
  </xsl:if>
/*
  IOFormat format;
<!--  <xsl:for-each select="/xips/datatypes/datatype">
  <dataDef>
//    format = avs_register_IOcontext_format( "<xsl:value-of select="@name"/><dataTypeName/>", <xsl:value-of select="@name"/><dataTypeName/>FieldList, ioc );
//    if( !format )
//    {
//      fprintf(stderr,"Error registering format <xsl:value-of select="@name"/><dataTypeName/>.\n");
//      return -1;
//    }
  </dataDef>
  </xsl:for-each> -->
*/
  gen_pthread_init();
  cm = CManager_create();
  <xsl:if test="not(/xip/pipes/pipe[@name=$thisPipeClass]/inport)">
  forked = CMfork_comm_thread(cm);
  if( !forked ) {
    fprintf(stderr, "Fork of comm thread failed. Exiting.\n");
    exit(1);
  }
  </xsl:if>

  ec = ECho_CM_init(cm);
  return 0;
}

<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
<xsl:variable name="portName" select="@name"/>
<nodePipeInConnect mark="true">
// incoming channel vars for receive
static ECSinkHandle <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>ReceiveHandle;
static EChannel <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>ReceiveChannel;
</nodePipeInConnect>
</xsl:for-each>


<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
  <xsl:variable name="portName" select="@name"/>
  // Filtering code for pipe "<xsl:value-of select="$thisPipeName"/>" port "<xsl:value-of select="$portName"/>"
  <xsl:call-template name="filterCode">
      <xsl:with-param name="thisPipeName" select="$thisPipeName"/>
      <xsl:with-param name="portName" select="$portName"/></xsl:call-template>
  // end Filtering code
</xsl:for-each>
<!--
<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
//int <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit( <dataTypeName/>CarrierStruct * );
//int <xsl:value-of select="$thisPipeName"/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit(  );
//int <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit( <xsl:value-of select="@type"/>Struct_ptr  );
</nodePipeOutConnect>
</xsl:for-each>
-->

<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
<xsl:variable name="portName" select="@name"/>
<nodePipeInConnect mark="true">
void <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Handler( void *, void *, attr_list);
</nodePipeInConnect>
</xsl:for-each>

int firstTime = 1;
/* start this sub-pipe */
void infopipeStart_<xsl:value-of select="@name"/>(  )
{
  char lookupStr[512];

  if( firstTime )
  {
    firstTime = 0;
    doPipeInit();
  }


<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
  /////////////////////////////////
  // OUTGOING 
  // create the outgoing channel
  <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel = EChannel_typed_create( ec, <xsl:value-of select="@type"/><dataTypeName/>FieldList, <xsl:value-of select="@type"/><dataTypeName/>FormatList ); 
  if(<xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel == NULL )
    fprintf(stderr, "Error creating <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel\n");
  publish( "<xsl:value-of select="$thisPipeName"/><pipeName/>", "<connType/>","<xsl:value-of select="$portName"/><connName/>", ECglobal_id(<xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel), PUBLISH_FILE );
  //<xsl:value-of select="$thisPipeName"/><pipeName/> = EChannel_open( ec, ECglobal_id(<xsl:value-of select="$thisPipeName"/><pipeName/>SendChannel));
  // subscribe as a source 
  <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendHandle = ECsource_typed_subscribe( <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel, <xsl:value-of select="@type"/><dataTypeName/>FieldList, <xsl:value-of select="@type"/><dataTypeName/>FormatList);
</nodePipeOutConnect>
</xsl:for-each>

<xsl:variable name="frompipe" select="../../connections/connection/from/@pipe"/>
<xsl:variable name="fromport" select="../../connections/connection/from/@port"/>
<!--
-->
<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
<xsl:variable name="portName" select="@name"/>
<nodePipeInConnect mark="true">
  /////////////////////////////////
  // INCOMING LINK -- open channel
  incomingGlobalID = lookup("<xsl:value-of select="$frompipe"/>","","<xsl:value-of select="$fromport"/><connName/>", PUBLISH_FILE, lookupStr);
  <!-- 
  -->
  <xsl:value-of select="$thisPipeName"/>_<connType/>_<xsl:value-of select="$portName"/>ReceiveChannel = EChannel_open( ec,  incomingGlobalID);

  printf("lookup string = %s\n", lookupStr);
  // subscribe as a sink on all incoming channels
  ECsink_typed_subscribe( <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>ReceiveChannel, <xsl:value-of select="@type"/><dataTypeName/>FieldList, NULL, <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Handler, NULL);
</nodePipeInConnect>
</xsl:for-each>

  //ec = ECho_CM_init();
  <xsl:if test="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
  CMrun_network(cm);
  </xsl:if>
}

// shuts down the infopipe system and cleans out directory entries
void infopipeStop_<xsl:value-of select="@name"/>( )
{

// shutdown our outgoing pipes - the incoming pipes will
// be shutdown by the nodes upstream from us
<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
  EChannel_destroy( <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel );
</nodePipeOutConnect>
</xsl:for-each>

<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
  // unpublish our server info
  <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel = EChannel_typed_create( ec, <xsl:value-of select="@type"/><dataTypeName/>FieldList, NULL ); 
  if(<xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel == NULL )
    fprintf(stderr,"Error creating <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendChannel\n");
  unpublish( "<xsl:value-of select="$thisPipeName"/><pipeName/>", "<connType/>","<xsl:value-of select="$portName"/><connName/>", PUBLISH_FILE );
</nodePipeOutConnect>
</xsl:for-each>

}

<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/inport">
<xsl:variable name="portName" select="@name"/>
<nodePipeInConnect mark="true">

extern void update_handler( void *, void *, attr_list );

/////////////////////////////////////////////////////////////////////////////
// handler function for an incoming pipe 
// This handler function accepts data coming from the pipe. It runs
// once for each set of data that comes into this machine from
// the pipe/channel.
void <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Handler( void *event, void *client_data, attr_list as)
{
  int li;
  int lnlr;
  // Typecast incoming data to workable type
  //<xsl:value-of select="@type"/><dataTypeName/>CarrierStruct *incomingData = (<xsl:value-of select="@type"/><dataTypeName/>CarrierStruct *)event;
  <xsl:value-of select="@type"/><dataTypeName/>Struct_ptr incomingData = (<xsl:value-of select="@type"/><dataTypeName/>Struct *)event;

  <aspect name="latency">  //////////////
  // ASPECT
  // when receiving a packet, we need to process some QoS data
  qosReceiveEvent( &amp;(incomingData->qosInfo) );
  //////////////
  // copy QoS info
  lnlr = incomingData->qosInfo.numLatencyRecords;
  <xsl:value-of select="$portName"/><connName/>_outgoingData.qosInfo.latencyRecords = (long *)malloc( sizeof(long) * ( lnlr + 1));
  <xsl:value-of select="$portName"/><connName/>_outgoingData.qosInfo.numLatencyRecords = lnlr + 1;
  for( li = 0; li &lt; lnlr - 1; li++ )
    <xsl:value-of select="$portName"/><connName/>_outgoingData.qosInfo.latencyRecords[li] = incomingData->qosInfo.latencyRecords[li];
  <aspect name="debugLatency">
  // print out observed latencies
  long totalLat = 0;
  for( li = 0; li &lt; lnlr; li++ )
    printf("%5i",li);
  printf(" |   Total\n");
  for( li = 0; li &lt; lnlr; li++ )
    printf("-----");
  printf("----------\n");
  for( li = 0; li &lt; lnlr; li++ )
    {
    printf("%5ld",incomingData->qosInfo.latencyRecords[li]);
    totalLat += incomingData->qosInfo.latencyRecords[li];
    }
  printf(" | %7ld\n", totalLat);
  </aspect>
  </aspect>

  
  // USER: Add code here to operate on incoming data
  // memory pointed to by incomingData (and event) may be reclaimed by calling 
  update_handler(event, client_data, as );

}
</nodePipeInConnect>
</xsl:for-each>

<xsl:for-each select="/xip/pipes/pipe[@name=$thisPipeClass]/outport">
<xsl:variable name="portName" select="@name"/>
<nodePipeOutConnect mark="true">
/////////////////////////////////////////////////////////////////////////////
// functions for submitting data to net
<!--
//int <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit( <xsl:value-of select="@type"/><dataTypeName/>CarrierStruct *outgoingData )
//int <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit(  )
-->
int <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>Submit( <xsl:value-of select="@type"/><dataTypeName/>Struct_ptr outgoingData )
{

  ECsubmit_typed_event( <xsl:value-of select="$thisPipeName"/><pipeName/>_<connType/>_<xsl:value-of select="$portName"/><connName/>SendHandle, outgoingData );
  return 0;
}
</nodePipeOutConnect>
</xsl:for-each>
// ENDING OF IMP FILE FOR A NODE

int test_main( int argc, char *argv )
{
  //infopipeEnvironStartup();
  // start the pipe
  infopipeStart_<xsl:value-of select="@name"/>();
  
  while( 1 )
    sleep( 1 );

  // close down the pipes
  infopipeStop_<xsl:value-of select="@name"/>();

  // normal termination
  return 0;
}

// &lt;/block&gt;
</content>
</file>

</xsl:for-each>
</xsl:template>
</xsl:stylesheet>
