<?xml version="1.0" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:template name="javaSocketClasses">
<file><name>InfopipeSocketServerStub.java</name>
<content>
import java.net.*;
import java.util.*;
import java.io.*;

public class InfopipeSocketServerStub extends Thread {
	private ServerSocket serverSocket = null;
	private Socket incomingSocket = null;

	private BufferedReader socketIn = null;
	private PrintWriter socketOut = null;
	private String recvBuf = null;

	private String stubName = null;

	public InfopipeSocketServerStub(int portNum, String stubName) {
		
		try {
			serverSocket = new ServerSocket(portNum); 
			this.stubName = stubName;
			
		} catch (IOException e) {
			if (e.getMessage().equals("Address already in use")) {
				System.out.println(stubName + ": Bind error!");
				System.exit(-1);
			}
		}

		recvBuf = new String();

		// Start thread 
		this.start();
	}

	public synchronized void run() {
		
		try {
			incomingSocket = serverSocket.accept();

			socketIn = new BufferedReader(new InputStreamReader(
						incomingSocket.getInputStream()));
			socketOut = new PrintWriter(incomingSocket.getOutputStream()); 

			System.out.println(stubName + ": Connected!");

			/* awake wait() in the getIn() */
			notifyAll();

		} catch (IOException e) {
			System.err.println(e);
			System.exit(-1);
		}
	}

	public Reader getIn() {
		return getIn("infopipeDataFormat");
	}

	public synchronized Reader getIn(String rootName) {

		/* wait until connected */
		try {
			while(socketIn == null) {
				try {
					wait();
				} catch (InterruptedException e) {
				}
			}

		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}


		/* read data from socket */
		int bufSize = 4096;
		int total_rb = 0;
		int rb = 0;
		char buf[] = new char[bufSize];
		String recvData = null;

		try {
			do {
				rb = socketIn.read(buf, 0, bufSize);

				recvBuf = recvBuf + new String(buf, 0, rb);

				String compareStr = "&lt;/" + rootName + "&gt;";
				int index = recvBuf.indexOf(compareStr);
				if (index &lt; 0) { continue; }

				recvData = recvBuf.substring(0, index + compareStr.length());
				//System.out.println("rb = " + recvData.length() + " recv: " + recvData);
				recvBuf = recvBuf.substring(index + compareStr.length());
				break;

			} while (true);
		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}
		
		return new StringReader(recvData);
	}

	public synchronized Writer getOut() {

		/* wait until connected */
		try {
			if(socketOut == null) {
				try {
					wait();
				} catch (InterruptedException e) {
				}
			}

		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}

		return socketOut;
	}
}
</content>
</file>
<file><name>InfopipeSocketClientStub.java</name>
<content>
import java.net.*;
import java.util.*;
import java.io.*;

public class InfopipeSocketClientStub extends Thread {
	private Socket clientSocket = null;
	private String destAddr = null;
	private int portNum = 0;

	private BufferedReader socketIn = null;
	private PrintWriter socketOut = null;
	private String recvBuf = null;

	private String stubName = null;

	public InfopipeSocketClientStub(String destAddr, int portNum, String stubName) {
		
		this.destAddr = destAddr;
		this.portNum = portNum;
	
		this.stubName = stubName;

		// Start thread 
		this.start();
	}

	public synchronized void run() {

		while(clientSocket == null) {
			try {
				clientSocket = new Socket(destAddr, portNum); 
				socketOut = new PrintWriter(clientSocket.getOutputStream());

				System.out.println(stubName + ": Connected!");

				/* awake wait() in the getOut(), if someone waits */
				notifyAll();

				break;
			} catch (IOException e) {
				//System.out.print(stubName + ": " + e.getMessage());
				//System.out.println(" - retrying...");
			}

			try {
				/* retry in 3 seconds */
				Thread.sleep(3*1000);
			} catch (Exception e) {
				System.err.println(stubName + ": " + e.getMessage());
				e.printStackTrace();
				System.exit(-1);
			}
		}
	} 

	public Reader getIn() {
		return getIn("infopipeDataFormat");
	}

	public synchronized Reader getIn(String rootName) {

		/* wait until connected */
		try {
			while(socketIn == null) {
				try {
					wait();
				} catch (InterruptedException e) {
				}
			}

		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}


		/* read data from socket */
		int bufSize = 4096;
		int total_rb = 0;
		int rb = 0;
		char buf[] = new char[bufSize];
		String recvData = null;

		try {
			do {
				rb = socketIn.read(buf, 0, bufSize);

				recvBuf = recvBuf + new String(buf, 0, rb);

				String compareStr = "&lt;/" + rootName + "&gt;";
				int index = recvBuf.indexOf(compareStr);
				if (index &lt; 0) { continue; }

				recvData = recvBuf.substring(0, index + compareStr.length());
				//System.out.println("rb = " + recvData.length() + " recv: " + recvData);
				recvBuf = recvBuf.substring(index + compareStr.length());
				break;

			} while (true);
		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}
		
		return new StringReader(recvData);
	}

	public synchronized Writer getOut() {

		/* wait until connected */
		try {
			if(socketOut == null) {
				try {
					wait();
				} catch (InterruptedException e) {
				}
			}

		} catch (Exception e) {
			System.err.println(e);
			System.exit(-1);
		}

		return socketOut;
	}
}
</content>
</file>
</xsl:template>
</xsl:stylesheet>

