/* -*-C-*-
   @author: Yan, Wenchang
*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>      
#include <getopt.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <signal.h>
/*for tprintf */
#include <sys/timeb.h>
#include <stdarg.h>
//#include "display.h"

extern int showPPM_wrapper(char* ppmFrame, int width, int height, int bytesPerPixel,
              unsigned long PPMFrameNum,
              char magic1, char magic2);



#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 1
#define DEBUG_WHAT 1



int verbose = 1;
int ctrlced = 0;/*^C*/


//The characteristics of each PPM frame.
//This infomation is needed both at the PPM frame
//Sender and Receiver side.
typedef struct {
  int tag;          //Reserved
  char ppm1;        //'P'
  char ppm2;        //'1'-'6'
  int size;         //width*height*pixel_size_in_bytes
  int width;
  int height;
  char* buff;       //Reserved
} PPM_frame_info, *PPM_frame_info_ptr;


/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};


/* 
 * PPM stuff
 *
 */

typedef unsigned char pixval;



#define IMAGE_SIZE  921600 /* 3*640*480 */

/*
#define MUS 57600 
*//* bytes in each UDP packet sent */
#define MUS 15360
#define MAXPIXELSIZE 3
#define MAXLINESIZE 640

#define PPM_OVERALLMAXVAL 65535
#define PPM_MAXMAXVAL 255

#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
#define PPM_FORMAT (PPM_MAGIC1 * 256 + PPM_MAGIC2)
#define RPPM_FORMAT (PPM_MAGIC1 * 256 + RPPM_MAGIC2)
#define PPM_TYPE PPM_FORMAT

#define PPM_ASSIGN(p,red,grn,blu) \
   do { (p).r = (red); (p).g = (grn); (p).b = (blu); } while ( 0 )

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
 
void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] -n num-frames [-M MTU] [-P maxpixel] [-L maxlinesize] [-f format] [-H receiver host] [-p portno]\n"
	          "\t -v --verbose            \t set verbose mode\n"
	          "\t -h --help               \t give this help\n"
	          "\t -n --nframes num-frames \t number of images. Please use 40\n"
                  "\t -M --MTU value          \t the approx. package size\n"
                  "\t -P --Pixel maxpixel     \t 3 is default\n"
                  "\t -L --Line maxlinesize   \t 640 pixels is default\n"
	          "\t -f --format  format     \t RGB format (dflt 2, RGB)\n" 
         	  "\t -H --Host               \t the receiver host name\n"
	          "\t -p --port port_no       \t receiver port number\n",
	  "reader");
  exit(status);
} /* end usage */

void ctrlc(int signal)
{
    static char text[] = "^C - one moment please\n";
    fprintf(stdout,text,strlen(text));
    ctrlced=1;
}

int
main (int argc, char* argv[])
{
  char snapshotFile[20];/* 640*480 color file 1-30
			   320*240 color file 31-40*/
  char duplicateFile[20];

  int shotfd,duplicate; 
  char header[20];//header[20] for 640x480 sample, header[15] for 352x240
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;
  char* image;
  char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;

  int optc;

  int mus=MUS;
  int maxpixelsize=MAXPIXELSIZE;
  int maxlinesize=MAXLINESIZE;

  int sendresult,readresult;
  char* buf;
  int format = 2;
  int header_idx;
  int actual_size;/*actual size of the packet sent each time*/

  int ii;
  //uint32 icopy;
  /* For PPM frame information*/
  PPM_frame_info pfi;
  int line_size,num_lines, pixel_size, lines_send_per_time, num_times;
  int switch_file=60;/* now have only 30 640x480 color PPM files,
			10 640x480 grey PPM files,
			10 352x240 color PPM files,
		        10 352x240 grey PPM files*/
  int partial_name;
  int DEBUG;

  while ((optc = getopt_long(argc, argv, "+vhn:M:P:L:f:", 
			     longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = 1;
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break; 
    case 'M':
      mus = atoi(optarg);
      break;
    case 'P':
      maxpixelsize = atoi(optarg);
      break;
    case 'L':
      maxlinesize = atoi(optarg);
      break;

    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }

  if (numFrames < 1) usage(stderr, 1);

  /* no extra args */
  if (optind > argc) usage(stderr, 1);

  if (!(buf = (char*)malloc(IMAGE_SIZE*sizeof(char))))
  {
    printf(" [imagereader::main(): 0ut of mem\n");
    exit(-1);
  }

  signal(SIGINT,ctrlc);

  DEBUG=1;//1 == forever

  ///////////////////////////////////////////////////

  //VideoSink_X11 displayer;

  do {
    for (i = 0; i < numFrames && !ctrlced; i++) {
      //sleep(2);
      time(&nowt);
      now = localtime(&nowt);
      
      /*choose between 2 kinds of PPM files*/
      partial_name = i % switch_file ;

      if (partial_name <30 && partial_name >5) {
	printf(" partial_name: %d, continue\n", partial_name);
        continue;
      }
	
      if (0<=partial_name && partial_name<30) {//640x480 color
	line_size = 640;
	num_lines = 480;
	pixel_size = 3;/* 3 bytes */
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/sample%d.ppm",partial_name+1);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	line_size = 352;
	num_lines = 240;
	pixel_size = 3;
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/img0000%d.ppm",partial_name-30);
      }
      /*
      else if (40<=partial_name && partial_name<50) {//640x480 grey
	line_size = 640;
	num_lines = 480;
	pixel_size = 1;
	sprintf(snapshotFile,"../My/grey%d.ppm",partial_name-39);
      }
      else if (50<=partial_name && partial_name<60) {//352x240 grey
	line_size = 352;
	num_lines = 240;
	pixel_size = 1;
	sprintf(snapshotFile,"../My/imgGrey%d.ppm",partial_name-50);
      }
      */
      if ((shotfd = open(snapshotFile, O_RDONLY )) < 0) {
	perror("Problems opening snapshot file");
	exit(-1);
      }
      
      pfi.tag = i;//reserved for future extension
      pfi.ppm1= 'P';
      pfi.ppm2=pixel_size < 3?'5':'6';
      pfi.size=line_size*num_lines*pixel_size;
      pfi.width=line_size;
      pfi.height=num_lines;
      pfi.buff=NULL;/* not used for udp between dvdview client/server*/
 
      j = 0;

      /* send the image */
      /* Forget the first 20 bytes */
      if (0<=partial_name && partial_name<30) {//640x480 color
	read(shotfd,buf,15);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	read(shotfd,buf,15);/*img0000*.ppm has 15 bytes as header*/
      }
      /*
      else if (40<=partial_name && partial_name<50) {//640x480 grey
	read(shotfd,buf,60);
      }
      else if (50<=partial_name && partial_name<60) {//352x240 grey
	read(shotfd,buf,60);
      }
      */

      readresult=read(shotfd,buf,pfi.size);   

      if (readresult<0) {
	printf(" [reader::main(): read error \n");
	exit(-1);
      }
      
      printf(" %d bytes read from file\n",readresult);

      showPPM_wrapper(buf, pfi.width, pfi.height, pfi.size/pfi.width/pfi.height,
			pfi.tag, 
			pfi.ppm1, pfi.ppm2);
      
      if (DEBUG_WHAT)
	printf("   The %dth file\n",i);

      
      j += sendresult;
      usleep(1000000);
    }

    if (DEBUG_WHERE)
      printf(" [reader::main(): %d bytes sent \n",j);
    
    close(shotfd);
  } while (DEBUG && !ctrlced);


  if (buf) free(buf);
  //close(sd);  

  return 0;
} /* end main */
