/* -*-C-*-
   @author: Yan, Wenchang
   @time: Dec, 2001
*/

#include <src/config.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>      
#include <getopt.h>
#include <vdev.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include "connect.h"


#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 0
#define DEBUG_WHAT 0

/*for tprintf */
#include <sys/timeb.h>
#include <stdarg.h>

typedef enum {FALSE, TRUE} boolean;

static char* local_progname;
/*static boolean verbose = FALSE;*/
static boolean verbose = TRUE;

/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};

/* for debug */
extern int tprintf(const char* fmt, ...);


/* 
 * PPM stuff
 *
 */

typedef unsigned char pixval;

typedef struct {
  pixval r, g, b;
} pixel;


#define RECEIVER_HOST "localhost"
#define SENDER_HOST "localhost"
#define RECEIVER_PORT 32333
#define IMAGE_SIZE  921600 /* 3*640*480 */
#define MUS 57600 /* bytes in each UDP packet sent */

#define PPM_OVERALLMAXVAL 65535
#define PPM_MAXMAXVAL 255

#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
#define PPM_FORMAT (PPM_MAGIC1 * 256 + PPM_MAGIC2)
#define RPPM_FORMAT (PPM_MAGIC1 * 256 + RPPM_MAGIC2)
#define PPM_TYPE PPM_FORMAT

#define PPM_ASSIGN(p,red,grn,blu) \
   do { (p).r = (red); (p).g = (grn); (p).b = (blu); } while ( 0 )


/*
 * tprintf
 */

int tprintf(const char *fmt, ...) {
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
 
static void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] -n num-frames [-f format] [-p portno]\n"
	          "\t -v --verbose            \t set verbose mode\n"
	          "\t -h --help               \t give this help\n"
	          "\t -n --nframes num-frames \t number of images\n"
	          "\t -f --format  format     \t RGB format (dflt 2, RGB)\n" 
	          "\t -p --port port_no       \t port number\n",
	  local_progname);
  exit(status);
} /* end usage */



int
main (int argc, char* argv[])
{
  VDev_T vdev;
  char snapshotFile[40];
  int shotfd; 
  char header[20];
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;
  pixel *pixrow, *pP;
  u_char* image;
  u_char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;
  int format = RBG;
  struct VDev_imageSize size;
  int optc;

  struct hostent* h;
  struct sockaddr_in receiver;
  int sd;/*socket descriptor */
  int sendresult;
  int portno = RECEIVER_PORT;
  u_char* buf;

  local_progname = argv[0];
  while ((optc = getopt_long(argc, argv, "+vhn:f:p:", 
			     longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = TRUE;
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break; 
    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    case 'p':
      portno = atoi(optarg);
      if (portno<0) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }

  h = gethostbyname(RECEIVER_HOST);
  if(h==NULL) {
    printf(" [imagesender::main(): unknown host '%s' \n", RECEIVER_HOST);
    exit(1);
  }

  /*  printf("%s: sending data to '%s' (IP : %s) \n", argv[0], h->h_name,
         inet_ntoa(*(struct in_addr *)h->h_addr_list[0]));
  */

  receiver.sin_family = h->h_addrtype;
  memcpy((char *) &receiver.sin_addr.s_addr, 
         h->h_addr_list[0], h->h_length);
  receiver.sin_port = htons(portno);

  /*set_server(&receiver,h->h_addr_list[0],portno);*/

  
  if (numFrames < 1) usage(stderr, 1);

  /* no extra args */
  if (optind > argc) usage(stderr, 1);

  VDev_init();
  if (verbose) tprintf(" [imagesender::main(): Num devices: %d\n", VDev_numVDevices());

  vdev = VDev_getVDevice(0);
  if (vdev == NULL) fprintf(stderr, "Wrong device?\n");
  if (verbose) {
    printf("Valid? %c\n", VDev_isValid(vdev) ? 'T':'F');
    printf("V4L name:    %s\n", VDev_getIntfName(vdev));
    printf("Device name: %s\n", VDev_getNodeName(vdev));
    VDev_getMaxSize(vdev, &size);
    printf("Max size (width, height): (%d, %d)\n", size.width, size.height);
    VDev_getMinSize(vdev, &size);
    printf("Min size (width, height): (%d, %d)\n", size.width, size.height);  
  }

  VDev_getSize(vdev, &size);
  if (verbose) printf("Size (width, heigth): (%d, %d)\n", size.width, size.height);
  
  VDev_open(vdev, 1);
  if (verbose) {
    printf("File descriptor: %d\n", VDev_getDescriptor(vdev));  
    printf("Frame rate: %d\n", VDev_getFramerate(vdev));
    printf("Brightness: %u\n", VDev_getBrightness(vdev));
    printf("Contrast: %u\n", VDev_getContrast(vdev));
    printf("Hue: %u\n", VDev_getHue(vdev));
    printf("Color: %u\n", VDev_getColor(vdev));
    printf("Whiteness: %u\n", VDev_getWhiteness(vdev));
  }

  /* Set up the udp connection */
  sd = connect_udp(SENDER_HOST,0);
  if (sd<0) {
    fprintf(stderr, "[imagesender::main(): can't create socket\n");
    exit(-1);
  }
  
  for (i = 0; i < numFrames; i++) {
    /*
    tprintf(" [imagesender::main(): Ready for another picture? [ENTER]\n");
    getchar();  
    */
    time(&nowt);
    now = localtime(&nowt);

    /* Raw image file */
    sprintf(snapshotFile, "snapshot-raw%d-%d-%d-%d", 
	    i+1, now->tm_hour, now->tm_min, now->tm_sec);
    if ((shotfd = open(snapshotFile, O_CREAT|O_WRONLY, 
		       S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
      perror("Problems opening raw snapshot file");
      exit(1);
    }
    image = VDev_getImage(vdev);
    rawsize = VDev_getRawSize(vdev);
    printf ("RAWSIZE %d\n", rawsize);

    write(shotfd, image, rawsize);
    close(shotfd);

    /* PPM image file locally derived from our raw image */
    sprintf(snapshotFile, "snapshot-derived%d-%d-%d-%d.ppm", 
	    i+1, now->tm_hour, now->tm_min, now->tm_sec);

    if ((shotfd = open(snapshotFile, O_CREAT|O_WRONLY, 
		       S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
      perror("Problems opening PPM-derived snapshot file");
      exit(1);
    }

    sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,  
	    size.width, size.height, 255);  
    write(shotfd, header, sizeof(header));
    
    if (!(pixrow = (pixel*)malloc(size.width * sizeof(pixel)))) {
      perror("Out of memory");
      exit(1);
    }
    
    if (DEBUG_WHERE)
      tprintf(" [imagesender::main(): pixrow size = %d\n",size.width*sizeof(pixel));

    idx = 0;
    for ( row = 0; row < size.height; ++row) {
      for ( col = 0, pP = pixrow; col < size.width; ++col, ++pP ) {
	val1 = image[idx+0];
	val2 = image[idx+1];
	val3 = image[idx+2];
	
	/* PPM_ASSIGN( *pP, val1, val2, val3 ); RGB */
	PPM_ASSIGN( *pP, val1, val3, val2 ); /* RBG */
	/* PPM_ASSIGN( *pP, val2, val1, val3 ); GRB */
	/* PPM_ASSIGN( *pP, val2, val3, val1 ); BRG */
	/* PPM_ASSIGN( *pP, val3, val1, val2 ); GBR */
	/* PPM_ASSIGN( *pP, val3, val2, val1 ); BGR */
	
	idx += 3;
      }


      for (j = 0, pP = pixrow; j < size.width; ++j, ++pP) {
	sprintf(pixelline, "%c%c%c", pP->r, pP->g, pP->b);
	write(shotfd, pixelline, 3);
      }
    }
      /* allocate mem */
      if (!(buf = (u_char*)malloc(MUS))) {
        tprintf(" [imagereceiver::main(): 0ut of mem\n");
        exit(-1);
      }
      
      j=0;
      read(shotfd,header,sizeof(header));/* skip the header*/

      if (DEBUG_WHERE)
	tprintf(" [imagesender::main(): before sent\n");
      /* send the image */
      for (row=0;row<16;row++) {
       sleep(1);
       read(shotfd,buf,MUS);
       sendresult = sendto(sd, buf, MUS, 0,
			(struct sockaddr*)&receiver,sizeof(receiver));
      if (sendresult<0) {
	tprintf(" [imagesender::main(): cannot send \n");
	exit(-1);
      }
      j += rawsize/16;
    }

      if (DEBUG_WHERE)
	tprintf(" [imagesender::main(): %d bytes sent to %s:%d\n",j,
		inet_ntoa(receiver.sin_addr),ntohs(receiver.sin_port));



    close(shotfd);


    /* PPM image file as given by VDev */
    /*
    sprintf(snapshotFile, "snapshot%d-%d-%d-%d.ppm", 
	    i+1, now->tm_hour, now->tm_min, now->tm_sec);
    
    if ((shotfd = open(snapshotFile, O_CREAT|O_WRONLY, 
		       S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
      perror("Problems opening PPM snapshot file");
      exit(1);
    }
    
    sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,  
	    size.width, size.height, 255);  
    write(shotfd, header, sizeof(header));
    image = VDev_getPPMImage(vdev, format);
    write(shotfd, image, rawsize);
    close(shotfd);
    */

  }


  close(sd);  

  VDev_close(vdev);

  VDev_finalize();
  
  return 0;
} /* end main */
