
#include "output/accumulator.hh"
#include <string.h>
#include <algorithm>
using namespace std;

extern "C" int tprintf(const char* fmt, ...);


#define DEBUGINFO 0

/*vvv*/
#undef DEBUG_WHERE
#define DEBUG_WHERE 0
/*^^^*/

#if DEBUGINFO

void ShowDIMG(DecodedImageData* dimg)
{
  cout << dimg->m_src_y_start << "-" << dimg->m_src_y_end << " to "
       << dimg->m_dst_y_start << "-" << dimg->m_dst_y_start+dimg->m_src_y_end-dimg->m_src_y_start
       << (dimg->m_may_modify ? " may modify" : " may not modify") << endl;
}
#endif


void ImageDataAccumulator::StartAccumulation(int first,int last,bool modify)
{
  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::StartAccumulation(int first,int last,bool modify)\n");

  // Round range up multiples of 4, so we don't have any problems with choma even in
  // field pictures.

  //first &= ~3;
  //last = (last+3) & ~3;   DOESN't WORK (watch chroma with "spaceneedle.mpg")

  // cout << "START ACCUMULATION " << first << " -> " << last << endl;

  d_line_available.CreateRange(first,last);
  bool* p = d_line_available.Data();
  for (int i=first;i<=last;i++)
    p[i]=false;

  d_modifyable_copy = modify;
}


DecodedImageData* ImageDataAccumulator::Accumulate(DecodedImageData* dimg)
{
  const int first = d_line_available.AskStartIdx();                      // first line to accumulate into
  const int last  = min(dimg->m_height-1,d_line_available.AskEndIdx());  // last line to accumulate into

  const int ys = dimg->m_dst_y_start;                        // first output line
  const int ye = ys + dimg->m_src_y_end-dimg->m_src_y_start; // last output line

#if DEBUGINFO
  cout << "--- Accumulate into: " << first << " to " << last << endl;
  ShowDIMG(dimg);
#endif

  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate(DecodedImageData* dimg)\n");
  if (DEBUG_WHERE) tprintf("           [first=%d, last=%d, ys=%d, ye=%d, m_src_y_end=%d, m_src_y_start=%d\n", first, last, ys, ye, dimg->m_src_y_end, dimg->m_src_y_start);


  /* If image area is completely out of the range used, it can simply be sent to
     the next postprocessor.
  */

  if (ye < first) { if (DEBUGINFO) cout << "show all (above)\n"; d_sink->ShowMBRows(dimg); return NULL; }
  if (ys > last)  { if (DEBUGINFO) cout << "show all (below)\n"; d_sink->ShowMBRows(dimg); return NULL; }

  /* If image area completely contains the used range and it may be modified or no
     write-access is required, simply return the area. */

  if (ys<=first && ye>=last && (!d_modifyable_copy || dimg->m_may_modify) && !dimg->m_field_lines)
    {
#if DEBUGINFO
      cout << "return complete region\n";
#endif
      return dimg;
    }


  /* Save values out of dimg as we may leave it unchanged at the end of our function. */

  const int old_src_y_start = dimg->m_src_y_start;
  const int old_src_y_end   = dimg->m_src_y_end;
  const int old_dst_y_start = dimg->m_dst_y_start;
  const int old_may_modify  = dimg->m_may_modify;


  /* Forward the areas that are not used to the next postprocessor. */

  if (ys < first)
    {
      // show lines above drawing region
  if (DEBUG_WHERE) tprintf("           [if (ys < first)\n");

      int before = first-ys;
      //cout << "lines before: " << before << endl;


      // show unmodified lines

      dimg->m_src_y_end = dimg->m_src_y_start+before-1;

      if (dimg->m_field_lines && (before&1)==0) dimg->m_src_y_end--;  // make displayed line range tight

      Assert(d_sink);
      d_sink->ShowMBRows(dimg);
#if DEBUGINFO
      cout << "show unmodified above: ";  ShowDIMG(dimg);
#endif

      // remove already shown lines

      if (dimg->m_field_lines && (before&1)==1) before++;  // obey parity of accumulated lines

      dimg->m_src_y_end = old_src_y_end;
      dimg->m_src_y_start += before;
      dimg->m_dst_y_start += before;
    }

  if (ye>last)
    {
  if (DEBUG_WHERE) tprintf("           [if (ys > last)\n");

      // show lines below drawing region

      int after = ye-last;
      //cout << "lines after: " << after << endl;

      dimg->m_src_y_start = dimg->m_src_y_end-after+1;
      dimg->m_dst_y_start = ye-after+1;

      if (dimg->m_field_lines && (after&1)==0)
	{
	  dimg->m_src_y_start++;
	  dimg->m_dst_y_start++;
	}

      Assert(d_sink);
      d_sink->ShowMBRows(dimg);
#if DEBUGINFO
      cout << "show unmodified below: ";  ShowDIMG(dimg);
#endif

      // remove already shown lines

      if (dimg->m_field_lines && (after&1)==1) after++;  // obey parity of accumulated lines

      dimg->m_src_y_start = old_src_y_start;
      dimg->m_dst_y_start = old_dst_y_start;
      dimg->m_src_y_end -= after;
    }



  ImageParam_YUV param;
  dimg->m_image.GetParam(param);

  if (d_width != param.width ||
      d_height < d_line_available.AskSize())
    {
      ImageSpec_YUV spec; ((ImageParam_YUV&)spec)=param;
      spec.height = d_line_available.AskSize();
      d_dimg.m_image.Create(spec);

      d_width  = spec.width;
      d_height = spec.height;

      d_dimg.m_width      =dimg->m_width;
      d_dimg.m_height     =dimg->m_height;
      d_dimg.m_src_y_start=0;
      d_dimg.m_src_y_end  =d_line_available.AskSize()-1;
      d_dimg.m_dst_y_start=d_line_available.AskStartIdx();
      d_dimg.m_field_lines=false;
      d_dimg.m_may_modify =true;
    }

  d_dimg.m_picdata1   =dimg->m_picdata1;
  d_dimg.m_picdata2   =dimg->m_picdata2;
  d_dimg.m_pichdr1    =dimg->m_pichdr1;
  d_dimg.m_pichdr2    =dimg->m_pichdr2;
  d_dimg.m_timing     =dimg->m_timing;

  int lineskip = (dimg->m_field_lines ? 2 : 1);
  int y0 = -dimg->m_src_y_start+dimg->m_dst_y_start-first;

  bool* line = d_line_available.Data();

#if DEBUGINFO
  cout << "Copy from " << dimg->m_src_y_start << " to " << dimg->m_src_y_end << endl;
  cout << "y0: " << y0 << endl;
#endif

  // accumulate luminance data (and 4:2:2 or 4:4:4 chrominance data)

  int chromawidth = param.GetChromaWidth();

  for (int y=0; ;y+=lineskip)
    {
      int src_y = dimg->m_src_y_start+y;
      int dst_y = dimg->m_dst_y_start+y;
      int dat_y = dimg->m_dst_y_start-first+y;

      if (src_y > dimg->m_src_y_end)
	break;

      // cout << "copy line " << y << " to " << y+y0 << endl;
      memcpy(d_dimg.m_image.AskFrameY()[dat_y],
	     dimg->m_image.AskFrameY()[src_y],d_width);
      line[dst_y]=true;

      if (param.chroma==Chroma422 || param.chroma==Chroma444)
	{
	  memcpy(d_dimg.m_image.AskFrameU()[dat_y],
		 dimg->m_image.AskFrameU()[src_y],chromawidth);
	  memcpy(d_dimg.m_image.AskFrameV()[dat_y],
		 dimg->m_image.AskFrameV()[src_y],chromawidth);
	}
    }

  // accumulate 4:2:0 chrominance data

  if (param.chroma==Chroma420)
    {
      int parity = (dimg->m_src_y_start&1);

      for (int y=0; ;y+=lineskip)
	{
	  int src_y,dst_y,dat_y;

	  if (dimg->m_field_lines)
	    {
	      src_y = ((dimg->m_src_y_start/2)&~1) +parity+y;
	      dst_y = ((dimg->m_dst_y_start/2)&~1) +parity+y;
	      dat_y = (((dimg->m_dst_y_start-first)/2)&~1) +parity+y;
	    }
	  else
	    {
	      src_y = dimg->m_src_y_start/2+y;
	      dst_y = dimg->m_dst_y_start/2+y;
	      dat_y = (dimg->m_dst_y_start-first)/2+y;
	    }

	  if (src_y > dimg->m_src_y_end/2)
	    break;

	  memcpy(d_dimg.m_image.AskFrameU()[dat_y],
		 dimg->m_image.AskFrameU()[src_y],d_width/2);
	  memcpy(d_dimg.m_image.AskFrameV()[dat_y],
		 dimg->m_image.AskFrameV()[src_y],d_width/2);
	}
    }


  // restore old values into dimg

  dimg->m_src_y_start = old_src_y_start;
  dimg->m_src_y_end   = old_src_y_end;
  dimg->m_dst_y_start = old_dst_y_start;
  dimg->m_may_modify  = old_may_modify;


  for (int i=last;i>=first;i--)
    if (line[i]==false)
      {
	if (DEBUGINFO) cout << "line " << i << " is still missing...\n";
	return NULL;
      }

  if (DEBUGINFO) cout << "image data range is complete..!\n";

  return &d_dimg;
}


/*vvv*/

void ImageDataAccumulator::StartAccumulation_PPM(RawImageData* img,int first,int last,bool modify,int d_start_line,int d_end_line,int i_start_line,bool field_line)
{
  if (DEBUG_WHERE) {
    tprintf(" [ImageDataAccumulator::StartAccumulation_PPM(): img.width=%d, img.height=%d, data_start_line=%d, data_end_line=%d, img_start_line=%d\n",img->width,img->height,d_start_line,d_end_line,i_start_line);
    tprintf(" [mus: %d, maxpixelline: %d, maxlinesize: %d\n",mus,maxpixelsize,
	    maxlinesize);
  }

  char filename[20];
  /*
  if (fileidx%2==0) {
    strcpy(filename,PPMfilename[0]);
  }
  else {
    strcpy(filename,PPMfilename[1]);
  }
  */

  /*
  sprintf(filename,"received-%d.ppm",fileidx);

  if ((PPMfile[fileidx%2] = open(filename,O_CREAT|O_WRONLY,S_IRWXU|S_IRGRP|
                      S_IROTH)) < 0 ) {
    if (DEBUG_WHERE) tprintf(" [VideoDecoder::Cstr(): file open error\n");
    exit(1);
  }
  */

  d_line_available.CreateRange(first,last);
  bool* p = d_line_available.Data();
  for (int i=first;i<=last;i++)
    p[i]=false;

  d_modifyable_copy = modify;

  //Hack for now
  //640*480 ppm file has 921600 bytes, ie, 921600/480=1920 bytes/line
  //=> 480/30 = 16 times, 30*1920 = 57600 bytes/time
  //line_to_accumulate = 30;
  line_accumulated = 0;

  data_start_line = d_start_line;
  data_end_line = d_end_line;
  img_start_line = i_start_line;
  field_lines = field_line;

  /* For debugging*/
  BytesReceived = 0;
  //char header[15];
  
  //sprintf(header, "%c%c\n%d %d\n%d\n", img->ppm1, img->ppm2,  
  //        img->width, img->height, 255/*grey and pixmap are same*/);  
  //write(PPMfile[fileidx%2], header, sizeof(header));
  
  /*For debugging*/
}

RawImageData* ImageDataAccumulator::Accumulate_PPM(RawImageData* dimg,bool flushOrNot)
{
  const int first = d_line_available.AskStartIdx();
  const int last  = min(dimg->m_height-1,d_line_available.AskEndIdx());

  const int ys = img_start_line;               // first output line
  const int ye = ys + data_end_line - data_start_line;//ys + dimg->m_src_y_end-dimg->m_src_y_start;//last output line

  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(DecodedImageData* dimg)\n");
  if (DEBUG_WHERE) tprintf("       [first=%d, last=%d, ys=%d, ye=%d, m_src_y_end=%d, m_src_y_start=%d\n", first, last, ys, ye, data_end_line, data_start_line);


  //int y0 = -dimg->m_src_y_start+dimg->m_dst_y_start-first;
  //int y0 = -data_start_line + img_start_line - first;

  bool* line = d_line_available.Data();

  int bytesPerPixel = dimg->size/dimg->width/dimg->height;

  //allocate mem for d_rdimg if it's the 1st time
  if (d_rdimg.image.IsEmpty()) {
    if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(): d_rdimg.image is EMPTY\n");
    d_rdimg.Create(dimg->m_width*bytesPerPixel,dimg->m_height);
  }

  //If frame # conflicts, flush the curr frame
  if (flushOrNot) {
    return &d_rdimg;
  }

  //May change each time, depends on MUS and img size ...
  int line_to_accumulate = min(data_end_line + 1,dimg->m_height-line_accumulated);
    
  for (int y=0; y<line_to_accumulate;y++)
    {
      memcpy((void*)d_rdimg.AskFrame_const()[y + ys],
	     dimg->AskFrame_const()[y],dimg->m_width*bytesPerPixel);
      line[y + ys]=true;
      line_accumulated ++;
  
      // if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(): line[%d] true\n",y+ys);
      /*For debugging*/
      /*
      write(PPMfile[fileidx%2],dimg->AskFrame_const()[y],dimg->m_width*bytesPerPixel);
      */
      BytesReceived += dimg->m_width*bytesPerPixel;
      /*For debuggin*/
    }

  img_start_line += line_to_accumulate;

  /*For debugging*/
  /*
    for (int row = 0; row <= line_to_accumulate; ++row) {
      unsigned char* pP = (unsigned char*)dimg->AskFrame_const()[row];
      
      for (int col = 0; col < 640; ++col ) {
        unsigned char val1 = (unsigned char)*pP++;
        unsigned char val2 = (unsigned char)*pP++;
        unsigned char val3 = (unsigned char)*pP;

	char pixelline[3];
        sprintf(pixelline,"%c%c%c",val1,val3,val2);
        write(PPMfile,pixelline,3);
        BytesReceived += 3;
      }
    } 
  */

  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(): bytesPerPixel=%d, line_to_accumulate=%d, BytesReceived=%d, line_accumulated=%d\n", bytesPerPixel,line_to_accumulate,BytesReceived,line_accumulated);

  if (dimg->size <= BytesReceived) {
    if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(): BytesReceived=%d\n",BytesReceived);
    /*
      close (PPMfile[fileidx%2]);

      //      if (fileidx <38 )
      // remove(PPMfilename[fileidx%2]);//remove file
    */
      fileidx++;
      BytesReceived = 0;

  }
    /*For debugging*/
  for (int i=last;i>=first;i--)
    if (line[i]==false)
      {
	if (0) cout << "line " << i << " is still missing...\n";
	return NULL;
      }


  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Accumulate_PPM(): A whole frame accumulated -------------------------\n");
  
  return &d_rdimg;
}

void ImageDataAccumulator::Free_d_rdimg()
{
  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Free_d_rdimg()\n");
  d_rdimg.FreePictureData();
}

/*^^^*/

template class Array<bool>;
#include "libvideogfx/containers/array.cc"
