/********************************************************************************
  output/accumulator.hh
    Build continuous regions of image data from output stripes.

  purpose:

  notes:

  to do:

  author(s):
   - Dirk Farin, Kapellenweg 15, 72070 Tuebingen, Germany,
     email: farindk@trick.informatik.uni-stuttgart.de

  modifications:
   20/Sep/2000 - Dirk Farin
     - Moved class definition to a file of its own.
   19/Sep/2000 - Dirk Farin
     - Accumulate helper class to ease postprocessor and display writing.
 ********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef DVDVIEW_OUTPUT_ACCUMULATOR_HH
#define DVDVIEW_OUTPUT_ACCUMULATOR_HH

#include "video12/output.hh"
#include "libvideogfx/containers/array.hh"

/*For debugging */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#undef DEBUG_WHERE
#define DEBUG_WHERE 0
//extern "C"  int open(const char *pathname, int flags, mode_t mode);
extern "C" int close(int fd);
extern "C" ssize_t write(int fd, const void *buf, size_t count);
/*^^^*/


extern "C" int tprintf(const char* fmt, ...);

class ImageDataAccumulator
{
public:
  ImageDataAccumulator() : d_width(0), d_height(0), d_sink(NULL) {
  if (DEBUG_WHERE) tprintf(" [ImageDataAccumulator::Cstr()\n");

  /* For debuggin */
  /*
  if ((PPMfile = open("received.ppm",O_CREAT|O_WRONLY,S_IRWXU|S_IRGRP|
                      S_IROTH)) < 0 ) {
    tprintf(" [VideoDecoder::Cstr(): file open error\n");
    exit(1);
  }
  */
  fileidx =0;
  BytesReceived=0;
  strcpy(PPMfilename[0],"received-A.ppm");
  strcpy(PPMfilename[1],"received-B.ppm");
  }
 
  virtual ~ImageDataAccumulator() {
    if (DEBUG_WHERE)
      tprintf(" [ImageDataAccumulator::Dstr()\n"); 
  /*For debugging*/
  //close(PPMfile);
  /*^^^*/

  }

  void SetNext(DecodedPictureSink* next) { d_sink = next; } /* The image sink, where unused
							       image strip data is sent to. */
  void StartAccumulation(int first,int last,         // range to accumulate
				      bool modifyable_copy=true); // if we are going to write to the data
  DecodedImageData* Accumulate(DecodedImageData*);  // !NULL -> image is complete


  /*vvv*/
  void ImageDataAccumulator::StartAccumulation_PPM(RawImageData* img,
						   int first,int last,
						   bool modify,
						   int d_start_line,
						   int d_end_line,
						   int i_start_line,
						   bool field_line);

  RawImageData* Accumulate_PPM(RawImageData*,bool flushOrNot);  // !NULL -> image is complete
  
  void Free_d_rdimg();

  //setup the frag/defrag param's
  void set_fragDefrag_param(int mtu, int pixelsize, int linesize) 
  {
    mus=mtu; maxpixelsize=pixelsize; maxlinesize=linesize;
  }

  //The relative PPM frag/defragment parameter
  int mus;
  int maxpixelsize;
  int maxlinesize;
  
  /*^^^*/
  
private:
  DecodedImageData d_dimg;

  Array<bool> d_line_available;
  bool        d_modifyable_copy;

  int         d_width,d_height;

  class DecodedPictureSink* d_sink;

  /*vvv*/
  //The so-far-accumulated img.
  RawImageData d_rdimg;

  int line_accumulated;
  //data_start_line: the start line # of the passed RawImageData
  //data_end_line:   the end line # of the passed RawImageData
  //img_start_line:  the starting line # in the so-far-accumulated img.
  int data_start_line,data_end_line,img_start_line;
  //Actually not used in PPM, just keep it for compatibility
  bool field_lines;

  /*For debuggin */
  int BytesReceived;
  int PPMfile[2];
  char PPMfilename[2][20];
  int fileidx;
  /*^^^*/

  /*^^^*/
};

#endif
