/********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#include "output/out_x11.hh"
#include "config.h"
#include "libvideogfx/x11/imgwin.hh"

/*vvv*/
#include "video_fmt.hh" /* for various X fmt */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#undef DEBUG_WHERE
#define DEBUG_WHERE 0

//extern "C"  int open(const char *pathname, int flags, mode_t mode);
extern "C" int close(int fd);
extern "C" ssize_t write(int fd, const void *buf, size_t count);
/*^^^*/

extern "C" {
  int tprintf(const char *fmt, ...);
}

VideoSink_X11::VideoSink_X11()
  : imgwin(NULL), first(true), pic_available(false)
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::Cstr()\n");
  /*vvv*/
  /*
  if ((outfile = open("snapshot.ppm",O_CREAT|O_WRONLY,S_IRWXU|S_IRGRP|
		      S_IROTH)) < 0 ) {
    if (DEBUG_WHERE) tprintf(" [VideoDecoder::Cstr(): file open error\n");
    exit(1);
  }
  */
  bytes_received=0;
  //Hack for first time use
  curr_img_width = curr_img_height = 0;
  /*^^^*/
}


VideoSink_X11::~VideoSink_X11()
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::Dstr()\n");

  if (imgwin)
    {
      delete imgwin;
      delete x11img;
      delete transformation;
    }
  /*vvv*/
  /*
  close(outfile);
  */
  /*^^^*/
}

#if 0
void VideoSink_X11::ShowMBRows(DecodedImageData* dimg)
{
  const Image_YUV<Pixel>& img = dimg->m_image;

  nextpts = dimg->m_timing.pts;

  if (first)
    {
      ImageParam_YUV param;
      img.GetParam(param);
      d_height = param.height;

      //StartAccumulation(0,d_height-1,false);

      assert(imgwin==NULL);
      imgwin=new ImageWindow_X11;
      x11img=new DisplayImage_X11;
      transformation=new Image2Raw;

      imgwin->SetPosition(10,10);
      imgwin->Create(dimg->m_width,dimg->m_height,
		     "DVDview (c) 1998-2000 Dirk Farin");
      x11img->Create(dimg->m_width,dimg->m_height,imgwin->AskWindow());

      XImage& ximg = x11img->AskXImage();

      RawImageSpec_RGB spec;
      spec.bytes_per_line = bpl = ximg.bytes_per_line;
      spec.bits_per_pixel = ximg.bits_per_pixel;
      spec.little_endian  = (ximg.byte_order==LSBFirst);
      spec.SetRGBMasks(ximg.red_mask,ximg.green_mask,ximg.blue_mask);

      transformation->SetOutputSpec(spec);
  
      first=false;
    }

  DecodedImageData* decimg = dimg; //Accumulate(dimg);
  //if (!decimg)
  //return;

  transformation->TransformYUV(decimg->m_image,
			       (uint8*)(x11img->AskXImage().data+bpl*decimg->m_dst_y_start),
			       decimg->m_src_y_start,decimg->m_src_y_end);


  if (decimg->m_dst_y_start - decimg->m_src_y_start + decimg->m_src_y_end==decimg->m_height-1)
    {
      x11img->PutImage();
    }
  //imgwin->Display_const(decimg->m_image);
}


void VideoSink_X11::FinishedPicture()
{
  pic_available=true;
  StartAccumulation(0,d_height-1,false);
}

#else
void VideoSink_X11::ShowMBRows(DecodedImageData* dimg)
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_x11::ShowMBRows()\n");

  const Image_YUV<Pixel>& img = dimg->m_image;

  nextpts = dimg->m_timing.pts;

  if (first)
    {
      /*vvv*/
      if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowMBRows: if (first)\n");
      /*^^^*/

      ImageParam_YUV param;
      img.GetParam(param);
      d_height = param.height;

      StartAccumulation(0,d_height-1,false);

      assert(imgwin==NULL);
      imgwin=new ImageWindow_X11;
      x11img=new DisplayImage_X11;
      transformation=new Image2Raw;

      imgwin->SetPosition(10,10);
      imgwin->Create(dimg->m_width,dimg->m_height,
		     "DVDview (c) 1998-2000 Dirk Farin");
      x11img->Create(dimg->m_width,dimg->m_height,imgwin->AskWindow());

      XImage& ximg = x11img->AskXImage();

      RawImageSpec_RGB spec;
      spec.bytes_per_line = bpl = ximg.bytes_per_line;
      spec.bits_per_pixel = ximg.bits_per_pixel;
      spec.little_endian  = (ximg.byte_order==LSBFirst);
      spec.SetRGBMasks(ximg.red_mask,ximg.green_mask,ximg.blue_mask);

      transformation->SetOutputSpec(spec);
  
      first=false;
    }

  DecodedImageData* decimg = Accumulate(dimg);
  if (!decimg)
    return;

  transformation->TransformYUV(decimg->m_image , (uint8*)(x11img->AskXImage().data));

  x11img->PutImage();
}


void VideoSink_X11::FinishedPicture()
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::FinishedPicture()\n");

  pic_available=true;
  /*vvv*/
  //StartAccumulation(0,d_height-1,false);
  /*^^^*/
}
#endif


/*vvv*/
int VideoSink_X11::ShowPPM(RawImageData* dimg, uint32 PPMFrameNum)
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_x11::ShowPPM()\n");

  if (first)
    {
      if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowPPM(): if (first)\n");

      //Hack for AskPTSOfNextToBeDisplayed()
      nextpts = 1;

      curr_PPM_frame_num = PPMFrameNum;
 
      //P1/P4 : bitmap
      //P2/P5 : greymap
      //P3/P6 : pixmap
      //Update later for X11 display
      bits_per_pixel = (dimg->ppm2 == '6'? 24:8);
      bytes_per_line = dimg->width * bits_per_pixel / 8;
      little_endian = false; //hack

      d_height = dimg->height; //480;

      img_start_line = 0;

      StartAccumulation_PPM(dimg,0,d_height-1,false,0,
			    (int)((double)mus/(double)bytes_per_line+.9999)-1,
			    img_start_line,false);

      if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowPPM(): curr_img_width=%d, curr_img_height=%d, dimg.width=%d, dimg.height=%d, curr_PPM_frame_num=%d\n", curr_img_width,curr_img_height,dimg->width,dimg->height, curr_PPM_frame_num);


      /* For debugging on network display. Removed later.........
       */
      if (imgwin == NULL) {
	//assert(imgwin==NULL);
	if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowPPM(): imgwin == NULL\n");

	imgwin=new ImageWindow_X11;
	x11img=new DisplayImage_X11;
	//transformation=new Image2Raw;

	imgwin->SetPosition(10,10);
	imgwin->Create(dimg->m_width,dimg->m_height,
		     "dvdview (c) 1998-2000 Dirk Farin");
	x11img->Create(dimg->m_width,dimg->m_height,imgwin->AskWindow());

	//XImage& ximg = x11img->AskXImage();
      }
      /**/
      /* Try this */
      /**/
      else if ((curr_img_width!=0 && curr_img_height!=0) && 
               (curr_img_width!=dimg->width || curr_img_height!=dimg->height))
      {
	if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowPPM(): imgwin reconstructed !!!!!!!\n");

	delete x11img;
	delete imgwin;
	imgwin=new ImageWindow_X11;
	x11img=new DisplayImage_X11;
	imgwin->SetPosition(10,10);
	imgwin->Create(dimg->m_width,dimg->m_height,
		     "dvdview (c) 1998-2000 Dirk Farin");
	x11img->Create(dimg->m_width,dimg->m_height,imgwin->AskWindow());
      }	
      /**/

      XImage& ximg = x11img->AskXImage();

      imgspec.bytes_per_line = bpl = ximg.bytes_per_line;
      imgspec.bits_per_pixel = ximg.bits_per_pixel;
      imgspec.little_endian  = (ximg.byte_order==LSBFirst);
      imgspec.SetRGBMasks(ximg.red_mask,ximg.green_mask,ximg.blue_mask);

      if (DEBUG_WHERE)
	imgspec.Debug_ShowParam();

      first=false;
    }
  
  bool image_flush = false;
  RawImageData* decimg;

  if (PPMFrameNum != curr_PPM_frame_num) {
    //frame number conflicts. Flush the current accumulated image
    image_flush = true;

    first = true;
    //return false;//Be careful
  }

  decimg = Accumulate_PPM(dimg, image_flush);

  //increment       dimg->m_dst_y_start 30 at a time
  //dimg->m_dst_y_start += dimg->m_src_y_end -dimg->m_src_y_start + 1;
  img_start_line += (int)((double)mus/(double)bytes_per_line+.9999);
  
  if (!decimg)
    return 1;//Not displayed

  /*For debugging on network display ..............
    In case of incomplete image : Use the minimum height...
  */ 
  
  if (dimg->ppm2 == '6') {
    TransformPixmap(decimg->image , (uint8*)(x11img->AskXImage().data), 0, (dimg->m_height-1 < img_start_line-1? dimg->m_height-1:img_start_line-1), dimg->m_width, imgspec);
  } else if (dimg->ppm2 == '5') {
    TransformGreymap(decimg->image , (uint8*)(x11img->AskXImage().data), 0, (dimg->m_height-1 < img_start_line-1? dimg->m_height-1:img_start_line-1), dimg->m_width, imgspec);
  }


  x11img->PutImage();

  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::ShowPPM(): after PutImage()\n");
  /**/

  //Hack for FinishedPicture()
  pic_available=true;  
  first = true;
  Free_d_rdimg();//d_rdimg is the accumulated RawImageData -- Needs to recreate
                 //each time when the source img size changes.

  //store the curr img info in case the ImageWindow needs to be recontrusted.
  // Init value = 0 for both. 
  curr_img_width = dimg->width;
  curr_img_height = dimg->height;

  if (image_flush)
    return 2;//incomplete but displayed
  else
    return 3;//complete img displayed
}

void VideoSink_X11::TransformPixmap(const Bitmap<Pixel>& dimg, 
			      uint8* mem, int firstline, int lastline,
			      int width, RawImageSpec_RGB& spec)
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::Transform():\n");
  if (DEBUG_WHERE) tprintf("      firstline=%d, lastline=%d, width=%d, depth=%d, little_endian=%c\n",firstline,lastline,width,(spec.bits_per_pixel+7)/8,spec.little_endian==true?'y':'n');

  char PPM_MAGIC1='P';
  char RPPM_MAGIC2='6';
  char header[20];
  char pixelline[3];

  int row,col;
  unsigned char val1,val2,val3;

  int bpp = (spec.bits_per_pixel+7)/8;

  const Pixel*const* pPP = dimg.AskFrame_const();

  if (16==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_RGB16_LE
    //bbbbbggg-gggrrrrr

    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {

        val1 = (unsigned char)*pP++; 
        val2 = (unsigned char)*pP++;
        val3 = (unsigned char)*pP++;

	uint16 val;
	val = (val1 << 8) & spec.r_mask;
	val |= (val2 << 3) & spec.g_mask;
	val |= val3 >> 3;

	mem[0] = (val & 0xff);
	mem[1] = (val & 0xff00) >>8 ;

        mem += bpp;
      }
    }
  }
  else if (32==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_BGR32
    //bgr-
    rpos = 2; gpos = 1; bpos = 0;

    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {
        val1 = (unsigned char)*pP++; 
        val2 = (unsigned char)*pP++;
        val3 = (unsigned char)*pP++;

	/* RBG */
	mem[rpos]=val1;
	mem[gpos]=val2;
	mem[bpos]=val3;

	//For debug, write to output file
	/*
	if (row==0) {//write header
           sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,  
            640, 480, 255);  
	   write(outfile, header, sizeof(header));
	}
 
	sprintf(pixelline,"%c%c%c",val1,val3,val2);
	write(outfile,pixelline,3);
	bytes_received += 3;
	*/
        mem += bpp;
      }
    }
  }
  else {
    //TO DO
    //Other X fmt
    if (DEBUG_WHERE) 
      tprintf(" [VideoSink_X11::TransformPixmap(): Video Screen NOT supported yet !\n");
  }
    /*
    if (921600 <= bytes_received) 
      close (outfile);
    */
}

void VideoSink_X11::TransformGreymap(const Bitmap<Pixel>& dimg, 
			      uint8* mem, int firstline, int lastline,
			      int width, RawImageSpec_RGB& spec)
{
  if (DEBUG_WHERE) tprintf(" [VideoSink_X11::TransformGreymap():\n");
  if (DEBUG_WHERE) tprintf("      firstline=%d, lastline=%d, width=%d, depth=%d, little_endian=%c\n",firstline,lastline,width,(spec.bits_per_pixel+7)/8,spec.little_endian==true?'y':'n');

  int row,col;
  unsigned char val1,val2,val3;

  int bpp = (spec.bits_per_pixel+7)/8;

  const Pixel*const* pPP = dimg.AskFrame_const();

  if (16==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_RGB16_LE
    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {
	int tmp = (unsigned char)(*pP);
        val1 = val2 = val3 = tmp;

	uint16 val;
	val = (val1 << 8) & spec.r_mask;
	val |= (val2 << 3) & spec.g_mask;
	val |= val3 >> 3;

	mem[0] = (val & 0xff);
	mem[1] = (val & 0xff00) >>8 ;

	pP++;
        mem += bpp;
      }
    }
  }
  else if (32==spec.bits_per_pixel && true==spec.little_endian) {
    //VID_BGR32
    rpos = 2; gpos = 1; bpos = 0;

    for ( row = firstline; row <= lastline; ++row) {
      const Pixel* pP = pPP[row];
      
      for ( col = 0; col < width; ++col ) {
	/*Take the value and divided by 3*/
	int tmp = (unsigned char)(*pP);
        val1 = val2 = val3 = tmp; 
        //val2 = (unsigned char)*pP++;
        //val3 = (unsigned char)*pP++;

	mem[rpos]=val2;
	mem[gpos]=val3;
	mem[bpos]=val1;

	pP++;
        mem += bpp;
      }
    }
  }
  else {
  }
}
/*^^^*/
