/********************************************************************************
    Copyright (C) 1999  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/


#include "system/sysdec1.hh"
#include "input/streamsrc.hh"
#include "input/errors.hh"
#include "error.hh"

#include <stdio.h>

#include <iostream.h>

#include <stdlib.h>
#include <string.h>
/*vvv*/
#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 0
#define DEBUG_WHAT 0

extern "C" int tprintf(const char* fmt, ...);

const int c_ElementaryPacketSize = 63*1024;

int SystemDecoder_MPEG1::s_next_pck_nr = 0;


SystemDecoder_MPEG1::SystemDecoder_MPEG1()
  : d_strsrc(NULL),
    d_streamtype(Unknown),
    d_readahead_packet(NULL),
    d_synced(false),
    d_bytebufparams(3*1024,20)
{
  //tprintf(" [SystemDecoder_MPEG1::SystemDecoder_MPEG1()\n");
}


SystemDecoder_MPEG1::~SystemDecoder_MPEG1()
{
  //if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::~SystemDecoder_MPEG1()\n");

  if (d_readahead_packet)
    {
      delete d_readahead_packet;
    }
}


void SystemDecoder_MPEG1::SetSource(StreamSource& strsrc)
{
  //if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::SetSource(StreamSource& strsrc)\n");

  d_strsrc=&strsrc;
  HardReset();
}

/*vvv*/
void SystemDecoder_MPEG1::SetSource_PPM(StreamSource& strsrc)
{
  //if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::SetSource_PPM(StreamSource& strsrc)\n");

  d_strsrc=&strsrc;
  HardReset_PPM();
}

void SystemDecoder_MPEG1::HardReset_PPM()
{
  //if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM()\n");

  SoftReset();


  // If streamtype is now known, guess it from the next startcode.

  if (d_streamtype == Unknown)
    {
      //if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM(): if (d_stramtype==Unknown)\n");

      // Read first four bytes of stream to determine type of stream.
      // As we have read some bytes we have to read a complete startcode unit then.

      uint8 buf[sizeof(unsigned long)+sizeof(PPM_frame_info)];
      int BytesRead = d_strsrc->FillBuffer(buf,sizeof(unsigned long) 
					   +sizeof(PPM_frame_info));

      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM(): buf[4]=%x\n",buf);
      /* New 17/May/99: Skip extra leading 0-bytes. */
      while (buf[0]==0 && buf[1]==0 && buf[2]==0)
	{
	  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM(): while \n");
 
	  buf[0]=buf[1];
	  buf[1]=buf[2];
	  buf[2]=buf[3];
	  BytesRead--;
	  BytesRead += d_strsrc->FillBuffer(&buf[3],1);

	  // Leave loop if no complete startcode is possible any more.
	  if (BytesRead != 4)
	    break;
	}

      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM(): after while :\n");
      if (DEBUG_WHERE) tprintf("         buf[4]=%x, %x, %x, %x\n",buf[0],buf[1],buf[2],buf[3]);

      if (BytesRead!=sizeof(unsigned long)+sizeof(PPM_frame_info) || buf[0]!=0x00 || buf[1]!=0x00 || buf[2]!=0x01)
	{
	  d_streamtype = Illegal;
	}
      else if (buf[3]==0xBA)
	{
	  d_streamtype = System;
	  try
	    {
	      d_readahead_packet = ReadPack();
	    }
	  catch (Excpt_Error_UnexpectedEndOfStream&)
	    {
	      // Stream is far too short to be useable.
	      d_streamtype = Illegal;
	    }
	}
      /*vvv*/
      /*
      else
	{
	  d_streamtype = Video;
	  d_readahead_packet = ReadElemPacket(true,buf[3]);
	}
      */
      else if (buf[3]==0xFF) {//PPM Sequence header
	d_streamtype = PPM;
	d_readahead_packet = ReadPacket_PPM_Header(buf);
	//if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM(): after calling ReadPacket_PPM_Header\n");

      }
      else if (buf[3]==0xF0) {
	//PPM frame data header
	//d_streamtype = PPM;
	d_readahead_packet = ReadPacket_PPM(buf[3]==0xF0 ? true:false,buf[3]);
      }
      else {
	  d_streamtype = Video;
	  d_readahead_packet = ReadElemPacket(true,buf[3]);
      }	
      /*^^^*/
    }

  //  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset_PPM() Return \n");
}

/*^^^*/

void SystemDecoder_MPEG1::HardReset()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset()\n");

  SoftReset();


  // If streamtype is now known, guess it from the next startcode.

  if (d_streamtype == Unknown)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset(): if (d_stramtype==Unknown)\n");

      // Read first four bytes of stream to determine type of stream.
      // As we have read some bytes we have to read a complete startcode unit then.

      uint8 buf[4];
      int BytesRead = d_strsrc->FillBuffer(buf,4);

      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset(): buf[4]=%x\n",buf);
      /* New 17/May/99: Skip extra leading 0-bytes. */
      while (buf[0]==0 && buf[1]==0 && buf[2]==0)
	{
	  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset(): while \n");
 
	  buf[0]=buf[1];
	  buf[1]=buf[2];
	  buf[2]=buf[3];
	  BytesRead--;
	  BytesRead += d_strsrc->FillBuffer(&buf[3],1);

	  // Leave loop if no complete startcode is possible any more.
	  if (BytesRead != 4)
	    break;
	}

      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::HardReset(): after while :\n");
      if (DEBUG_WHERE) tprintf("         buf[4]=%x, %x, %x, %x\n",buf[0],buf[1],buf[2],buf[3]);

      if (BytesRead!=4 || buf[0]!=0x00 || buf[1]!=0x00 || buf[2]!=0x01)
	{
	  d_streamtype = Illegal;
	}
      else if (buf[3]==0xBA)
	{
	  d_streamtype = System;
	  try
	    {
	      d_readahead_packet = ReadPack();
	    }
	  catch (Excpt_Error_UnexpectedEndOfStream&)
	    {
	      // Stream is far too short to be useable.
	      d_streamtype = Illegal;
	    }
	}
      else
	{
	  d_streamtype = Video;
	  d_readahead_packet = ReadElemPacket(true,buf[3]);
	}
    }
}


void SystemDecoder_MPEG1::SoftReset()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::SoftReset()\n");

  if (d_readahead_packet)
    {
      delete d_readahead_packet;
      d_readahead_packet=NULL;
    }

  d_synced=false;
}


SysPacket* SystemDecoder_MPEG1::GetNextPacket()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::GetNextPacket()\n");

  // While there are previously read packets pending in the queue, return these.

  if (d_readahead_packet)
    {
      SysPacket* pck = d_readahead_packet;
      d_readahead_packet = NULL;
      return pck;
    }

  // If end of input reached return nothing (as no more packets are in the queue).

  if (!d_strsrc->MoreDataPending() &&
      !d_synced) // This test because we have read 3 bytes already.
    return NULL;

  // Remember 'd_synced' flag and clear it.

  bool sync=d_synced;
  d_synced=false;

  // There is more data to be read. Generate more SysPackets if possible.

  if (d_streamtype==System)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::GetNextPacket(): if (d_streamtype==System)\n");

      // Find next startcode.

      uint8 buf[4];
      int BytesRead;
      if (!sync)
	{
	  BytesRead = d_strsrc->FillBuffer(buf,4);
	}
      else
	{
	  buf[0]=buf[1]=0; buf[2]=1;
	  BytesRead = d_strsrc->FillBuffer(&buf[3],1) + 3;
	}

      // Incomplete startcode.
      if (BytesRead!=4)
        {
	  if (BytesRead==0)
	    return NULL;	// End of stream reached, no more startcodes.
	  else
	    throw Excpt_Error_UnexpectedEndOfStream();
	}

      while (!(buf[0]==0 && buf[1]==0 && buf[2]==1)) // While startcode sync not found.
        {
	  // Shift buffer forward one byte.
	  buf[0]=buf[1]; buf[1]=buf[2]; buf[2]=buf[3];
	  if (d_strsrc->FillBuffer(&buf[3],1) != 1)
	    {
	      throw Excpt_Error_UnexpectedEndOfStream();
	    }
        }


      // According to the startcode read, get complete startcode-unit.

      if (buf[3]>=0xBD && buf[3]<=0xEF)
        {
	  //printf("%p\n",buf[3]);
          return ReadPacket(buf[3]);
        }
      else
        {
          switch (buf[3])
            {
            case 0xB9:
              {
                SysPacket* syspck = new SysPacket_Iso11172EndCode;
                syspck->pck_nr = s_next_pck_nr++;
                syspck->absolute_filepos = d_strsrc->AskCurrentPosition()-4;
                return syspck;
              }
              break;
            case 0xBA: return ReadPack(); break;
            case 0xBB: return ReadSystemHeader(); break;
            default:
	      {
		char errtxt[100];
		sprintf(errtxt,"Undefined system start code read (%p).",0x100|buf[3]);
		throw Excpt_Error_InvalidData(errtxt);
		break;
	      }
            }
        }
    }
  else
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::GetNextPacket(): else (d_streamtype==System)\n");
      return ReadElemPacket(false);
    }

  Assert(0);
}


void SystemDecoder_MPEG1::Resync()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::Resync()\n");

  if (d_streamtype==System)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::Resync(): if (d_streamtype==System)\n");
      
      int state=0;
      for (;;)
	{
	  unsigned char c;

	  // End of stream reached -> exit.
	  if (d_strsrc->FillBuffer(&c,1) != 1)
	    return;

	  /*                      -0-
                                  | |
                                  \ /
	    [0] --0--> (1) --0--> (2) --9--> (X)
              ^         |          |
              \        /          /
               ---?---------------
	   */

	  // simple state machine
	  if (state <= 1 && c==0)
	    state++;
	  else if (state==2 && c==0)
	    {
	    }
	  else if (state==2 && c==1)
	    {
	      d_synced=true;
	      return;
	    }
	  else
	    state=0;
	}
    }
}


SysPacket* SystemDecoder_MPEG1::ReadPack()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPack()\n");

  uint8 buf[10];
  int BytesRead = d_strsrc->FillBuffer(buf,8);

  if (BytesRead!=8)
    {
      throw Excpt_Error_UnexpectedEndOfStream();
    }

  SysPacket_Pack* syspck = new SysPacket_Pack;
  syspck->absolute_filepos = d_strsrc->AskCurrentPosition()-4-8;
  syspck->pck_nr = s_next_pck_nr++;
  syspck->SCR     = (((uint64)(buf[0]&0x0E))<<29) |
                    (((uint64)(buf[1]&0xFF))<<22) |
                    (((uint64)(buf[2]&0xFE))<<14) |
                    (((uint64)(buf[3]&0xFF))<< 7) |
                    (((uint64)(buf[4]&0xFE))>> 1);
  syspck->MuxRate = (((uint32)(buf[5]&0x7F))<<15) |
                    (((uint32)(buf[6]&0xFF))<< 7) |
                    (((uint32)(buf[7]&0xFE))>> 1);

  return syspck;
}


SysPacket* SystemDecoder_MPEG1::ReadPacket(uint8 Channel)
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket(uint8 Channel)\n");

  SysPacket_Packet* syspck = new SysPacket_Packet(d_bytebufparams);
  syspck->absolute_filepos = d_strsrc->AskCurrentPosition()-4;
  syspck->Channel=Channel;
  syspck->timing.HasPTS=false;
  syspck->timing.HasDTS=false;
  syspck->HasSTDBufferSpecs=false;

  // Read packet_length.

  uint8 buf[10];
  if (d_strsrc->FillBuffer(buf,2)!=2)
    {
      delete syspck;
      throw Excpt_Error_UnexpectedEndOfStream();
    }

  int packet_length = (((uint32)buf[0])<<8) | (((uint32)buf[1])<<0);

  if (Channel == 0xBF)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket(uint8 Channel):if (Channel==0xBF)\n");

      d_strsrc->SetCurrentPosition(d_strsrc->AskCurrentPosition()+packet_length);
      return syspck;
    }

  bool MPEG2_PES=false;

  if (Channel != 0xBE)   // FIX: 99/01/31     was: 0xBF
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket(uint8 Channel):if (Channel!=0xBE)\n");

      if (d_strsrc->FillBuffer(buf,1)!=1)
	{
	  delete syspck;
	  throw Excpt_Error_UnexpectedEndOfStream();
	}

      if ((buf[0]>>6)==0x02)
	{
	  // MPEG-2 PES packet

	  MPEG2_PES=true;

	  d_strsrc->SetCurrentPosition(d_strsrc->AskCurrentPosition()+1);
	  packet_length-=2;

	  d_strsrc->FillBuffer(buf,1);
	  d_strsrc->SetCurrentPosition(d_strsrc->AskCurrentPosition()+buf[0]);
	  packet_length-=buf[0]+1;
	}
      else
	{
	  // Read byte-padding.

	  for(;;)
	    {
	      if (buf[0] != 0xFF)
		break;
	      else
		packet_length--;

	      // read another byte

	      if (d_strsrc->FillBuffer(buf,1)!=1)
		{
		  delete syspck;
		  throw Excpt_Error_UnexpectedEndOfStream();
		}
	    }
	}


      if (!MPEG2_PES)
	{
	  // Read STDBufferSpecs.

	  if ((buf[0]&0xC0) == 0x40)
	    {
	      if (d_strsrc->FillBuffer(&buf[1],1)!=1)
		{
		  delete syspck;
		  throw Excpt_Error_UnexpectedEndOfStream();
		}

	      syspck->HasSTDBufferSpecs=true;
	      syspck->STDBufferScale = ((buf[0]&0x20)!=0);
	      syspck->STDBufferSize  = (((uint32)(buf[0]&0x1F))<<8) | ((uint32)(buf[1]));
	      packet_length -= 2;

	      if (d_strsrc->FillBuffer(buf,1)!=1)
		{
		  delete syspck;
		  throw Excpt_Error_UnexpectedEndOfStream();
		}
	    }
	  else
	    syspck->HasSTDBufferSpecs=false;


	  // Read PTS/DTS

	  if ((buf[0] & 0xE0) == 0x20)
	    {
	      bool ReadDTS = ((buf[0]&0xF0)==0x30);

	      syspck->timing.HasPTS=true;
	      syspck->timing.HasDTS=ReadDTS;

	      uint32 BytesToRead = (ReadDTS ? 10 : 5);
	      if (d_strsrc->FillBuffer(&buf[1],BytesToRead-1) != BytesToRead-1)
		{
		  delete syspck;
		  throw Excpt_Error_UnexpectedEndOfStream();
		}

	      syspck->timing.pts = (((PTS)(buf[0]&0x0E))<<29) |
		(((PTS)(buf[1]&0xFF))<<22) |
		(((PTS)(buf[2]&0xFE))<<14) |
		(((PTS)(buf[3]&0xFF))<< 7) |
		(((PTS)(buf[4]&0xFE))>> 1);

	      if (ReadDTS)
		{
		  syspck->timing.dts = (((DTS)(buf[5]&0x0E))<<29) |
		    (((DTS)(buf[6]&0xFF))<<22) |
		    (((DTS)(buf[7]&0xFE))<<14) |
		    (((DTS)(buf[8]&0xFF))<< 7) |
		    (((DTS)(buf[9]&0xFE))>> 1);
		}

	      packet_length -= BytesToRead;
	    }
	  else
	    {
	      if (buf[0])
		{
		  if (buf[0] != 0x0F)
		    {
		      delete syspck;
		      throw Excpt_Error_InvalidData("Invalid data read: packet header does not contain 00001111 where it should.");
		    }

		  packet_length--;
		}
	    }
	}
    }

  // Read packet data.

  if (packet_length<=0)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket(uint8 Channel):if (packet_length<=0)\n");

      delete syspck;
      throw Excpt_Error_InvalidData("Invalid data read: packet data length <= 0.");
    }

  unsigned char* mem = syspck->data.GetPtrToAppendToBuffer(packet_length);
  uint32 nBytesRead = d_strsrc->FillBuffer(mem,packet_length);
  if (nBytesRead<packet_length)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket(uint8 Channel):if (nBytesRead<packet_length)\n");

      delete syspck;
      throw Excpt_Error_UnexpectedEndOfStream();

      //syspck->data.TruncateBuffer(packet_length - nBytesRead);
    }


#if 0
  // throw away padding stream
  if (syspck->Channel == 0xBE)
    {
      return NULL;
    }
#endif

  syspck->pck_nr = s_next_pck_nr++;
  return syspck;
}


SysPacket* SystemDecoder_MPEG1::ReadElemPacket(bool AppendStartcode,uint8 startcode)
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadElemPacket(bool AppendStartcode,uint8 startcode)\n");


  /* As it's a video or audio stream, cut input into arbitrary pieces and
     return as SysPacket without PTS/DTS-times. */
  
  SysPacket_Packet* syspck = new SysPacket_Packet(d_bytebufparams);
  syspck->absolute_filepos = d_strsrc->AskCurrentPosition();
  syspck->timing.HasPTS = false;
  syspck->timing.HasDTS = false;
  syspck->HasSTDBufferSpecs = false;

       if (d_streamtype==Video) syspck->Channel = 0xE0;
  else if (d_streamtype==Audio) syspck->Channel = 0xC0;
  else { cerr << "Not implemented streamtype: " << d_streamtype << endl; NotImplemented }

  if (AppendStartcode)
    {
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadElemPacket(bool AppendStartcode,uint8 startcode): if (AppendStartcode)\n");

      syspck->absolute_filepos -= 4;
      uint8 buf[4]; buf[0]=buf[1]=0; buf[2]=1; buf[3]=startcode;
      syspck->data.AppendBytes(buf,4);
    }

  unsigned char* mem = syspck->data.GetPtrToAppendToBuffer(c_ElementaryPacketSize);
  int nBytesRead = d_strsrc->FillBuffer(mem,c_ElementaryPacketSize);

  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadElemPacket(bool AppendStartcode,uint8 startcode): nBytesRead=%d\n",nBytesRead);
  if (nBytesRead<c_ElementaryPacketSize)
    { 
      if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadElemPacket(bool AppendStartcode,uint8 startcode): if (nBytesRead<c_ElementaryPacketSize)\n");

  syspck->data.TruncateBuffer(c_ElementaryPacketSize - nBytesRead); }

  syspck->pck_nr = s_next_pck_nr++;
  return syspck;
}


SysPacket* SystemDecoder_MPEG1::ReadSystemHeader()
{

  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadSystemHeader()\n");

  // Read packet_length.

  uint8 buf[8];
  if (d_strsrc->FillBuffer(buf,8)!=8)
    {
      throw Excpt_Error_UnexpectedEndOfStream();
    }

  SysPacket_SystemHeader* syspck = new SysPacket_SystemHeader;

  uint32 packet_length = (((uint32)buf[0])<<8) | (((uint32)buf[1])<<0);

  syspck->absolute_filepos = d_strsrc->AskCurrentPosition()-4-8;
  syspck->RateBound = (((int)(buf[2]&0x7F))<<15) |
                      (((int)(buf[3]&0xFF))<< 7) |
                      (((int)(buf[4]&0xFE))>> 1);
  syspck->AudioBound = (buf[5]&0xFC)>>2;
  syspck->VideoBound = (buf[6]&0x1F);
  syspck->FixedBitRate         = ((buf[5]&0x02) != 0);
  syspck->ConstrainedBitstream = ((buf[5]&0x01) != 0);
  syspck->AudioLock  = ((buf[6]&0x80) != 0);
  syspck->VideoLock  = ((buf[6]&0x40) != 0);

  packet_length -= 6;

  while (packet_length>0)
    {
      if (packet_length<3)
        {
          delete syspck;
          throw Excpt_Error_InvalidData("Invalid data read: Junk at end of system header detected.");
        }

      if (d_strsrc->FillBuffer(buf,3)!=3)
        {
	  delete syspck;
	  throw Excpt_Error_UnexpectedEndOfStream();
	}

      if ((buf[0]&0x80) == 0)
        break;

      int scale;
      if ((buf[1]&0x20) != 0)
        scale=128;
      else
        scale=1024;

      uint32 size = (((uint32)(buf[1]&0x1F))<<8) | ((uint32)(buf[2]));

      if (buf[0]==0xB8)
        {
          // special StreadID 0xB8: BufferSize is set for all audio channels
          for (int i=0;i<32;i++) syspck->STDBufferSizeAudio[i] = size*scale;
        }
      else if (buf[0]==0xB9)
        {
          // special StreadID 0xB9: BufferSize is set for all video channels
          for (int i=0;i<16;i++) syspck->STDBufferSizeVideo[i] = size*scale;
        }
      else if (buf[0]<0xC0 || buf[0]>0xEF)
        {
	  /*
          delete syspck;
          throw Excpt_Error_InvalidData("Invalid data read: stream ID out of range.");
	  */
        }
      else
        {
          if (buf[0]<0xE0) syspck->STDBufferSizeAudio[buf[0]-0xC0] = size*scale;
          else             syspck->STDBufferSizeVideo[buf[0]-0xE0] = size*scale;
        }

      packet_length-=3;
    }

  syspck->pck_nr = s_next_pck_nr++;
  return syspck;
}


/*vvv*/
SysPacket* SystemDecoder_MPEG1::GetNextPacket_PPM()
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::GetNextPacket_PPM()\n");
  if (d_readahead_packet)
    {
      SysPacket* pck = d_readahead_packet;
      d_readahead_packet = NULL;

#     if (DEBUG_WHERE)
 tprintf(" [SystemDecoder_MPEG1::GetNextPacket_PPM(): d_readahead_packet exists. &pck=%x, &mem=%x\n",pck,((SysPacket_Packet*)pck)->data.AskContents());
#     endif
      //debugging
      /*      
      for (int i=0;i<16;i++) {
	printf("%x:",((SysPacket_Packet*)pck)->data.AskContents()[i]);
      }
      if (DEBUG_WHERE) tprintf("\n");
      */
      return pck;
    }

  if (!d_strsrc->MoreDataPending() &&
      !d_synced) // This test because we have read 3 bytes already.
    return NULL;

  bool sync=d_synced;
  d_synced=false;

  return ReadPacket_PPM(false);
}


SysPacket* SystemDecoder_MPEG1::ReadPacket_PPM(
					       bool AppendStartcode/*=false*/, uint8 startcode/* =0xFF*/)
{
#  if (DEBUG_WHERE) 
  tprintf(" [SystemDecoder_MPEG1::ReadPacket_PPM(): append=%s\n",AppendStartcode?"true":"false");
#  endif

  //SysPacket_Packet* syspck = new SysPacket_Packet(d_bytebufparams);
  if (curr_pck) {
    curr_pck->data.Clear();
  }
  else {
    curr_pck = new SysPacket_Packet(d_bytebufparams);
  }

  curr_pck->absolute_filepos = d_strsrc->AskCurrentPosition();
  curr_pck->Channel=0xE0;//Hack for now ......
  //Audio Stream starts at Channel 0xBD=189, Video Stream starts at
  //0xE0 = 224, All streams end at 0xEF=239.
  curr_pck->timing.HasPTS=false;
  curr_pck->timing.HasDTS=false;
  curr_pck->HasSTDBufferSpecs=false;

  if (AppendStartcode) {
    //When the dvdview were brought up if the PPM stream already started
    //sending to it. Then the PPM header will be missed. Here's the hack.
    curr_pck->absolute_filepos -=4;
    uint8 buf[32]; 
    buf[0]=buf[1]=0;
    buf[2]=1; 
    buf[3]=0xFF;//startcode;
    //Pretend we have a 320x240 color = largest in our avs demo
    PPM_frame_info default_pfi;
    default_pfi.tag=0;
    default_pfi.ppm1='P';
    default_pfi.ppm2='6';
    default_pfi.size=320*240*3;
    default_pfi.width=320;
    default_pfi.height=240;
    default_pfi.buff=0;
    memcpy(&buf[8],&default_pfi,sizeof(default_pfi));
    curr_pck->data.AppendBytes(buf,sizeof(unsigned long)+sizeof(default_pfi)
			       +sizeof(unsigned long));
    d_readahead_packet = ReadPacket_PPM_Header(buf);
    return curr_pck;
  }

  int packet_length = (mus+maxpixelsize*maxlinesize+sizeof(uint32)+
		       PPM_HEADER_LENGTH+sizeof(PPM_frame_info))*sizeof(uint8);

  //Large doesn't matter
  unsigned char* mem =  curr_pck->data.GetPtrToAppendToBuffer(packet_length);

  uint32 nBytesRead = d_strsrc->FillBuffer(mem,packet_length);

#  if (DEBUG_WHERE) 
  tprintf(" [SystxemDecoder_MPEG1::ReadPacket_PPM(): nBytesRead=%d, packet_length=%d, &curr_pck=%x, &mem=%x\n",nBytesRead, packet_length,curr_pck,mem);
#  endif
  
  //debugging
#  if (DEBUG_WHAT) 
    for (int i=0;i<16;i++) {
      printf("%x:",mem[i]);
    }
    tprintf("\n");
    fflush(stdout);
#  endif

  if (nBytesRead < packet_length) {
    curr_pck->data.TruncateBuffer(packet_length-nBytesRead);
  }
  
  return curr_pck;
}  

SysPacket* SystemDecoder_MPEG1::ReadPacket_PPM_Header(uint8* buf)
{
  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket_PPM_Header(): buf=%x, %x, %x, %x, %x, %x, %x, %x, HeaderLength:%d\n",buf[0],buf[1],buf[2],buf[3],buf[4],buf[5],buf[6],buf[7],4+sizeof(PPM_frame_info));

  if (curr_pck) {
    curr_pck->data.Clear();
  }
  else {
    curr_pck = new SysPacket_Packet(d_bytebufparams);
  }

  curr_pck->absolute_filepos = d_strsrc->AskCurrentPosition();
  curr_pck->Channel=0xE0;//Hack for now ......
  //Audio Stream starts at Channel 0xBD=189, Video Stream starts at
  //0xE0 = 224, All streams end at 0xEF=239.
  curr_pck->timing.HasPTS=false;
  curr_pck->timing.HasDTS=false;
  curr_pck->HasSTDBufferSpecs=false;

  curr_pck->data.AppendBytes(buf,sizeof(unsigned long)+sizeof(PPM_frame_info));

  uint8* tmp = curr_pck->data.AskContents();

  if (DEBUG_WHERE) tprintf(" [SystemDecoder_MPEG1::ReadPacket_PPM_Header(): buf=%x, %x, %x, %x, %x, %x, %x, %x, HeaderLength:%d\n",tmp[0],tmp[1],tmp[2],tmp[3],tmp[4],tmp[5],tmp[6],tmp[7],curr_pck->data.AskLength());

  return curr_pck;
}  
    

/*^^^*/
