/* -*-C-*-
*******************************************************************************
*
* File:         vdev.h
* RCS:          $Id: vdev.h,v 1.1.1.1 2003/06/11 19:05:40 wyan Exp $
* Description:  Library to access USB video cams in Linux (RH 7.1, Kernel 2.4) 
*               Look at the included README file.
* Author:       Fabian E. Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Sun May 13, 2001 at 10:15:10
* Modified:     Sun Jun 10, 2001 at 11:32:05 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#ifndef VDEV_INCLUDED
#define VDEV_INCLUDED 1

#ifdef __cplusplus
#  define BEGIN_C_DECLS         extern "C" {
#  define END_C_DECLS           }
#else
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif


BEGIN_C_DECLS

#define VDEV_NAMELEN   32
#define VDEV_MAXNUMDEV 64

/* Raw format flags */
#define RGB 1
#define RBG 2
#define GRB 3
#define GBR 4
#define BRG 5
#define BGR 6

typedef struct VDev_T *VDev_T;

struct VDev_imageSize {
  int width;
  int height;
};

#define VDEV_IMGSIZECOLOR(width, height) width * height * 3 * sizeof(u_char); 
#define VDEV_IMGSIZEGREY(width, height) width * height * sizeof(u_char); 

/*vvv*/
/*
extern int
VDev_init();
*/
extern int VDev_init(int w,int h);
/*^^^*/

extern void
VDev_finalize();

extern int 
VDev_numVDevices();

extern VDev_T 
VDev_getVDevice(int n);

/*vvv*/
/*
extern VDev_T
VDev_new(char* devname);
*/
extern VDev_T VDev_new(char* devname, int w, int h);
/*^^^*/

extern void
VDev_free(VDev_T vdev);

extern int
VDev_open(VDev_T vdev, int buf);

extern int
VDev_close(VDev_T vdev);

extern int
VDev_getDescriptor(VDev_T vdev);

extern int
VDev_isValid(VDev_T vdev);

extern char* 
VDev_getIntfName(VDev_T vdev);

extern char* 
VDev_getNodeName(VDev_T vdev);

extern int
VDev_readImage(VDev_T vdev);

extern int
VDev_getRawSize(VDev_T vdev);

extern u_char*
VDev_getImage(VDev_T vdev);

extern u_char*
VDev_getPPMImage(VDev_T vdev, int format);

extern int
VDev_getMinSize(VDev_T vdev, struct VDev_imageSize *size);

extern int
VDev_getMaxSize(VDev_T vdev, struct VDev_imageSize *size);

extern int
VDev_getSize(VDev_T vdev, struct VDev_imageSize *size);

extern int
VDev_setSize(VDev_T vdev, struct VDev_imageSize size);

extern u_int
VDev_getBrightness(VDev_T vdev);

extern u_int
VDev_getColor(VDev_T vdev);

extern u_int
VDev_getContrast(VDev_T vdev);

extern int
VDev_getFramerate(VDev_T vdev);

extern u_int
VDev_getHue(VDev_T vdev);

extern u_int
VDev_getWhiteness(VDev_T vdev);

extern int
VDev_setBrightness(VDev_T vdev, u_int val);

extern int
VDev_setColor(VDev_T vdev, u_int val);

extern int
VDev_setContrast(VDev_T vdev, u_int val);

extern int
VDev_setFramerate(VDev_T vdev, int fps, int snap);

extern int
VDev_setHue(VDev_T vdev, u_int val);

extern int
VDev_setWhiteness(VDev_T vdev, u_int val);

END_C_DECLS

#endif /* !VDEV_INCLUDED */
