dnl @synopsis AC_PKG_PYTHON(X.X)
dnl This macro is modified from the qt by Bastiaan.N.Veelo@immtek.ntnu.no
dnl $Id: acinclude.m4,v 1.1.1.1 2003/06/11 21:16:00 wyan Exp $
dnl @author Wenchang
dnl This macro searches for a Python installation on your system.
dnl The argument of --with-python-include-dir specifies the full pathname 
dnl of Python.h. The argument of --with-python-lib-dir specifies the full
dnl path name of libpython*.a. The version number X.X is the minimum ver
dnl number required. If the found python lib version is lower than the 
dnl required version. The configuration will stop.
dnl 
dnl The python lib is assumed to be installed under /usr/lib/python*.*/
dnl config or /usr/local/lib/python*.*/config /usr/local/public/python*.*
dnl /config
dnl
dnl It defines the symbol PYTHON_INCLUDE and PYTHON_LIB if the include
dnl file and the library are found.
dnl
dnl Example of use of these variables in your Makefile.in:
dnl
dnl   INCLUDES += @PYTHON_INCLUDE@
dnl   LDFLAGS += @PYTHON_LIB@
dnl

AC_DEFUN([AC_PKG_PYTHON],
[
#
# first parse the argument given in configure.in to AC_PKG_PYTHON
# 
PYTHON_REQUEST_VERSION=
PYTHON_INCLUDE=
PYTHON_LIB=

python_header=Python.h
python_lib_ext=".a"

changequote(<<, >>)

for a in $1 $2 $3 $4 $5 $6 $7 $8 $9 x; do
    case "$a" in
        x) break;;
        [0-9]*.[0-9]*) PYTHON_REQUEST_VERSION="$a";;
        *) AC_MSG_ERROR("argument to the PKG_PYTHON macro must be of 'X.X' as the required version number");;
    esac
done

changequote([, ])

echo "PYTHON_REQUEST_VERSION: $PYTHON_REQUEST_VERSION"

AC_MSG_CHECKING(python include and lib)
AC_ARG_WITH(python-include-dir,
    [  --with-python-include-dir=PATH absolute path name of python's include file 'Python.h'])
AC_ARG_WITH(python-lib-dir,
    [  --with-python-lib-dir=PATH absolute path name of python's library])

## Look for header files ##
if test x"$with_python_include_dir" != x; then
   if test -r "$with_python_include_dir/$python_header"; then
      AC_MSG_ERROR(" $with_python_include_dir is not valid")
   else 
      python_include_dir="$with_python_include_dir"
   fi
else
#
# If python path not specified by caller, search in standard places
# and public dir here at CoC
#
   # Look for header files `Python.h' ##
   python_include_dir=
   # Look for the header file in a standard set of common directories.
   python_include_path_list="
 	     /usr/include
      	     `ls -dr /usr/include/python* 2>/dev/null`
	     `ls -dr /usr/local/public/include/python* 2>/dev/null`
	     `ls -dr /usr/local/lib/python* 2>/dev/null`
        "

   for python_dir in $python_include_path_list; do
   	   if test -r "$python_dir/$python_header"; then
          	python_dirs="$python_dirs $python_dir"
      	   fi	
   done	

   changequote(<<, >>)

   # Now look for the newest in this list
   python_prev_ver=0
   for python_dir in $python_dirs; do
          python_this_ver=`eval echo "$python_dir" | sed s/[a-zA-Z/-]*//`
          if expr $python_this_ver '>' $python_prev_ver > /dev/null; then
             python_include_dir=$python_dir
             python_prev_ver=$python_this_ver
          fi
   done

   # Check if version is new enough
   if expr $python_prev_ver '<' $PYTHON_REQUEST_VERSION > /dev/null; then
   	AC_MSG_ERROR(" This python version $python_prev_ver shall be upgraded to version $PYTHON_REQUEST_VERSION")
   fi
   dnl Found header files.
fi

python_dirs=
python_lib_dir=
## Look for python lib ##
if test x"$with_python_lib_dir" != x; then
   python_lib_dir="$with_python_lib_dir"
else
#
# If python path not specified by caller, search in standard places
# and public dir here at CoC
#
 if test x"$python_lib_dir" = x; then
   python_lib_path_list="
 	     /usr/lib
      	     `ls -dr /usr/lib/python* 2>/dev/null`
	     `ls -dr /usr/local/public/lib/python* 2>/dev/null`
        "
   for python_dir in $python_lib_path_list; do
      python_this_ver=`eval echo "$python_dir" | sed s/[a-zA-Z/-]*//`
      if test -r "$python_dir/config/libpython${python_this_ver}${python_lib_ext}"; then
       	 python_dirs="$python_dirs $python_dir"
      fi	
   done	

   # Now look for the newest in this list
   python_prev_ver=0
   for python_dir in $python_dirs; do
          python_this_ver=`eval echo "$python_dir" | sed s/[a-zA-Z/-]*//`
          if expr $python_this_ver '>' $python_prev_ver > /dev/null; then
             python_lib_dir=$python_dir
             python_prev_ver=$python_this_ver
          fi
   done
 else # if test x"$python_lib_dir" = x
   python_prev_ver=`eval echo "$python_lib_dir" |sed s/[a-zA-Z/-]*//`
 fi

 if expr $python_prev_ver '<' $PYTHON_REQUEST_VERSION > /dev/null; then
    AC_MSG_ERROR(" This python version $python_prev_ver shall be upgraded to version $PYTHON_REQUEST_VERSION")
 fi
 dnl Found python lib.
fi


changequote([, ])

PYTHON_INCLUDE=-I$python_include_dir
PYTHON_LIB="-L$python_lib_dir/config -lpython$python_prev_ver"

AC_SUBST(PYTHON_INCLUDE)
AC_SUBST(PYTHON_LIB)


])


dnl This macro is to check SIP to block SIP / block XIP python converter
AC_DEFUN([AC_PKG_CSIPLEX],
[
CSIPLEX_INCLUDE=
CSIPLEX_LIB=

changequote(<<, >>)

for a in $1 $2 $3 $4 $5 $6 $7 $8 $9 x; do
    case "$a" in
        x) break;;
        *.h) csiplex_include_file="$a";;
        *.a) csiplex_lib_name="$a";;
        *) AC_MSG_ERROR("argument to the PKG_CSIPLEX macro must be of 'csiplex.h' 'libcsiplex.a' ");;
    esac
done


AC_MSG_CHECKING(custom csiplex block SIP/XIP include and lib)

cur_dir=`pwd`

## Check for header files ##
if test -r "$cur_dir/../sip2xml/$csiplex_include_file"; then
      csiplex_include_dir="$cur_dir/../sip2xml"
else 
      AC_MSG_ERROR(" $csiplex_include_file not exist ")
fi

if test -r "$cur_dir/../sip2xml/$csiplex_lib_name"; then
      csiplex_lib_dir="$cur_dir/../sip2xml"
else
      AC_MSG_ERROR(" $csiplex_lib_name not exist")
fi

csiplex_lib_name=`eval echo "$csiplex_lib_name" | sed s/lib// | sed s/\.a//`

changequote([, ])


CSIPLEX_INCLUDE=-I$csiplex_include_dir
CSIPLEX_LIB="-L$csiplex_lib_dir -l$csiplex_lib_name"

AC_SUBST(CSIPLEX_INCLUDE)
AC_SUBST(CSIPLEX_LIB)


])
