
// <block name="pipe_uavSender">
/////////////////////////////////////////////////////////////////////////////
// sender.c -- the implementation file for this node
/////////////////////////////////////////////////////////////////////////////
#include "sender.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>


static EControlContext ec;
static CManager cm;

// this probably shouldn't go here (bad form), but we'll cope for now
char *incomingGlobalID;

// definition of publish types
#define PUBLISH_FILE  0x0001


// functions publishing and looking up
static int publish(char *pipe, char *connType, char *connName, char *str, int type);
static int publishFile(char *pipe, char *connType, char *connName, char *str);
static int unpublish(char *pipe, char *connType, char *connName, int type);
static int unpublishFile(char *pipe, char *connType, char *connName);
static char *lookup(char *pipe, char *nodeOrBranch, char *connPoint, int type, char *str);
static char *lookupFile(char *pipe, char *connType, char *connName, char *str);


// <block name="datatype_Raw_data">

static IOField Raw_dataFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_dataStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_dataStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, maxval)},
  
  {"buff", "char[size]",sizeof(char), IOOffset( Raw_dataStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_dataFormatList[] = 
{
  {"Raw_dataStruct",Raw_dataFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1C">

static IOField Raw_data_1CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, maxval)},
  
  {"buff", "char[921600]",sizeof(char), IOOffset( Raw_data_1CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1CFormatList[] = 
{
  {"Raw_data_1CStruct",Raw_data_1CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1G">

static IOField Raw_data_1GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, maxval)},
  
  {"buff", "char[307200]",sizeof(char), IOOffset( Raw_data_1GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1GFormatList[] = 
{
  {"Raw_data_1GStruct",Raw_data_1GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1DC">

static IOField Raw_data_1DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_1DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1DCFormatList[] = 
{
  {"Raw_data_1DCStruct",Raw_data_1DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1DG">

static IOField Raw_data_1DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_1DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1DGFormatList[] = 
{
  {"Raw_data_1DGStruct",Raw_data_1DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2C">

static IOField Raw_data_2CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, maxval)},
  
  {"buff", "char[230400]",sizeof(char), IOOffset( Raw_data_2CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2CFormatList[] = 
{
  {"Raw_data_2CStruct",Raw_data_2CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2G">

static IOField Raw_data_2GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, maxval)},
  
  {"buff", "char[76800]",sizeof(char), IOOffset( Raw_data_2GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2GFormatList[] = 
{
  {"Raw_data_2GStruct",Raw_data_2GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2DC">

static IOField Raw_data_2DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_2DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2DCFormatList[] = 
{
  {"Raw_data_2DCStruct",Raw_data_2DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2DG">

static IOField Raw_data_2DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_2DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2DGFormatList[] = 
{
  {"Raw_data_2DGStruct",Raw_data_2DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3C">

static IOField Raw_data_3CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, maxval)},
  
  {"buff", "char[57600]",sizeof(char), IOOffset( Raw_data_3CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3CFormatList[] = 
{
  {"Raw_data_3CStruct",Raw_data_3CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3G">

static IOField Raw_data_3GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, maxval)},
  
  {"buff", "char[19200]",sizeof(char), IOOffset( Raw_data_3GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3GFormatList[] = 
{
  {"Raw_data_3GStruct",Raw_data_3GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3DC">

static IOField Raw_data_3DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_3DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3DCFormatList[] = 
{
  {"Raw_data_3DCStruct",Raw_data_3DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3DG">

static IOField Raw_data_3DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_3DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3DGFormatList[] = 
{
  {"Raw_data_3DGStruct",Raw_data_3DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4C">

static IOField Raw_data_4CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, maxval)},
  
  {"buff", "char[14400]",sizeof(char), IOOffset( Raw_data_4CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4CFormatList[] = 
{
  {"Raw_data_4CStruct",Raw_data_4CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4G">

static IOField Raw_data_4GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, maxval)},
  
  {"buff", "char[4800]",sizeof(char), IOOffset( Raw_data_4GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4GFormatList[] = 
{
  {"Raw_data_4GStruct",Raw_data_4GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4DC">

static IOField Raw_data_4DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_4DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4DCFormatList[] = 
{
  {"Raw_data_4DCStruct",Raw_data_4DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4DG">

static IOField Raw_data_4DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_4DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4DGFormatList[] = 
{
  {"Raw_data_4DGStruct",Raw_data_4DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_rectangle">

static IOField rectangleFieldList[] = {
  
  {"x", "integer ",sizeof(int), IOOffset( rectangleStruct*, x)},
  
  {"y", "integer ",sizeof(int), IOOffset( rectangleStruct*, y)},
  
  {"width", "integer ",sizeof(int), IOOffset( rectangleStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( rectangleStruct*, height)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec rectangleFormatList[] = 
{
  {"rectangleStruct",rectangleFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_factor">

static IOField factorFieldList[] = {
  
  {"factor", "integer ",sizeof(int), IOOffset( factorStruct*, factor)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec factorFormatList[] = 
{
  {"factorStruct",factorFieldList},
  { NULL, NULL}
};


// </block>
 


int makePathDir(char *path, mode_t mode)
{
  char *dir; 
  char *p;
  mode = 0777;

  dir = (char *)malloc(sizeof(char) * (strlen(path) + 1));

  p = path;

  while(1)
  {
    p = strchr(p, '/');
    if (p == NULL) 
    {
      free( dir );
      return 0;
    }

    p++;
    // get first dir name including '/'
    memcpy(dir, path, p - path);
    dir[p - path] = 0;

    // make directory
    if (mkdir(dir, mode))
    {
      // if dir exists already, continue
      // otherwise(error occurs), return -1
      if (errno != EEXIST)
      {
        free(dir);
        return -1;
      }
    }
  }

  // cannot reach here
  return 0;
}

// publish
static int publish(char *pipe, char *connType, char *connName, char *str, int type)
{
  if (type & PUBLISH_FILE)
  {
    publishFile(pipe, connType, connName, str);
  }

  return 0;
}

static int publishFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ofile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc( sizeof(char) * (strlen(publishRoot) + strlen(pipe) 
                                             + strlen(connType) + strlen(connName)
                                             + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  makePathDir(filename,0);

  printf("publishing file = %s\n",  filename);

  ofile = fopen(filename,"w");
  if (ofile)
  {
    fprintf(ofile,"%s\n",str);
  }
  else
    printf("Error in publishing file %s\n", filename);
 
  // close and write info
  fclose(ofile);
  return 0;
}


static int unpublish(char *pipe, char *connType, char *connName, int type)
{
  if (type & PUBLISH_FILE)
  {
    unpublishFile(pipe, connType, connName);
  }

  return 0;
}

static int unpublishFile(char *pipe, char *connType, char *connName)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *) malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                             strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  return remove(filename);
}


static char *lookup(char *pipe, char *connType, char *connName, int type, char *str)
{
  if (type & PUBLISH_FILE)
  {
    return lookupFile(pipe, connType, connName, str);
  }

}


static char *lookupFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ifile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                  		      strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  ifile = fopen(filename, "r");
  if (ifile && !feof(ifile))
  {
    fgets(str,strlen(str),ifile);
  }
  else
    fprintf(stderr,"Error in looking up file %s\n",filename); 

  return str;
}




// variables to track

// channel vars for sending info 
static ECSourceHandle sender__send1SendHandle;
static EChannel sender__send1SendChannel;
//Raw_dataCarrierStruct send1_outgoingData;




extern IOContext ioc;
static int doPipeInit(void)
{
/*
  IOFormat format;

*/
  gen_pthread_init();
  cm = CManager_create();
//  forked = CMfork_comm_thread(cm);
//  if( !forked ) {
//    fprintf(stderr, "Fork of comm thread failed. Exiting.\n");
//    exit(1);
//  }

  ec = ECho_CM_init(cm);
  return 0;
}



int firstTime = 1;
/* start this sub-pipe */
void infopipeStart_sender(  )
{
  char lookupStr[512];

  if( firstTime )
  {
    firstTime = 0;
    doPipeInit();
  }



  /////////////////////////////////
  // OUTGOING 
  // create the outgoing channel
  sender__send1SendChannel = EChannel_typed_create( ec, Raw_dataFieldList, Raw_dataFormatList ); 
  if(sender__send1SendChannel == NULL )
    fprintf(stderr, "Error creating sender__send1SendChannel\n");
  publish( "sender", "","send1", ECglobal_id(sender__send1SendChannel), PUBLISH_FILE );
  //sender = EChannel_open( ec, ECglobal_id(senderSendChannel));
  // subscribe as a source 
  sender__send1SendHandle = ECsource_typed_subscribe( sender__send1SendChannel, Raw_dataFieldList, Raw_dataFormatList);


  //ec = ECho_CM_init();
  CMrun_network(cm);
}

// shuts down the infopipe system and cleans out directory entries
void infopipeStop_sender( )
{

// shutdown our outgoing pipes - the incoming pipes will
// be shutdown by the nodes upstream from us

  EChannel_destroy( sender__send1SendChannel );

  // unpublish our server info
  sender__send1SendChannel = EChannel_typed_create( ec, Raw_dataFieldList, NULL ); 
  if(sender__send1SendChannel == NULL )
    fprintf(stderr,"Error creating sender__send1SendChannel\n");
  unpublish( "sender", "","send1", PUBLISH_FILE );


}


/////////////////////////////////////////////////////////////////////////////
// functions for submitting data to net

int sender__send1Submit( Raw_dataStruct_ptr outgoingData )
{

  ECsubmit_typed_event( sender__send1SendHandle, outgoingData );
  return 0;
}

// ENDING OF IMP FILE FOR A NODE

int test_main( int argc, char *argv )
{
  //infopipeEnvironStartup();
  // start the pipe
  infopipeStart_sender();
  
  while( 1 )
    sleep( 1 );

  // close down the pipes
  infopipeStop_sender();

  // normal termination
  return 0;
}

// </block>
