/*
 * $Id: cfg_info_parse.c,v 1.1.1.1 2003/06/11 21:16:01 wyan Exp $
 * $Log: cfg_info_parse.c,v $
 * Revision 1.1.1.1  2003/06/11 21:16:01  wyan
 * demoHarness
 *
 * Revision 1.2  2002/11/22 00:34:10  wyan
 * *** empty log message ***
 *
 * $Revision: 1.1.1.1 $
 */
#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include "cfg_info_parse.h"
#include "const.h"

/* struct demo_info: see const.h*/
demo_info* curr_demo;

GScannerConfig gsconfig =
{
  (
   " \t\r\n"
   )                    /* cset_skip_characters */,
  (
   G_CSET_a_2_z
   "_"
   G_CSET_A_2_Z
   )                    /* cset_identifier_first */,
  (
   G_CSET_a_2_z
   "_-0123456789"
   G_CSET_A_2_Z
   )                    /* cset_identifier_nth */,
  ( "#\n" )             /* cpair_comment_single */,
  
  TRUE                  /* case_sensitive */,
  
  TRUE                  /* skip_comment_multi */,
  TRUE                  /* skip_comment_single */,
  TRUE                  /* scan_comment_multi */,
  TRUE                  /* scan_identifier */,
  FALSE                 /* scan_identifier_1char */,
  FALSE                 /* scan_identifier_NULL */,
  TRUE                  /* scan_symbols */,
  TRUE                  /* scan_binary */,
  TRUE                  /* scan_octal */,
  TRUE                  /* scan_float */,
  TRUE                  /* scan_hex */,
  TRUE                  /* scan_hex_dollar */,
  TRUE                  /* scan_string_sq */,
  TRUE                  /* scan_string_dq */,
  TRUE                  /* numbers_2_int */,
  FALSE                 /* int_2_float */,
  FALSE                 /* identifier_2_string */,
  TRUE                  /* char_2_token */,
  TRUE                  /* symbol_2_token */,
  FALSE                 /* scope_0_fallback */,
};


typedef enum {
  ISG_TOKEN_INVALID = G_TOKEN_LAST,
  ISG_TOKEN_demo,
  ISG_TOKEN_cfg_file,
  ISG_TOKEN_demo_src_file,
  ISG_TOKEN_demo_isg_file,
  ISG_TOKEN_gencode_file,
  ISG_TOKEN_SIP_file,
  ISG_TOKEN_XIP_file,
  ISG_TOKEN_REAL_SIP_file,
  ISG_TOKEN_REAL_XIP_file,
  ISG_TOKEN_stats_file,
  ISG_TOKEN_run_isg_script,
  ISG_TOKEN_file_name,
  BLK_TOKEN_TAG,/*This is used in looking for blk info in SIP, XIP*/
  BLK_TOKEN_ATTR,/*This is used in looking for blk info in SIP, XIP*/
  ISG_TOKEN_LAST
} ISG_token_type;

struct
{
  gchar *name;
  guint token;
} symbols[] = {
  { "demo",             ISG_TOKEN_demo },
  { "cfg_file",         ISG_TOKEN_cfg_file },
  { "demo_src_file",    ISG_TOKEN_demo_src_file},
  { "demo_isg_file",ISG_TOKEN_demo_isg_file},
  { "gencode_file",     ISG_TOKEN_gencode_file},
  { "SIP_file",         ISG_TOKEN_SIP_file},
  { "XIP_file",         ISG_TOKEN_XIP_file},
  { "REAL_SIP_file",    ISG_TOKEN_REAL_SIP_file},
  { "REAL_XIP_file",    ISG_TOKEN_REAL_XIP_file},
  { "stats_file",       ISG_TOKEN_stats_file},
  { "run_isg_script",   ISG_TOKEN_run_isg_script},
  { "file_name",        ISG_TOKEN_file_name},
  { "block",            BLK_TOKEN_TAG},
  { "num",              BLK_TOKEN_ATTR}
};


//static int get_value = 0;

gint open_file_fd (char* path, int flags)
{
  guint file;
  if ( (file=open(path, flags )) == -1) {
    ERR1(" [open_file_fd(): Error reading file %s\n",path);
    return -1;
  }

  return file;
}

void parse_error(guint token)
{
  int i;
  for (i=0; i < G_N_ELEMENTS (symbols) ; i++) {
    if (token == symbols[i].token) {
      ERR1(" Error: token %s expected!\n",symbols[i].name);
      return;
    }
  }
 
  ERR(" Error: unexpected token!\n");
}

/*
 * Get the name of each demo.
 *  Used in parsing 'demos.rc'
 */
guint parse_demo_name (GScanner* scanner)
{
  guint token;
  token = g_scanner_get_next_token(scanner);//eat 'demo'
  token = g_scanner_peek_next_token(scanner);

  if (token != G_TOKEN_STRING) {
    parse_error(G_TOKEN_STRING);
  }

  curr_demo = (demo_info*)g_malloc(sizeof(demo_info));
  //Init to NULL
  curr_demo->name = curr_demo->cfg_file = curr_demo->demo_src_file =
    curr_demo->demo_isg_file = curr_demo->gencode_file = 
    curr_demo->SIP_file = curr_demo->XIP_file = curr_demo->REAL_SIP_file
    = curr_demo->REAL_XIP_file = curr_demo->stats_file 
    = curr_demo->run_isg_script = NULL;
  curr_demo->name = g_strdup(scanner->next_value.v_string);

  return G_TOKEN_STRING;
}

/*
 * Get the file name of the cfg file for each demo.
 * Used in parsing 'demos.rc'
 */
guint parse_demo_cfg_file (GScanner* scanner)
{
  guint token;
  token = g_scanner_get_next_token(scanner);//eat 'cfg_file'
  token = g_scanner_peek_next_token(scanner);

  if (token != '=') {
    parse_error('=');
  }

  token = g_scanner_get_next_token(scanner);//eat '='
  token = g_scanner_peek_next_token(scanner);
  if (token != G_TOKEN_STRING) {
    parse_error(G_TOKEN_STRING);
  }
  
  curr_demo->cfg_file = g_strdup(scanner->next_value.v_string);
  ALL_DEMO = g_list_append (ALL_DEMO, curr_demo);

  return G_TOKEN_STRING;
}
  
/* 
 * Start the parsing for 'demos.rc'
 */
guint parse_all_demo_statement (GScanner *scanner) 
{
  guint token;
  token = g_scanner_peek_next_token(scanner);

  switch (token) 
    {
    case '{':
      token = g_scanner_get_next_token(scanner);
      return token;

    case ISG_TOKEN_demo:
      parse_demo_name(scanner);
      token = g_scanner_get_next_token(scanner);
      return ISG_TOKEN_demo;

    case '}':
      token = g_scanner_get_next_token(scanner);
      return '}';
	
    case ISG_TOKEN_cfg_file:     
      parse_demo_cfg_file(scanner);
      token = g_scanner_get_next_token(scanner);
      return ISG_TOKEN_cfg_file;

    default:
      token = g_scanner_get_next_token(scanner);
      return G_TOKEN_NONE;
    }
}

/*
 *  Used in parsing 'demos.rc'
 */
void
parse_all_demo (GScanner* scanner)
{
  gint i;
  gboolean done;

  for (i = 0; i < G_N_ELEMENTS (symbols); i++)
      g_scanner_scope_add_symbol (scanner, 0, symbols[i].name, GINT_TO_POINTER (symbols[i].token));
      
  done = FALSE;

  while (!done)
    {
      if (g_scanner_peek_next_token (scanner) == G_TOKEN_EOF)
        done=TRUE;
      else
        {
          guint expected_token;
          
          expected_token = parse_all_demo_statement (scanner);

          if (expected_token != G_TOKEN_NONE)
            {
              gchar *symbol_name;
              gchar *msg;
              //MSG2( " Symbol: %d, Curr_pos: %d", expected_token,
	      //      g_scanner_cur_position(scanner));
              msg = NULL;
              symbol_name = NULL;

            }
	  else {
	    ERR("G_TOKEN_NONE");
	  }
	}
    }
}



/* 
 * Get the root dir to be used in fill in the various string file name list
 * Used in parsing each demo cfg file like 'UAV.conf'.
 * demo: the name of the current demo
 * root: the returned root dir stored here.
 */
guint parse_root_path (GScanner* scanner,  gchar** root)
{
  guint  token;

  token = g_scanner_get_next_token(scanner);//eat token
  token = g_scanner_peek_next_token(scanner);

  if (token != G_TOKEN_STRING) {
    parse_error(G_TOKEN_STRING);
  }

  *root = g_strdup(scanner->next_value.v_string);

  return G_TOKEN_STRING;
}


 
/* 
 * Append the found file names to various string file name list
 * Used in parsing each demo cfg file like 'UAV.conf'.
 * demo: the name of the current demo
 * root: the returned root dir stored here.
 * file_type: which file name list should be appended.
 */
guint parse_file_names (gchar* demo,
		       GScanner* scanner,  gchar* root, guint file_type)
{
  guint i, token;
  demo_info* df = NULL;
  gchar* str;

  //find the related element in ALL_DEMO fisrt
  for (i=0; i< g_list_length(ALL_DEMO); i++) {
    df = (demo_info*)g_list_nth_data(ALL_DEMO, i);
    if (NULL != df) {
      if ( 0 == g_ascii_strcasecmp(df->name, demo) ) {
	break;
      }
    }
  }

  if (NULL == df) {
    ERR1(" [parse_file_names(): Name %s doesn't match!\n", demo);
    exit(-1);
  }

  token = g_scanner_get_next_token(scanner);//eat ISG_TOKEN_file_name
  token = g_scanner_peek_next_token(scanner);

  if (token != '=') {
    parse_error('=');
  }

  token = g_scanner_get_next_token(scanner);//eat '='
  token = g_scanner_peek_next_token(scanner);
  if (token != G_TOKEN_STRING) {
    parse_error(G_TOKEN_STRING);
  }

  str = g_strconcat(root,"/", scanner->next_value.v_string,NULL);
 
  switch (file_type) 
    {
    case ISG_TOKEN_demo_src_file:
      df->demo_src_file = g_list_append(df->demo_src_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_demo_isg_file:
      df->demo_isg_file = g_list_append(df->demo_isg_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_gencode_file:
      df->gencode_file = g_list_append(df->gencode_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_SIP_file:
      df->SIP_file = g_list_append(df->SIP_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_XIP_file:
      df->XIP_file = g_list_append(df->XIP_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_REAL_SIP_file:
      df->REAL_SIP_file = g_list_append(df->REAL_SIP_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_REAL_XIP_file:
      df->REAL_XIP_file = g_list_append(df->REAL_XIP_file,
					      g_strdup(str));
      break;
    case ISG_TOKEN_stats_file:
      df->stats_file = g_strdup(str);
      break;
    case ISG_TOKEN_run_isg_script:
      df->run_isg_script = g_strdup(str);
      break;
    default:
      ERR(" [parse_file_names(): Error file_type");
    }

  g_free(str);

  return G_TOKEN_STRING;  
}



/* 
 *  Start parsing.
 *  Used in parsing each demo cfg file like 'UAV.conf'.
 * demo: the name of the current demo
 * root: the returned root dir stored here.
 */
guint parse_conf_statement (gchar* demo, GScanner *scanner) 
{
  guint token;
  static guint file_type;
  static gchar* root;
  token = g_scanner_peek_next_token(scanner);


  switch (token) 
    {
    case ISG_TOKEN_demo_src_file:
    case ISG_TOKEN_demo_isg_file:
    case ISG_TOKEN_gencode_file:
    case ISG_TOKEN_SIP_file:
    case ISG_TOKEN_XIP_file:
    case ISG_TOKEN_REAL_SIP_file:
    case ISG_TOKEN_REAL_XIP_file:
    case ISG_TOKEN_stats_file:
    case ISG_TOKEN_run_isg_script:
      parse_root_path (scanner , &root);
      file_type = token;
      token = g_scanner_get_next_token(scanner);
      return token;

    case ISG_TOKEN_file_name:
      parse_file_names (demo, scanner, root, file_type);
      token = g_scanner_get_next_token(scanner);
      return ISG_TOKEN_file_name;

    case '{':
      token = g_scanner_get_next_token(scanner);
      return token;

    case '}':
      token = g_scanner_get_next_token(scanner);
      return token;

    default:
      token = g_scanner_get_next_token(scanner);
      return G_TOKEN_NONE;
    }
}


/* 
 *  Used in parsing each demo cfg file like 'UAV.conf'.
 * demo: the name of the current demo
 */
void parse_conf_file(gchar* demo, GScanner* scanner)
{
  gint i;
  gboolean done;

  if (!scanner) 
    for (i = 0; i < G_N_ELEMENTS (symbols); i++)
      g_scanner_scope_add_symbol (scanner, 0, symbols[i].name, GINT_TO_POINTER (symbols[i].token));
      
  done = FALSE;

  while (!done)
    {
      if (g_scanner_peek_next_token (scanner) == G_TOKEN_EOF)
        done=TRUE;
      else
        {
          guint expected_token;
          
          expected_token = parse_conf_statement (demo, scanner);

          if (expected_token != G_TOKEN_NONE)
            {
              gchar *symbol_name;
              gchar *msg;
              
              msg = NULL;
              symbol_name = NULL;
            }
	  else {
	    ERR("G_TOKEN_NONE\n");
	  }
	}
    }

}
  
