/* 
 * $Id: pixbufs.c,v 1.1.1.1 2003/06/11 21:16:01 wyan Exp $
 * file: pixbufs.c
 * Some of them are borrowed from the examples coming with Gtk+-2.0.6
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>
#include <errno.h>
#include "pixbufs.h"



/* This notebook container is the top level widget holds animation stuff */
GtkWidget* notebook_img = NULL;


#if 0
GtkWidget* window_img_main = NULL;
#endif


guint timeout_id =0;
guint load_timeout = 0;

GdkPixbufLoader *pixbuf_loader = NULL;
FILE* text_img_stream = NULL;
gboolean text_img_loaded = TRUE;

GtkWidget *frame_text=NULL;
GtkWidget *image_event_box=NULL;
GtkWidget *image_text=NULL;

/* Current frame */
GdkPixbuf *pixbuf_logo;
GtkWidget* frame_logo= NULL;
int frame_num;

/* Background image */
GdkPixbuf *background;
gint back_width, back_height;

/* Images */
GdkPixbuf *images[N_IMAGES];

GtkWidget *da;



/*Error*/
#if 0
void error_msg(GtkWidget* window, GError* error)
{
  GtkWidget *dialog;
  dialog = gtk_message_dialog_new (GTK_WINDOW (window),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_ERROR,
				   GTK_BUTTONS_CLOSE,
				   "Failed to load an image: %s",
				   error->message);
  
  g_error_free (error);
  g_signal_connect (dialog, "response",
		    G_CALLBACK (gtk_widget_destroy), NULL);
  gtk_widget_show (dialog);
}
#endif


/* Loads the images and returns whether the operation succeeded */
gboolean
load_pixbufs (GError **error)
{
  gint i;

  if (background)
    return TRUE; /* already loaded earlier */

  background = gdk_pixbuf_new_from_file (BACKGROUND_NAME, error);
  
  if (!background)
    return FALSE; /* Note that "error" was filled with a GError */

  back_width = gdk_pixbuf_get_width (background);
  back_height = gdk_pixbuf_get_height (background);

  for (i = 0; i < N_IMAGES; i++)
    {
      images[i] = gdk_pixbuf_new_from_file (image_names[i], error);
      
      if (!images[i])
	return FALSE; /* Note that "error" was filled with a GError */
    }

  return TRUE;
}






/* Expose callback for the drawing area */
gint
expose_cb (GtkWidget	  *widget,
	   GdkEventExpose *event,
	   gpointer	   data)
{
  guchar *pixels;
  int rowstride;

  rowstride = gdk_pixbuf_get_rowstride (pixbuf_logo);

  pixels = gdk_pixbuf_get_pixels (pixbuf_logo) + rowstride * event->area.y 
           + event->area.x * 3;

  gdk_draw_rgb_image_dithalign (widget->window,
				widget->style->black_gc,
				event->area.x, event->area.y,
				event->area.width, event->area.height,
				GDK_RGB_DITHER_NORMAL,
				pixels, rowstride,
				event->area.x, event->area.y);

  return TRUE;
}










/* Timeout handler to regenerate the frame */
gint
timeout (gpointer data)
{
  double f;
  int i;
  double xmid, ymid;
  static double factor = 0;

  if (text_img_loaded) {
 
    gdk_pixbuf_copy_area (background, 0, 0, back_width, back_height,
			  pixbuf_logo, 0, 0);

    f = (double) (frame_num % CYCLE_LEN) / CYCLE_LEN;

    xmid = back_width / 2.0;
    ymid = back_height / 2.0;

    for (i = 0; i < N_IMAGES; i++)
      {
	int xpos, ypos;
	int iw, ih;

	GdkRectangle r1, r2, dest;
	
	iw = gdk_pixbuf_get_width (images[i]);
	ih = gdk_pixbuf_get_height (images[i]);
	
	ypos = f * (back_height-ih);

	r1.y = ypos;

	/* If the logo passes the middle of the window, it prolongs back 
	   in the x-direction again */
	if (r1.y + ih/2 < back_height/2) {
	  factor += (double)1/CYCLE_LEN;	
	}
	else {
	  factor -= (double)1/CYCLE_LEN;
	}

	if (frame_num % CYCLE_LEN == 0)
	  factor = 0;

	/* The logo coordinates */
	r1.x = floor (xmid - iw*(1-factor) / 2.0 + 0.5);
	r1.width = (1-factor)*iw;
	r1.height = (1+factor) *ih;
	xpos = r1.x;

	/* The rectangular */
	r2.x = 0;
	r2.y = 0;
	r2.width = back_width;
	r2.height = back_height;

	if (gdk_rectangle_intersect (&r1, &r2, &dest))
	  gdk_pixbuf_composite (images[i],
				pixbuf_logo,
				dest.x, dest.y,
				dest.width, dest.height,
				xpos, ypos,
				1-factor, 1+factor,
				GDK_INTERP_NEAREST,
				255);
        /* If the logo reaches the bottom of the window, starts to 
	   progressive loading of the text img. 
	   The timeout signal shall be updated, because it seems like
	   Gtk+ has problem if more than 2 timeout callbacks in a process.
	*/
	if (r1.y + ih/ADJUST >= back_height-ih) {
	  if (timeout_id>0) g_source_remove (timeout_id);	  
	  /* Indicates the timeout shall be reinstalled*/
	  timeout_id = 0;

	  if (load_timeout<=0)
	    load_timeout = gtk_timeout_add (150,
					    progressive_timeout,
					    image_text);
	  text_img_loaded = FALSE;
	  /* Just to refresh the previous text img even before it
	     has been displayed
	  */
	  progressive_timeout(image_text);
	  progressive_timeout(image_text);
	  /* Change to the text img page */
	  gtk_notebook_set_page(GTK_NOTEBOOK(notebook_img), 1);
	  return TRUE;
	}
      }

    gtk_widget_queue_draw (da);
    frame_num++;
    /* Must return TRUE to let the event signal processing stop here
       and prevent it from further propagating to the parent widgets.
    */
    return TRUE;
  }

}






gboolean
cleanup_callback (GtkObject *object,
		  gpointer   data)
{
  if (timeout_id>0) g_source_remove (timeout_id);
  if (load_timeout>0) g_source_remove (load_timeout);

  timeout_id = 0;
  load_timeout =0 ;

  gtk_object_sink(GTK_OBJECT(notebook_img));
  if (notebook_img) gtk_widget_destroy(notebook_img);
  //if (window_img_main) gtk_widget_destroy(window_img_main);
  return TRUE;
}









void do_pixbufs ()
{
      GError *error;

      if (!frame_logo) {

	error = NULL;
	if (!load_pixbufs (&error))
	  {
	    fprintf(OUT, " [do_pixbufs(): Error\n");
	  }
	else
	{
	  frame_logo = gtk_frame_new(NULL);	  

	  gtk_widget_set_size_request (GTK_WIDGET(frame_logo), back_width, 
	  		       back_height);
	  gtk_widget_show(frame_logo);
	  gtk_container_add(GTK_CONTAINER(notebook_img), frame_logo);

	  pixbuf_logo = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, 
				  back_width, back_height);

	  da = gtk_drawing_area_new ();

	  gtk_container_add (GTK_CONTAINER (frame_logo), da);
	  gtk_widget_show(da);

	  g_signal_connect (da, "expose_event",
			    G_CALLBACK (expose_cb), NULL);
	  
	  g_signal_connect (G_OBJECT(da), "button_press_event", 
			    G_CALLBACK(on_window_clicked),NULL);	  
	  /* This should be added after the window has been realized??*/
	  gtk_widget_set_events (da, GDK_EXPOSURE_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_POINTER_MOTION_MASK
                         | GDK_POINTER_MOTION_HINT_MASK);

	  if (timeout_id<=0)
	    timeout_id = g_timeout_add (FRAME_DELAY, timeout, NULL);
	}
      }
}







void
progressive_prepared_callback (GdkPixbufLoader *loader,
                               gpointer         data)
{
  GdkPixbuf *pixbuf;
  GtkWidget *image;

  image = GTK_WIDGET (data);
  
  pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);

  gdk_pixbuf_fill (pixbuf, 0xaaaaaaff);
  
  gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
}







void
progressive_updated_callback (GdkPixbufLoader *loader,
                              gint                 x,
                              gint                 y,
                              gint                 width,
                              gint                 height,
                              gpointer     data)
{
  GtkWidget *image;
  
  image = GTK_WIDGET (data);

  gtk_widget_queue_draw (image);
}







gint
progressive_timeout (gpointer data)
{
  GtkWidget *img;

  img = GTK_WIDGET (data);

  if (!text_img_loaded) {
    if (text_img_stream)
      {
	size_t bytes_read;
	guchar buf[256];
	GError *error = NULL;
	
	bytes_read = fread (buf, 1, 256, text_img_stream);
	
	if (ferror (text_img_stream))
	  {
	    //error_msg(window_img_main, error);
	    fprintf(OUT, " [progressive_timeout(): Error\n");

	    fclose (text_img_stream);
	    text_img_stream = NULL;
	    
	    load_timeout = 0;

	    return FALSE; /* uninstall the timeout */
	  }

	if (!gdk_pixbuf_loader_write (pixbuf_loader,
				      buf, bytes_read,
				      &error))
	  {
	    //error_msg(window_img_main, error);
	    fprintf(OUT, " [progressive_timeout(): Error\n");
	    
	    fclose (text_img_stream);
	    text_img_stream = NULL;

	    load_timeout = 0;

	    return FALSE; /* uninstall the timeout */
	  }

	if (feof (text_img_stream))
	  {
	    fclose (text_img_stream);
	    text_img_stream = NULL;

	    error = NULL;
	    if (!gdk_pixbuf_loader_close (pixbuf_loader,
					  &error))
	      {
		//error_msg(window_img_main, error);
		fprintf(OUT, " [progressive_timeout(): Error\n");
		g_object_unref (G_OBJECT (pixbuf_loader));
		pixbuf_loader = NULL;
              
		load_timeout = 0;

		return FALSE; /* uninstall the timeout */
	      }
          
	    g_object_unref (G_OBJECT (pixbuf_loader));
	    pixbuf_loader = NULL;

	    /* Process reaches here means the text img has been fully
	       loaded/displayed. We need then switch back to load the
	       logo. */
	    frame_num = 0;
	    if (load_timeout>0) g_source_remove(load_timeout);
	    /* Indicates load_timeout shall be reinstalled */
	    load_timeout = 0;

	    if (timeout_id<=0)
	      timeout_id = g_timeout_add (FRAME_DELAY, timeout, NULL);
	    text_img_loaded = TRUE;
	    /* Refresh the previously displayed logo before it even
	       will be displayed */
	    timeout(NULL);
	    gtk_notebook_set_page(GTK_NOTEBOOK(notebook_img), 0);
	  }
      }
    else
      {
	//gchar *error_message = NULL;
	//GError *error = NULL; 

	if (!(text_img_stream = fopen (TEXT_IMAGE, "r")))
	  {
	    fprintf(OUT, " [progressive_timeout(): Error: open file %s\n",
		    TEXT_IMAGE);
	  }

	if (text_img_stream == NULL)
	  {
	    //error_msg(window_img_main, error);
	    fprintf(OUT, " [progresssive_timeout(): Error\n");
	    load_timeout = 0;

	    return FALSE;
	  }

	if (pixbuf_loader)
	  {
	    gdk_pixbuf_loader_close (pixbuf_loader, NULL);
	    g_object_unref (G_OBJECT (pixbuf_loader));
	    pixbuf_loader = NULL;
	  }
      
	pixbuf_loader = gdk_pixbuf_loader_new ();

	g_signal_connect (G_OBJECT (pixbuf_loader), "area_prepared",
			  G_CALLBACK (progressive_prepared_callback), img);
      
	g_signal_connect (G_OBJECT (pixbuf_loader), "area_updated",
			  G_CALLBACK (progressive_updated_callback), img);
      }
    /* leave timeout installed */
    return TRUE;
  }
}






void
do_load_text_image (void)
{
   if (!frame_text)
    {
      frame_text = gtk_frame_new (NULL);
      gtk_widget_show(frame_text);
      gtk_frame_set_shadow_type (GTK_FRAME (frame_text), GTK_SHADOW_IN);
      gtk_container_add (GTK_CONTAINER (notebook_img), frame_text);

      image_event_box = gtk_event_box_new();
      gtk_widget_show(image_event_box);
      gtk_container_add(GTK_CONTAINER(frame_text), image_event_box);

      image_text = gtk_image_new_from_pixbuf (NULL);
      gtk_widget_show(image_text);
      gtk_container_add(GTK_CONTAINER(image_event_box), image_text);

      g_signal_connect (G_OBJECT(image_event_box), "button_press_event", G_CALLBACK(on_window_clicked),NULL);

      if (load_timeout<=0)
	load_timeout = gtk_timeout_add (150,
				      progressive_timeout,
				      image_text);

    }
}



      
gboolean on_window_clicked(GtkWidget* widget, GdkEventButton* event,
			   gpointer data)
{
  cleanup_callback(NULL,NULL);

  //gtk_widget_destroy(window_img_main);

  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook_top), 1);
  //gtk_exit(0);
  return TRUE;
}






void init_load(GtkWidget* container)
{
#if 0
  if (!window_img_main) {
    window_img_main = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window_img_main), "Animation");
    gtk_window_set_resizable (GTK_WINDOW (window_img_main), TRUE);

    g_object_set_data (G_OBJECT (window_img_main), "window_img_main", 
		       window_img_main);

  }

  gtk_container_add(GTK_CONTAINER(notebook_top), window_img_main);
#endif

  notebook_img = gtk_notebook_new();    
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook_img), FALSE);
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook_img), FALSE);

  gtk_widget_show(notebook_img);
  gtk_container_add(GTK_CONTAINER(/*window_img_main*/container),
		    notebook_img);

  do_pixbufs();
  do_load_text_image();

}





#if 0

int main(int argc, gchar* argv[])
{
  gtk_set_locale();
  gtk_init(&argc, &argv);

  init_load();

  if (!GTK_WIDGET_VISIBLE (window_img_main))
    {
      gtk_widget_show_all(window_img_main);
    }
  else
    {
      gtk_widget_destroy (window_img_main);
      window_img_main = NULL;
    }
  
  gtk_main();
  
  return 0;
}

#endif
