#ifndef __CMRPC_H__
#define __CMRPC_H__

/*
 *
 *  $Id: cmrpc.h,v 1.17 2000/10/27 19:28:54 pmw Exp $
 *
 */
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>

#include "atl.h"
#include "cm.h"


#if defined(FUNCPROTO) || defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
#ifndef ARGS
#define ARGS(args) args
#endif
#else
#ifndef ARGS
#define ARGS(args) (/*args*/)
#endif
#endif

extern int CM_RPC_errno;

typedef struct
{
  CManager cm;
  void* channel;
  unsigned long flags;
} *CM_RPC_options, CM_RPC_options_t;

typedef void (*CM_RPC_HandlerFunc) ARGS((void* in_param_msg,
                                         void* out_param_msg,
                                         CM_RPC_options options));

typedef void (*CM_RPC_OnewayHandlerFunc) ARGS((void* in_param_msg, CM_RPC_options options));

/*
 * option flags for handler writers to use
 */
#define CM_RPC_set_option(X,OPT) ((X)->flags |= (OPT))
#define CM_RPC_clear_option(X,OPT) ((X)->flags &= ~(OPT))

#define CM_RPC_call_cleanup_function (0x1)


/*
 * option flags for RPC calls
 */
#define CM_RPC_async_call (0x1)


typedef void (*CM_RPC_HandlerCleanupFunc) ARGS((CManager cm,
                                                void* out_msg));

struct _CM_RPC_ticket_struct;
typedef struct _CM_RPC_ticket_struct *CM_RPC_ticket;


extern int
CM_RPC_register_rpc_format ARGS((CManager cm,
                                 char* rpc_name,
                                 IOFieldList in_field_list,
                                 CMFormatList in_subformats,
                                 IOFieldList out_field_list,
                                 CMFormatList out_subformats));


extern CM_RPC_ticket
CM_RPC_call_rpc ARGS((CManager cm,
                      char* rpc_name,
                      attr_list contact_info,
                      void* in_data,
                      void* out_data,
                      unsigned long flags));

extern void
CM_RPC_call_rpc_oneway ARGS((CManager cm,
                             char* rpc_name,
                             attr_list contact_info,
                             void* in_data,
                             unsigned long flags));

extern int
CM_RPC_register_rpc_contact_info ARGS((CManager cm,
                                       char* rpc_name,
                                       attr_list contact_info));

extern CM_RPC_ticket
CM_RPC_call_anon_rpc ARGS((CManager cm,
                           char* rpc_name,
                           attr_list target_spec,
                           IOContext anon_in_context,
                           void* in_data,
                           IOContext anon_out_context,
                           void* out_data,
                           unsigned long flags));

extern int
CM_RPC_redeem_ticket ARGS((CManager cm,
                           CM_RPC_ticket ticket,
                           void* out_data,
                           int wait));

extern int
CM_RPC_register_rpc_handler ARGS((CManager cm,
                                  char* rpc_name,
                                  CM_RPC_HandlerFunc func,
                                  CM_RPC_OnewayHandlerFunc oneway_func,
                                  CM_RPC_HandlerCleanupFunc cleanup_func));

extern int
CM_RPC_unregister_rpc ARGS((char* rpc_name));

#ifdef CMRPC_WITH_CHANNEL

/*
 *  I cheaply steal the typedef from ECho so I don't have to include
 *  the header file.
 */
typedef void (*CM_RPC_Chan_HandlerFunction) ARGS((void *event, int length,
                                                  void *client_data));

extern int
CM_RPC_call_rpc_with_channel ARGS((CManager cm,
                                   char* rpc_name,
                                   attr_list contact_info,
                                   void* in_data,
                                   void* out_data,
                                   unsigned long flags));

extern int
CM_RPC_register_chan_handler ARGS((CManager cm,
                                   char* rpc_name,
                                   CM_RPC_Chan_HandlerFunction chan_handler));

#endif /* CMRPC_WITH_CHANNEL */
                                   

/*
 *
 *  $Log: cmrpc.h,v $
 *  Revision 1.17  2000/10/27 19:28:54  pmw
 *  Removed CM_RPC_register_rpc_subformat().  I didn't understand how CM
 *  worked well enough; this function is really not necessary.
 *
 *  Revision 1.16  2000/10/23 20:52:09  pmw
 *  Added contact_info parameter to call_rpc calls.  This is to be used
 *  if, for a specific invocation of an RPC, you wish to override the
 *  registered contact information.
 *
 *  Revision 1.15  2000/10/16 03:45:35  pmw
 *  Added oneway RPC calls.  This has been tested somewhat.
 *
 *  Revision 1.14  2000/10/16 02:31:21  pmw
 *  1 - fixed gotcha-in-waiting concerning IOcontext management and anonymous
 *  calls.  The IOcontext allocated for anon calls was getting freed before the
 *  user ever saw the result structure from the call, and the underlying PBIO
 *  buffer was suspect.  This meant revising the anon-call interface back to
 *  where the user supplies his own IOcontext values.
 *  2 - Added rpc-with-channel functionality, not yet tested.
 *
 *  Revision 1.13  2000/09/29 20:52:07  pmw
 *  fixed lingering problems from subcontext changes.  hopefully this closes it.
 *  make check works correctly, so I'll believe it for the moment.
 *
 *  Revision 1.12  2000/09/19 20:54:18  pmw
 *  changed syntax of handlers to eliminate explicit passing of CManager value - now contained in options structure
 *
 *  Revision 1.11  2000/05/12 20:54:17  pmw
 *  Re-did anonymous call syntax to avoid PBIO memory-handling issues
 *
 *  Revision 1.10  2000/05/10 19:49:11  pmw
 *  removed extra free() which caused async test bug
 *
 *  Revision 1.9  2000/04/11 18:40:33  pmw
 *  added CM_RPC_errno
 *
 *  Revision 1.8  2000/04/10 21:22:17  pmw
 *  added options flag to handlers
 *
 *  Revision 1.7  2000/04/10 20:28:32  pmw
 *  Added cleanup handler functionality
 *
 *  Revision 1.6  2000/04/10 18:10:14  pmw
 *  more rearrangement
 *
 *  Revision 1.5  2000/04/06 22:39:18  pmw
 *  Now does sane handling of IO conversions
 *
 *  Revision 1.4  2000/04/05 03:00:05  pmw
 *  added high-level convenience functions
 *
 *  Revision 1.3  2000/04/04 20:40:04  pmw
 *  Restructured for asyncronous calls
 *
 *  Revision 1.2  2000/04/04 18:35:29  pmw
 *  added RCS headers
 *
 *
 */

#endif /* ifndef __CMRPC_H__ */
